package com.gx.obe.server.common.enumeration;

import com.gx.obe.server.common.utils.StringUtils;

public class EvaluationFactorEnum {
	
	/**
	 * 指标评审类型 [00:符合性评审 01:打分评审 02:价格 03:清单]
	 */
	public static final String AUDIT_COMPLIANCE = "00";
	public static final String AUDIT_SCORE = "01";
	public static final String AUDIT_PRICE = "02";
	public static final String AUDIT_BILL = "03";
	public static final String AUDIT_CREDIT = "04";
	
	/**
	 * 评标类型[00:默认为商务技术;01:商务标;02:技术标;03:经济标;04:综合信用]
	 */
	public static final String EVAL_BID_DETAIL = "00";
	public static final String EVAL_BID_BUS = "01";
	public static final String EVAL_BID_TEC = "02";
	public static final String EVAL_BID_PRICE = "03";
	public static final String EVAL_BID_CREDIT = "04";
	
	public static String getEvalBidTypeCN(String evalBidType) {
		if (null == evalBidType || evalBidType.length() == 0) {
			return "";
		}
		if (evalBidType.equals(EVAL_BID_BUS)) {
			return "商务";
		}
		if (evalBidType.equals(EVAL_BID_TEC)) {
			return "技术";
		}
		if (evalBidType.equals(EVAL_BID_PRICE)) {
			return "经济";
		}
		if (evalBidType.equals(EVAL_BID_CREDIT)) {
			return "综合信用";
		}
		if (evalBidType.equals(EVAL_BID_DETAIL)) {
			return "";
		}
		return "";
	}
	
	
	/**
	 * @Description: 获得评标分组
	 * @author guoyr
	 * @param evalBid
	 * @param index
	 * @return 
	 */
	public static final String[] getEvalGroup(String evalGroup){
		if(null != evalGroup && evalGroup.length() > 0){
			String[] evalGroupArray = evalGroup.toString().split("_");
			if(evalGroupArray.length == 2){
				return getEvalGroup(evalGroupArray[0], Integer.parseInt(evalGroupArray[1]));
			}
		}
		return new String[]{null, ""};
	}
	
	/**
	 * @Description: 获得评标分组
	 * @author wangxiang
	 * @param evalGroup
	 * @return 
	 */
	public static final String getEvalGroupName(String evalGroup){
		if(null != evalGroup && evalGroup.length() > 0){
			String[] evalGroupArray = evalGroup.toString().split("_");
			if(evalGroupArray.length == 2){
				if(null != evalGroupArray[0] && evalGroupArray[0].equals(EvaluationFactorEnum.EVAL_BID_BUS)){
					return "B"+Integer.parseInt(evalGroupArray[1]);
				}else if(null != evalGroupArray[0] && evalGroupArray[0].equals(EvaluationFactorEnum.EVAL_BID_TEC)){
					return "T"+Integer.parseInt(evalGroupArray[1]);
				}
			}
		}
		return "G";
	}
		
		/**
		 * @Description: 获得评标分组
		 * @author guoyr
		 * @param evalBid
		 * @param index
		 * @return 
		 */
		public static final String[] getEvalGroup(String evalBid, int index){
		if(null != evalBid && evalBid.equals(EvaluationFactorEnum.EVAL_BID_BUS)){
			return new String[]{EvaluationFactorEnum.EVAL_BID_BUS+"_" + index, "商务" + index + "单元"};
		}else if(null != evalBid && evalBid.equals(EvaluationFactorEnum.EVAL_BID_TEC)){
				return new String[]{EvaluationFactorEnum.EVAL_BID_TEC+"_" + index, "技术" + index + "单元"};
		}else {
			return new String[]{EvaluationFactorEnum.EVAL_BID_DETAIL+"_" + index, "评审" + index + "单元"};
		}
	}
	/**
	 * 指标类型[00:默认指标;01:基础指标;02:浮动指标;10:工程量清单指标:11:清价总价;12：分部分项;13：措施项]
	 */
	public static final String FACTOR_DEFAULT = "00";
	public static final String FACTOR_BASE = "01";
	public static final String FACTOR_FLOAT = "02";
	
	public static final String FACTOR_BILL = "10";
	
	public static final String FACTOR_TOTAL_PRICE = "11";
	public static final String FACTOR_ITEMIZE = "12";
	public static final String FACTOR_MEASURE = "13";
	
	public static final String TYPE_PRICE_CHECK = "21";
	public static final String TYPE_TOTAL_PRICE = "22";
	public static final String TYPE_ITEMIZE_PRICE = "23";
	public static final String TYPE_ITEMIZE_PROJECT = "24";
	public static final String TYPE_MEASURE = "25";
	public static final String TYPE_OTHER = "26";
	public static final String TYPE_MATERIAL_PRICE = "27";
	
	public static final String BZ_TYPE_TOTAL_PRICE = "31";
	public static final String BZ_TYPE_MAIN_PROJECT = "32";
	
	/**
	 * 检查类型[00:不是客观项;01:客观项检查;02:预警检查]
	 */
	public static final String OBJECTIVE_NO = "00";
	public static final String OBJECTIVE_YES = "01";
	public static final String OBJECTIVE_WARING = "02";
	
	/**
	 * 打分方式[01:主观直接打分;02:客观范围打分;03:通过性评审;04:智能评审]
	 */
	public static final String SCORE_TYPE_DIRECT = "01";
	public static final String SCORE_TYPE_OBJECTIVE = "02";
	public static final String SCORE_TYPE_PASS = "03";
	public static final String SCORE_TYPE_AI = "04";
	public static final String SCORE_TYPE_PASS_PARENT = "-1";
	
	public static final String getScoreTypeCN(String str) {
		String scoreTypeCN = "";
		if (StringUtils.isEmpty(str) || str.equals(SCORE_TYPE_DIRECT)) {
			scoreTypeCN = "主观直接打分";
		} else if (str.equals(SCORE_TYPE_OBJECTIVE)) {
			scoreTypeCN = "客观范围打分";
		} else if (str.equals(SCORE_TYPE_PASS)) {
			scoreTypeCN = "通过性评审";
		}
		return scoreTypeCN;
	}
	
	/**
	 * 条款类型[01:星号条款;02:偏离性条款]
	 */
	public static final String CLAUSE_TYPE_STAR = "01";
	public static final String CLAUSE_TYPE_DEVIATE = "02";
	
	/**
	 * 符合性评审状态[01：符合；02：不符合]
	 */
	public static final String CONFORM = "01";
	public static final String UNCONFORM = "02";
	
}
