package com.gx.obe.server.common.sftp;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.concurrent.TimeUnit;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import com.jcraft.jsch.ChannelSftp;
import com.jcraft.jsch.SftpException;

@Component
public class SFTPClientUtils {

	private static Logger LOGGER = LoggerFactory.getLogger(SFTPClientUtils.class);

	/**
	 * 下载文件。
	 * 
	 * @param directory    下载目录
	 * @param downloadFile 下载的文件
	 * @param saveFileFolder     存在本地的路径
	 */
	public synchronized static boolean download(String downloadFile, String saveFileFolder) {
		Path target = Paths.get(saveFileFolder);
		if (Files.notExists(target)) {
			try {
				Files.createDirectories(target.getParent());
			} catch (IOException e) {
				return false;
			}
		}
		boolean result = false;
		Integer i = 0;
		String directory = SftpConfiguration.ROOT;
		while (!result) {
			ChannelSftp sftp = SFTPConnectionFactory.getInstance().makeConnection();
			if (!StringUtils.isEmpty(directory)) {
				try {
					sftp.cd(directory);
				} catch (SftpException e) {
					LOGGER.error("sftp文件下载，目录不存在，错误信息" + e.getMessage());
				}
			}
			File file = new File(saveFileFolder + "\\" + downloadFile);
			FileOutputStream fileOutputStream = null;
			try {
				fileOutputStream = new FileOutputStream(file);
			} catch (FileNotFoundException e1) {
				LOGGER.error("sftp文件下载失败，本地目录不存在" + e1.getMessage());
			}
			try {
				sftp.get(downloadFile, fileOutputStream);
				if (i > 0) {
					LOGGER.info("sftp文件重试下载成功,sftp地址:" + directory + ",本地文件地址:" + saveFileFolder);
				} else {
					LOGGER.info("sftp文件下载成功,sftp地址:" + directory + ",本地文件地址:" + saveFileFolder);
				}
				result = true;
			} catch (SftpException e1) {
				i++;
				LOGGER.error("sftp文件下载失败，重试中。。。第" + i + "次，错误信息" + e1.getMessage());
				if (i > 3) {
					LOGGER.error("ftp文件下载失败，超过重试次数结束重试，错误信息" + e1.getMessage());
					return result;
				}
				try {
					TimeUnit.MILLISECONDS.sleep(15000);
				} catch (Exception e2) {
					e2.printStackTrace();
				}
			} finally {
				try {
					if (fileOutputStream != null) {
						fileOutputStream.close();
					}
				} catch (IOException e) {

					e.printStackTrace();
				}
			}
		}
		return result;
	}

	/**
	 * 删除文件
	 * 
	 * @param directory  要删除文件所在目录
	 * @param deleteFile 要删除的文件
	 */
	public synchronized static boolean delete(String directory, String deleteFile) {
		boolean result = false;
		ChannelSftp sftp = SFTPConnectionFactory.getInstance().makeConnection();
		try {
			sftp.cd(directory);
			sftp.rm(deleteFile);
		} catch (SftpException e) {
 			e.printStackTrace();
		}
		result = true;
		return result;
	}

}
