package com.gx.obe.server.common.sftp;

import java.util.Properties;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import com.jcraft.jsch.Channel;
import com.jcraft.jsch.ChannelSftp;
import com.jcraft.jsch.JSch;
import com.jcraft.jsch.JSchException;
import com.jcraft.jsch.Session;

@Component
 public class SFTPConnectionFactory {
	private static Logger LOGGER = LoggerFactory.getLogger(SFTPConnectionFactory.class);
	private static final SFTPConnectionFactory factory = new SFTPConnectionFactory();
	private ChannelSftp client;
	private Session session;
	
	private SFTPConnectionFactory() {

	}

	public static SFTPConnectionFactory getInstance() {
		return factory;
	}

	synchronized public ChannelSftp makeConnection() {
		if (client == null || session == null || !client.isConnected() || !session.isConnected()) {
			try {
				JSch jsch = new JSch();
				session = jsch.getSession(SftpConfiguration.USERNAME, SftpConfiguration.HOST, SftpConfiguration.PORT);
				if (SftpConfiguration.PASSWORD != null) {
					session.setPassword(SftpConfiguration.PASSWORD);
				}
				Properties config = new Properties();
				config.put("StrictHostKeyChecking",SftpConfiguration.SESSIONSTRICTHOSTKEYCHECKING);
				session.setConfig(config);
				session.connect();
				Channel channel = session.openChannel(SftpConfiguration.PROTOCOL);
				channel.connect();
				client = (ChannelSftp) channel;
				LOGGER.info("sftp服务器连接成功");
			} catch (JSchException e) {
				LOGGER.error("sftp登录失败，检测登录ip，端口号，用户名密码是否正确，错误信息为" + e.getMessage());
			}
		}

		return client;
	}

	/**
	 *     * 关闭连接 server     
	 */
	public void logout() {
		if (client != null) {
			if (client.isConnected()) {
				client.disconnect();
			}
		}
		if (session != null) {
			if (session.isConnected()) {
				session.disconnect();
			}
		}
	}
}
