package com.gx.obe.server.common.utils;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * @Description:
 * @author chenxw
 */
public class CollectionUtils {
	
	/**
	 * @Description: 根据过滤条件获取集合中的一个元素
	 * @author chenxw
	 * @param <T>
	 * @param collection
	 * @param filter
	 * @return
	 */
	public static <T> T get(Collection<? extends T> collection, Filter<T> filter) {
		if (null != collection) for (T t : collection) if (filter.select(t)) return t;
		return null;
	}
	
	/**
	 * @Description: 根据过滤条件获取集合的条数
	 * @author chenxw
	 * @param <T>
	 * @param collection
	 * @param filter
	 * @return
	 */
	public static <T> int getCount(Collection<? extends T> collection, Filter<T> filter) {
		int count = 0;
		if (null != collection) for (T t : collection) if (filter.select(t)) count++;
		return count;
	}
	
	/**
	 * @Description: 根据过滤条件过滤集合中的元素
	 * @author chenxw
	 * @param <T>
	 * @param collection
	 * @param filter
	 */
	public static <T> void filter(Collection<T> collection, Filter<T> filter) {
		if (null == collection) return;
		Iterator<T> iterator = collection.iterator();
		while (iterator.hasNext())
			if (!filter.select(iterator.next())) iterator.remove();
	}
	
	/**
	 * @Description: 转换为Map
	 * @author chenxw
	 * @param <K>
	 * @param <T>
	 * @param collection
	 * @param newMap
	 * @return
	 */
	public static <K, T> Map<K, T> toHashMap(Collection<? extends T> collection, NewMap<K, T> newMap) {
		if (null == collection) return null;
		Map<K, T> map = new HashMap<K, T>();
		for (T t : collection)
			map.put(newMap.getKey(t), t);
		return map;
	}
	
	/**
	 * @Description: 转换为Map
	 * @author chenxw
	 * @param <K>
	 * @param <T>
	 * @param collection
	 * @param newMap
	 * @return
	 */
	public static <K, T> Map<K, List<T>> toArrayListHashMap(Collection<? extends T> collection, NewMap<K, T> newMap) {
		if (null == collection) return null;
		Map<K, List<T>> listMap = new HashMap<K, List<T>>();
		for (T t : collection) {
			K k = newMap.getKey(t);
			if (listMap.containsKey(k)) {
				listMap.get(k).add(t);
			} else {
				List<T> tList = new ArrayList<T>();
				tList.add(t);
				listMap.put(k, tList);
			}
		}
		return listMap;
	}
	
	/**
	 * @Description: Map迭代转换为ArrayList
	 * @author chenxw
	 * @param <T>
	 * @param map
	 * @return
	 */
	public static <T> ArrayList<T> mapToArrayList(Map<?, ?> map) {
		ArrayList<T> arrayList = new ArrayList<T>();
		mapToCollection(arrayList, map);
		return arrayList;
	}
	
	/**
	 * @Description: Map迭代转换为Collection
	 * @author chenxw
	 * @param <T>
	 * @param collection
	 * @param map
	 */
	public static <T> void mapToCollection(Collection<T> collection, Map<?, ?> map) {
		if (null == map) return;
		for (Object v : map.values()) {
			if (v instanceof Map) {
				mapToCollection(collection, (Map<?, ?>) v);
			} else {
				@SuppressWarnings("unchecked")
				T t = (T) v;
				collection.add(t);
			}
		}
	}
	
	public static boolean isNull(Collection<?> collection) {
		return null == collection || collection.isEmpty();
	}
	
	public static boolean isNotNull(Collection<?> collection) {
		return null != collection && collection.size() > 0;
	}
	
	public interface Filter<T> {
		boolean select(T t);
	}
	
	public interface NewMap<K, T> {
		K getKey(T t);
	}
	
}
