package com.gx.obe.server.management.boq.controller;

import com.gx.obe.server.common.base.BaseController;
import com.gx.obe.server.management.boq.entity.ObeMaterialSummary;
import com.gx.obe.server.management.boq.service.ObeMaterialSummaryService;
import io.swagger.annotations.Api;
import lombok.AllArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author mazc
 * @Description:
 */
@Api(tags = "工料机汇总")
@RestController
@AllArgsConstructor
@RequestMapping("/obeMaterialSummary")
public class ObeMaterialSummaryController extends BaseController<ObeMaterialSummaryService, ObeMaterialSummary> {
    @Autowired
    public ObeMaterialSummaryService ObeMaterialSummaryServices;

    /**
     * @param ObeMaterialSummaryList
     * @Description: 批量插入
     * @author mazc
     */
    @PostMapping("/insertByBatch")
    public int insertByBatch(@RequestBody List<ObeMaterialSummary> ObeMaterialSummaryList) {
        return ObeMaterialSummaryServices.insertByBatch(ObeMaterialSummaryList);
    }

    /**
     * @param ObeMaterialSummaryList
     * @Description: 批量更新
     * @author mazc
     */
    @PostMapping("/updateByBatch")
    public int updateByBatch(@RequestBody List<ObeMaterialSummary> ObeMaterialSummaryList) {
        return ObeMaterialSummaryServices.updateByBatch(ObeMaterialSummaryList);
    }

    /**
     * @param projectId
     * @param kind      材料类型(00：其他；01：人工；02：材料；03：机械；04：主材；05：设备)
     * @return List<MaterialSummary>
     * @Description: 根据工程Id，获得指定类型的工料机汇总。
     * @author guoyr
     */
    @GetMapping("/getMaterialSummaryList")
    public List<ObeMaterialSummary> getMaterialSummaryList(String projectId, String kind) {
        return ObeMaterialSummaryServices.getMaterialSummaryList(projectId, kind);
    }

    /**
     * @param projectId
     * @param kind      材料类型(00：其他；01：人工；02：材料；03：机械；04：主材；05：设备)
     * @return List<MaterialSummary>
     * @Description: 根据工程Id，获得指定类型的工料机汇总。
     * @author guoyr
     */
    @GetMapping("/getMaterialSummary")
    public ObeMaterialSummary getMaterialSummary(String projectId, String kind) {
        return ObeMaterialSummaryServices.getMaterialSummary(projectId, kind);
    }

    /**
     * @param projectId
     * @return List<MaterialSummary>
     * @Description: 根据工程Id，获得工料机汇总。
     * @author guoyr
     */
    @GetMapping("/getMaterialSummaryAndChildList")
    public List<ObeMaterialSummary> getMaterialSummaryAndChildList(String projectId) {
        return ObeMaterialSummaryServices.getMaterialSummaryAndChildList(projectId);
    }

}