package com.gx.obe.server.management.boq.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.utils.CollectionUtils;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.boq.dao.*;
import com.gx.obe.server.management.boq.entity.*;
import com.gx.obe.server.management.boq.service.ObeConstructProjectService;
import com.gx.obe.server.management.vo.AnalyzeDataSource;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

import static com.gx.obe.server.common.utils.CollectionUtils.isNotNull;

/**
 * @author mazc
 * @Description:
 */
@Service
@Transactional
public class ObeConstructProjectServiceImpl extends ServiceImpl<ObeConstructProjectMapper, ObeConstructProject> implements ObeConstructProjectService {

    @Autowired
    private ObeAppraisalPriceMapper obeAppraisalPriceMapper;
    @Autowired
    private ObeBillTableMapper obeBillTableMapper;
    @Autowired
    private ObeConstructProjectMapper obeConstructProjectMapper;
    @Autowired
    private ObeDayWorksMapper obeDayWorksMapper;
    @Autowired
    private ObeExtraTableMapper obeExtraTableMapper;
    @Autowired
    private ObeMaterialSummaryMapper obeMaterialSummaryMapper;
    @Autowired
    private ObeMeasureTableMapper obeMeasureTableMapper;
    @Autowired
    private ObeProjectSummaryMapper obeProjectSummaryMapper;
    @Autowired
    private ObeProvisionalSumsMapper obeProvisionalSumsMapper;
    @Autowired
    private ObeQuotaItemMapper obeQuotaItemMapper;
    @Autowired
    private ObeRatesTableMapper obeRatesTableMapper;
    @Autowired
    private ObeServiceCostsMapper obeServiceCostsMapper;
    @Autowired
    private ObeSupplyMaterialDetailMapper obeSupplyMaterialDetailMapper;

    /**
     * @param ObeConstructProjectList
     * @Description: 批量更新
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer updateByBatch(List<ObeConstructProject> ObeConstructProjectList) {
        return obeConstructProjectMapper.updateBatchList(ObeConstructProjectList);
    }

    /**
     * @param ObeConstructProjectList
     * @Description: 批量插入
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer insertByBatch(List<ObeConstructProject> ObeConstructProjectList) {
        return obeConstructProjectMapper.insertByBatch(ObeConstructProjectList);
    }

    /**
     * @param ObeConstructProject
     * @param attributes
     * @Description: 指定字段修改
     * @author chenxw
     */
    @Override
    public boolean updateAssignProperty(ObeConstructProject ObeConstructProject, String[] attributes) {
        return obeConstructProjectMapper.updateAssignProperty(ObeConstructProject, attributes) > 0;
    }

    /**
     * @param ObeConstructProjectList
     * @param attributes
     * @Description: 批量指定字段修改
     * @author chenxw
     */
    @Override
    public Integer batchUpdateProperty(List<ObeConstructProject> ObeConstructProjectList, String[] attributes) {
        return obeConstructProjectMapper.batchUpdateProperty(ObeConstructProjectList, attributes);
    }

    /**
     * @param ObeConstructProjectList
     * @param attributes
     * @return
     * @Description: 批量添加或跟新
     * @author chenxw
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer batchSaveOrUpdate(List<ObeConstructProject> ObeConstructProjectList, String[] attributes) {
        List<ObeConstructProject> insertObeConstructProjectList = new ArrayList<>();
        List<ObeConstructProject> updateObeConstructProjectList = new ArrayList<>();
        for (ObeConstructProject ObeConstructProject : ObeConstructProjectList) {
            if (StringUtils.isEmpty(ObeConstructProject.getId())) {
                ObeConstructProject.setId(IDUtils.getId());
                insertObeConstructProjectList.add(ObeConstructProject);
            } else {
                updateObeConstructProjectList.add(ObeConstructProject);
            }
        }
        int count = 0;
        if (insertObeConstructProjectList.size() > 0) {
            count += obeConstructProjectMapper.insertByBatch(insertObeConstructProjectList);
        }
        if (updateObeConstructProjectList.size() > 0) {
            count += obeConstructProjectMapper.batchUpdateProperty(updateObeConstructProjectList, attributes);
        }
        return count;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer deleteAllConstructProjectDetail(String tenderId) {
        int deleteCount = 0;
        deleteCount += obeAppraisalPriceMapper.deleteByTenderId(tenderId);
        deleteCount += obeBillTableMapper.deleteByTenderId(tenderId);
        deleteCount += obeConstructProjectMapper.deleteByTenderId(tenderId);
        deleteCount += obeDayWorksMapper.deleteByTenderId(tenderId);
        deleteCount += obeExtraTableMapper.deleteByTenderId(tenderId);
        deleteCount += obeMaterialSummaryMapper.deleteByTenderId(tenderId);
        deleteCount += obeMeasureTableMapper.deleteByTenderId(tenderId);
        deleteCount += obeProjectSummaryMapper.deleteByTenderId(tenderId);
        deleteCount += obeProvisionalSumsMapper.deleteByTenderId(tenderId);
        deleteCount += obeQuotaItemMapper.deleteByTenderId(tenderId);
        deleteCount += obeRatesTableMapper.deleteByTenderId(tenderId);
        deleteCount += obeServiceCostsMapper.deleteByTenderId(tenderId);
        deleteCount += obeSupplyMaterialDetailMapper.deleteByTenderId(tenderId);
        return deleteCount;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean saveConstructProjectDetail(AnalyzeDataSource analyzeDataSource) {
        List<ObeConstructProject> projectList = analyzeDataSource.getProjectList();
        if (CollectionUtils.isNull(projectList)) {
            return false;
        }
        boolean success = true;
        if (isNotNull(projectList)) {
            if (obeConstructProjectMapper.insertByBatch(projectList) <= 0) {
                success = false;
            }
        }
        List<ObeBillTable> billList = analyzeDataSource.getBillTableList();
        if (isNotNull(billList)) {
            if (obeBillTableMapper.insertByBatch(billList) <= 0) {
                success = false;
            }
        }
        List<ObeMeasureTable> measureList = analyzeDataSource.getMeasureTableList();
        if (isNotNull(measureList)) {
            if (obeMeasureTableMapper.insertByBatch(measureList) <= 0) {
                success = false;
            }
        }

        List<ObeExtraTable> extraList = analyzeDataSource.getExtraTableList();
        if (isNotNull(extraList)) {
            if (obeExtraTableMapper.insertByBatch(extraList) <= 0) {
                success = false;
            }
        }
        List<ObeProvisionalSums> provisionalSumsList = analyzeDataSource.getProvisionalSumsList();
        if (isNotNull(provisionalSumsList)) {
            if (obeProvisionalSumsMapper.insertByBatch(provisionalSumsList) <= 0) {
                success = false;
            }
        }
        List<ObeProjectSummary> projectSummaryList = analyzeDataSource.getProjectSummaryList();
        if (isNotNull(projectSummaryList)) {
            if (obeProjectSummaryMapper.insertByBatch(projectSummaryList) <= 0) {
                success = false;
            }
        }

        List<ObeAppraisalPrice> appraisalPriceList = analyzeDataSource.getAppraisalPriceList();
        if (isNotNull(appraisalPriceList)) {
            if (obeAppraisalPriceMapper.insertByBatch(appraisalPriceList) <= 0) {
                success = false;
            }
        }
        List<ObeServiceCosts> serviceCostsList = analyzeDataSource.getServiceCostsList();
        if (isNotNull(serviceCostsList)) {
            if (obeServiceCostsMapper.insertByBatch(serviceCostsList) <= 0) {
                success = false;
            }
        }
        List<ObeDayWorks> dayWorksList = analyzeDataSource.getDayWorksList();
        if (isNotNull(dayWorksList)) {
            if (obeDayWorksMapper.insertByBatch(dayWorksList) <= 0) {
                success = false;
            }
        }
        List<ObeRatesTable> ratesTableList = analyzeDataSource.getRatesTableList();
        if (isNotNull(ratesTableList)) {
            if (obeRatesTableMapper.insertByBatch(ratesTableList) <= 0) {
                success = false;
            }
        }
        List<ObeQuotaItem> quotaItemList = analyzeDataSource.getQuotaItemList();
        if (isNotNull(quotaItemList)) {
            if (obeQuotaItemMapper.insertByBatch(quotaItemList) <= 0) {
                success = false;
            }
        }
        List<ObeMaterialSummary> materialSummaryList = analyzeDataSource.getMaterialSummaryList();
        if (isNotNull(materialSummaryList)) {
            if (obeMaterialSummaryMapper.insertByBatch(materialSummaryList) <= 0) {
                success = false;
            }
        }
        List<ObeSupplyMaterialDetail> supplyMaterialDetailList = analyzeDataSource.getSupplyMaterialDetailList();
        if (isNotNull(supplyMaterialDetailList)) {
            if (obeSupplyMaterialDetailMapper.insertByBatch(supplyMaterialDetailList) <= 0) {
                success = false;
            }
        }
        return success;
    }

    /**
     * @param tenderId
     * @param supplierId
     * @return
     * @Description: 根据项目Id和供应商Id，获得工程信息(包括单项和单位工程)。
     * @author guoyr
     */
    public ObeConstructProject getAllConstructProject(String tenderId, String supplierId) {
        return initConstructProjectChild(obeConstructProjectMapper.getConstructProjectListBySupplierId(tenderId, supplierId));
    }

    /**
     * @param list
     * @return
     * @Description: 项目信息封装树节点(双重循环实现拼接)
     * @author guoyr
     */
    private ObeConstructProject initConstructProjectChild(List<ObeConstructProject> list) {
        ObeConstructProject constructProject = null;
        if (null != list && list.size() > 0) {
            for (ObeConstructProject parentProject : list) {
                for (ObeConstructProject childProject : list) {
                    if (StringUtils.equals(parentProject.getId(), childProject.getParentId())) {
                        parentProject.addChild(childProject);
                        childProject.setParentProject(parentProject);
                    }
                }
                // 如果是顶级指标
                if (StringUtils.isEmpty(parentProject.getParentId())) {
                    constructProject = parentProject;
                }
            }
        }
        return constructProject;
    }

}
