package com.gx.obe.server.management.boq.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.boq.dao.ObeMaterialSummaryMapper;
import com.gx.obe.server.management.boq.entity.ObeMaterialSummary;
import com.gx.obe.server.management.boq.service.ObeMaterialSummaryService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * @author mazc
 * @Description:
 */
@Service
@Transactional
public class ObeMaterialSummaryServiceImpl extends ServiceImpl<ObeMaterialSummaryMapper, ObeMaterialSummary> implements ObeMaterialSummaryService {

    @Autowired
    private ObeMaterialSummaryMapper obeMaterialSummaryMapper;

    /**
     * @param ObeMaterialSummaryList
     * @Description: 批量更新
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer updateByBatch(List<ObeMaterialSummary> ObeMaterialSummaryList) {
        return obeMaterialSummaryMapper.updateBatchList(ObeMaterialSummaryList);
    }

    /**
     * @param ObeMaterialSummaryList
     * @Description: 批量插入
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer insertByBatch(List<ObeMaterialSummary> ObeMaterialSummaryList) {
        return obeMaterialSummaryMapper.insertByBatch(ObeMaterialSummaryList);
    }

    /**
     * @param ObeMaterialSummary
     * @param attributes
     * @Description: 指定字段修改
     * @author chenxw
     */
    @Override
    public boolean updateAssignProperty(ObeMaterialSummary ObeMaterialSummary, String[] attributes) {
        return obeMaterialSummaryMapper.updateAssignProperty(ObeMaterialSummary, attributes) > 0;
    }

    /**
     * @param ObeMaterialSummaryList
     * @param attributes
     * @Description: 批量指定字段修改
     * @author chenxw
     */
    @Override
    public Integer batchUpdateProperty(List<ObeMaterialSummary> ObeMaterialSummaryList, String[] attributes) {
        return obeMaterialSummaryMapper.batchUpdateProperty(ObeMaterialSummaryList, attributes);
    }

    /**
     * @param ObeMaterialSummaryList
     * @param attributes
     * @return
     * @Description: 批量添加或跟新
     * @author chenxw
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer batchSaveOrUpdate(List<ObeMaterialSummary> ObeMaterialSummaryList, String[] attributes) {
        List<ObeMaterialSummary> insertObeMaterialSummaryList = new ArrayList<>();
        List<ObeMaterialSummary> updateObeMaterialSummaryList = new ArrayList<>();
        for (ObeMaterialSummary ObeMaterialSummary : ObeMaterialSummaryList) {
            if (StringUtils.isEmpty(ObeMaterialSummary.getId())) {
                ObeMaterialSummary.setId(IDUtils.getId());
                insertObeMaterialSummaryList.add(ObeMaterialSummary);
            } else {
                updateObeMaterialSummaryList.add(ObeMaterialSummary);
            }
        }
        int count = 0;
        if (insertObeMaterialSummaryList.size() > 0) {
            count += obeMaterialSummaryMapper.insertByBatch(insertObeMaterialSummaryList);
        }
        if (updateObeMaterialSummaryList.size() > 0) {
            count += obeMaterialSummaryMapper.batchUpdateProperty(updateObeMaterialSummaryList, attributes);
        }
        return count;
    }

    /**
     * @param projectId
     * @param kind      材料类型(00：其他；01：人工；02：材料；03：机械；04：主材；05：设备)
     * @return List<MaterialSummary>
     * @Description: 根据工程Id，获得指定类型的工料机汇总。
     * @author guoyr
     */
    public List<ObeMaterialSummary> getMaterialSummaryList(String projectId, String kind) {
        return obeMaterialSummaryMapper.getMaterialSummaryList(projectId, kind);
    }

    /**
     * @param projectId
     * @param kind      材料类型(00：其他；01：人工；02：材料；03：机械；04：主材；05：设备)
     * @return List<MaterialSummary>
     * @Description: 根据工程Id，获得指定类型的工料机汇总。
     * @author guoyr
     */
    public ObeMaterialSummary getMaterialSummary(String projectId, String kind) {
        List<ObeMaterialSummary> materialSummaryList = initChildMeasure(obeMaterialSummaryMapper.getMaterialSummaryList(projectId, kind));
        if (null != materialSummaryList && materialSummaryList.size() > 0) {
            return materialSummaryList.get(0);
        }
        return null;
    }

    /**
     * @param projectId
     * @return List<MaterialSummary>
     * @Description: 根据工程Id，获得工料机汇总。
     * @author guoyr
     */
    public List<ObeMaterialSummary> getMaterialSummaryAndChildList(String projectId) {
        return initChildMeasure(obeMaterialSummaryMapper.getMaterialSummaryAndChildList(projectId));
    }

    /**
     * @Description:  获得工料机汇总封装树节点(双重循环实现拼接)
     * @author guoyr
     * @param list
     * @return
     */
    private List<ObeMaterialSummary> initChildMeasure(List<ObeMaterialSummary> list) {
        List<ObeMaterialSummary> measureTableList = new ArrayList<>();
        if (null != list && list.size() > 0) {
            for (ObeMaterialSummary parentMaterial : list) {
                for (ObeMaterialSummary childMaterial : list) {
                    if (null != childMaterial.getParentId() && parentMaterial.getId().equals(childMaterial.getParentId())) {
                        parentMaterial.addChild(childMaterial);
                        childMaterial.setParentMaterial(parentMaterial);
                    }
                }
                // 如果是顶级
                if (StringUtils.isEmpty(parentMaterial.getParentId())) {
                    measureTableList.add(parentMaterial);
                }
            }
        }
        return measureTableList;
    }

}
