package com.gx.obe.server.management.boq.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.boq.dao.ObeProjectSummaryMapper;
import com.gx.obe.server.management.boq.entity.ObeProjectSummary;
import com.gx.obe.server.management.boq.service.ObeProjectSummaryService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * @author mazc
 * @Description:
 */
@Service
@Transactional
public class ObeProjectSummaryServiceImpl extends ServiceImpl<ObeProjectSummaryMapper, ObeProjectSummary> implements ObeProjectSummaryService {

    @Autowired
    private ObeProjectSummaryMapper obeProjectSummaryMapper;

    /**
     * @param ObeProjectSummaryList
     * @Description: 批量更新
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer updateByBatch(List<ObeProjectSummary> ObeProjectSummaryList) {
        return obeProjectSummaryMapper.updateBatchList(ObeProjectSummaryList);
    }

    /**
     * @param ObeProjectSummaryList
     * @Description: 批量插入
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer insertByBatch(List<ObeProjectSummary> ObeProjectSummaryList) {
        return obeProjectSummaryMapper.insertByBatch(ObeProjectSummaryList);
    }

    /**
     * @param ObeProjectSummary
     * @param attributes
     * @Description: 指定字段修改
     * @author chenxw
     */
    @Override
    public boolean updateAssignProperty(ObeProjectSummary ObeProjectSummary, String[] attributes) {
        return obeProjectSummaryMapper.updateAssignProperty(ObeProjectSummary, attributes) > 0;
    }

    /**
     * @param ObeProjectSummaryList
     * @param attributes
     * @Description: 批量指定字段修改
     * @author chenxw
     */
    @Override
    public Integer batchUpdateProperty(List<ObeProjectSummary> ObeProjectSummaryList, String[] attributes) {
        return obeProjectSummaryMapper.batchUpdateProperty(ObeProjectSummaryList, attributes);
    }

    /**
     * @param ObeProjectSummaryList
     * @param attributes
     * @return
     * @Description: 批量添加或跟新
     * @author chenxw
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer batchSaveOrUpdate(List<ObeProjectSummary> ObeProjectSummaryList, String[] attributes) {
        List<ObeProjectSummary> insertObeProjectSummaryList = new ArrayList<>();
        List<ObeProjectSummary> updateObeProjectSummaryList = new ArrayList<>();
        for (ObeProjectSummary ObeProjectSummary : ObeProjectSummaryList) {
            if (StringUtils.isEmpty(ObeProjectSummary.getId())) {
                ObeProjectSummary.setId(IDUtils.getId());
                insertObeProjectSummaryList.add(ObeProjectSummary);
            } else {
                updateObeProjectSummaryList.add(ObeProjectSummary);
            }
        }
        int count = 0;
        if (insertObeProjectSummaryList.size() > 0) {
            count += obeProjectSummaryMapper.insertByBatch(insertObeProjectSummaryList);
        }
        if (updateObeProjectSummaryList.size() > 0) {
            count += obeProjectSummaryMapper.batchUpdateProperty(updateObeProjectSummaryList, attributes);
        }
        return count;
    }

    /**
     * @param projectId
     * @return
     * @Description: 获得项目的汇总列表
     * @author guoyr
     */
    public List<ObeProjectSummary> getProjectSummaryAndChildList(String projectId) {
        return obeProjectSummaryMapper.getProjectSummaryList(projectId);
    }

}
