package com.gx.obe.server.management.boq.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.boq.dao.ObeProvisionalSumsMapper;
import com.gx.obe.server.management.boq.entity.ObeProvisionalSums;
import com.gx.obe.server.management.boq.service.ObeProvisionalSumsService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * @author mazc
 * @Description:
 */
@Service
@Transactional
public class ObeProvisionalSumsServiceImpl extends ServiceImpl<ObeProvisionalSumsMapper, ObeProvisionalSums> implements ObeProvisionalSumsService {

    @Autowired
    private ObeProvisionalSumsMapper obeProvisionalSumsMapper;

    /**
     * @param ObeProvisionalSumsList
     * @Description: 批量更新
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer updateByBatch(List<ObeProvisionalSums> ObeProvisionalSumsList) {
        return obeProvisionalSumsMapper.updateBatchList(ObeProvisionalSumsList);
    }

    /**
     * @param ObeProvisionalSumsList
     * @Description: 批量插入
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer insertByBatch(List<ObeProvisionalSums> ObeProvisionalSumsList) {
        return obeProvisionalSumsMapper.insertByBatch(ObeProvisionalSumsList);
    }

    /**
     * @param ObeProvisionalSums
     * @param attributes
     * @Description: 指定字段修改
     * @author chenxw
     */
    @Override
    public boolean updateAssignProperty(ObeProvisionalSums ObeProvisionalSums, String[] attributes) {
        return obeProvisionalSumsMapper.updateAssignProperty(ObeProvisionalSums, attributes) > 0;
    }

    /**
     * @param ObeProvisionalSumsList
     * @param attributes
     * @Description: 批量指定字段修改
     * @author chenxw
     */
    @Override
    public Integer batchUpdateProperty(List<ObeProvisionalSums> ObeProvisionalSumsList, String[] attributes) {
        return obeProvisionalSumsMapper.batchUpdateProperty(ObeProvisionalSumsList, attributes);
    }

    /**
     * @param ObeProvisionalSumsList
     * @param attributes
     * @return
     * @Description: 批量添加或跟新
     * @author chenxw
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer batchSaveOrUpdate(List<ObeProvisionalSums> ObeProvisionalSumsList, String[] attributes) {
        List<ObeProvisionalSums> insertObeProvisionalSumsList = new ArrayList<>();
        List<ObeProvisionalSums> updateObeProvisionalSumsList = new ArrayList<>();
        for (ObeProvisionalSums ObeProvisionalSums : ObeProvisionalSumsList) {
            if (StringUtils.isEmpty(ObeProvisionalSums.getId())) {
                ObeProvisionalSums.setId(IDUtils.getId());
                insertObeProvisionalSumsList.add(ObeProvisionalSums);
            } else {
                updateObeProvisionalSumsList.add(ObeProvisionalSums);
            }
        }
        int count = 0;
        if (insertObeProvisionalSumsList.size() > 0) {
            count += obeProvisionalSumsMapper.insertByBatch(insertObeProvisionalSumsList);
        }
        if (updateObeProvisionalSumsList.size() > 0) {
            count += obeProvisionalSumsMapper.batchUpdateProperty(updateObeProvisionalSumsList, attributes);
        }
        return count;
    }

    /**
     * @Description:  根据工程Id，获得暂列金额
     * @author guoyr
     * @param projectId
     * @return List<ProvisionalSums>
     */
    public List<ObeProvisionalSums> getProvisionalSumsListByProjectId(String projectId){
        return initChildProvisional(obeProvisionalSumsMapper.getProvisionalSumsListByProjectId(projectId));
    }

    /**
     * @Description:  暂列金额封装树节点(双重循环实现拼接)
     * @author guoyr
     * @param list
     * @return
     */
    private List<ObeProvisionalSums> initChildProvisional(List<ObeProvisionalSums> list) {
        List<ObeProvisionalSums> provisionalSumsList = new ArrayList<>();
        if (null != list && list.size() > 0) {
            for (ObeProvisionalSums parentProvisional : list) {
                for (ObeProvisionalSums childProvisional : list) {
                    if (null != childProvisional.getParentId() && childProvisional.getParentId().equals(parentProvisional.getId())) {
                        parentProvisional.addChild(childProvisional);
                        childProvisional.setParentProvisional(parentProvisional);
                    }
                }
                // 如果是顶级
                if (StringUtils.isEmpty(parentProvisional.getParentId())) {
                    provisionalSumsList.add(parentProvisional);
                }
            }
        }
        return provisionalSumsList;
    }

}
