package com.gx.obe.server.management.config.service.impl;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.enumeration.CommonEnum;
import com.gx.obe.server.management.config.dao.SysConfigMapper;
import com.gx.obe.server.management.config.dao.SysConfigValueMapper;
import com.gx.obe.server.management.config.entity.SysConfig;
import com.gx.obe.server.management.config.entity.SysConfigValue;
import com.gx.obe.server.management.config.service.SysConfigService;

/** 
 * @Description: 
 * @author mazc 
 */
@Service
@Transactional
public class SysConfigServiceImpl extends ServiceImpl<SysConfigMapper, SysConfig> implements SysConfigService {

	@Autowired
	private SysConfigMapper sysConfigMapper;
	private SysConfigValueMapper sysConfigValueMapper;
	
	/**
	 * @Description: 批量更新
	 * @author mazc
	 * @param SysConfigList 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public int updateByBatch(List<SysConfig> SysConfigList) {
		return sysConfigMapper.updateBatchList(SysConfigList);
	}
	
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param SysConfigList 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public int insertByBatch(List<SysConfig> SysConfigList) {
		return sysConfigMapper.insertByBatch(SysConfigList);
	}
	
	/**
     * @Description: 指定字段修改
     * @author mazc
     * @param sysConfig
	 * @param attributes
     */
	@Override
    public boolean updateAssignProperty(SysConfig sysConfig, String[] attributes) {
		return sysConfigMapper.updateAssignProperty(sysConfig, attributes) > 0;
	}
    
    /**
     * @Description: 批量指定字段修改
     * @author mazc
     * @param SysConfigList
	 * @param attributes
     */
	@Override
    public int batchUpdateProperty(List<SysConfig> SysConfigList, String[] attributes) {
		return sysConfigMapper.batchUpdateProperty(SysConfigList, attributes);
	}
	
	
	/** 
	 * @Description: 验证配置名称唯一性
	 * @author mazc
	 * @param id
	 * @param configName
	 * @return 
	 */
	@Override
	public boolean isUniqueConfigName(String id, String configName) {
		return sysConfigMapper.isUniqueConfigName(id, configName) > 0 ? false : true;
	}

	@Override
	public boolean isUniqueConfigCode(String id, String configCode) {
 		return sysConfigMapper.isUniqueConfigCode(id, configCode) > 0 ? false : true;
	}

	@Override
	public SysConfig getConfigByCode(String configCode) {
		return sysConfigMapper.getConfigByCode(configCode);
	}

	@Override
	public List<SysConfigValue> getConfigValueList(String configCode) {
		return sysConfigValueMapper.getConfigValueList(configCode, CommonEnum.REAL_STATUS);
	}
}
