package com.gx.obe.server.management.dictionary.controller;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.gx.obe.server.common.base.BaseController;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.dictionary.entity.AwardCertificate;
import com.gx.obe.server.management.dictionary.service.AwardCertificateService;

import lombok.AllArgsConstructor;
/** 
 * @Description: 
 * @author mazc 
 */
@RestController
@AllArgsConstructor
@RequestMapping("/awardCertificate")
public class AwardCertificateController extends BaseController<AwardCertificateService,AwardCertificate>{
	@Autowired
	public AwardCertificateService awardCertificateServices;
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param AwardCertificateList 
	 */
	@PostMapping("/insertByBatch")
	public int insertByBatch(@RequestBody List<AwardCertificate> AwardCertificateList) {
		return awardCertificateServices.insertByBatch(AwardCertificateList);
	}
	/**
	 * @Description: 批量更新
	 * @author mazc
	 * @param AwardCertificateList 
	 */
	@PostMapping("/updateByBatch")
	public int updateByBatch(@RequestBody List<AwardCertificate> AwardCertificateList) {
		return awardCertificateServices.updateByBatch(AwardCertificateList);
	}
	
	/**
	 * @Description: 获得当前奖项类型下全部供应商
	 * @author mazc
	 * @param awardName
	 * @return 
	 */
	@GetMapping("/getAwardCertificateList")
	public List<AwardCertificate> getAwardCertificateList(String awardName){
		return awardCertificateServices.getAwardCertificateList(awardName);
	}

	/**
	 * @Description: 是否含有此供应商
	 * @author mazc
	 * @param orgName
	 * @param awardName
	 * @return 
	 */
	@GetMapping("/isHasSupplier")
	public boolean isHasSupplier(String orgName, String awardName) {
		return awardCertificateServices.isHasSupplier(orgName, awardName);
	}
	
	/**
	 * @Description: 判断厂商是否已经存在
	 * @author mazc
	 * @param orgName
	 * @param awardName
	 * @return 
	 */
	@GetMapping("/hasSupplier")
	public boolean hasSupplier(String orgName, String awardName) {
		return awardCertificateServices.hasSupplier(orgName, awardName);
	}
	
	
	/**
	 * @Description: 
	 * @author mazc
	 * @return 
	 */
	@GetMapping("/getAllList")
	public List<AwardCertificate> getAllList(){
		QueryWrapper<AwardCertificate> queryWrapper = new QueryWrapper<AwardCertificate>();
		return awardCertificateServices.list(queryWrapper);
	}
	
	
	/**
	 * @Description: 通过id修改
	 * @author mazc
	 * @param awardCertificate
	 * @return 
	 */
	@PostMapping("/updateById")
	public int updateById(@RequestBody AwardCertificate awardCertificate) {
		return awardCertificateServices.update(awardCertificate);
	}
	
	
	/**
	 * @Description: 添加或按照id更新非空字段
	 * @author mazc
	 * @param awardCertificate
	 * @return 
	 */
	@PostMapping("/saveOrUpdateReturn")
	public boolean saveOrUpdateReturn(@RequestBody AwardCertificate awardCertificate){
		if(StringUtils.isEmpty(awardCertificate.getId())) {
			awardCertificate.setId(IDUtils.getId());
			return awardCertificateServices.save(awardCertificate);
		}else {
			return awardCertificateServices.updateById(awardCertificate);
		}
	}
}