package com.gx.obe.server.management.dictionary.controller;
import java.util.List;

import com.gx.obe.server.management.dictionary.entity.BaseDictionaryValue;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gx.obe.server.common.base.BaseController;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.dictionary.entity.BaseDictionary;
import com.gx.obe.server.management.dictionary.service.BaseDictionaryService;

import io.swagger.annotations.Api;
import lombok.AllArgsConstructor;
/** 
 * @Description: 
 * @author mazc 
 */
@Api(tags = "数据字典表")
@RestController
@AllArgsConstructor
@RequestMapping("/baseDictionary")
public class BaseDictionaryController extends BaseController<BaseDictionaryService,BaseDictionary>{
	@Autowired
	public BaseDictionaryService baseDictionaryServices;
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param BaseDictionaryList 
	 */
	@PostMapping("/insertByBatch")
	public int insertByBatch(@RequestBody List<BaseDictionary> BaseDictionaryList) {
		return baseDictionaryServices.insertByBatch(BaseDictionaryList);
	}
	/**
	 * @Description: 批量更新
	 * @author mazc
	 * @param BaseDictionaryList 
	 */
	@PostMapping("/updateByBatch")
	public int updateByBatch(@RequestBody List<BaseDictionary> BaseDictionaryList) {
		return baseDictionaryServices.updateByBatch(BaseDictionaryList);
	}
	
	/**
	 * @Description: 验证字典名称唯一性
	 * @author mazc
	 * @param id
	 * @param dictName
	 * @return boolean true：是唯一；false:不是唯一
	 */
	@GetMapping("/isUniqueDictName")
	public boolean isUniqueDictName(String id, String dictName){
		return baseDictionaryServices.isUniqueDictName(id, dictName);
	}
	
	/**
	 * @Description: 验证字典编码唯一性
	 * @author mazc
	 * @param id
	 * @param dictCode
	 * @return
	 */
	@GetMapping("/isUniqueDictCode")
	public boolean isUniqueDictCode(String id, String dictCode){
		return baseDictionaryServices.isUniqueDictCode(id, dictCode);
	}
	
	/**
	 * @Description: 根据编码，获得数据字典对象。
	 * @author mazc
	 * @param dictCode
	 * @return
	 */
	@GetMapping("/getDictionaryByCode")
	public BaseDictionary getDictionaryByCode(String dictCode){
		return baseDictionaryServices.getDictionaryByCode(dictCode);
	}
	
	/**
	 * @Description: 根据编码，获得字典和字典值。
	 * @author mazc
	 * @param dictCode
	 * @return
	 */
	@GetMapping("/getDictionaryAndValuesByCode")
	public BaseDictionary getDictionaryAndValuesByCode(String dictCode){
		return baseDictionaryServices.getDictionaryAndValuesByCode(dictCode);
	}
	
	/**
	 * @Description: 根据字典编码，获得有效字典值集合。
	 * @author mazc
	 * @param dictCode 字典编码
	 * @return List<DictionaryValue>
	 */
	@GetMapping("/getDictionaryValueList")
	public List<BaseDictionaryValue> getDictionaryValueList(String dictCode){
		return baseDictionaryServices.getDictionaryValueList(dictCode);
	}

	
	/**
	 * @Description: 添加或按照id更新非空字段
	 * @author mazc
	 * @param awardCertificate
	 * @return 
	 */
	@PostMapping("/saveOrUpdateReturn")
	public boolean saveOrUpdateReturn(@RequestBody BaseDictionary baseDictionary){
		if(StringUtils.isEmpty(baseDictionary.getId())) {
			baseDictionary.setId(IDUtils.getId());
			return baseDictionaryServices.save(baseDictionary);
		}else {
			return baseDictionaryServices.updateById(baseDictionary);
		}
	}
	
	@GetMapping("/getDictionaryListOrderByCreateTime")
	public List<BaseDictionary> getDictionaryListOrderByCreateTime(){
 		return baseDictionaryServices.getDictionaryListOrderByCreateTime();
	}
	
}