package com.gx.obe.server.management.dictionary.controller;

import com.gx.obe.server.common.base.BaseController;
import com.gx.obe.server.common.utils.DateUtils;
import com.gx.obe.server.management.dictionary.entity.ProjectTemplate;
import com.gx.obe.server.management.dictionary.service.ProjectTemplateService;
import com.gx.obe.server.management.vo.UpdateAssignPropertyVo;
import lombok.AllArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author mazc
 * @Description:
 */
@RestController
@AllArgsConstructor
@RequestMapping("/projectTemplate")
public class ProjectTemplateController extends BaseController<ProjectTemplateService, ProjectTemplate> {

	@Autowired
	public ProjectTemplateService projectTemplateServices;

	/**
	 * @param ObeProjectTemplateList
	 * @Description: 批量插入
	 * @author mazc
	 */
	@PostMapping("/insertByBatch")
	public int insertByBatch(@RequestBody List<ProjectTemplate> ObeProjectTemplateList) {
		return projectTemplateServices.insertByBatch(ObeProjectTemplateList);
	}

	/**
	 * @param ObeProjectTemplateList
	 * @Description: 批量更新
	 * @author mazc
	 */
	@PostMapping("/updateByBatch")
	public int updateByBatch(@RequestBody List<ProjectTemplate> ObeProjectTemplateList) {
		return projectTemplateServices.updateByBatch(ObeProjectTemplateList);
	}

	/**
	 * @return
	 * @Description: 获取所有模板
	 * @author chenxw
	 */
	@GetMapping("/getAllList")
	public List<ProjectTemplate> getAllList() {
		return projectTemplateServices.getAllList();
	}

	/**
	 * @Description: 根据项目id及模板类型获取列表
	 * @author chenxw
	 * @param tenderId
	 * @param templateType
	 * @return
	 */
	@GetMapping("/getListByTenderIdAndTemplateType")
	public List<ProjectTemplate> getListByTenderIdAndTemplateType(String tenderId, String templateType) {
		return projectTemplateServices.getListByTenderIdAndTemplateType(tenderId, templateType);
	}

	/**
	 * @param updateAssignPropertyVo
	 * @return
	 * @Description: 指定字段修改
	 * @author chenxw
	 */
	@PostMapping("/updateAssignProperty")
	public boolean updateAssignProperty(@RequestBody UpdateAssignPropertyVo<ProjectTemplate> updateAssignPropertyVo) {
		ProjectTemplate projectTemplate = updateAssignPropertyVo.getEntity();
		String[] attributes = updateAssignPropertyVo.getAttributes();
		return projectTemplateServices.updateAssignProperty(projectTemplate, attributes);
	}

	/**
	 * @Description: 删除项目模板
	 * @author chenxw
	 * @param projectTemplate
	 * @return
	 */
	@PostMapping("/deleteProjectTemplate")
	public boolean deleteProjectTemplate(@RequestBody ProjectTemplate projectTemplate) {
		return projectTemplateServices.deleteProjectTemplate(projectTemplate);
	}

	/**
	 * @Description: 获取最晚修改时间
	 * @author chenxw
	 * @return
	 */
	@GetMapping("/getLatestModifyTime")
	public String getLatestModifyTime() {
		return DateUtils.getTimestampFormat(projectTemplateServices.getLatestModifyTime());
	}

}