package com.gx.obe.server.management.dictionary.dao;

import java.util.List;
import java.util.Map;

import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.gx.obe.server.management.dictionary.entity.BaseDictionaryValue;

 /**  
 * 
 * @author 
 * @Title: BaseDictionaryValue
 * @Package com.gx.obe.server.management.dictionary.entity  
 * @Description:   数据字典值表BaseDictionaryValueMapper 接口
 * @author: XiaoMa Pro - myzhichao@163.com
 * @date:   2019-05-13   
 * @version V1.0 
 * @Copyright: 2019 www.msypro.com Inc. All rights reserved. 
 *  
 */
 @Mapper
public interface BaseDictionaryValueMapper extends BaseMapper<BaseDictionaryValue> {

	  /**   
	 * @Title: selectDictionaryValueList   
	 * @Description: 查询数据字典
	 * @param: @param dictCode
	 * @param: @param useStatus
	 * @param: @return      
	 * @return: List<BaseDictionaryValue>      
	 * @throws   
	 */  
	List<BaseDictionaryValue> selectDictionaryValueList(@Param("dicCode")String dicCode, @Param("useStatus")String useStatus);
	
	/**
	 * @Description: 根据字典编号获得数据字典值列表
	 * @author wangxiang
	 * @param dicCode
	 * @return
	 */
	List<BaseDictionaryValue> getDictionaryValuesByCode(@Param("dictId")String dictId);
	
	
	/**
	 * @Description: 根据字典编码字典值集合。
	 * @author mazc
	 * @param dictCode
	 * @return 
	 */
	List<BaseDictionaryValue> getDictionaryValueListByDictCode(@Param("dictCode")String dictCode, @Param("useStatus")String useStatus);
	
	
	
	/**
	 * @Description: 验证字典值名称唯一性
	 * @author mazc
	 * @param dictId
	 * @param id
	 * @param dictValueName
	 * @return 
	 */
	Integer isUniqueDictValueName(@Param("dictId")String dictId, @Param("id")String id, @Param("dictValueName")String dictValueName);
	
	
	/**
	 * @Description: 验证字典值编码唯一性
	 * @author mazc
	 * @param dictId
	 * @param id
	 * @param dictValueCode
	 * @return 
	 */
	Integer isUniqueDictValueCode(@Param("dictId")String dictId,  @Param("id")String id,  @Param("dictValueCode")String dictValueCode);
	
	/**
	 * @Description: 获得数据字典值的最大序号
	 * @author mazc
	 * @return 
	 */
	Integer getMaxDictValueSortNo();
	
	
	/**
	 * @Description: 获得字典值列表
	 * @author mazc
	 * @param objIds
	 * @return 
	 */
	List<BaseDictionaryValue> getDictValueList(@Param("objIds") List<String> objIds);

	/**
	 * @Description: 获得字典ID下的字典值集合
	 * @author mazc
	 * @param dictId
	 * @param useStatus
	 * @return 
	 */
	List<BaseDictionaryValue> getDictionaryValueListByDicId(@Param("dictId")String dictId, @Param("useStatus")String useStatus);
	
	
	 /**
	 * @Description: 根据字典编号获得数据字典值
	 * @author mazc
	 * @param dicCode
	 * @return 
	 */
	List<Map<String, Object>> getDictMapValueByDicCode(@Param("dicCode") String dicCode);
	
	/**
	 * @Description: 
	 * @author mazc
	 * @param dictCode
	 * @param valueCode
	 * @return 
	 */
	BaseDictionaryValue getDictValueBydictCodeAndValue(@Param("dictCode")String dictCode, @Param("valueCode")String valueCode);
	
	
	/**
	 * @Description: 获得特征码检查项
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	List<BaseDictionaryValue> getFeatureCheckItemList(@Param("tenderId")String tenderId);
	
	
	/**
	 * @Description: 获得清标检查项
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	List<BaseDictionaryValue> getCleanTenderItemList(@Param("tenderId")String tenderId);
	
	/**
	 * @Description:  判断字典值是否存在 
	 * @author mazc
	 * @param dictValueName
	 * @return 
	 */
	Integer hasDictValueName(@Param("dictValueName")String dictValueName);
	
	/**
	 * @Description: 获得指定数据字典的值
	 * @author mazc
	 * @param dictValueName
	 * @return 
	 */
	BaseDictionaryValue getDictValueByDictValueName(@Param("dictValueName")String dictValueName);

	 /**
	  * @Description: 根据字典编码，获得有效字典值集合。
	  * @author mazc
	  * @param dictCode 字典编码
	  * @return List<DictionaryValue>
	  */
	 List<BaseDictionaryValue> getDictionaryValueList(@Param("dictCode") String dictCode);

 }