package com.gx.obe.server.management.dictionary.service.impl;

import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.enumeration.CommonEnum;
import com.gx.obe.server.management.dictionary.dao.BaseDictionaryValueMapper;
import com.gx.obe.server.management.dictionary.entity.BaseDictionaryValue;
import com.gx.obe.server.management.dictionary.service.BaseDictionaryValueService;

  /**  
 * 
 * @author 
 * @Title: BaseDictionaryValueService
 * @Package com.gx.obe.server.management.dictionary.service  
 * @Description:   数据字典值表  
 * @author: XiaoMa Pro - myzhichao@163.com
 * @date:   2019-05-13   
 * @version V1.0 
 * @Copyright: 2019 www.msypro.com Inc. All rights reserved. 
 *  
 */
@Service
@Transactional
public class BaseDictionaryValueServiceImpl extends ServiceImpl<BaseDictionaryValueMapper, BaseDictionaryValue> 
																				 implements BaseDictionaryValueService {

	@Autowired
	private BaseDictionaryValueMapper baseDictionaryValueMapper;

	@Override
	public List<BaseDictionaryValue> getDictionaryValueList(String dictCode, String useStatus) {
 		return baseDictionaryValueMapper.selectDictionaryValueList(dictCode, useStatus);
	}

	@Override
	public List<BaseDictionaryValue> getDictValueListByDicCode(String dicCode) {
		return baseDictionaryValueMapper.getDictionaryValueListByDictCode(dicCode, CommonEnum.REAL_STATUS);
	}
	/**
	 * @Description: 根据字典编码，获得有效字典值集合。
	 * @author mazc
	 * @param dictCode 字典编码
	 * @return List<DictionaryValue>
	 */
	@Override
	public List<BaseDictionaryValue> getDictionaryValueListByDictCode(String dictCode) {
		return baseDictionaryValueMapper.getDictionaryValueListByDictCode(dictCode, CommonEnum.REAL_STATUS);
	}

	/** 
	 * @Description: 验证字典值名称唯一性
	 * @author mazc
	 * @param dictId
	 * @param id
	 * @param dictValueName
	 * @return 
	 */
	@Override
	public boolean isUniqueDictValueName(String dictId, String id, String dictValueName) {
		return baseDictionaryValueMapper.isUniqueDictValueName(dictId, id, dictValueName) > 0 ? false : true;
	}

	/** 
	 * @Description: 验证字典值编码唯一性
	 * @author mazc
	 * @param dictId
	 * @param id
	 * @param dictValueCode
	 * @return 
	 */
	@Override
	public boolean isUniqueDictValueCode(String dictId, String id, String dictValueCode) {
		return baseDictionaryValueMapper.isUniqueDictValueCode(dictId, id, dictValueCode) > 0 ? false : true;
	}

	/** 
	 * @Description: 获得数据字典值的最大序号
	 * @author mazc
	 * @return 
	 */
	@Override
	public int getMaxDictValueSortNo() {
		Integer maxSort = baseDictionaryValueMapper.getMaxDictValueSortNo();
		return null != maxSort ? maxSort.intValue() : 0;
	}

	
	/** 
	 * @Description: 交换数据字典值的排序
	 * @author mazc
	 * @param sourceSortNo
	 * @param sourceid
	 * @param targetSortNo
	 * @param targetId
	 * @return 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public boolean changeDictValueSortNo(Integer sourceSortNo, String sourceid, Integer targetSortNo, String targetId) {
		int count = 0;
		UpdateWrapper<BaseDictionaryValue> sourceUpdateWrapper = new UpdateWrapper<>();
		sourceUpdateWrapper.set(BaseDictionaryValue.SORT_NO,targetSortNo);
		sourceUpdateWrapper.lambda().eq(BaseDictionaryValue::getId, sourceid);
		count += baseDictionaryValueMapper.update(null, sourceUpdateWrapper);
		
		UpdateWrapper<BaseDictionaryValue> targetUpdateWrapper = new UpdateWrapper<>();
		targetUpdateWrapper.set(BaseDictionaryValue.SORT_NO,sourceSortNo);
		targetUpdateWrapper.lambda().eq(BaseDictionaryValue::getId, targetId);
		count +=  baseDictionaryValueMapper.update(null, targetUpdateWrapper); 
		
		return count == 2;
 	}
 	
	
	
	@Override
	public List<BaseDictionaryValue> getDictValueList(List<String> objIds) {
		return baseDictionaryValueMapper.getDictValueList(objIds);
	}

	
	/** 
	 * @Description:  获得字典ID下的字典值集合
	 * @author mazc
	 * @param dictId
	 * @param useStatus
	 * @return 
	 */
	@Override
	public List<BaseDictionaryValue> getDictionaryValueListByDicId(String dictId, String useStatus) {
		return baseDictionaryValueMapper.getDictionaryValueListByDicId(dictId, useStatus);
	}

	/** 
	 * @Description: 根据字典编号获得数据字典值
	 * @author mazc
	 * @param dicCode
	 * @return 
	 */
	@Override
	public List<Map<String, Object>> getDictMapValueByDicCode(String dicCode) {
		return baseDictionaryValueMapper.getDictMapValueByDicCode(dicCode);
 	}

	/** 
	 * @Description: 获得指定数据字典的值
	 * @author mazc
	 * @param dictCode
	 * @param valueCode
	 * @return 
	 */
	@Override
	public BaseDictionaryValue getDictValue(String dictCode, String valueCode) {
		return baseDictionaryValueMapper.getDictValueBydictCodeAndValue(dictCode,valueCode);
	}

	/** 
	 * @Description: 获得特征码检查项
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	@Override
	public List<BaseDictionaryValue> getFeatureCheckItemList(String tenderId) {
		return baseDictionaryValueMapper.getFeatureCheckItemList(tenderId);
	}

	/** 
	 * @Description: 获得清标检查项
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	@Override
	public List<BaseDictionaryValue> getCleanTenderItemList(String tenderId) {
		return baseDictionaryValueMapper.getCleanTenderItemList(tenderId);
	}

	/** 
	 * @Description:判断字典值是否存在
	 * @author mazc
	 * @param dictValueName
	 * @return 
	 */
	@Override
	public boolean hasDictValueName(String dictValueName) {
		return baseDictionaryValueMapper.hasDictValueName(dictValueName) > 0 ? true : false;
	}

	/** 
	 * @Description: 获得指定数据字典的值
	 * @author mazc
	 * @param dictValueName
	 * @return 
	 */
	@Override
	public BaseDictionaryValue getDictValue(String dictValueName) {
		return baseDictionaryValueMapper.getDictValueByDictValueName(dictValueName);
	}

	@Override
	public int batchDelete(List<BaseDictionaryValue> list) {
		return baseDictionaryValueMapper.deleteBatchIds(list);	
	}

	@Override
	public boolean updateAssignProperty(BaseDictionaryValue baseDictionaryValue) {
 		return baseDictionaryValueMapper.updateById(baseDictionaryValue) > 0;
	}
	
	
	
}
