package com.gx.obe.server.management.dictionary.service.impl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.dictionary.dao.SuperviseMapper;
import com.gx.obe.server.management.dictionary.entity.Supervise;
import com.gx.obe.server.management.dictionary.service.SuperviseService;

/** 
 * @Description: 
 * @author mazc 
 */
@Service
public class SuperviseServiceImpl extends ServiceImpl<SuperviseMapper, Supervise> implements SuperviseService {

	@Autowired
	private SuperviseMapper superviseMapper;
	
	/**
	 * @Description: 批量更新
	 * @author mazc
	 * @param ObeSuperviseList 
	 */
	 @Override
	 @Transactional(rollbackFor = Exception.class)
	public Integer updateByBatch (List<Supervise> ObeSuperviseList){
		return superviseMapper.updateBatchList(ObeSuperviseList);
	}
	
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param ObeSuperviseList 
	 */
	 @Override
	 @Transactional(rollbackFor = Exception.class)
	public Integer insertByBatch (List<Supervise> ObeSuperviseList){
		return superviseMapper.insertByBatch(ObeSuperviseList);
	}

	/** 
	 * @Description: 查询监督人列表
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	@Override
	public List<Supervise> getSuperviseList(String tenderId) {
		return superviseMapper.getSuperviseList(tenderId);
	}

	/** 
	 * @Description:  删除监督人
	 * @author mazc
	 * @param tenderId
	 * @param userId
	 * @return 
	 */
	@Override
	public boolean deleteSupervise(String tenderId, String userId) {
		return superviseMapper.deleteSupervise(tenderId, userId) > 0;
	}

	@Override
	public List<Object> getSuperviseIdByTenderIdOrUserId(String tenderId, String userId) {
		QueryWrapper<Supervise> queryWrapper = new QueryWrapper<Supervise>();
		queryWrapper.select(Supervise.USER_ID).lambda().eq(Supervise::getTenderId, tenderId);
		if(StringUtils.isNotEmpty(userId)) {
			queryWrapper.lambda().eq(Supervise::getUserId, userId);
		}
  		return superviseMapper.selectObjs(queryWrapper);
	}
	

}
