package com.gx.obe.server.management.dictionary.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.management.dictionary.dao.ProjectTemplateMapper;
import com.gx.obe.server.management.dictionary.dao.TemplateFileMapper;
import com.gx.obe.server.management.dictionary.entity.ProjectTemplate;
import com.gx.obe.server.management.dictionary.entity.TemplateFile;
import com.gx.obe.server.management.dictionary.service.TemplateFileService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/** 
 * @Description: 
 * @author mazc 
 */
@Service
@Transactional
public class TemplateFileServiceImpl extends ServiceImpl<TemplateFileMapper, TemplateFile> implements TemplateFileService {

	@Autowired
	private TemplateFileMapper templateFileMapper;
	
	@Autowired
	private ProjectTemplateMapper projectTemplateMapper;

	/**
	 * @Description: 批量更新
	 * @author mazc
	 * @param ObeTemplateFileList 
	 */
	 @Override
	 @Transactional(rollbackFor = Exception.class)
	public Integer updateByBatch (List<TemplateFile> ObeTemplateFileList){
		return templateFileMapper.updateBatchList(ObeTemplateFileList);
	}
	
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param ObeTemplateFileList 
	 */
	 @Override
	 @Transactional(rollbackFor = Exception.class)
	public Integer insertByBatch (List<TemplateFile> ObeTemplateFileList){
		return templateFileMapper.insertByBatch(ObeTemplateFileList);
	}

	/** 
	 * @Description: 根据模板库 组，查询当前组下的模板信息
	 * @author mazc
	 * @param groupId
	 * @return 
	 */
	@Override
	public List<TemplateFile> getTemplateLibraryFileListByGroupID(String groupId) {
		QueryWrapper<TemplateFile> queryWrapper = new QueryWrapper<TemplateFile>();
		queryWrapper.lambda().eq(TemplateFile::getTemplateLibraryId, groupId);
		return templateFileMapper.selectList(queryWrapper);
	}

	@Override
	public List<TemplateFile> getTemplateLibraryFileByFileName(String groupId, String templateFileName) {
		String fileName = templateFileName.substring(0,templateFileName.indexOf("."));
		String suffix = templateFileName.substring(templateFileName.indexOf(".")+1);
		QueryWrapper<TemplateFile> queryWrapper = new QueryWrapper<TemplateFile>();
		queryWrapper.lambda().eq(TemplateFile::getTemplateLibraryId, groupId).eq(TemplateFile::getTemplateName, fileName)
		.eq(TemplateFile::getTemplateSuffix, suffix);
		return templateFileMapper.selectList(queryWrapper);
	}

	@Override
	public int delTemplateLibraryFileByGroupId(String groupId) {
		QueryWrapper<TemplateFile> queryWrapper = new QueryWrapper<TemplateFile>();
		queryWrapper.lambda().eq(TemplateFile::getTemplateLibraryId, groupId);
		return templateFileMapper.delete(queryWrapper);
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public int deleteTemplateLibraryFileByObjId(String id) {
		int count = 0;
		QueryWrapper<ProjectTemplate> queryWrapper = new QueryWrapper<ProjectTemplate>();
		queryWrapper.lambda().eq(ProjectTemplate::getTemplateId, id);
		count += projectTemplateMapper.delete(queryWrapper);
		count += templateFileMapper.deleteById(id);
		
		return count;
	}

	@Override
	public TemplateFile saveOrUpdateNoEmptyReturn(TemplateFile templateFile) {
		if(null == templateFile.getId()) {
			templateFile.setId(IDUtils.getId());
			templateFileMapper.insert(templateFile);
			return templateFile;
		}else {
			templateFileMapper.updateById(templateFile);
			return templateFileMapper.selectById(templateFile.getId());
		}
	}

	

}
