package com.gx.obe.server.management.evaluation.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.gx.obe.server.common.base.BaseController;
import com.gx.obe.server.management.evaluation.entity.BidPriceItem;
import com.gx.obe.server.management.evaluation.service.BidPriceItemService;

import io.swagger.annotations.Api;
import lombok.AllArgsConstructor;
/** 
 * @Description: 
 * @author mazc 
 */
@Api(tags = "投标报价明细（开标一览表明细）")
@RestController
@AllArgsConstructor
@RequestMapping("/bidPriceItem")
public class BidPriceItemController extends BaseController<BidPriceItemService,BidPriceItem>{
	
	@Autowired
	private BidPriceItemService bidPriceItemService;
	
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param ObeBidPriceItemList 
	 */
	@PostMapping("/insertByBatch")
	public int insertByBatch(@RequestBody List<BidPriceItem> ObeBidPriceItemList) {
		return bidPriceItemService.insertByBatch(ObeBidPriceItemList);
	}
	/**
	 * @Description: 批量更新
	 * @author mazc
	 * @param ObeBidPriceItemList 
	 */
	@PostMapping("/updateByBatch")
	public int updateByBatch(@RequestBody List<BidPriceItem> ObeBidPriceItemList) {
		return bidPriceItemService.updateByBatch(ObeBidPriceItemList);
	}
	/**
	 * @Description: 删除分项报价条目
	 * @author mazc
	 * @param bidPriceItemId
	 * @return 
	 */
	@GetMapping("/deleteBidPriceItem")
	public boolean deleteBidPriceItem(String bidPriceItemId){
		return bidPriceItemService.deleteBidPriceItem(bidPriceItemId);
	}
	
	/**
	 * @Description: 交换两个分项报价明细的排序
	 * @author mazc
	 * @param sourceSortNo
	 * @param sourceid
	 * @param targetSortNo
	 * @param targetId
	 * @return 
	 */
	@GetMapping("/changeBidPriceItemSortNo")
	public boolean changeBidPriceItemSortNo(String sourceSortNo, String sourceid, String targetSortNo, String targetId ){
		return bidPriceItemService.changeBidPriceItemSortNo(sourceSortNo, sourceid, targetSortNo, targetId);
	}

	@GetMapping("/getBidPriceItemList")
	public List<BidPriceItem> getBidPriceItemList(String priceId){
		QueryWrapper<BidPriceItem> QueryWrapper = new QueryWrapper<BidPriceItem>();
		QueryWrapper.lambda().eq(BidPriceItem::getBidPriceId, priceId);
		return bidPriceItemService.list(QueryWrapper);
	}
	
	@GetMapping("/getItemByBidPriceIdAndSortNoOrderBy")
	public List<BidPriceItem> getItemByBidPriceIdAndSortNoOrderBy(String bidPriceId){
		return bidPriceItemService.getItemByBidPriceIdAndSortNoOrderBy(bidPriceId);
	}
	
	
	/**
	 * @Description: 
	 * @author mazc
	 * @param item
	 * @return 
	 */
	@PostMapping("/saveOrUpdate")
	public boolean saveOrUpdate(@RequestBody BidPriceItem item){
		return bidPriceItemService.saveOrUpdate(item);
	}
	
	
	   /**
	 * @Description: 获得最大序号
	 * @author mazc
	 * @return 
	 */
	@GetMapping("/getMaxDictValueSortNo")
	public int getMaxDictValueSortNo(String bidPriceId){
		return bidPriceItemService.getMaxDictValueSortNo(bidPriceId);
	}
	
	
}