package com.gx.obe.server.management.evaluation.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.gx.obe.server.common.vo.RequestBodyEntity;
import com.gx.obe.server.management.evaluation.entity.BidRegulations;
import com.gx.obe.server.management.evaluation.service.BidRegulationsService;

import io.swagger.annotations.Api;
import lombok.AllArgsConstructor;

/** 
 * @Description: 
 * @author mazc 
 */
@Api(tags = "废标条款管理")
@RestController
@AllArgsConstructor
@RequestMapping("/bidRegulations")
public class BidRegulationsController {
	
	@Autowired
	BidRegulationsService bidRegulationsService;
	
	@GetMapping("/byTenderIdList")
	public List<BidRegulations> regulationsByIdToList(String tenderId){
		QueryWrapper<BidRegulations> wrapper = new QueryWrapper<BidRegulations>();
		wrapper.lambda().eq(BidRegulations::getTenderId, tenderId);
		wrapper.orderByAsc("SORT_NO");
		return 	bidRegulationsService.list(wrapper);
	}
	
	
	/**
	 * @Description: 统计当前项目下废标条款数量
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	@GetMapping("/byTenderIdSize")
	public Integer regulationsByIdToSJize(String tenderId){
		 return bidRegulationsService.byTenderIdToSize(tenderId);
	}
	
	/**
	 * @Description: 验证当前编号是否存在
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	@GetMapping("/isExistByTenderIdAndNumber")
	public boolean VerificationBidRegulationsSortNo(String tenderId ,int number){
		QueryWrapper<BidRegulations> wrapper = new QueryWrapper<BidRegulations>();
		wrapper.select("BID_REGULATIONS_ID").lambda().eq(BidRegulations::getTenderId, tenderId).eq(BidRegulations::getSortNo, number);
		return bidRegulationsService.list(wrapper).size() > 0;
	}
	
	
	/**
	 * @Description: 通过id删除记录
	 * @author mazc
	 * @param id
	 * @return 
	 */
	@GetMapping("/deleteById")
	public boolean deleteById(String id) {
		return bidRegulationsService.removeById(id);
	}
	
	/**
	 * @Description: 通过id修改条款内容
	 * @author mazc
	 * @param bidRegulationslist
	 * @return 
	 */
	@PostMapping("/updateBatchById")
	public boolean updateBatchById(@RequestBody List<BidRegulations> bidRegulationsList) {
		return bidRegulationsService.updateBatchById(bidRegulationsList);
	}
	
	
	
	/**
	 * @Description: 导入废弃指标
	 * @author mazc
	 * @param requestBodyEntity
	 * @return 
	 */
	@PostMapping("/importBidRegulations")
	public boolean importBidRegulations(@RequestBody RequestBodyEntity<BidRegulations> requestBodyEntity) {
		List<BidRegulations> list = requestBodyEntity.getEntityList();
		String id = (String) requestBodyEntity.getParam("id");
		return 	bidRegulationsService.importBidRegulations(list , id);
	}
	
	/**
	 * @Description: 插入一条指标
	 * @author mazc
	 * @param bidRegulations
	 * @return 
	 */
	@PostMapping("/insert")
	public boolean insert(@RequestBody BidRegulations bidRegulations) {
		return bidRegulationsService.save(bidRegulations);
	}
	
	/**
	 * @Description: 更新指标
	 * @author mazc
	 * @param bidRegulations
	 * @return 
	 */
	@PostMapping("/update")
	public boolean update(@RequestBody BidRegulations bidRegulations) {
		UpdateWrapper<BidRegulations> updateWrapper = new UpdateWrapper<BidRegulations>();
		updateWrapper.lambda().eq(BidRegulations::getId,bidRegulations.getId());
		return  bidRegulationsService.update(updateWrapper);
	}
	

	
}