package com.gx.obe.server.management.evaluation.service;

import java.math.BigDecimal;
import java.util.List;

import com.baomidou.mybatisplus.extension.service.IService;
import com.gx.obe.server.management.evaluation.entity.EvaluationFactor;
import com.gx.obe.server.management.evaluation.entity.EvaluationStep;
import com.gx.obe.server.management.evaluation.entity.EvaluationStepFactor;
import com.gx.obe.server.management.project.entity.TenderProject;
import com.gx.obe.server.management.supplier.entity.Supplier;
 
/** 
 * @Description: 
 * @author mazc 
 */
public interface EvaluationStepService extends IService<EvaluationStep> {
 
	/**
	 * @Description: 通过指标获得该指标所在的评标步骤
	 * @author mazc
	 * @param tenderId
	 * @param factorId
	 * @return 
	 */
	public  EvaluationStep selectEvaluationStepByFactor(String tenderId, String factorId);
	
	/**
	 * @Description: 获得评审要素下可评审的评标步骤
	 * @author mazc
	 * @param tenderId
	 * @param evalGroup
	 * @return 
	 */
	public List<String> getEvalGroupEvalStepList(String tenderId,String evalGroup);
	
	/**
	 * @Description:  获得指定评标步骤以前未评审的节点
	 * @author mazc
	 * @param tenderId
	 * @param sortNo
	 * @return 
	 */
	public EvaluationStep getPreNoStarEvaluationStep(String tenderId, int sortNo);
	
	/**
	 * @Description: 通过项目id删除评标步骤
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	public int deleteStepByTenderId(String tenderId);
	
	
 	/**
	 * @Description: 批量更新
	 * @author mazc
	 * @param ObeEvaluationStepList 
	 */
 	Integer updateByBatch (List<EvaluationStep> ObeEvaluationStepList);
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param ObeEvaluationStepList 
	 */
	Integer insertByBatch (List<EvaluationStep> ObeEvaluationStepList);
	
	 /**
	 * @Description: 根据id跟新指定的字段
	 * @author mazc
	 * @param evaluationStep
	 * @param strs 
	 */
	boolean updateAssignProperty(EvaluationStep evaluationStep, String[] attributes);
	
	/**
     * @Description: 获得项目的评标步骤
     * @author guoyr
     * @param tenderId
     * @return
     */
    public List<EvaluationStep> getEvaluationStepList(String tenderId);
    
    /**
     * @Description: 获得指定序号的评标步骤
     * @author guoyr
     * @param tenderId
     * @param sortNo
     * @return
     */
    public EvaluationStep getEvaluationStepBySortNo(String tenderId, int sortNo);
    
    /**
     * @Description: 查询指定评审类型的评标步骤列表
     * @author guoyr
     * @param tenderId
     * @return
     */
    public List<EvaluationStep> getEvaluationStepListByType(String tenderId, String stepType);
    
    /**
     * @Description: 获得评标步骤的最大序号
     * @author guoyr
     * @param tenderId
     * @return
     */
    public int getMaxEvaluationStepSortNo(String tenderId);
    
    /**
     * @Description: 交换评审结点的顺序
     * @author guoyr
     * @param source
     * @param target
     */
    public boolean changeEvaluationStepSortNo(String source, String target);
    
    /**
     * @Description: 删除评标步骤
     * @author guoyr
     * @param EvaluationStep
     * @return
     */
    public boolean deleteEvaluationStep(EvaluationStep EvaluationStep);
    
    /**
     * @Description: 保存签到评标步骤
     * @author guoyr
     * @param evaluationStep
     * @return
     */
    public EvaluationStep saveSignEvaluationStep(EvaluationStep evaluationStep);
    
    /**
     * @Description: 保存特征码检查评标步骤
     * @author guoyr
     * @param evaluationStep
     * @param stepExpertUserList
     * @param featureItemObjIdList
     * @return
     */
    public EvaluationStep saveFeatureEvaluationStep(EvaluationStep evaluationStep, List<String> stepExpertUserList, List<String> featureItemObjIdList);
    
    /**
     * @Description: 保存清标检查评标步骤
     * @author guoyr
     * @param evaluationStep
     * @param stepExpertUserList
     * @param cleanTenderItemObjIdList
     * @return
     */
    public EvaluationStep saveCleanTenderEvaluationStep(EvaluationStep evaluationStep, List<String> stepExpertUserList, List<String> cleanTenderItemObjIdList);
    
    /**
     * @Description: 保存评标步骤和评审指标
     * @author guoyr
     * @param evaluationStep
     * @param evaluationFactorList
     * @return
     */
    public EvaluationStep saveEvaluationStepAndFactors(EvaluationStep evaluationStep, List<EvaluationFactor> evaluationFactorList, List<String> factorWeightList, List<String> stepExpertUserList);
    
    
    /**
     * @Description: 保存评标步骤
     * @author guoyr
     * @param evaluationStep
     * @return
     */
    public EvaluationStep saveEvaluationStep(EvaluationStep evaluationStep);
    
    /**
     * @Description: 创建废标步骤
     * @author guoyr
     * @param tenderProjectEntity
     * @param evaluationStep
     */
    public EvaluationStep saveWasteBidStep(TenderProject tenderProjectEntity, EvaluationStep evaluationStep);
    
    /**
     * @Description: 验证评标步骤的名称是否已经存在
     * @author guoyr
     * @param tenderId
     * @param id
     * @param stepName
     * @return
     */
    public boolean valEvaluationStepNameExist(String tenderId, String id, String stepName);
    
    /**
     * @Description: 设置默认评标步骤
     * @author guoyr
     * @param tenderId
     * @return
     */
    public boolean createDefaultEvaluationSteps(String tenderId, String siginStepName, String complianceStepName, String detailStepName, String priceStepName, String recommonedStepName, String SHOW_SUMMARY_COMPLIANCE, String SHOW_SUMMARY_SCORE);
    
    /**
     * @Description: 设置默认评标步骤
     * @author guoyr
     * @param tenderId
     * @return
     */
    public boolean createDefaultEvaluationSteps(String tenderId, List<EvaluationStep> nodeList, List<EvaluationStepFactor> evaluationStepFactorList);
    
    /**
     * @Description: 清空项目的评标步骤
     * @author guoyr
     * @param tenderId
     * @return
     */
    public boolean deleteAllEvaluationSteps(String tenderId);
    
    /**
     * @Description: 重新授权评标步骤，并重新授权推荐
     * @author chenxw
     * @param evaluationStep
     * @param userId 
     */
    public void reBackEvaluationStep(EvaluationStep evaluationStep, String userId);
    
    /**
     * @Description: 获得最后一个评标步骤
     * @author guoyr
     * @param tenderId
     * @return
     */
    public EvaluationStep getLastEvaluationStep(String tenderId);
    
    /**
     * @Description: 获得存在评审不通过投标人的评标步骤
     * @author guoyr
     * @param tenderId
     * @return
     */
    public List<EvaluationStep> getDisAccessmentNode(String tenderId);
    
    /**
     * @Description: 通过指标获得该指标所在的评标步骤
     * @author guoyr
     * @return
     */
    public EvaluationStep getEvaluationStepByFactor(String tenderId, String factorId);
    
    /**
     * @Description: 验证评标步骤是否正在评标步骤中被退回
     * @author guoyr
     * @param evaluationStep
     * @return
     */
    public boolean isEvaluationStepRebackedInProgress(EvaluationStep evaluationStep);
    
    

    /**
     * @Description: 获得评标步骤的权重
     * @author guoyr
     * @param evaluationStep
     * @return 
     */
    public BigDecimal getEvaluationStepWeight(EvaluationStep evaluationStep);
    
    /**
     * @Description: 获得评标步骤的分值
     * @author guoyr
     * @param evaluationStep
     * @param weightScore 权重分值
     * @return 
     */
    public BigDecimal getEvaluationStepScore(EvaluationStep evaluationStep, boolean weightScore);
    
    /**
     * @Description: 根据项目id,专家用户id(userId)查找当前专家所拥有的评标步骤
     * @author guoyr
     * @param tendertId
     * @param userId
     * @return 
     */
    public List<EvaluationStep> getEvaluationStepByUserId(String tenderId, String userId);
    
    /**
     * @Description: 根据项目评审点id查询当前评审点所属评标步骤
     * @author guoyr
     * @param tendertId
     * @param factorId
     * @return 
     */
    public EvaluationStep getEvaluationStepByFactorId(String tenderId, String factorId);
    
    /**
     * @Description: 获得已完成步骤最大编号
     * @author wangxiang
     * @param tenderId
     * @param stepStatus
     * @return 
     */
    public int getMAXStepCount(String tenderId, String stepStatus);

    /**
     * @Description: 更新添加步骤后后面步骤+1
     * @author wangxiang 
     * @param tenderId 
     * @param maxComplateIndex 
     */
    public void updateSortNo(String tenderId, int maxComplateIndex);
    
    /**
     * @Description:获取未完成的评审步骤列表
     * @author chenxw
     * @param tenderId
     * @param stepSortNo
     * @return 
     */
    public List<EvaluationStep> getNotComplateStepList(String tenderId, int stepSortNo);
    
    /** 
     * @Description: 获取存在废标的未完成评审步骤列表
     * @author chenxw
     * @param tenderId
     * @param stepSortNo
     * @return 
     */
    public List<EvaluationStep> getNotComplateFilterStepList(String tenderId, int stepSortNo);
    
    /**
     * @Description: 当前步骤是否在评审中
     * @author guoyr
     * @param tenderId
     * @param stepId
     * @return 
     */
    public boolean isInEvaluationStep(String tenderId, String stepId);
    
    /**
     * @Description: 删除专家组长评标结果
     * @author chenxw
     * @param tenderId 
     */
    public void deleteExpertLeaderEvaluationStepResult(String tenderId);
    
    /**
     * @Description:是否完成商务技术评审 
     * @author guoyr
     * @param tenderId
     * @return 
     */
    public boolean isComplateBusAndTecEvaluation(String tenderId);
    
    /**
     * @Description: 保存评标步骤
     * @author wangx
     * @param tenderId 
     * @param evaluationStep
     * @param supplierList 
     * @return
     */
    public EvaluationStep saveReEvaluationStep(TenderProject tender, EvaluationStep evaluationStep, List<Supplier> supplierList);

    /**
     * @Description: 是否含有评标步骤
     * @author chenxw
     * @param tenderId
     * @return 
     */
    public boolean isHasEvaluationStep(String tenderId);

    /**
     * @Description: 批量指定字段修改
     * @author chenxw
     * @param evaluationStepList
     * @param attributes
     * @return 
     */
    public int batchUpdateProperty(List<EvaluationStep> evaluationStepList, String[] attributes);

	public EvaluationStep saveTianYanEvaluationStep(EvaluationStep evaluationStep, List<String> stepExpertUserList);
	
}

