package com.gx.obe.server.management.expert.service;

import java.util.Date;
import java.util.List;
import java.util.Map;

import com.baomidou.mybatisplus.extension.service.IService;
import com.gx.obe.server.management.check.entity.CheckResult;
import com.gx.obe.server.management.editprice.entity.EditPriceItemResult;
import com.gx.obe.server.management.evaluation.entity.BidPriceResult;
import com.gx.obe.server.management.evaluation.entity.EvaluationFactorResult;
import com.gx.obe.server.management.evaluation.entity.EvaluationStep;
import com.gx.obe.server.management.expert.entity.ExpertEntity;
import com.gx.obe.server.management.expert.entity.ExpertEvaluationResult;
import com.gx.obe.server.management.expert.entity.ExpertEvaluationStep;
import com.gx.obe.server.management.supplier.entity.Supplier;
import com.gx.obe.server.management.supplier.entity.SupplierEvaluationResult;
import com.gx.obe.server.management.supplier.entity.SupplierEvaluationStep;
import com.gx.obe.server.management.vo.Audit;
import com.gx.obe.server.management.vo.SaveEvaluationResultVo;
import com.gx.obe.server.management.vo.SubmitEvaluationResultVo;

/**
 * @author mazc
 * @Description:
 */
public interface ExpertEvaluationStepService extends IService<ExpertEvaluationStep> {
    /**
     * @param ObeExpertEvaluationStepList
     * @Description: 批量插入
     * @author mazc
     */
    Integer updateByBatch(List<ExpertEvaluationStep> ObeExpertEvaluationStepList);

    /**
     * @param ObeExpertEvaluationStepList
     * @Description: 批量更新
     * @author mazc
     */
    Integer insertByBatch(List<ExpertEvaluationStep> ObeExpertEvaluationStepList);

    /**
     * @param tenderId 项目id
     * @return
     * @Description: 删除评审成员分配的评标步骤
     * @author mazc
     */
    public int deleteStepByTenderId(String tenderId);

    /**
     * @param status
     * @param tenderId
     * @param evaluationStep
     * @param userId
     * @return
     * @Description: 修改专家对当前评标步骤的评审进度
     * @author mazc
     */
    ExpertEvaluationStep updateExpertEvaluationStatus(String status, String tenderId, EvaluationStep evaluationStep, String userId);

    /**
     * @param status
     * @param endTime
     * @param tenderId
     * @param stepId
     * @param userId
     * @return
     * @Description: 修改专家对当前评标步骤的评审进度
     * @author mazc
     */
    boolean updateExpertEvaluationStepStatus(String status, Date endTime, String tenderId, String stepId, String userId);

    /**
     * @param saveEvaluationResultVo
     * @return
     * @Description: 保存评标结果
     * @author mazc
     */
    boolean saveEvaluationResult(SaveEvaluationResultVo saveEvaluationResultVo);

    /**
     * @param submitEvaluationResultVo
     * @return
     * @Description: 提交评标结果
     * @author mazc
     */
    boolean submitEvaluationResult(SubmitEvaluationResultVo submitEvaluationResultVo);

    /**
     * @param tenderId
     * @param evaluationStep
     * @param userIdList
     * @return
     * @Description: 删除评审结果记录
     * @author mazc
     */
    Integer deleteDirtyEvaluationResult(String tenderId, EvaluationStep evaluationStep, List<String> userIdList);

    /**
     * @param tenderId
     * @param sort
     * @param userId
     * @return
     * @Description: 获得评审成员对指定项目的指定序号的评标步骤
     * @author wangxiang
     */
    ExpertEvaluationStep getExpertEvaluationStepBySort(String tenderId, int sort, String userId);

    /**
     * @param tenderId
     * @param stepId
     * @return
     * @Description: 获得某一评标步骤评标专家评审完成的数量
     * @author wangxiang
     */
    int getExpertEvaluationComplateCount(String tenderId, String stepId);

    /**
     * @param tenderId
     * @param userId
     * @return
     * @Description: 评标专家签到
     * @author wangxiang
     */
    boolean submitExpertSignEvaluation(String tenderId, String userId, EvaluationStep evaluationStep, String signType, String signResult);

    /**
     * @param tenderId
     * @param userId
     * @param evaluationStep
     * @param supplierEvalResultList
     * @return
     * @Description: 提交推荐中标候选人结果
     * @author wangxiang
     */
    boolean submitRecommonedEvaluationResult(String tenderId, String userId, String evaluationOpinion, EvaluationStep evaluationStep, List<SupplierEvaluationResult> supplierEvalResultList);

    /**
     * @param saveFeatureEvaluationResultVo
     * @return
     * @Description: 保存特征码检查结果
     * @author wangxiang
     */
    boolean saveFeatureEvaluationResult(String tenderId, String userId, EvaluationStep evaluationStep, List<CheckResult> checkResultList, List<ExpertEvaluationResult> expertEvaluationResultList);


    /**
     * @param tenderId
     * @param userId
     * @param evaluationStep
     * @param supplierList
     * @return
     * @Description: 提交特征码检查结果
     * @author wangxiang
     */
    boolean submitFeatureEvaluationResult(String tenderId, String userId, EvaluationStep evaluationStep, List<Supplier> supplierList);


    /**
     * @param tenderId
     * @param userId
     * @param evaluationStep
     * @param expertEvaluationResultMap
     * @return
     * @Description: 保存限价检查结果
     * @author wangxiang
     */
    boolean saveLimitPriceEvaluationResult(String tenderId, String userId, EvaluationStep evaluationStep, Map<String, ExpertEvaluationResult> expertEvaluationResultMap);

    /**
     * @param submitFeatureEvaluationResultVo
     * @return
     * @Description: 提交限价检查结果
     * @author wangxiang
     */
    boolean submitLimitPriceEvaluationResult(String tenderId, String userId, EvaluationStep evaluationStep, List<Supplier> supplierList);

    /**
     * @param tenderId
     * @param stepId
     * @param nodeSortNo
     * @param ifFilter
     * @Description: 重新授权专家的评标步骤
     * @author wangxiang
     */
    boolean reBackExpertEvaluationStep(String tenderId, String stepId, int nodeSortNo, boolean ifFilter, String userId);

    /**
     * @param tenderId
     * @param stepId
     * @return
     * @Description: 获得专家确认修正评标价结果列表
     * @author guoyr
     */
    List<ExpertEvaluationStep> getEvalPriceConfirmList(String tenderId, String stepId);

    /**
     * @param tenderId
     * @param stepId
     * @return
     * @Description: 获得步骤下已经评审完毕的专家
     * @author wangxiang
     */
    List<ExpertEntity> getExpertListByStepFinish(String tenderId, String stepId);


    /**
     * 保存推荐排名结果
     *
     * @param tenderId
     * @param userId
     * @param evaluationStep
     * @param expertEvaluationResultList
     * @return
     * @Description:
     * @author wangxiang
     */
    boolean saveRecommendRankEvaluationResult(String tenderId, String userId, EvaluationStep evaluationStep, List<ExpertEvaluationResult> expertEvaluationResultList);


    /**
     * @param tenderId
     * @param userId
     * @param evaluationStep
     * @param supplierList
     * @param count
     * @return
     * @Description: 提交推荐排名结果
     * @author wangxiang
     */
    boolean submitRecommendRankEvaluationResultVo(String tenderId, String userId, EvaluationStep evaluationStep, List<Supplier> supplierList, int count);

    /**
     * @param tenderId
     * @param userId
     * @param evaluationStep
     * @param expertEvaluationResultMap
     * @return
     * @Description: 保存否决投标结果
     * @author wangxiang
     */
    boolean saveVetoBidEvaluationResult(String tenderId, String userId, EvaluationStep evaluationStep, Map<String, ExpertEvaluationResult> expertEvaluationResultMap);

    /**
     * @param tenderId
     * @param userId
     * @param evaluationStep
     * @param supplierList
     * @return
     * @Description: 提交否决投标结果
     * @author wangxiang
     */
    boolean submitVetoBidEvaluationResult(String tenderId, String userId, EvaluationStep evaluationStep, List<Supplier> supplierList);

    /**
     * @param tenderId
     * @param userId
     * @param evaluationStep
     * @param supplierList
     * @param expertEvaluationResultMap
     * @return
     * @Description: 保存信用得分结果
     * @author wangxiang
     */
    boolean submitCreditEvaluationResult(String tenderId, String userId, EvaluationStep evaluationStep, List<Supplier> supplierList, Map<String, ExpertEvaluationResult> expertEvaluationResultMap);

    /**
     * @param tenderId
     * @param userId
     * @param evaluationStep
     * @param expertEvaluationResultMap
     * @return
     * @Description: 保存清单评审结果
     * @author wangxiang
     */
    boolean saveBillEvaluationResult(String tenderId, String userId, EvaluationStep evaluationStep, Map<String, ExpertEvaluationResult> expertEvaluationResultMap);

    /**
     * @param status
     * @param tenderId
     * @param evaluationStep
     * @param userId
     * @return
     * @Description: 提交多轮报价结果
     * @author wangxiang
     */
    boolean submitMultiPriceResult(String status, String tenderId, EvaluationStep evaluationStep, String userId);

    /**
     * @param tenderId
     * @param userId
     * @param curStepIndex
     * @return
     * @Description: 获得当前评标步骤之前未完成评审的个数
     * @author wangxiang
     */
    Integer getNotComplateStepCount(String tenderId, String userId, int curStepIndex);

    /**
     * @param id
     * @param commonExpert
     * @param evaluationStep
     * @param supplierList
     * @param expertEvaluationResultMap
     * @return
     * @Description: 保存合量评审结果
     * @author wangxiang
     */
    boolean saveTotalPriceEvaluationResult(String id, String commonExpert, EvaluationStep evaluationStep, List<Supplier> supplierList, Map<String, ExpertEvaluationResult> expertEvaluationResultMap);

    /**
     * @param tenderId
     * @param userId
     * @param evaluationStep
     * @param supplierList
     * @return
     * @Description: 提交合量评审结果
     * @author wangxiang
     */
    boolean submitTotalPriceEvaluationResult(String tenderId, String userId, EvaluationStep evaluationStep, List<Supplier> supplierList);

    /**
     * @param tenderId
     * @param evaluationStep
     * @return
     * @Description: 提交预评审结果
     * @author wangxiang
     */
    boolean submitPreEvaluationResult(String tenderId, EvaluationStep evaluationStep);

    /**
     * @param tenderId
     * @param evaluationStep
     * @param editPriceItemResultList
     * @param bidPriceResultList
     * @return
     * @Description: 保存修正报价结果
     * @author wangxiang
     */
    boolean submitTotalPriceEvaluationResult(String tenderId, EvaluationStep evaluationStep, List<EditPriceItemResult> editPriceItemResultList, List<BidPriceResult> bidPriceResultList);

    /**
     * @param tenderId
     * @param evaluationStep
     * @param userId
     * @return
     * @Description: 提交修正报价结果
     * @author wangxiang
     */
    boolean submitEditPriceResult(String tenderId, EvaluationStep evaluationStep, String userId);

    /**
     * @param tenderId
     * @param stepId
     * @return
     * @Description: 删除评审结果记录
     * @author chenxw
     */
    int deleteExpertEvaluationStep(String tenderId, String stepId);

    /**
     * @param tenderId
     * @param stepType
     * @return
     * @Description: 根据步骤类型获取审核列表
     * @author chenxw
     */
    List<Audit> getAuditListByStepType(String tenderId, String stepType);

    /**
     * @param ObeExpertEvaluationStep
     * @Description: 指定字段修改
     * @author chenxw
     */
    boolean updateAssignProperty(ExpertEvaluationStep expertEvaluationStep, String[] attributes);

    /**
     * @param ObeExpertEvaluationStepList
     * @Description: 批量指定字段修改
     * @author chenxw
     */
    Integer batchUpdateProperty(List<ExpertEvaluationStep> expertEvaluationStepList, String[] attributes);

    /**
     * @param tenderId
     * @param stepType
     * @Description: 根据步骤类型删除专家评审步骤
     * @author chenxw
     */
    Integer deleteExpertEvaluationStepByStepType(String tenderId, String stepType);

    /**
     * @param tenderId
     * @param userId
     * @param evaluationStep
     * @param expertEvaluationResultList
     * @return
     * @Description: 保存清标检查结果
     * @author guoyr
     */
    boolean saveCleanTenderEvaluationResult(String tenderId, String userId, EvaluationStep evaluationStep, List<ExpertEvaluationResult> expertEvaluationResultList);

    /**
     * @param tenderId
     * @param userId
     * @param evaluationStep
     * @param memberEvaluationStep
     * @param supplierList
     * @return
     * @Description: 提交清标检查结果
     * @author guoyr
     */
    boolean submitCleanTenderEvaluationResult(String tenderId, String userId, EvaluationStep evaluationStep, ExpertEvaluationStep memberEvaluationStep, List<Supplier> supplierList);

    /**
     * @Description: 保存商务否决评审结果
     * @author chenxw
     * @param tenderId
     * @param evaluationStep
     * @param userId
     * @param expertEvaluationResultList
     * @return
     */
    boolean saveBusinessVetoEvaluationResult(String tenderId, EvaluationStep evaluationStep, String userId, List<ExpertEvaluationResult> expertEvaluationResultList);

    /**
     * @Description: 提交商务评审结果
     * @author chenxw
     * @param tenderId
     * @param evaluationStep
     * @param userId
     * @param supplierList
     * @return
     */
    boolean submitBusinessVetoEvaluationResult(String tenderId, EvaluationStep evaluationStep, String userId, List<Supplier> supplierList);

    /**
     * @Description: 保存商务评审结果
     * @author chenxw
     * @param tenderId
     * @param userId
     * @param evaluationStep
     * @param evaluationFactorResultList
     * @param expertEvaluationResultList
     * @return
     */
    boolean saveBusinessEvaluationResult(String tenderId, String userId, EvaluationStep evaluationStep, List<EvaluationFactorResult> evaluationFactorResultList, List<ExpertEvaluationResult> expertEvaluationResultList);

    /**
     * @Description: 提交商务评审结果
     * @author chenxw
     * @param tenderId
     * @param userId
     * @param evaluationStep
     * @param supplierEvaluationStepList
     * @return
     */
    boolean submitBusinessEvaluationResult(String tenderId, String userId, EvaluationStep evaluationStep, List<SupplierEvaluationStep> supplierEvaluationStepList);

    /**
     * @param tenderId
     * @param stepId
     * @param userId
     * @return
     * @Description: 提交商务评审结果
     * @author chenxw
     */
    boolean submitEvaluationStep(String tenderId, String stepId, String userId);
}
