package com.gx.obe.server.management.expert.service;

import java.util.List;

import org.springframework.web.bind.annotation.GetMapping;

import com.baomidou.mybatisplus.extension.service.IService;
import com.gx.obe.server.management.evaluation.entity.EvaluationStep;
import com.gx.obe.server.management.expert.entity.ExpertEntity;
import com.gx.obe.server.management.user.entity.AuthUserEntity;
import com.gx.obe.server.management.user.entity.AuthUserRoleEntity;
 
/** 
 * @Description: 
 * @author mazc 
 */
public interface ExpertService extends IService<ExpertEntity> {
	
	
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param ObeExpertList 
	 */
	Integer insertByBatch (List<ExpertEntity> ObeExpertList);

	/**
	 * @Description: 指定字段修改
	 * @author chenxw
	 * @param ObeExpert
	 * @param attributes
	 */
	boolean updateAssignProperty(ExpertEntity ObeExpert, String[] attributes);

	/**
	 * @Description: 批量指定字段修改
	 * @author chenxw
	 * @param ObeExpertList
	 * @param attributes
	 */
	Integer batchUpdateProperty(List<ExpertEntity> ObeExpertList, String[] attributes);

	/**
	 * @Description: 批量添加或跟新
	 * @author chenxw
	 * @param ObeExpertList
	 * @param attributes
	 * @return
	 */
	Integer batchSaveOrUpdate(List<ExpertEntity> ObeExpertList, String[] attributes);

	/**
	 * @Description: 获得指定项目的评审成员
	 * @author mazc
	 * @param tenderId
	 * @param userId
	 * @return 
	 */
	ExpertEntity findExpertBatch(String tenderId, String userId);

	/**
	 * @Description:  修改专家的签到状态
	 * @author mazc
	 * @param signType
	 * @param tenderId
	 * @return 
	 */
	int updateSignType(String signType, String tenderId);
	
	/**
	 * @Description: 获得评标专家列表
	 * @author mazc
	 * @param tenderId
	 * @param signType
	 * @return 
	 */
	List<ExpertEntity> findExpertByTenderIdAndSignTypeBatch(String tenderId, String signType);
	
	
	/**
	 * @Description: 通过项目id 获得的评标专家
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	List<ExpertEntity> findExpertByTenderId(String tenderId);

	/**
	 * @Description: 获得评标专家组长
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	ExpertEntity findExpertLeader(String tenderId);
	
	
	/**
	 * @Description: 获得评标步骤的评标专家
	 * @author mazc
	 * @param tenderId
	 * @param evaluationStep
	 * @return 
	 */
	List<ExpertEntity> findEvaluationStepExpertList(String tenderId, EvaluationStep evaluationStep);
	
	/**
	 * @Description: 获得评标步骤特定的专家
	 * @author mazc
	 * @param tenderId
	 * @param evaluationStep
	 * @return 
	 */
	ExpertEntity findEvaluationStepExpert(String tenderId, EvaluationStep evaluationStep);

	
	/**
	 * @Description: 根据项目父级ID查询当前项目（包含子项目）下所有的评审专家
	 * @author mazc
	 * @param projectId
	 * @return 
	 */
	List<ExpertEntity> findExpertListByTenderId(String projectId);
	
	/**
	 * @Description: 获得参与当前评标步骤的专家数
	 * @author mazc
	 * @param tenderId
	 * @param evaluationStep
	 * @return 
	 */
	Integer findEvaluationStepExpertCount (String tenderId, EvaluationStep evaluationStep);
	
	
	/**
	 * @Description:  获得评标专家列表
	 * @author mazc
	 * @param tenderId
	 * @param signType
	 * @return 
	 */
	List<ExpertEntity> findExpertList(String tenderId, String signType);
	
	
	/**
	 * @Description: 查询userId votesNum 、votes  字段
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	List<ExpertEntity> findExperColumnstByTenderId(String tenderId);

	/**
	 * @Description: 获得系统中所有的评标专家
	 * @author wangxiang
	 * @return 
	 */
	List<ExpertEntity> getSystemExpertList(String userType);

	/**
	 * @Description: 获得指定评标步骤的自定义评标专家小组列表
	 * @author wangxiang
	 * @param tenderId 项目id
	 * @param stepId 步骤Id
	 * @return
	 */
	 List<ExpertEntity> getCustomExpertList(String tenderId, String stepId);

	/**
	 * @Description: 获得最大的专家账号
	 * @author wangxiang
	 * @return
	 */
	 Integer getMaxExpertAccountName();

	/**
	 * @Description: 为专家组长投票
	 * @author wangxiang
	 * @param myId    投票者id
	 * @param voterId 投票对象的id
	 * @return
	 */
	 boolean votesExpertLeader(String myId, String voterId);
	 
	 /**
		 * @Description: 新增临时专家
		 * @author wangxiang
		 * @param expert
		 * @param userRoleList
		 * @param user 
		 */
	 boolean saveTemExpert(ExpertEntity expert, List<AuthUserRoleEntity> userRoleList, AuthUserEntity user);

	 /**
		 * @Description: 删除所有评审专家
		 * @author wangxiang
		 * @param tenderId
		 * @return 
		 */
	 boolean deleteAllExperts(String tenderId);
	 
	 /**
		 * @Description: 获取专家可分配的分组列表
		 * @author wangxiang
		 * @param tenderId
		 * @param expertId
		 * @return 
		 */
	 List<Object> getEvaGroupList(String tenderId, String expertId);

	 /**
		 * @Description: 获得评标单元指定的专家
		 * @author wangxiang
		 * @param tenderId
		 * @param evalGroup
		 * @return
		 */
	 ExpertEntity getExpertByevalGroup(String tenderId, String evalGroup);

	 /**
		 * @Description: 获取所有指标单元
		 * @author wangxiang
		 * @param tenderId
		 * @return
		 */
	 List<Object> getAllEvaGroupList(String tenderId);

	 /**
		 * @Description: 删除垃圾专家
		 * @author wangxiang
		 * @param tenderId 
		 */
	 boolean deleteDirtyExpert(String tenderId);

    /**
     * @Description: 获得评标专家列表
     * @author chenxw
     * @param tenderId
     * @param signType
     * @return 
     */
    List<ExpertEntity> getExpertList(String tenderId, String signType);
    
    List<Object> getEvaGroupList(String tenderId);
    
    
    
    boolean deleteExpert(String tenderId, String userId);
    
    
    List<ExpertEntity> getExpertListByTenderId(String tenderId);
    
    
    boolean getIdListByTenderIdAndUserId( String tenderId,String userId);
    boolean updateVotesAndVotesNumByTenderId(int votes ,int votesNum,String tenderId );
    List<Object> getUserIdByTenderId(String tenderId);
    List<Object> getExpertIdListByTenderIdAndSignTypeisNull(String tenderId);
    
    ExpertEntity getExpertByUserIdAndTenderId(String userId,String tenderId);
    
    ExpertEntity saveOrUpdateReturn (ExpertEntity expert);
    
    boolean batchSaveOrUpdate(List<ExpertEntity> expertList);
    
    boolean updateSignTypeByTenderIdOrUserId(String signType,String tenderId,String userId);

    /**
     * @Description: 插入
     * @author chenxw
     * @param expertEntity
     * @return 
     */
    ExpertEntity insert(ExpertEntity expertEntity);
	
    List<ExpertEntity> findUserIdByTenderIdAndStepId(String tenderId,String stepId);

    /**
     * @Description: 根据项目父级ID查询当前项目（包含子项目）下所有的评审专家
     * @author mazc
     * @param projectId
     * @return 
     */
    List<ExpertEntity> getExpertListByProjectId(String projectId);

	/**更新专家
	 * @param selectExpert
	 * @param curExpert
	 * @param user
	 * @return
	 */
	boolean updateExpertInfo(ExpertEntity selectExpert, ExpertEntity curExpert, AuthUserEntity user);

	/**
	 * @Description: 两位专家是否在同一项目中
	 * @author chenxw
	 * @param selectUserId
	 * @param tempUserId
	 * @return
	 */
	boolean isTenderExpertContain(String tenderId, String selectUserId, String tempUserId);

	/**
	 * @Description: 修改临时账号信息
	 * @author chenxw
	 * @param selectUserId
	 * @param tempUserId
	 * @return
	 */
	boolean updateTempExpertMessgae(String selectUserId, String tempUserId);

	/**
	 * @Description: 获得专家姓名的专家
	 * @author wangxiang
	 * @return 
	 */
	@GetMapping("/getExpertListByUserName")
	public List<ExpertEntity> getExpertListByUserName(String userName);

}

