package com.gx.obe.server.management.expert.service.impl;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.utils.CollectionUtils;
import com.gx.obe.server.management.evaluation.dao.EvaluationFactorResultMapper;
import com.gx.obe.server.management.expert.dao.ExpertEvaluationResultMapper;
import com.gx.obe.server.management.expert.entity.ExpertEvaluationResult;
import com.gx.obe.server.management.expert.service.ExpertEvaluationResultService;

/** 
 * @Description: 
 * @author mazc 
 */
@Service
@Transactional
public class ExpertEvaluationResultServiceImpl extends ServiceImpl<ExpertEvaluationResultMapper, ExpertEvaluationResult> implements ExpertEvaluationResultService {

	@Autowired
	private ExpertEvaluationResultMapper expertEvaluationResultMapper;
	@Autowired
	private EvaluationFactorResultMapper evaluationFactorResultMapper;
	
	/**
	 * @Description: 批量更新
	 * @author mazc
	 * @param ObeExpertEvaluationResultList 
	 */
	 @Override
	 @Transactional(rollbackFor = Exception.class)
	public Integer updateByBatch (List<ExpertEvaluationResult> ObeExpertEvaluationResultList){
		return expertEvaluationResultMapper.updateBatchList(ObeExpertEvaluationResultList);
	}
	
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param ObeExpertEvaluationResultList 
	 */
	 @Override
	 @Transactional(rollbackFor = Exception.class)
	public Integer insertByBatch (List<ExpertEvaluationResult> ObeExpertEvaluationResultList){
	     if (CollectionUtils.isNull(ObeExpertEvaluationResultList)) {
            return 0;
        }
		return expertEvaluationResultMapper.insertByBatch(ObeExpertEvaluationResultList);
	}

	/** 
	 * @Description: 删除所有专家为投标人的评标结果
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	@Override
	public int deleteStepByTenderId(String tenderId) {
		QueryWrapper<ExpertEvaluationResult> queryWrapper = new QueryWrapper<ExpertEvaluationResult>();
		queryWrapper.lambda().eq(ExpertEvaluationResult::getTenderId, tenderId);
		return expertEvaluationResultMapper	.delete(queryWrapper);
	}

	/** 
	 * @Description: 获得评标专家对指定评标步骤对投标人的评审记录
	 * @author mazc
	 * @param tenderId
	 * @param stepId
	 * @param userId
	 * @return 
	 */
	@Override
	public List<ExpertEvaluationResult> getStepAllExpertEvaluationResultList(String tenderId, String stepId) {
		return expertEvaluationResultMapper.getStepAllExpertEvaluationResultList(tenderId, stepId);
	}

	/** 
	 * @Description: 获得供应商在指定评标步骤的平均得分
	 * @author mazc
	 * @param tenderId
	 * @param stepId
	 * @param supplierId
	 * @param removeMaxAndMin
	 * @return 
	 */
	@Override
	public List<BigDecimal> getSupplierStepAvgResult(String tenderId, String stepId, String supplierId,
			boolean removeMaxAndMin) {
		List<BigDecimal> bigDecimalList = new ArrayList<BigDecimal>();
		Map<String, Object> resultMap = expertEvaluationResultMapper.getSupplierStepAvgResult(tenderId, stepId, supplierId, removeMaxAndMin);
		if(null != resultMap) {
			bigDecimalList.add((BigDecimal)resultMap.get("AVG_SCORE"));
			bigDecimalList.add((BigDecimal)resultMap.get("AVG_FINAL_SCORE"));
		}
		return bigDecimalList;
	}

	/** 
	 * @Description: 获得供应商在指定评标步骤的平均得分
	 * @author mazc
	 * @param tenderId
	 * @param stepId
	 * @param supplierId
	 * @param removeMaxAndMin
	 * @return 
	 */
	@Override
	public List<BigDecimal> getSupplierStepAvgResultByFactor(String tenderId, String stepId, String supplierId,
			boolean removeMaxAndMin) {
		List<BigDecimal> bigDecimalList = new ArrayList<BigDecimal>();
		Map<String, Object> resultMap = expertEvaluationResultMapper.getSupplierStepAvgResultByFactor(tenderId, stepId, supplierId, removeMaxAndMin);
		if(null != resultMap) {
			bigDecimalList.add((BigDecimal)resultMap.get("AVG_SCORE"));
			bigDecimalList.add((BigDecimal)resultMap.get("AVG_FINAL_SCORE"));
		}
		return bigDecimalList;
	}

	/**
	 * @Description: 获得供应商在指定评标步骤的所有专家的评标结果
	 * @author wangxiang
	 * @param tenderId
	 * @param stepId
	 * @param supplierId
	 * @return 
	 */
	@Override
	public List<ExpertEvaluationResult> getSupplierStepAllExpertEvaluationResultList(String tenderId, String stepId, String supplierId) {
		return expertEvaluationResultMapper.getSupplierStepAllExpertEvaluationResultList(tenderId,stepId,supplierId);
	}

	/**
	 * @Description: 获得供应商在指定评标步骤的指定专家的评标结果
	 * @param tenderId
	 * @param userId
	 * @param stepId
	 * @param supplierId
	 * @return
	 */
	@Override
	public List<ExpertEvaluationResult> getSupplierExpertEvaluationResultList(String tenderId, String stepId, String supplierId) {
		return expertEvaluationResultMapper.getSupplierExpertEvaluationResultList(tenderId,stepId,supplierId);
	}

	/**
	 * @Description: 删除评标专家对指定评标步骤的所有评审记录
	 * @author wangxiang
	 * @param tenderId
	 * @param stepId
	 * @param userId
	 * @return
	 */
	@Override
	public Integer deleteExpertEvaluationResult(String tenderId, String stepId, String userId) {
		return expertEvaluationResultMapper.deleteExpertEvaluationResult(tenderId,stepId,userId);
	}

	/**
	 * @Description: 按供应商分组获得每家供应商的评分汇总 
	 * @author wangxiang
	 * @param tenderId
	 * @param stepId
	 * @param userId
	 * @return 
	 */
	@Override
	public List<Map<String, Object>> getSupplierSumEvaluationResultList(String tenderId, String stepId, String userId) {
		return expertEvaluationResultMapper.getSupplierSumEvaluationResultList(tenderId,stepId,userId);
	}

	/**
	 * @Description: 删除专家评审结果
	 * @author wangxiang
	 * @param tenderId
	 * @param stepId
	 * @param userId 
	 * @return 
	 */
	@Override
	public boolean deleteExpertEvaluationAndFactorResult(String tenderId, String stepId, String userId) {
		int count = 0;
		count += expertEvaluationResultMapper.deleteExpertEvaluationResult(tenderId,stepId,userId);
		count += evaluationFactorResultMapper.deleteEvaluationFactorResult(tenderId,stepId,userId);
		
		return count >= 0;
	}
	
	/** 
	 * @Description: 删除评审结果记录
	 * @author chenxw
	 * @param tenderId
	 * @param stepId
	 * @return 
	 */
	@Override
	public int deleteExpertEvaluationResult(String tenderId, String stepId) {
	    QueryWrapper<ExpertEvaluationResult> queryWrapper = new QueryWrapper<ExpertEvaluationResult>();
        queryWrapper.lambda().eq(ExpertEvaluationResult::getTenderId, tenderId)
                             .eq(ExpertEvaluationResult::getStepId, stepId);
        return expertEvaluationResultMapper.delete(queryWrapper);
	}

	@Override
	public List<ExpertEvaluationResult> getSupplierAllEvaluationResultList(String tenderId,
			String supplierId) {
		  QueryWrapper<ExpertEvaluationResult> queryWrapper = new QueryWrapper<ExpertEvaluationResult>();
	        queryWrapper.lambda().eq(ExpertEvaluationResult::getTenderId, tenderId)
	                             .eq(ExpertEvaluationResult::getSupplierId, supplierId);
		return expertEvaluationResultMapper.selectList(queryWrapper);
	}

	@Override
	public List<ExpertEvaluationResult> getAllExpertEvaluationResultList(String tenderId) {
		QueryWrapper<ExpertEvaluationResult> queryWrapper = new QueryWrapper<ExpertEvaluationResult>();
        queryWrapper.lambda().eq(ExpertEvaluationResult::getTenderId, tenderId);
		return  expertEvaluationResultMapper.selectList(queryWrapper);
	}

	@Override
	public List<ExpertEvaluationResult> getExpertSupplierEvaluationResultList(String tenderId,
			String supplierId, String userId) {
		QueryWrapper<ExpertEvaluationResult> queryWrapper = new QueryWrapper<ExpertEvaluationResult>();
        queryWrapper.lambda().eq(ExpertEvaluationResult::getTenderId, tenderId)
        .eq(ExpertEvaluationResult::getSupplierId, supplierId)
        .eq(ExpertEvaluationResult::getUserId, userId);
		return  expertEvaluationResultMapper.selectList(queryWrapper);
	}


	@Override
	public ExpertEvaluationResult getExpertEvaluationResult(String tenderId, String stepId,	String supplierId, String userId) {
		QueryWrapper<ExpertEvaluationResult> queryWrapper = new QueryWrapper<ExpertEvaluationResult>();
        queryWrapper.lambda().eq(ExpertEvaluationResult::getTenderId, tenderId)
        .eq(ExpertEvaluationResult::getStepId, stepId)
        .eq(ExpertEvaluationResult::getSupplierId, supplierId)
        .eq(ExpertEvaluationResult::getUserId, userId);
		return  expertEvaluationResultMapper.selectOne(queryWrapper);
	}

	@Override
	public List<Object> getSupplierStepAllExpertEvaluationAccessmentList(String tenderId,String supplierId, String stepId) {
		QueryWrapper<ExpertEvaluationResult> queryWrapper = new QueryWrapper<ExpertEvaluationResult>();
		queryWrapper.select(ExpertEvaluationResult.ACCESSMENT);
        queryWrapper.lambda().eq(ExpertEvaluationResult::getTenderId, tenderId).eq(ExpertEvaluationResult::getSupplierId, supplierId)
        .eq(ExpertEvaluationResult::getStepId, stepId);
		return expertEvaluationResultMapper.selectObjs(queryWrapper);
	}

	@Override
	public List<ExpertEvaluationResult> getExpertEvaluationResult(String tenderId, String stepId, String userId) {
		 return expertEvaluationResultMapper.getExpertEvaluationResultList(tenderId, stepId, userId);
	}
	
}
