package com.gx.obe.server.management.expert.service.impl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.expert.dao.ObeExpertLibraryMapper;
import com.gx.obe.server.management.expert.entity.ExpertLibrary;
import com.gx.obe.server.management.expert.service.ExpertLibraryService;

/** 
 * @Description: 
 * @author mazc 
 */
@Service
@Transactional
public class ExpertLibraryServiceImpl extends ServiceImpl<ObeExpertLibraryMapper, ExpertLibrary> implements ExpertLibraryService {

	@Autowired
	private ObeExpertLibraryMapper expertLibraryMapper;
	
	/**
	 * @Description: 批量更新
	 * @author mazc
	 * @param ObeExpertLibraryList 
	 */
	 @Override
	 @Transactional(rollbackFor = Exception.class)
	public Integer updateByBatch (List<ExpertLibrary> ObeExpertLibraryList){
		return expertLibraryMapper.updateBatchList(ObeExpertLibraryList);
	}
	
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param ObeExpertLibraryList 
	 */
	 @Override
	 @Transactional(rollbackFor = Exception.class)
	public Integer insertByBatch (List<ExpertLibrary> ObeExpertLibraryList){
		return expertLibraryMapper.insertByBatch(ObeExpertLibraryList);
	}

	@Override
	public boolean changeExpertLibrarySortNo(String sourceSortNo, String sourceid, String targetSortNo,
			String targetId) {
		int count = 0;
		UpdateWrapper<ExpertLibrary> sourceUpdateWrapper = new UpdateWrapper<>();
		sourceUpdateWrapper.set(ExpertLibrary.SORT_NO,targetSortNo);
		sourceUpdateWrapper.lambda().eq(ExpertLibrary::getId, sourceid);
		count += expertLibraryMapper.update(null, sourceUpdateWrapper);
		
		UpdateWrapper<ExpertLibrary> targetUpdateWrapper = new UpdateWrapper<>();
		targetUpdateWrapper.set(ExpertLibrary.SORT_NO,sourceSortNo);
		targetUpdateWrapper.lambda().eq(ExpertLibrary::getId, targetId);
		count +=  expertLibraryMapper.update(null, targetUpdateWrapper); 
		
		return count == 2;
 	}

	/** 
	 * @Description: 获得专家库的最大序号
	 * @author mazc
	 * @return 
	 */
	@Override
	public int getMaxExpertLibrarySortNo() {
		Integer maxSortNo = expertLibraryMapper.getMaxExpertLibrarySortNo();
		return null != maxSortNo ? maxSortNo.intValue() : 0;
	}

	@Override
	public boolean updateAssignProperty(ExpertLibrary expertLibrary) {
		return expertLibraryMapper.updateById(expertLibrary) > 0;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public ExpertLibrary saveOrUpdateReturn(ExpertLibrary expertLibrary) {
		int size = 0;
		if(expertLibrary == null) {
			return null;
		}
		if(null == expertLibrary.getId()) {
			expertLibrary.setId(IDUtils.getId());
			expertLibraryMapper.insert(expertLibrary);
			return expertLibrary;
		}else {
			size =  expertLibraryMapper.updateById(expertLibrary);
			if(size > 0) {
				return expertLibraryMapper.selectById(expertLibrary.getId());
			}
			return expertLibrary;
		}
 	}

	@Override
	public List<Object> getExpertLibByExpertLibNameOrId(String id, String expertLibName) {
		QueryWrapper<ExpertLibrary> query = new QueryWrapper<ExpertLibrary>();
		query.lambda().eq(ExpertLibrary::getExpertLibName, expertLibName);
		if(StringUtils.isNotEmpty(id)) {
			query.lambda().ne(ExpertLibrary::getId, id);
		}
		return expertLibraryMapper.selectObjs(query);
	}

	@Override
	public List<ExpertLibrary> getExpertLibraryByOrderByorUseStatus(String useStatus) {
		QueryWrapper<ExpertLibrary> query = new QueryWrapper<ExpertLibrary>();
		if(StringUtils.isNotEmpty(useStatus)) {
			query.lambda().eq(ExpertLibrary::getUseStatus, useStatus);
		}
		query.lambda().orderByAsc(ExpertLibrary::getSortNo);
		return expertLibraryMapper.selectList(query);
	}
	

}
