package com.gx.obe.server.management.expert.service.impl;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.GetMapping;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.enumeration.CommonEnum;
import com.gx.obe.server.common.enumeration.EvaluationStepEnum;
import com.gx.obe.server.common.enumeration.ExpertEnum;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.ObjectUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.check.dao.CheckResultMapper;
import com.gx.obe.server.management.evaluation.dao.EvaluationFactorResultMapper;
import com.gx.obe.server.management.evaluation.entity.EvaluationStep;
import com.gx.obe.server.management.expert.dao.EvaluationStepExpertMapper;
import com.gx.obe.server.management.expert.dao.ExpertEvaluationResultMapper;
import com.gx.obe.server.management.expert.dao.ExpertEvaluationStepMapper;
import com.gx.obe.server.management.expert.dao.ExpertMapper;
import com.gx.obe.server.management.expert.entity.EvaluationStepExpert;
import com.gx.obe.server.management.expert.entity.ExpertEntity;
import com.gx.obe.server.management.expert.service.EvaluationStepExpertService;
import com.gx.obe.server.management.expert.service.ExpertService;
import com.gx.obe.server.management.other.dao.InquireEnvelopMapper;
import com.gx.obe.server.management.user.dao.AuthUserMapper;
import com.gx.obe.server.management.user.dao.AuthUserRoleMapper;
import com.gx.obe.server.management.user.entity.AuthUserEntity;
import com.gx.obe.server.management.user.entity.AuthUserRoleEntity;
import com.gx.obe.server.management.user.service.AuthUserService;

/** 
 * @Description: 
 * @author mazc 
 */
@Service
@Transactional
public class ExpertServiceImpl extends ServiceImpl<ExpertMapper, ExpertEntity> implements ExpertService {

	@Autowired
	private ExpertMapper expertMapper;
	
 	
	@Autowired
	private AuthUserService authUserService;
	
	@Autowired
	private AuthUserMapper authUserMapper;
	
	@Autowired
	private AuthUserRoleMapper authUserRoleMapper;
	
	@Autowired
	private CheckResultMapper checkResultMapper;
	
	@Autowired
	private ExpertEvaluationResultMapper expertEvaluationResultMapper;
	
	@Autowired
	private InquireEnvelopMapper inquireEnvelopMapper;
	
	@Autowired
	private ExpertEvaluationStepMapper expertEvaluationStepMapper;
	
	@Autowired
	private EvaluationStepExpertMapper evaluationStepExpertMapper;
	
	@Autowired
	private EvaluationFactorResultMapper evaluationFactorResultMapper;
	
	@Autowired
	public EvaluationStepExpertService evaluationStepExpertServices;
	
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param ObeExpertList 
	 */
	 @Override
	 @Transactional(rollbackFor = Exception.class)
	public Integer insertByBatch (List<ExpertEntity> ObeExpertList){
		return expertMapper.insertByBatch(ObeExpertList);
	}

	/**
	 * @Description: 指定字段修改
	 * @author chenxw
	 * @param ObeExpert
	 * @param attributes
	 */
	@Override
	public boolean updateAssignProperty(ExpertEntity ObeExpert, String[] attributes) {
		return expertMapper.updateAssignProperty(ObeExpert, attributes) > 0;
	}

	/**
	 * @Description: 批量指定字段修改
	 * @author chenxw
	 * @param ObeExpertList
	 * @param attributes
	 */
	@Override
	public Integer batchUpdateProperty(List<ExpertEntity> ObeExpertList, String[] attributes) {
		return expertMapper.batchUpdateProperty(ObeExpertList, attributes);
	}

	/**
	 * @Description: 批量添加或跟新
	 * @author chenxw
	 * @param ObeExpertList
	 * @param attributes
	 * @return
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public Integer batchSaveOrUpdate(List<ExpertEntity> ObeExpertList, String[] attributes) {
		List<ExpertEntity> insertObeExpertList = new ArrayList<>();
		List<ExpertEntity> updateObeExpertList = new ArrayList<>();
		for (ExpertEntity ObeExpert : ObeExpertList) {
			if (StringUtils.isEmpty(ObeExpert.getId())) {
				ObeExpert.setId(IDUtils.getId());
				insertObeExpertList.add(ObeExpert);
			} else {
				updateObeExpertList.add(ObeExpert);
			}
		}
		int count = 0;
		if (insertObeExpertList.size() > 0) {
			count += expertMapper.insertByBatch(insertObeExpertList);
		}
		if (updateObeExpertList.size() > 0) {
			count += expertMapper.batchUpdateProperty(updateObeExpertList, attributes);
		}
		return count;
	}

	/** 
	 * @Description: 获得指定项目的评审成员
	 * @author mazc
	 * @param tenderId
	 * @param userId
	 * @return 
	 */
	@Override
	public ExpertEntity findExpertBatch(String tenderId, String userId) {
		return expertMapper.findExpertBatch(tenderId, userId);
	}

	/** 
	 * @Description: 修改专家的签到状态
	 * @author mazc
	 * @param signType
	 * @param tenderId
	 * @return 
	 */
	@Override
	public int updateSignType(String signType, String tenderId) {
		return expertMapper.updateSignType(signType, tenderId);
	}

	/** 
	 * @Description: 获得评标专家列表
	 * @author mazc
	 * @param tenderId
	 * @param signType
	 * @return 
	 */
	@Override
	public List<ExpertEntity> findExpertByTenderIdAndSignTypeBatch(String tenderId, String signType) {
		return expertMapper.findExpertByTenderIdAndSignTypeBatch(tenderId, signType);
	}

	/** 
	 * @Description: 通过项目id 获得的评标专家
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	@Override
	public List<ExpertEntity> findExpertByTenderId(String tenderId) {
		return expertMapper.findExpertByTenderId(tenderId);
	}

	/** 
	 * @Description: 获得评标专家组长
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	@Override
	public ExpertEntity findExpertLeader(String tenderId) {
		return expertMapper.findExpertLeader(tenderId);
	}

	/** 
	 * @Description: 获得评标步骤的评标专家
	 * @author mazc
	 * @param tenderId
	 * @param evaluationStep
	 * @return 
	 */
	@Override
	public List<ExpertEntity> findEvaluationStepExpertList(String tenderId, EvaluationStep evaluationStep) {
		return expertMapper.findEvaluationStepExpertList(tenderId, evaluationStep);
	}

	
	@Override
	@Transactional(rollbackFor = Exception.class)
	public ExpertEntity findEvaluationStepExpert(String tenderId, EvaluationStep evaluationStep) {
		ExpertEntity expert = null;
		if (ObjectUtils.equals(evaluationStep.getRoleType(), EvaluationStepEnum.EXPERT_LEADER)) {
			expert = findExpertLeader(tenderId);
			if (null == expert) {
				expert = new ExpertEntity();
				expert.setIsLeader(ExpertEnum.LEADER_YES);
				expert.setUserId(ExpertEnum.COMMON_EXPERT);
				expert.setUserName("非组长");
				AuthUserEntity user = new AuthUserEntity();
				user.setId(ExpertEnum.COMMON_EXPERT);
				user.setUserName("非组长");
				expert.setUser(user);
			}
		} else if (ObjectUtils.equals(evaluationStep.getRoleType(), EvaluationStepEnum.OPERATOR)) {
			expert = new ExpertEntity();
			AuthUserEntity user = authUserMapper.findUserInfoByTenderId(tenderId);
			expert.setId(user.getId());
			expert.setUserName(user.getUserName());
			expert.setUserId(user.getId());
			expert.setUser(user);
			expert.setEvalBidType(ExpertEnum.EXPERT_BUS | ExpertEnum.EXPERT_TEC | ExpertEnum.EXPERT_PRICE);
		} else {
			expert = new ExpertEntity();
			expert.setUserId(ExpertEnum.COMMON_EXPERT);
			expert.setUserName("--");
			AuthUserEntity user = new AuthUserEntity();
			user.setId(ExpertEnum.COMMON_EXPERT);
			user.setUserName("--");
			expert.setUser(user);
		}
		return expert;
	}

	/** 
	 * @Description: 根据项目父级ID查询当前项目（包含子项目）下所有的评审专家
	 * @author mazc
	 * @param projectId
	 * @return 
	 */
	@Override
	public List<ExpertEntity> findExpertListByTenderId(String projectId) {
		return expertMapper.findExpertListByTenderId(projectId);
	}

	/** 
	 * @Description: 获得参与当前评标步骤的专家数
	 * @author mazc
	 * @param tenderId
	 * @param evaluationStep
	 * @return 
	 */
	@Override
	public Integer findEvaluationStepExpertCount(String tenderId, EvaluationStep evaluationStep) {
		return expertMapper.findEvaluationStepExpertCount(tenderId, evaluationStep);
	}

	/** 
	 * @Description:  获得评标专家列表
	 * @author mazc
	 * @param tenderId
	 * @param signType
	 * @return 
	 */
	@Override
	public List<ExpertEntity> findExpertList(String tenderId, String signType) {
		return expertMapper.findExpertList(tenderId, signType);
	}

	/** 
	 * @Description: 查询userId votesNum 、votes  
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	@Override
	public List<ExpertEntity> findExperColumnstByTenderId(String tenderId) {
		QueryWrapper<ExpertEntity> queryWrapper = new QueryWrapper<ExpertEntity>();
		queryWrapper.lambda().select(ExpertEntity::getId,ExpertEntity::getUserId,ExpertEntity::getVotesNum,ExpertEntity::getVotes)
		.eq(ExpertEntity::getTenderId, tenderId);
 		return expertMapper.selectList(queryWrapper);
	}
	
	/**
	 * @Description: 获得系统中所有的评标专家
	 * @author wangxiang
	 * @return 
	 */
	@Override
	public List<ExpertEntity> getSystemExpertList(String userType){
		return expertMapper.getSystemExpertList(userType);
	}

	/**
	 * @Description: 获得指定评标步骤的自定义评标专家小组列表
	 * @author wangxiang
	 * @param tenderId 项目id
	 * @param stepId 步骤Id
	 * @return
	 */
	@Override
	public List<ExpertEntity> getCustomExpertList(String tenderId, String stepId) {
		return expertMapper.getCustomExpertList(tenderId,stepId,ExpertEnum.AVOID);
	}

	/**
	 * @Description: 获得最大的专家账号
	 * @author wangxiang
	 * @return
	 */
	@Override
	public Integer getMaxExpertAccountName() {
		return expertMapper.getMaxExpertAccountName();
	}

	/**
	 * @Description: 为专家组长投票
	 * @author wangxiang
	 * @param myId    投票者id
	 * @param voterId 投票对象的id
	 * @return
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public boolean votesExpertLeader(String myId, String voterId) {
		int voteCount = 0;
		voteCount += expertMapper.votesVoterIdCount(voterId);
		voteCount += expertMapper.votesMyIdCount(myId);
		return voteCount >= 0;
	}

	/**
	 * @Description: 新增临时专家
	 * @author wangxiang
	 * @param expert
	 * @param userRoleList
	 * @param user 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public boolean saveTemExpert(ExpertEntity expert, List<AuthUserRoleEntity> userRoleList, AuthUserEntity user) {
		if(null != userRoleList && userRoleList.size() > 0) {
			int count = 0;
			count += authUserMapper.insert(user);
			count += authUserRoleMapper.insertByBatch(userRoleList);
			count += expertMapper.insert(expert);
			return count >= 0;
		}else {
			return false;
		}
	}

	/**
	 * @Description: 删除所有评审专家
	 * @author wangxiang
	 * @param tenderId
	 * @return 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public boolean deleteAllExperts(String tenderId) {
		int count = 0;
		count += checkResultMapper.deleteAllByTenderId(tenderId);
		count += expertEvaluationResultMapper.deleteRestartEvaluation(tenderId);
		count += inquireEnvelopMapper.deleteAllByTenderId(tenderId);
		count += expertEvaluationStepMapper.deleteRestartEvaluation(tenderId);
		count += evaluationStepExpertMapper.deleteAllByTenderId(tenderId);
		count += evaluationFactorResultMapper.deleteRestartEvaluation(tenderId);
		count += expertMapper.deleteAllByTenderId(tenderId);
		return count >= 0;
	}

	/**
	 * @Description: 获取专家可分配的分组列表
	 * @author wangxiang
	 * @param tenderId
	 * @param expertId
	 * @return 
	 */
	@Override
	public List<Object> getEvaGroupList(String tenderId, String expertId) {
		return expertMapper.getEvaGroupList(tenderId,expertId);
	}

	/**
	 * @Description: 获得评标单元指定的专家
	 * @author wangxiang
	 * @param tenderId
	 * @param evalGroup
	 * @return
	 */
	@Override
	public ExpertEntity getExpertByevalGroup(String tenderId, String evalGroup) {
		return expertMapper.getExpertByevalGroup(tenderId,evalGroup);
	}

	/**
	 * @Description: 获取所有指标单元
	 * @author wangxiang
	 * @param tenderId
	 * @return
	 */
	@Override
	public List<Object> getAllEvaGroupList(String tenderId) {
		return expertMapper.getAllEvaGroupList(tenderId);
	}

	/**
	 * @Description: 删除垃圾专家
	 * @author wangxiang
	 * @param tenderId 
	 */
	@Override
	public boolean deleteDirtyExpert(String tenderId) {
		return expertMapper.deleteDirtyExpert(tenderId) > 0;
	};
	
	/** 
	 * @Description: 获得评标专家列表
	 * @author chenxw
	 * @param tenderId
	 * @param signType
	 * @return 
	 */
	@Override
	public List<ExpertEntity> getExpertList(String tenderId, String signType) {
	    return expertMapper.findExpertList(tenderId, signType);
	}

	@Override
	public List<Object> getEvaGroupList(String tenderId) {
		return expertMapper.getAllEvaGroupList(tenderId);
	}


	@Override
	@Transactional(rollbackFor = Exception.class)
	public boolean deleteExpert(String tenderId, String userId) {
		QueryWrapper<ExpertEntity> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda().eq(ExpertEntity::getTenderId, tenderId).eq(ExpertEntity::getUserId, userId);
		AuthUserEntity authUser = authUserMapper.selectById(userId);
		if (CommonEnum.TEMP_STATUS.equals(authUser.getUseStatus())) {
			authUserMapper.deleteById(userId);
		}
		return expertMapper.delete(queryWrapper) > 0;
	}


	@Override
	public List<ExpertEntity> getExpertListByTenderId(String tenderId) {
		return expertMapper.getExpertListByTenderId(tenderId);
	}


	@Override
	public boolean getIdListByTenderIdAndUserId(String tenderId, String userId) {
		QueryWrapper<ExpertEntity> queryWrapper = new QueryWrapper<ExpertEntity>();
		queryWrapper.lambda().eq(ExpertEntity::getTenderId, tenderId).eq(ExpertEntity::getUserId, userId);
		return expertMapper.selectList(queryWrapper).size() > 0;
	}


	@Override
	public boolean updateVotesAndVotesNumByTenderId(int votes, int votesNum, String tenderId) {
		UpdateWrapper<ExpertEntity> update =  new UpdateWrapper<ExpertEntity>();
		update.lambda().set(ExpertEntity::getVotes, votes).set(ExpertEntity::getVotesNum, votesNum).eq(ExpertEntity::getTenderId, tenderId);
		return expertMapper.update(null, update) > 0;
	}


	@Override
	public List<Object> getUserIdByTenderId(String tenderId) {
		QueryWrapper<ExpertEntity> queryWrapper = new QueryWrapper<ExpertEntity>();
		queryWrapper.lambda().select(ExpertEntity::getUserId).eq(ExpertEntity::getTenderId, tenderId);
		return expertMapper.selectObjs(queryWrapper);
	}


	@Override
	public List<Object> getExpertIdListByTenderIdAndSignTypeisNull(String tenderId) {
		QueryWrapper<ExpertEntity> queryWrapper = new QueryWrapper<ExpertEntity>();
		queryWrapper.lambda().select(ExpertEntity::getId).eq(ExpertEntity::getTenderId, tenderId).isNull(ExpertEntity::getSignType);
		return expertMapper.selectObjs(queryWrapper);
	}


	@Override
	public ExpertEntity getExpertByUserIdAndTenderId(String userId, String tenderId) {
		QueryWrapper<ExpertEntity> queryWrapper = new QueryWrapper<ExpertEntity>();
		queryWrapper.lambda().eq(ExpertEntity::getTenderId, tenderId).eq(ExpertEntity::getUserId,userId);
		return expertMapper.selectOne(queryWrapper);
	}


	@Override
	public ExpertEntity saveOrUpdateReturn(ExpertEntity expert) {
		if(null == expert) {
			return expert ;
		}
		if(StringUtils.isNotEmpty(expert.getId())) {
			int size = expertMapper.updateById(expert);
			if(size > 0) {
				return expertMapper.selectById(expert.getId());
			}
			return expert;
		}else {
			expertMapper.insert(expert);
			return expert;
		}
 	}


	@Override
	@Transactional(rollbackFor = Exception.class)
	public boolean batchSaveOrUpdate(List<ExpertEntity> expertList) {
		List<ExpertEntity> listInsert = new ArrayList<ExpertEntity>();
		List<ExpertEntity> listUpdate = new ArrayList<ExpertEntity>();
		for(ExpertEntity expertEntity : expertList) {
			if(StringUtils.isNotEmpty(expertEntity.getId())) {
				listUpdate.add(expertEntity);
			}else {
				expertEntity.setId(IDUtils.getId());
				listInsert.add(expertEntity);
			}
		}
		int size = 0;
		if(listInsert.size() > 0) {
			size += expertMapper.insertByBatch(listInsert);
		}
		if(listUpdate.size() > 0) {
			size += expertMapper.updateExpertBatchList(listUpdate);
		}
		return size > 0;
	}


	@Override
	public boolean updateSignTypeByTenderIdOrUserId(String signType, String tenderId, String userId) {
		UpdateWrapper<ExpertEntity> queryWrapper = new UpdateWrapper<ExpertEntity>();
		queryWrapper.lambda().set(ExpertEntity::getSignType, signType).eq(ExpertEntity::getTenderId, tenderId);
		if(StringUtils.isNotEmpty(userId)) {
			queryWrapper.lambda().eq(ExpertEntity::getUserId, userId);
		}
 		return expertMapper.update(null, queryWrapper) > 0;
	}

	 /**
     * @Description: 插入
     * @author chenxw
     * @param expertEntity
     * @return 
     */
    public ExpertEntity insert(ExpertEntity expertEntity) {
        if (expertEntity != null) {
            expertEntity.setId(IDUtils.getId());
            if (expertMapper.insert(expertEntity) > 0) {
                return expertEntity;
            }
        }
        return null;
    }


	@Override
	public List<ExpertEntity> findUserIdByTenderIdAndStepId(String tenderId, String stepId) {
		QueryWrapper<EvaluationStepExpert> queryWrapper = new QueryWrapper<EvaluationStepExpert>();
 		queryWrapper.select(EvaluationStepExpert.USER_ID);
		queryWrapper.lambda().eq(EvaluationStepExpert::getTenderId, tenderId).eq(EvaluationStepExpert::getStepId, stepId);
		List<Object> userid = evaluationStepExpertServices.listObjs(queryWrapper);
		QueryWrapper<ExpertEntity> queryWrapperExpertEntity = new QueryWrapper<ExpertEntity>();
		queryWrapperExpertEntity.lambda().in(ExpertEntity::getId, userid);
		List<ExpertEntity> expertList = expertMapper.selectList(queryWrapperExpertEntity);
		for(ExpertEntity expert: expertList) {
			if(StringUtils.isEmpty(expert.getId())) {
				continue;
			}
			AuthUserEntity user = authUserMapper.selectById(expert.getUserId());
			expert.setUser(user);
		}
		return expertList;
	}

	/** 
	 * @Description: 根据项目父级ID查询当前项目（包含子项目）下所有的评审专家
	 * @author mazc
	 * @param projectId
	 * @return 
	 */
	@Override
	public List<ExpertEntity> getExpertListByProjectId(String projectId) {
		return expertMapper.getExpertListByProjectId(projectId);
	}
	
	public	boolean updateExpertInfo(ExpertEntity selectExpert, ExpertEntity curExpert, AuthUserEntity updateUser) {
		boolean result = false;
		//如果账号存在
		if(null != updateUser){
			//如果选中的是一个账号
			if(selectExpert.getUser().getUserAccount().equals(curExpert.getUser().getUserAccount())){
				AuthUserEntity selectUser = selectExpert.getUser();
				selectUser.setId(curExpert.getUserId());
				selectUser.setUseStatus(CommonEnum.REAL_STATUS);
				authUserMapper.deleteById(curExpert.getUserId());
				authUserMapper.insert(selectUser);
				
				updateExpet(selectUser);
			}else{//选中的不是同一个账号
				expertMapper.deleteById(curExpert.getId());
				
				AuthUserEntity selectUser = selectExpert.getUser();
				selectUser.setId(updateUser.getId());
				selectUser.setUseStatus(CommonEnum.REAL_STATUS);
				
				authUserMapper.deleteById(updateUser.getId());
				authUserMapper.insert(selectUser);
				
				updateExpet(selectUser);
				
				selectExpert.setUser(selectUser);
				selectExpert.setUserId(selectUser.getId());
				expertMapper.insert(selectExpert);
				
			}
			//账号不存在
		}else{
			updateUser = selectExpert.getUser();
			updateUser.setId(IDUtils.getId());
			updateUser.setUseStatus(CommonEnum.REAL_STATUS);
			authUserMapper.insert(updateUser);	
			expertMapper.deleteById(curExpert.getId());
			selectExpert.setUser(updateUser);
			selectExpert.setUserId(updateUser.getId());
			expertMapper.insert(selectExpert);
			
		}
		
		return true;
	}

	private void updateExpet(AuthUserEntity selectUser) {
		QueryWrapper<ExpertEntity> wrapper = new QueryWrapper<ExpertEntity>();
		wrapper.lambda().eq(ExpertEntity::getUserId, selectUser.getId());
		List<ExpertEntity> expertList = expertMapper.selectList(wrapper);
		
		if(null != expertList && expertList.size() > 0){
			for(ExpertEntity expert : expertList){
				expert.setUserName(selectUser.getUserName());
				expert.setUser(selectUser);
			}
			expertMapper.updateExpertBatchList(expertList);
		}
	}

	/**
	 * @Description: 两位专家是否在同一项目中
	 * @author chenxw
	 * @param selectUserId
	 * @param tempUserId
	 * @return
	 */
	@Override
	public boolean isTenderExpertContain(String tenderId, String selectUserId, String tempUserId) {
		return expertMapper.getTenderExpertContainCount(tenderId, selectUserId, tempUserId) > 0;
	}

	/**
	 * @Description: 修改临时账号信息
	 * @author chenxw
	 * @param selectUserId
	 * @param tempUserId
	 * @return
	 */
	@Override
	public boolean updateTempExpertMessgae(String selectUserId, String tempUserId) {
		UpdateWrapper<ExpertEntity> updateWrapper = new UpdateWrapper<>();
		updateWrapper.lambda()
				.set(ExpertEntity::getUserId, selectUserId)
				.eq(ExpertEntity::getUserId, tempUserId);
		if (expertMapper.update(null, updateWrapper) > 0) {
			return authUserMapper.deleteById(tempUserId) > 0;
		}
		return false;
	}
	
	/**
	 * @Description: 获得专家姓名的专家
	 * @author wangxiang
	 * @return 
	 */
	@GetMapping("/getExpertListByUserName")
	public List<ExpertEntity> getExpertListByUserName(String userName) {
		return expertMapper.getExpertListByUserName(userName);
	}

}
