package com.gx.obe.server.management.fileload.utill;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;

import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.fileload.entity.FileChunk;
import com.gx.obe.server.management.fileload.entity.ParameterInfo;

import lombok.extern.slf4j.Slf4j;


/** 
 * @Description: 
 * @author mazc 
 */
@Slf4j
public class FileUtils {
	
	private static final org.slf4j.Logger log = org.slf4j.LoggerFactory.getLogger(FileUtils.class);

    /**
     * @Description:   folder/tenderid/supplierId/filetype/fileName  ||  folder/tenderid/filetype/fileName
     * 					||  folder/tenderid/fileName
     * @author mazc
     * @param uploadFolder
     * @param fileChunk
     * @return 
     */
    public static String generatePath(String uploadFolder, FileChunk fileChunk) {
        StringBuilder sb = new StringBuilder();
        sb.append(uploadFolder);
        if(StringUtils.isNotEmpty(fileChunk.getTenderId()) ) {
        	sb.append("/").append(fileChunk.getTenderId().trim());
        }
        if(StringUtils.isNotEmpty(fileChunk.getSupplierId()) ) {
        	sb.append("/").append(fileChunk.getSupplierId().trim());
        }
        
        if(StringUtils.isNotEmpty(fileChunk.getFileType()) ) {
        	sb.append("/").append(fileChunk.getFileType().trim());
        }
        
        //判断uploadFolder/identifier 路径是否存在，不存在则创建
        if (!Files.isWritable(Paths.get(sb.toString()))) {
            log.info("path not exist,create path: {}", sb.toString());
            try {
                Files.createDirectories(Paths.get(sb.toString()));
            } catch (IOException e) {
                log.error(e.getMessage(), e);
            }
        }
        if(fileChunk.getTotalChunks() == 1) {
        	return sb.append("/").append(fileChunk.getFilename()).toString();
        }

        return sb.append("/").append(fileChunk.getFilename()) .append("-").append(fileChunk.getChunkNumber()).toString();
    }
    
    public static String generatePath(String uploadFolder, ParameterInfo fileInfo) {
    	 StringBuilder sb = new StringBuilder();
         sb.append(uploadFolder);
         if(StringUtils.isNotEmpty(fileInfo.getTenderId()) ) {
         	sb.append("/").append(fileInfo.getTenderId().trim());
         }
         if(StringUtils.isNotEmpty(fileInfo.getSupplierId()) ) {
         	sb.append("/").append(fileInfo.getSupplierId().trim());
         }
         
         if(StringUtils.isNotEmpty(fileInfo.getFileType()) ) {
         	sb.append("/").append(fileInfo.getFileType().trim());
         }
         return sb.append("/").append(fileInfo.getFilename()).toString();
    }
    

    /**
     * @Description: 文件合并
     * @author mazc
     * @param targetFile
     * @param folder
     * @param filename 
     * @throws IOException 
     */
    public static boolean merge(String targetFile, String folder, String filename) throws IOException {  
    	Files.deleteIfExists(Paths.get(targetFile));
    	Files.createFile(Paths.get(targetFile));
    	Files.list(Paths.get(folder))
    	.filter(path -> !path.getFileName().toString().equals(filename) && path.getFileName().toString().startsWith(filename))
    	.sorted((o1, o2) -> {
    		String p1 = o1.getFileName().toString();
    		String p2 = o2.getFileName().toString();
    		int i1 = p1.lastIndexOf("-");
    		int i2 = p2.lastIndexOf("-");
    		return Integer.valueOf(p2.substring(i2)).compareTo(Integer.valueOf(p1.substring(i1)));
    	})
    	.forEach(path -> {
    		try {
    			//以追加的形式写入文件
    			Files.write(Paths.get(targetFile), Files.readAllBytes(path), StandardOpenOption.APPEND);
    			//合并后删除该块
    			Files.delete(path);
    		} catch (IOException e) {
    			log.error(e.getMessage(), e);
    		}
    	});
		return true;
    }
}
