package com.gx.obe.server.management.im.service.impl;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.enumeration.CommonEnum;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.im.enumeration.EmployeeEnum;
import com.gx.obe.server.im.util.MD5Utils;
import com.gx.obe.server.management.im.dao.EmployeeDepartmentMapper;
import com.gx.obe.server.management.im.dao.EmployeeMapper;
import com.gx.obe.server.management.im.entity.Employee;
import com.gx.obe.server.management.im.entity.EmployeeDepartment;
import com.gx.obe.server.management.im.service.EmployeeService;

/** 
 * @Description: 
 * @author mazc 
 */
@Service
public class EmployeeServiceImpl extends ServiceImpl<EmployeeMapper, Employee> implements EmployeeService {

	@Autowired
	private EmployeeMapper employeeMapper;
	
	@Autowired
	private EmployeeDepartmentMapper employeeDepartmentMapper;
	
	
	/**
	 * @Description: 批量更新
	 * @author mazc
	 * @param EmployeeList 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public Integer updateByBatch(List<Employee> EmployeeList) {
		return employeeMapper.updateBatchList(EmployeeList);
	}
	
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param EmployeeList 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public Integer insertByBatch(List<Employee> EmployeeList) {
		return employeeMapper.insertByBatch(EmployeeList);
	}
	
	/**
     * @Description: 指定字段修改
     * @author chenxw
     * @param Employee
	 * @param attributes
     */
	@Override
    public boolean updateAssignProperty(Employee Employee, String[] attributes) {
		return employeeMapper.updateAssignProperty(Employee, attributes) > 0;
	}
    
    /**
     * @Description: 批量指定字段修改
     * @author chenxw
     * @param EmployeeList
	 * @param attributes
     */
	@Override
    public Integer batchUpdateProperty(List<Employee> EmployeeList, String[] attributes) {
		return employeeMapper.batchUpdateProperty(EmployeeList, attributes);
	}
	
	/** 
	 * @Description: 批量添加或跟新
	 * @author chenxw
	 * @param EmployeeList
	 * @param attributes
	 * @return 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
    public Integer batchSaveOrUpdate(List<Employee> EmployeeList, String[] attributes) {
        List<Employee> insertEmployeeList = new ArrayList<>();
        List<Employee> updateEmployeeList = new ArrayList<>();
        for (Employee Employee : EmployeeList) {
            if (StringUtils.isEmpty(Employee.getId())) {
                Employee.setId(IDUtils.getId());
                insertEmployeeList.add(Employee);
            } else {
                updateEmployeeList.add(Employee);
            }
        }
        int count = 0;
        if (insertEmployeeList.size() > 0) {
            count += employeeMapper.insertByBatch(insertEmployeeList);
        }
        if (updateEmployeeList.size() > 0) {
            count += employeeMapper.batchUpdateProperty(updateEmployeeList, attributes);
        }
        return count;
    }

	/** 
	 * @Description:  获得最大的员工编号
	 * @author mazc
	 * @return 
	 */
	@Override
	public int getMaxEmployeeNumber() {
		return employeeMapper.getMaxEmployeeNumber();
		
	}

	@Override
	public Employee saveEmployee(Employee employee, List<String> assignedDepartmentIdList) {
		// 如果是修改，先删除所有已分配的部门 
		if(StringUtils.isNotEmpty(employee.getId())){
			QueryWrapper<EmployeeDepartment> queryWrapper = new QueryWrapper<EmployeeDepartment>();
			queryWrapper.lambda().eq(EmployeeDepartment::getEmployeeId, employee.getId());
			employeeDepartmentMapper.delete(queryWrapper);
			if(StringUtils.isEmpty(employee.getId())) {
				employee.setId(IDUtils.getId());
				employeeMapper.insert(employee);
			}else {
				employeeMapper.updateById(employee);
			}
		 
		}else {
			employee.setPassword(MD5Utils.getMD5String(null != employee.getPassword() && !"".equals(employee.getPassword())?employee.getPassword():"1"));
			employeeMapper.updateById(employee);
		}
		if(null != assignedDepartmentIdList && assignedDepartmentIdList.size() > 0){
			List<EmployeeDepartment> employeeDepartmentList = new ArrayList<EmployeeDepartment>();
			for(String depId : assignedDepartmentIdList){
				EmployeeDepartment employeeDepartment = new EmployeeDepartment();
				employeeDepartment.setEmployeeId(employee.getId());
				if(depId.startsWith(EmployeeEnum.DIRECT_DEPARTMENT)){
					employeeDepartment.setDepartmentId(depId.substring(EmployeeEnum.DIRECT_DEPARTMENT.length()));
					employeeDepartment.setDirectDepartment(CommonEnum.YES);
				}else {
					employeeDepartment.setDepartmentId(depId);
					employeeDepartment.setDirectDepartment(CommonEnum.NO);
				}
				employeeDepartmentList.add(employeeDepartment);
			}
			employeeDepartmentMapper.insertByBatch(employeeDepartmentList);
		}
		
		return employee;
	}

	@Override
	public List<Employee> getEmployeeOfDepartment(String depId, boolean isDirect) {
		return employeeMapper.getEmployeeOfDepartment(depId, isDirect);
	}

	@Override
	public List<Employee> getEmployeeOfGroup(String groupId) {
		return employeeMapper.getEmployeeOfGroup(groupId);
	}

	@Override
	public List<Employee> getEmployeeByLineStatus(String lineStatus) {
		QueryWrapper<Employee> queryWrapper = new QueryWrapper<Employee>();
		queryWrapper.lambda().eq(Employee::getLineStatus, lineStatus);
 		return employeeMapper.selectList(queryWrapper);
	}

	@Override
	public void updateLineStatusById(String myid, String status) {
		UpdateWrapper<Employee> updateWrapper = new UpdateWrapper<Employee>();
		updateWrapper.lambda().set(Employee::getLineStatus, status).eq(Employee::getId, myid);
	}

}
