package com.gx.obe.server.management.im.service.impl;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.im.dao.MessageFileMapper;
import com.gx.obe.server.management.im.entity.MessageFile;
import com.gx.obe.server.management.im.service.MessageFileService;

/** 
 * @Description: 
 * @author mazc 
 */
@Service
public class MessageFileServiceImpl extends ServiceImpl<MessageFileMapper, MessageFile> implements MessageFileService {

	@Autowired
	private MessageFileMapper messageFileMapper;
	
	/**
	 * @Description: 批量更新
	 * @author mazc
	 * @param MessageFileList 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public Integer updateByBatch(List<MessageFile> MessageFileList) {
		return messageFileMapper.updateBatchList(MessageFileList);
	}
	
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param MessageFileList 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public Integer insertByBatch(List<MessageFile> MessageFileList) {
		return messageFileMapper.insertByBatch(MessageFileList);
	}
	
	/**
     * @Description: 指定字段修改
     * @author chenxw
     * @param MessageFile
	 * @param attributes
     */
	@Override
    public boolean updateAssignProperty(MessageFile MessageFile, String[] attributes) {
		return messageFileMapper.updateAssignProperty(MessageFile, attributes) > 0;
	}
    
    /**
     * @Description: 批量指定字段修改
     * @author chenxw
     * @param MessageFileList
	 * @param attributes
     */
	@Override
    public Integer batchUpdateProperty(List<MessageFile> MessageFileList, String[] attributes) {
		return messageFileMapper.batchUpdateProperty(MessageFileList, attributes);
	}
	
	/** 
	 * @Description: 批量添加或跟新
	 * @author chenxw
	 * @param MessageFileList
	 * @param attributes
	 * @return 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
    public Integer batchSaveOrUpdate(List<MessageFile> MessageFileList, String[] attributes) {
        List<MessageFile> insertMessageFileList = new ArrayList<>();
        List<MessageFile> updateMessageFileList = new ArrayList<>();
        for (MessageFile MessageFile : MessageFileList) {
            if (StringUtils.isEmpty(MessageFile.getId())) {
                MessageFile.setId(IDUtils.getId());
                insertMessageFileList.add(MessageFile);
            } else {
                updateMessageFileList.add(MessageFile);
            }
        }
        int count = 0;
        if (insertMessageFileList.size() > 0) {
            count += messageFileMapper.insertByBatch(insertMessageFileList);
        }
        if (updateMessageFileList.size() > 0) {
            count += messageFileMapper.batchUpdateProperty(updateMessageFileList, attributes);
        }
        return count;
    }

	@Override
	public int getLixianFileCount(String fileName) {
		QueryWrapper<MessageFile> queryWrapper = new QueryWrapper<MessageFile>();
		queryWrapper.lambda().like(MessageFile::getFileName, fileName);
		List<MessageFile> objIdList = messageFileMapper.selectList(queryWrapper);
		return (null == objIdList || objIdList.isEmpty()) ? 0 : objIdList.size();
	}

	@Override
	public boolean saveUpdate(MessageFile messageFile) {
		if(null == messageFile.getId()) {
			messageFile.setId(IDUtils.getId());
			return  messageFileMapper.insert(messageFile) > 0;
		}else {
			return messageFileMapper.updateById(messageFile) > 0;
		}
	}

	@Override
	public List<MessageFile> getMessageFileListByReadTyepAndReceiveId(String readType, String id) {
		QueryWrapper<MessageFile> queryWrapper = new QueryWrapper<MessageFile>();
		queryWrapper.lambda().eq(MessageFile::getReadType, readType).eq(MessageFile::getReceiveId, id);
		return messageFileMapper.selectList(queryWrapper);
	}

}
