package com.gx.obe.server.management.menu.controller;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.gx.obe.server.common.base.BaseController;
import com.gx.obe.server.common.log.OperationType;
import com.gx.obe.server.common.log.SysLogAnnotation;
import com.gx.obe.server.management.menu.entity.AuthMenuEntity;
import com.gx.obe.server.management.menu.entity.AuthMenuFunction;
import com.gx.obe.server.management.menu.service.AuthMenuService;
import com.gx.obe.server.management.vo.BatchUpdatePropertyVo;

import io.swagger.annotations.Api;
import lombok.AllArgsConstructor;

 /**  
 * 
 * @author 
 * @Title: AuthMenuController  
 * @Package com.gx.obe.server.management.menu.controller   
 * @Description:   
 * @author: XiaoMa Pro - myzhichao@163.com
 * @date:   2019-05-09   
 * @version V1.0 
 * @Copyright: 2019 www.msypro.com Inc. All rights reserved. 
 *  
 */
@Api(tags = "菜单管理")
@RestController
@AllArgsConstructor
@RequestMapping("/authMenu")
public class AuthMenuController extends BaseController<AuthMenuService, AuthMenuEntity> {
    
	@Autowired
	private AuthMenuService authMenuService;
	
	@GetMapping("/getUserMenuListByRoleType")
	public List<AuthMenuEntity> getUserResourceMenuList(String userId, String roleType ,String useStatus){
		return authMenuService.selectUserResourceMenuList(userId, roleType, useStatus);
	}
	
	@GetMapping("/getUserRoleMenuList")
	public List<AuthMenuEntity> getUserRoleMenuList(String userId, String roleId ,String useStatus){
		return authMenuService.getUserRoleMenuList(userId, roleId, useStatus);
	}
	
	/**
	 * @Description: 获取顶级菜单信息
	 * @author wangxiang
	 * @return
	 */
	@GetMapping("/getTopResourceList")
	public List<AuthMenuEntity> getTopResourceList(String parentId){
		QueryWrapper<AuthMenuEntity> wrapper = new  QueryWrapper<AuthMenuEntity>();
		wrapper.lambda().isNull(AuthMenuEntity::getParentId);
		return authMenuService.list(wrapper);
	}
	
	/**
	 * @Description: 获取顶级菜单信息
	 * @author wangxiang
	 * @return
	 */
	@GetMapping("/getChildResourceList")
	public List<AuthMenuEntity> getChildResourceList(String parentId){
		QueryWrapper<AuthMenuEntity> wrapper = new  QueryWrapper<AuthMenuEntity>();
		wrapper.lambda().eq(AuthMenuEntity::getParentId,parentId);
		return authMenuService.list(wrapper);
	}
	
	/**
	 * @Description: 获得全部菜单信息(包含子菜单和父级菜单信息)
	 * @author wangxiang
	 * @param type
	 * @return 
	 */
	@GetMapping("/getAllResourceList")
	public List<AuthMenuEntity> getAllResourceList(String type){
		return authMenuService.getAllResourceList(type);
	}
	
	/**
	 * @Description: 获得菜单的最大序号
	 * @author wangxiang
	 * @param parentId
	 * @return
	 */
	@GetMapping("/getMaxResourceSortNo")
	public Integer getMaxResourceSortNo(String parentId, String menuType) {
		return authMenuService.getMaxResourceSortNo(parentId,menuType);
	}
	
	/**
	 * @Description: 交换两个菜单的排序
	 * @author chenxw
	 * @param sourceId
	 * @param targetId
	 * @return 
	 */
	@GetMapping("/changeResourceSortNo")
    public boolean changeResourceSortNo(String sourceId, String targetId) {
        return authMenuService.changeResourceSortNo(sourceId, targetId);
    }
	
	/**
	 * @Description: 查询用户权限菜单
	 * @author wangxiang
	 * @param userId
	 * @param roleType
	 * @param parentMenuId
	 * @return 
	 */
	@GetMapping("/getUserResourceList")
	public List<AuthMenuEntity> getUserResourceList(String userId, String roleType){
		return authMenuService.getUserResourceList(userId, roleType);
	}
	
	/**
	 * @Description: 根据类型获得菜单列表
	 * @author wangxiang
	 * @param menuType
	 * @return 
	 */
	@GetMapping("/getMenuListByMenuType")
	public List<AuthMenuEntity> getMenuListByMenuType(String menuType){
		return authMenuService.getMenuListByMenuType(menuType);
	}
	
	/**
	 * @Description: 菜单编号是否唯一
	 * @author wangxiang
	 * @param menuId
	 * @param menuCode
	 * @return 
	 */
	@GetMapping("/isUniqueMenuCode")
	public boolean isUniqueMenuCode(String menuId, String menuCode){
		return authMenuService.isUniqueMenuCode(menuId,menuCode);
	}
	
	/**
	 * @Description: 根据菜单编码获得菜单
	 * @author wangxiang
	 * @param menuCode
	 * @return 
	 */
	@GetMapping("/getMenuByMenuCode")
	public AuthMenuEntity getMenuByMenuCode(String menuCode){
		QueryWrapper<AuthMenuEntity>   wrapper  =  new QueryWrapper<AuthMenuEntity>();
		wrapper.lambda().eq(AuthMenuEntity::getMenuCode, menuCode);
		return authMenuService.getOne(wrapper);
	}
	
	/**
	 * @Description: 根据类型获得菜单列表
	 * @author wangxiang
	 * @param menuType
	 * @return 
	 */
	@GetMapping("/getMenuListByMenuCode")
	public List<AuthMenuEntity> getMenuListByMenuCode(String menuCode){
		return authMenuService.getMenuListByMenuCode(menuCode);
	}
	
	/**
	 * @Description: 获得下级菜单列表
	 * @author wangxiang
	 * @param menuType
	 * @return 
	 */
	@GetMapping("/getMenuChildList")
	public List<AuthMenuEntity> getMenuChildList(String parentId){
		QueryWrapper<AuthMenuEntity>   wrapper  =  new QueryWrapper<AuthMenuEntity>();
		wrapper.lambda().eq(AuthMenuEntity::getParentId, parentId);
		return authMenuService.list(wrapper);
	}
	
	/**
	 * @Description: 获得菜单下的功能集
	 * @author wangxiang
	 * @param menuId
	 * @return 
	 */
	@GetMapping("/getMenuFunctionMap")
	public Map<String, String> getMenuFunctionMap(String menuId){
		return authMenuService.getMenuFunctionMap(menuId);
	}
	
	@PostMapping("/saveOrUpdateNoEmptyReturn")
	public AuthMenuEntity saveOrUpdateNoEmptyReturn(@RequestBody AuthMenuEntity menu){
		return authMenuService.saveOrUpdateNoEmptyReturn(menu);	
	}
	
	/**
	 * @Description: 批量指定字段修改
	 * @author chenxw
	 * @param batchUpdatePropertyVo
	 * @return 
	 */
	@PostMapping("/batchUpdateProperty")
    public boolean batchUpdateProperty(@RequestBody BatchUpdatePropertyVo<AuthMenuEntity> batchUpdatePropertyVo) {
        List<AuthMenuEntity> menuList = batchUpdatePropertyVo.getEntityList();
        String[] attributes = batchUpdatePropertyVo.getAttributes();
        return authMenuService.batchUpdateProperty(menuList, attributes);
    }
	
	@PostMapping("/updateAssignProperty")
	public boolean updateAssignProperty(@RequestBody AuthMenuEntity menu){
		return authMenuService.updateAssignProperty(menu);
	} 
	
	/**
	 * @Description: 菜单编码获取菜单
	 * @author mazc
	 * @param menuCode
	 * @return 
	 */
	@GetMapping("/getMenutByMenuCode")
	public AuthMenuEntity getMenutByMenuCode(String menuCode){
		return authMenuService.getMenuByMenuCode(menuCode);
	}
	
	
	/**
	 * @Description: 根据menuId 删除
	 * @author wangxiang
	 * @param menuId 
	 */
	@GetMapping("/deleteByMenuId")
	public Integer deleteByMenuId(String menuId) {
		return authMenuService.deleteByMenuId(menuId);
	}
	
	
	
}