package com.gx.obe.server.management.menu.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.management.menu.dao.AuthMenuFunctionMapper;
import com.gx.obe.server.management.menu.entity.AuthMenuFunction;
import com.gx.obe.server.management.menu.service.AuthMenuFunctionService;

/**  
 * 
 * @author 
 * @Title: AuthMenuFunctionService
 * @Package com.gx.obe.server.management.menu.service  
 * @Description:     
 * @author: XiaoMa Pro - myzhichao@163.com
 * @date:   2019-05-10   
 * @version V1.0 
 * @Copyright: 2019 www.msypro.com Inc. All rights reserved. 
 *  
 */
@Service
public class AuthMenuFunctionServiceImpl extends ServiceImpl<AuthMenuFunctionMapper, AuthMenuFunction> implements AuthMenuFunctionService {

	@Autowired
	private AuthMenuFunctionMapper authMenuFunctionMapper;

	@Override
	public Map<String, String> getFunctionByMenuIdReturnMap(String menuId) {
 		Map<String, String> menuFunctionMap = null;
		 List<Map<String, Object>> list = authMenuFunctionMapper.getFunctionByMenuIdReturnMap(menuId);
			if(null != list && list.size() > 0){
				menuFunctionMap = new HashMap<String, String>();
				for(Map<String, Object> tempmap : list){
					menuFunctionMap.put((String)tempmap.get("FUNCTION_CODE"), (String)tempmap.get("FUNCTION_STATUS"));
				}
			}
		
		return menuFunctionMap;
	}

	/** 
	 * @Description:获得最大序号
	 * @author mazc
	 * @param menuId
	 * @return 
	 */
	@Override
	public int getMaxSortNo(String menuId) {
	    Integer maxSortNo = authMenuFunctionMapper.getMaxSortNo(menuId);
	    if (null == maxSortNo) {
	        maxSortNo = 0;
        }
		return maxSortNo;
	}

	/** 
	 * @Description: 是否统一编码
	 * @author mazc
	 * @param menuId
	 * @param functionId
	 * @param functionCode
	 * @return 
	 */
	@Override
    public boolean isUniqueFunctionCode(String menuId, String functionId, String functionCode) {
        return authMenuFunctionMapper.isUniqueFunctionCode(menuId, functionId, functionCode) <= 1;
    }

	/** 
	 * @Description: 批量保存
	 * @author wangxiang
	 * @param menuFunctionList
	 * @return 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public int batchSaveOrUpdate(List<AuthMenuFunction> menuFunctionList) {
		int count = 0;
		List<AuthMenuFunction>  insertAuthMenuFunctionList = new ArrayList<AuthMenuFunction>();
		for(AuthMenuFunction authMenuFunction : menuFunctionList) {
			authMenuFunction.setId(IDUtils.getId());
			insertAuthMenuFunctionList.add(authMenuFunction);
		}
		count +=  authMenuFunctionMapper.insertByBatch(insertAuthMenuFunctionList);			
		return count;
	}

	@Override
	public boolean changeResourceSortNo(String sourceId, String targetId) {
		return authMenuFunctionMapper.changeResourceSortNo(sourceId, targetId);
	}

}
