package com.gx.obe.server.management.negotiation.service;

import com.baomidou.mybatisplus.extension.service.IService;
import java.util.List;

import com.gx.obe.server.management.evaluation.entity.EvaluationStep;
import com.gx.obe.server.management.negotiation.entity.Negotiation;
import com.gx.obe.server.management.negotiation.entity.NegotiationResult;
import com.gx.obe.server.management.supplier.entity.SupplierEvaluationStep;
 
/** 
 * @Description: 
 * @author mazc 
 */
public interface NegotiationService extends IService<Negotiation> {
 	/**
	 * @Description: 批量更新
	 * @author mazc
	 * @param ObeNegotiationList 
	 */
 	Integer updateByBatch (List<Negotiation> ObeNegotiationList);
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param ObeNegotiationList 
	 */
	Integer insertByBatch (List<Negotiation> ObeNegotiationList);
	
	
	
	/** 
	 * @Description: 删除所有的谈判
	 * @author mazc
	 * @param tenderId 项目id
	 * @return 
	 */
	public int deleteStepByTenderId(String tenderId);
	
	/**
	 * @Description: 获得竞争性谈判评标结果
	 * @param tenderId 项目ID编号
	 * @param stepId 项目评标步骤ID编号
	 * @param supplierId 投标人ID编号
	 * @return List<NegotiationResult>
	 */
	List<NegotiationResult> getEvalNegotiationResult(String tenderId, String stepId, String supplierId);
	/**
	 * @Description: 保存竞争性谈判评标结果
	 * @param tenderId 项目ID编号
	 * @param userId 用户ID编号
	 * @param evaluationStep 项目评标步骤（评标步骤）
	 * @param nodeNegotiationResultList 评审竞争性谈判指标结果
	 * @param supplierEvalNodeList 投标人评标步骤
	 * @return boolean
	 */
	boolean saveNegotiationResult(String tenderId, String userId, EvaluationStep evaluationStep, List<NegotiationResult> nodeNegotiationResultList, List<SupplierEvaluationStep> supplierEvalNodeList);
	/**
	 * @Description: 提交竞争性谈判评标结果
	 * @param tenderId 项目ID编号
	 * @param userId 用户ID编号
	 * @param evaluationStep 项目评标步骤（评标步骤）
	 * @return boolean
	 */
	boolean submitNegotiationResult(String tenderId, String userId, EvaluationStep evaluationStep);
}



