package com.gx.obe.server.management.negotiation.service.impl;

import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.enumeration.CommonEnum;
import com.gx.obe.server.common.utils.ObjectUtils;
import com.gx.obe.server.management.evaluation.dao.EvaluationStepMapper;
import com.gx.obe.server.management.evaluation.entity.EvaluationStep;
import com.gx.obe.server.management.expert.dao.ExpertEvaluationStepMapper;
import com.gx.obe.server.management.expert.entity.ExpertEvaluationStep;
import com.gx.obe.server.management.negotiation.dao.NegotiationMapper;
import com.gx.obe.server.management.negotiation.entity.Negotiation;
import com.gx.obe.server.management.negotiation.entity.NegotiationResult;
import com.gx.obe.server.management.negotiation.service.NegotiationResultService;
import com.gx.obe.server.management.negotiation.service.NegotiationService;
import com.gx.obe.server.management.project.dao.TenderProjectMapper;
import com.gx.obe.server.management.supplier.entity.SupplierEvaluationStep;
import com.gx.obe.server.management.supplier.service.SupplierEvaluationStepService;

/** 
 * @Description: 
 * @author mazc 
 */
@Service
@Transactional
public class NegotiationServiceImpl extends ServiceImpl<NegotiationMapper, Negotiation> implements NegotiationService {

	@Autowired
	private NegotiationMapper negotiationMapper;
	
	@Autowired
	private NegotiationResultService negotiationResultService;
	
	@Autowired
	private SupplierEvaluationStepService supplierEvaluationStepService;
	
	@Autowired
	private ExpertEvaluationStepMapper expertEvaluationStepMapper;
	@Autowired
	private TenderProjectMapper tenderProjectMapper;
	@Autowired
	private EvaluationStepMapper evaluationStepMapper;
	/**
	 * @Description: 批量更新
	 * @author mazc
	 * @param ObeNegotiationList 
	 */
	 @Override
	 @Transactional(rollbackFor = Exception.class)
	public Integer updateByBatch (List<Negotiation> ObeNegotiationList){
		return negotiationMapper.updateBatchList(ObeNegotiationList);
	}
	
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param ObeNegotiationList 
	 */
	 @Override
	 @Transactional(rollbackFor = Exception.class)
	public Integer insertByBatch (List<Negotiation> ObeNegotiationList){
		return negotiationMapper.insertByBatch(ObeNegotiationList);
	}

	/** 
	 * @Description: 删除所有的谈判
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	@Override
	public int deleteStepByTenderId(String tenderId) {
		QueryWrapper<Negotiation> queryWrapper = new QueryWrapper<Negotiation>();
		queryWrapper.lambda().eq(Negotiation::getTenderId, tenderId);
		return negotiationMapper.delete(queryWrapper);
	}

	
	/**
	 * @Description: 获得竞争性谈判评标结果
	 * @param tenderId 项目ID编号
	 * @param stepId 项目评标步骤ID编号
	 * @param supplierId 投标人ID编号
	 * @return List<NegotiationResult>
	 */
	@Override
	public List<NegotiationResult> getEvalNegotiationResult(String tenderId, String stepId, String supplierId) {
		return negotiationMapper.getEvalNegotiationResult(tenderId,stepId,supplierId);
	}

	/**
	 * @Description: 保存竞争性谈判评标结果
	 * @param tenderId 项目ID编号
	 * @param userId 用户ID编号
	 * @param evaluationStep 项目评标步骤（评标步骤）
	 * @param nodeNegotiationResultList 评审竞争性谈判指标结果
	 * @param supplierEvalNodeList 投标人评标步骤
	 * @return boolean
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public boolean saveNegotiationResult(String tenderId, String userId, EvaluationStep evaluationStep, List<NegotiationResult> nodeNegotiationResultList, List<SupplierEvaluationStep> supplierEvalNodeList) {
		// 竞争性谈判评标结果
		if (null != nodeNegotiationResultList && nodeNegotiationResultList.size() > 0) {
			negotiationResultService.saveOrUpdateBatch(nodeNegotiationResultList);
		}
		// 投标人评标步骤
		if(null != supplierEvalNodeList && supplierEvalNodeList.size() > 0){
			supplierEvaluationStepService.saveOrUpdateBatch(supplierEvalNodeList);
		}
		
		return true;
	}

	
	/**
	 * @Description: 提交竞争性谈判评标结果
	 * @param tenderId 项目ID编号
	 * @param userId 用户ID编号
	 * @param evaluationStep 项目评标步骤（评标步骤）
	 * @return boolean
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public boolean submitNegotiationResult(String tenderId, String userId, EvaluationStep evaluationStep) {
		int count = 0;
		// 修改专家的评审进度为完成
		UpdateWrapper<ExpertEvaluationStep>  updateWrapper = new UpdateWrapper<ExpertEvaluationStep>();
		updateWrapper.set("STEP_STATUS", CommonEnum.COMPLETE).set("END_TIME", new Date()).lambda()
				.eq(ExpertEvaluationStep::getTenderId, tenderId).eq(ExpertEvaluationStep::getStepId, evaluationStep.getId())
				.eq(ExpertEvaluationStep::getUserId, userId);
		count += expertEvaluationStepMapper.update(null, updateWrapper);
		
		// 修改项目的进度状态
		if(!ObjectUtils.equals(evaluationStep.getStepStatus(), CommonEnum.RE_OPERATOR)){
			count += tenderProjectMapper.updateEvaluateStatus(tenderId, evaluationStep.getStepName()+"完成");
		}
		
		// 修改项目的评审状态为完成
		count += evaluationStepMapper.updateEvaluateStatus(CommonEnum.COMPLETE,evaluationStep.getId());
		
		return count >= 0;
	}
	

}
