package com.gx.obe.server.management.other.controller;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gx.obe.server.common.base.BaseController;
import com.gx.obe.server.management.other.entity.SignatureFile;
import com.gx.obe.server.management.other.service.SignatureFileService;
import com.gx.obe.server.management.vo.SaveBatchSignatureFileVo;
import com.gx.obe.server.management.vo.SaveSignatureFileVo;
import com.gx.obe.server.management.vo.UpdateAssignPropertyVo;

import io.swagger.annotations.Api;
import lombok.AllArgsConstructor;
/** 
 * @Description: 
 * @author mazc 
 */
@Api(tags = "")
@RestController
@AllArgsConstructor
@RequestMapping("/signatureFile")
public class SignatureFileController extends BaseController<SignatureFileService,SignatureFile>{
	@Autowired
	public SignatureFileService SignatureFileServices;
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param ObeSignatureFileList 
	 */
	@PostMapping("/insertByBatch")
	public int insertByBatch(@RequestBody List<SignatureFile> ObeSignatureFileList) {
		return SignatureFileServices.insertByBatch(ObeSignatureFileList);
	}
	/**
	 * @Description: 批量更新
	 * @author mazc
	 * @param ObeSignatureFileList 
	 */
	@PostMapping("/updateByBatch")
	public int updateByBatch(@RequestBody List<SignatureFile> ObeSignatureFileList) {
		return SignatureFileServices.updateByBatch(ObeSignatureFileList);
	}
	
	/**
	 * @Description: 获得当前项目的流转签章文件
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	@GetMapping("/getSignatureFileList")
	public List<SignatureFile> getSignatureFileList(String tenderId,String reportType){
		return SignatureFileServices.getSignatureFileList(tenderId,reportType);
	
	}
	
	/**
	 * @Description: 添加流转签章报表
	 * @author mazc
	 * @param saveBatchSignatureFileVo
	 * @return 
	 */
	@PostMapping("/saveBatchSignatureFile")
	public boolean saveBatchSignatureFile(@RequestBody SaveBatchSignatureFileVo saveBatchSignatureFileVo){
		if(null == saveBatchSignatureFileVo) {
			return false;
		}
		return SignatureFileServices.saveBatchSignatureFile(saveBatchSignatureFileVo.getSignatureFileList(),saveBatchSignatureFileVo.getSignatureUserList());
	}
	
	/**
	 * @Description: 获得签名文件
	 * @author mazc
	 * @param signatureId
	 * @return 
	 */
	@GetMapping("/getSignatureFileById")
	public SignatureFile getSignatureFileById(String id) {
		return SignatureFileServices.getSignatureFileById(id);
	}
	
	
	/**
	 * @Description: 根据文件名称获取待签名的文件
	 * @author mazc
	 * @param tenderId
	 * @param fileName
	 * @return 
	 */
	@GetMapping("/getSignatureFileByFileName")
	public SignatureFile getSignatureFileByFileName(String tenderId, String fileName) {
		return SignatureFileServices.getSignatureFileByFileName(tenderId, fileName);
	}
	 /**
	 * @Description: 批量申请流转签名
	 * @author mazc
	 * @param tenderId
	 * @param userId
	 * @param userName
	 * @return 
	 */
	@GetMapping("/batchApplySignatureFile")
	public Integer batchApplySignatureFile(String tenderId, String userId, String userName) {
		return SignatureFileServices.batchApplySignatureFile(tenderId, userId, userName);
	}
	
	/**
	 * @Description: 批量释放已申请签章
	 * @author mazc
	 * @param tenderId
	 * @param userId
	 * @return 
	 */
	@GetMapping("/batchReleaseApplySignatureFile")
	public void batchReleaseApplySignatureFile(String tenderId, String userId) {
		SignatureFileServices.batchReleaseApplySignatureFile(tenderId, userId);
	}
	/**
	 * @Description: 批量释放已申请签章
	 * @author mazc
	 * @param tenderId
	 * @param signatureId
	 * @return 
	 */
	@GetMapping("/releaseApplySignatureFile")
	public void releaseApplySignatureFile(String signatureId, String userId) {
		SignatureFileServices.releaseApplySignatureFile(signatureId, userId);
	}
	
	/**
	 * @Description: 申请签章
	 * @author mazc
	 * @param signatureId
	 * @param userId
	 * @param userName
	 * @return 
	 */
	@GetMapping("/applySignatureFile")
	public SignatureFile applySignatureFile(String signatureId, String userId, String userName){
		return SignatureFileServices.applySignatureFile(signatureId, userId, userName);
	}
	
	
	
	
	/**
	 * @Description: 保存签章文件
	 * @author mazc
	 * @param saveSignatureFileVo
	 * @return 
	 */
	@PostMapping("/saveSignatureFile")
	public boolean saveSignatureFile(@RequestBody SaveSignatureFileVo saveSignatureFileVo) {
		if(null == saveSignatureFileVo) {
			return false;
		}
		return SignatureFileServices.saveSignatureFile(saveSignatureFileVo.getSignatureId(), saveSignatureFileVo.getSignatureUser(), saveSignatureFileVo.getSignatureTime());
	}
	
	/**
	 * @Description: 删除流转签章文件
	 * @author mazc
	 * @param signatureId
	 * @return 
	 */
	@GetMapping("/deleteSignatureFile")
	public boolean deleteSignatureFile(String signatureId){
		return SignatureFileServices.deleteSignatureFile(signatureId);
	}
	
	@PostMapping("/updateAssignProperty")
	public boolean updateAssignProperty(@RequestBody UpdateAssignPropertyVo<SignatureFile> updateAssignPropertyVo) {
		SignatureFile signatureFile = updateAssignPropertyVo.getEntity();
		String[] attributes = updateAssignPropertyVo.getAttributes();
		return SignatureFileServices.updateAssignProperty(signatureFile, attributes);
	}

	@GetMapping("/getSignatureFileByTenderIdAndOrderByCreateTime")
	public List<SignatureFile> getSignatureFileByTenderIdAndOrderByCreateTime(String tenderId){
		return SignatureFileServices.getSignatureFileByTenderIdAndOrderByCreateTime(tenderId);
	}

	/**
	 * @Description: 检查报表PDF文件是否为流转文件
	 * @author chenxw
	 * @param tenderId
	 * @param pdfFileName
	 * @return
	 */
	@GetMapping("/checkIsReportFile")
	public boolean checkIsReportFile(String tenderId, String pdfFileName) {
		return SignatureFileServices.checkIsReportFile(tenderId, pdfFileName);
	}
	
	/**
	 * @Description: 获取当前签名文件的签名人ID
	 * @author mazc
	 * @param signatureId
	 * @return 
	 */
	@GetMapping("/getSignatureUserId")
	public String getSignatureUserId(String signatureId) {
		return SignatureFileServices.getSignatureUserId(signatureId);
	}

}