package com.gx.obe.server.management.other.dao;

import java.util.Date;
import java.util.List;

import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.gx.obe.server.management.other.entity.SignatureFile;
/** 
 * @Description: 
 * @author mazc 
 */
 @Mapper
public interface SignatureFileMapper extends BaseMapper<SignatureFile> {

	/**
	 * @Description: 批量更新
	 * @author mazc
	 * @param ObeSignatureFileList 
	 */
	Integer updateBatchList (@Param("ObeSignatureFileList") List<SignatureFile> ObeSignatureFileList);
	
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param ObeSignatureFileList 
	 */
	Integer insertByBatch (@Param("ObeSignatureFileList")List<SignatureFile> ObeSignatureFileList);

	/**
	 * @Description: 指定字段修改
	 * @author chenxw
	 * @param ObeSignatureFile
	 * @param attributes
	 */
	Integer updateAssignProperty(@Param("ObeSignatureFile")SignatureFile ObeSignatureFile, @Param("attributes")String[] attributes);

	/**
	 * @Description: 批量指定字段修改
	 * @author chenxw
	 * @param ObeSignatureFileList
	 * @param attributes
	 */
	Integer batchUpdateProperty(@Param("ObeSignatureFileList")List<SignatureFile> ObeSignatureFileList, @Param("attributes")String[] attributes);
	
	/**
	 * @Description: 获得当前项目的流转签章文件
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	List<SignatureFile> getSignatureFileList(@Param("tenderId")String tenderId,@Param("reportType")String reportType);
	
	/**
	 * @Description: 获得签名文件
	 * @author mazc
	 * @param signatureId
	 * @return 
	 */
	SignatureFile getSignatureFileById(@Param("id")String id);
	
	 /**
	 * @Description: 批量申请流转签名
	 * @author mazc
	 * @param tenderId
	 * @param userId
	 * @param userName
	 * @return 
	 */
	Integer batchApplySignatureFile(@Param("tenderId")String tenderId, @Param("userId")String userId, @Param("userName")String userName);
	
	/**
	 * @Description: 批量释放已申请签章
	 * @author mazc
	 * @param tenderId
	 * @param userId
	 * @return 
	 */
	void batchReleaseApplySignatureFile(@Param("tenderId")String tenderId, @Param("userId")String userId);
	
	/**
	 * @Description: 释放已申请签章
	 * @author mazc
	 * @param signatureId
	 * @return 
	 */
	void releaseApplySignatureFile(@Param("signatureId")String signatureId, @Param("userId")String userId);
	
	 /**
	 * @Description: 
	 * @author mazc
	 * @param signatureId
	 * @param userId
	 * @param userName
	 * @return 
	 */
	Integer updateApplySignatureFile(@Param("signatureId")String signatureId, @Param("userId")String userId, @Param("userName")String userName);
	
	
	/**
	 * @Description: 申请签章成功
	 * @author mazc
	 * @param signatureId
	 * @return 
	 */
	SignatureFile applySignatureFileSuccess(@Param("signatureId")String signatureId);
	
	/**
	 * @Description: 申请签章失败 不包含 签章文件字节 添加签章时间
	 * @author mazc
	 * @param signatureId
	 * @return 
	 */
	SignatureFile applySignatureFileFail(@Param("signatureId")String signatureId);
	
	/**
	 * @Description: 根据文件名称获取待签名的文件
	 * @author mazc
	 * @param tenderId
	 * @param fileName
	 * @return 
	 */
	SignatureFile getSignatureFileByFileName(@Param("tenderId")String tenderId, @Param("fileName")String fileName);
	
	
	
	 /**
	 * @Description: 保存签章文件
	 * @author mazc
	 * @param signatureId
	 * @param signatureFileByte
	 * @param userId
	 * @param signatureTime
	 * @return 
	 */
	Integer saveSignatureFile(@Param("signatureId")String signatureId, @Param("userId")String userId,@Param("signatureTime") Date signatureTime);
	
	/**
	 * @Description: 获取当前签名文件的签名人ID
	 * @author mazc
	 * @param signatureId
	 * @return 
	 */
	String getSignatureUserId(@Param("signatureId")String signatureId);

}