package com.gx.obe.server.management.other.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.enumeration.CommonEnum;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.other.dao.SignatureFileMapper;
import com.gx.obe.server.management.other.dao.SignatureUserMapper;
import com.gx.obe.server.management.other.entity.SignatureFile;
import com.gx.obe.server.management.other.entity.SignatureUser;
import com.gx.obe.server.management.other.service.SignatureFileService;
import com.gx.obe.server.management.other.service.SignatureUserService;

/** 
 * @Description: 
 * @author mazc 
 */
@Service
@Transactional
public class SignatureFileServiceImpl extends ServiceImpl<SignatureFileMapper, SignatureFile> implements SignatureFileService {

	@Autowired
	private SignatureFileMapper signatureFileMapper;
	
	@Autowired
	private SignatureUserMapper signatureUserMapper;
	
	@Autowired
	private SignatureUserService signatureUserService;
	/**
	 * @Description: 批量更新
	 * @author mazc
	 * @param ObeSignatureFileList 
	 */
	 @Override
	 @Transactional(rollbackFor = Exception.class)
	public Integer updateByBatch (List<SignatureFile> ObeSignatureFileList){
		return signatureFileMapper.updateBatchList(ObeSignatureFileList);
	}
	
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param ObeSignatureFileList 
	 */
	 @Override
	 @Transactional(rollbackFor = Exception.class)
	public Integer insertByBatch (List<SignatureFile> ObeSignatureFileList){
		return signatureFileMapper.insertByBatch(ObeSignatureFileList);
	}

	/**
	 * @Description: 指定字段修改
	 * @author chenxw
	 * @param ObeSignatureFile
	 * @param attributes
	 */
	@Override
	public boolean updateAssignProperty(SignatureFile ObeSignatureFile, String[] attributes) {
		return signatureFileMapper.updateAssignProperty(ObeSignatureFile, attributes) > 0;
	}

	/**
	 * @Description: 批量指定字段修改
	 * @author chenxw
	 * @param ObeSignatureFileList
	 * @param attributes
	 */
	@Override
	public Integer batchUpdateProperty(List<SignatureFile> ObeSignatureFileList, String[] attributes) {
		return signatureFileMapper.batchUpdateProperty(ObeSignatureFileList, attributes);
	}

	/**
	 * @Description: 批量添加或跟新
	 * @author chenxw
	 * @param ObeSignatureFileList
	 * @param attributes
	 * @return
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public Integer batchSaveOrUpdate(List<SignatureFile> ObeSignatureFileList, String[] attributes) {
		List<SignatureFile> insertObeSignatureFileList = new ArrayList<>();
		List<SignatureFile> updateObeSignatureFileList = new ArrayList<>();
		for (SignatureFile ObeSignatureFile : ObeSignatureFileList) {
			if (StringUtils.isEmpty(ObeSignatureFile.getId())) {
				ObeSignatureFile.setId(IDUtils.getId());
				insertObeSignatureFileList.add(ObeSignatureFile);
			} else {
				updateObeSignatureFileList.add(ObeSignatureFile);
			}
		}
		Integer count = 0;
		if (insertObeSignatureFileList.size() > 0) {
			count += signatureFileMapper.insertByBatch(insertObeSignatureFileList);
		}
		if (updateObeSignatureFileList.size() > 0) {
			count += signatureFileMapper.batchUpdateProperty(updateObeSignatureFileList, attributes);
		}
		return count;
	}

	/** 
	 * @Description: 获得当前项目的流转签章文件
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	@Override
	public List<SignatureFile> getSignatureFileList(String tenderId,String reportType) {
		return signatureFileMapper.getSignatureFileList(tenderId,reportType);
	}

	
	@Override
	@Transactional(rollbackFor = Exception.class)
	public boolean saveBatchSignatureFile(List<SignatureFile> signatureFileList,List<SignatureUser> signatureUserList) {
		int count = 0;
		count += signatureUserMapper.insertByBatch(signatureUserList);
		count += signatureFileMapper.insertByBatch(signatureFileList);
		return count > 0;
	}
 
	/**
	 * @Description: 获得签名文件
	 * @author mazc
	 * @param signatureId
	 * @return 
	 */
	public SignatureFile getSignatureFileById(String id) {
		return signatureFileMapper.getSignatureFileById(id);
	}
	
	
	/**
	 * @Description: 根据文件名称获取待签名的文件
	 * @author mazc
	 * @param tenderId
	 * @param fileName
	 * @return 
	 */
	@Override
	public SignatureFile getSignatureFileByFileName(String tenderId, String fileName) {
		return signatureFileMapper.getSignatureFileByFileName(tenderId, fileName);
	}
	
	 /**
	 * @Description: 批量申请流转签名
	 * @author mazc
	 * @param tenderId
	 * @param userId
	 * @param userName
	 * @return 
	 */
	public Integer batchApplySignatureFile(String tenderId, String userId, String userName) {
		return signatureFileMapper.batchApplySignatureFile(tenderId, userId, userName);
	}
	
	/**
	 * @Description: 批量释放已申请签章
	 * @author mazc
	 * @param tenderId
	 * @param userId
	 * @return 
	 */
	public void batchReleaseApplySignatureFile(String tenderId, String userId) {
		signatureFileMapper.batchReleaseApplySignatureFile(tenderId, userId);
	}
	
	/**
	 * @Description: 释放已申请签章
	 * @author mazc
	 * @param signatureId
	 * @return 
	 */
	public void releaseApplySignatureFile(String signatureId, String userId) {
		signatureFileMapper.releaseApplySignatureFile(signatureId, userId);
	}
	
	/** 
	 * @Description: 申请签章
	 * @author mazc
	 * @param signatureId
	 * @param userId
	 * @param userName
	 * @return 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public SignatureFile applySignatureFile(String signatureId, String userId, String userName) {
		boolean applay = signatureFileMapper.updateApplySignatureFile(signatureId, userId, userName) > 0;
		if(applay) {
			return signatureFileMapper.applySignatureFileSuccess(signatureId);
		}else {
			return	signatureFileMapper.applySignatureFileFail(signatureId);
		}
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public boolean saveSignatureFile(String signatureId, SignatureUser signatureUser,Date signatureTime) {		
		if(signatureFileMapper.saveSignatureFile(signatureId, signatureUser.getUserId(), signatureTime)>0) {
			signatureUser.setSignatureType(CommonEnum.YES);
			signatureUser.setSignatureTime(signatureTime);
			return signatureUserService.updateAssignProperty(signatureUser, new String[]{"signatureType", "signatureTime"});
		} 
		return false;
	}

	@Override
	public boolean deleteSignatureFile(String signatureId) {
		if(signatureFileMapper.deleteById(signatureId) > 0) {
			QueryWrapper<SignatureUser> queryWrapper = new QueryWrapper<SignatureUser>();
			queryWrapper.eq(SignatureUser.SIGNATURE_ID, signatureId);
			return signatureUserMapper.delete(queryWrapper) > 0;
		}
		return false;
	}

	@Override
	public List<SignatureFile> getSignatureFileByTenderIdAndOrderByCreateTime(String tenderId) {
		QueryWrapper<SignatureFile> queryWrapper = new QueryWrapper<SignatureFile>();
		queryWrapper.lambda().eq(SignatureFile::getTenderId, tenderId).orderByAsc(SignatureFile::getCreateTime);
		return signatureFileMapper.selectList(queryWrapper);
	}

	/**
	 * @Description: 检查报表PDF文件是否为流转文件
	 * @author chenxw
	 * @param tenderId
	 * @param pdfFileName
	 * @return
	 */
	@Override
	public boolean checkIsReportFile(String tenderId, String fileName) {
		QueryWrapper<SignatureFile> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda().eq(SignatureFile::getTenderId, tenderId)
							 .eq(SignatureFile::getFileName, fileName);
		return signatureFileMapper.selectCount(queryWrapper) > 0;
	}
	
	/**
	 * @Description: 获取当前签名文件的签名人ID
	 * @author mazc
	 * @param signatureId
	 * @return 
	 */
	@Override
	public String getSignatureUserId(String signatureId) {
		return signatureFileMapper.getSignatureUserId(signatureId);
	}
}
