package com.gx.obe.server.management.price.controller;
import com.gx.obe.server.common.base.BaseController;
import com.gx.obe.server.management.price.entity.BidPriceRoundStatus;
import com.gx.obe.server.management.price.entity.ReBidPriceRound;
import com.gx.obe.server.management.price.service.BidPriceRoundStatusService;
import com.gx.obe.server.management.price.service.ReBidPriceRoundService;
import io.swagger.annotations.Api;
import lombok.AllArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;
/** 
 * @Description: 
 * @author mazc 
 */
@Api(tags = "报价状态")
@RestController
@AllArgsConstructor
@RequestMapping("/bidPriceRoundStatus")
public class BidPriceRoundStatusController extends BaseController<BidPriceRoundStatusService,BidPriceRoundStatus>{
	@Autowired
	public BidPriceRoundStatusService BidPriceRoundStatusServices;
	
	@Autowired
	public  ReBidPriceRoundService reBidPriceRoundService;
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param ObeBidPriceRoundStatusList 
	 */
	@PostMapping("/insertByBatch")
	public int insertByBatch(@RequestBody List<BidPriceRoundStatus> ObeBidPriceRoundStatusList) {
		return BidPriceRoundStatusServices.insertByBatch(ObeBidPriceRoundStatusList);
	}
	/**
	 * @Description: 批量更新
	 * @author mazc
	 * @param ObeBidPriceRoundStatusList 
	 */
	@PostMapping("/updateByBatch")
	public int updateByBatch(@RequestBody List<BidPriceRoundStatus> ObeBidPriceRoundStatusList) {
		return BidPriceRoundStatusServices.updateByBatch(ObeBidPriceRoundStatusList);
	}

	/**
	 * 
	 * @Description: 当前轮次是否完成报价 
	 * @author mazc
	 * @param strRoundId
	 * @return
	 */
	@GetMapping("/isCompletePrice")
	public boolean isCompletePrice(String strRoundId) {
		return BidPriceRoundStatusServices.isCompletePrice(strRoundId);
	}
	
	/**
	 * @Description: 当前轮次指定供应商是否完成报价
	 * @author mazc
	 * @param strRoundId
	 * @param supplier
	 * @return 
	 */
	@GetMapping("/isCompletePriceBySupplier")
	public boolean isCompletePriceBySupplier(String strRoundId,String supplier) {
		return BidPriceRoundStatusServices.isCompletePriceBySupplier(strRoundId, supplier);
	}
	
	/**
	 * @Description: 当前轮次未完成报价的供应商数量
	 * @author mazc
	 * @param strRoundId
	 * @return 
	 */
	@GetMapping("/getNotCompletePriceNumber")
	public int getNotCompletePriceNumber(String strRoundId) {
		return BidPriceRoundStatusServices.getNotCompletePriceNumber(strRoundId);
	}
	
	
	/**
	 * @Description: 当前轮次已完成的供应商数量
	 * @author mazc
	 * @param strRoundId
	 * @return 
	 */
	@GetMapping("/getCompletePriceNumber")
	public int getCompletePriceNumber(String strRoundId) {
		return BidPriceRoundStatusServices.getCompletePriceNumber(strRoundId);
	}
	
	
	/**
	 * @Description: 更新轮次指定供应商报价状态
	 * @author mazc
	 * @param roundId
	 * @param supplier
	 * @return 
	 */
	@PostMapping("/updateCompletePriceStatusBySupplierId")
	public boolean updateCompletePriceStatusBySupplierId(String roundId,String supplier,String status) {
		return BidPriceRoundStatusServices.updateCompletePriceStatusBySupplierId(roundId, supplier,status);
	}
	
	
	
	@PostMapping("/updateCompletePriceStatus")
	public boolean updateCompletePriceStatus(String roundId ,String status) {
		return BidPriceRoundStatusServices.updateCompletePriceStatus(roundId,status);
	}
	
	/**
	 * 
	 * @Description: 获得轮次供应商状态信息
	 * @author mazc
	 * @param roundId
	 * @return
	 */
	@GetMapping("/getSupplierBidPriceRoundStatus")
	public Map<String, String> getSupplierBidPriceRoundComplateStatus(String roundId){
		return BidPriceRoundStatusServices.getSupplierBidPriceRoundComplateStatus(roundId);
	}

	/**
	 * @Description: 根据轮次Id及供应商Id获取数据
	 * @author chenxw
	 * @param priceRoundId
	 * @param supplierId
	 * @return
	 */
	@GetMapping("/getBidPriceRoundStatus")
	public BidPriceRoundStatus getBidPriceRoundStatus(String priceRoundId, String supplierId) {
		return BidPriceRoundStatusServices.getBidPriceRoundStatus(priceRoundId, supplierId);
	}

	@GetMapping("/getBidPriceRoundComplateStatus")
	public Map<String, String> getBidPriceRoundComplateStatus(String tenderId){
		return BidPriceRoundStatusServices.getBidPriceRoundComplateStatus(tenderId);
	}
	
	@PostMapping("/updatRoundStatus")
	public void updateRoundStatus(@RequestBody ReBidPriceRound reBidPriceRound) {
		BidPriceRoundStatusServices.updateRoundStatus(reBidPriceRound);
	}
}