package com.gx.obe.server.management.price.service.impl;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.entity.ReflecUtils;
import com.gx.obe.server.common.enumeration.BidPriceEnum;
import com.gx.obe.server.common.utils.CollectionUtils;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.management.evaluation.dao.BidPriceHisResultMapper;
import com.gx.obe.server.management.evaluation.dao.BidPriceItemResultMapper;
import com.gx.obe.server.management.evaluation.dao.BidPriceResultMapper;
import com.gx.obe.server.management.evaluation.entity.BidPriceHisResult;
import com.gx.obe.server.management.evaluation.entity.BidPriceItemResult;
import com.gx.obe.server.management.evaluation.entity.BidPriceResult;
import com.gx.obe.server.management.evaluation.service.BidPriceHisResultService;
import com.gx.obe.server.management.evaluation.service.BidPriceResultService;
import com.gx.obe.server.management.price.dao.BidPriceItemHisResultMapper;
import com.gx.obe.server.management.price.dao.BidPriceRoundStatusMapper;
import com.gx.obe.server.management.price.dao.ReBidPriceRoundMapper;
import com.gx.obe.server.management.price.entity.BidPriceItemHisResult;
import com.gx.obe.server.management.price.entity.BidPriceRoundStatus;
import com.gx.obe.server.management.price.entity.ReBidPriceRound;
import com.gx.obe.server.management.price.service.ReBidPriceRoundService;
import com.gx.obe.server.management.supplier.dao.SupplierMapper;
import com.gx.obe.server.management.supplier.dao.SupplierOpeningResultMapper;
import com.gx.obe.server.management.supplier.entity.Supplier;
import com.gx.obe.server.management.supplier.entity.SupplierOpeningResult;
import com.gx.obe.server.management.supplier.service.SupplierOpeningResultService;

/** 
 * @Description: 
 * @author wangxiang 
 */
@Service
@Transactional
public class ReBidPriceRoundServiceImpl extends ServiceImpl<ReBidPriceRoundMapper, ReBidPriceRound> implements ReBidPriceRoundService {
	
	@Autowired
	private ReBidPriceRoundMapper reBidPriceRoundMapper;
	
	@Autowired
	private BidPriceResultMapper bidPriceResultMapper;
	
	@Autowired
	private BidPriceHisResultMapper bidPriceHisResultMapper;
	
	@Autowired
	private BidPriceItemResultMapper bidPriceItemResultMapper;
	
	@Autowired
	private BidPriceItemHisResultMapper bidPriceItemHisResultMapper;
	
	@Autowired
	private SupplierOpeningResultMapper supplierOpeningResultMapper;
	
	@Autowired
	private BidPriceRoundStatusMapper bidPriceRoundStatusMapper;

	@Autowired
	private SupplierMapper supplierMapper;
	
	@Autowired
	private BidPriceResultService bidPriceResultService;
	
	@Autowired
	private BidPriceHisResultService bidPriceHisResultService;
	
	@Autowired
	private SupplierOpeningResultService supplierOpeningResultService;
	
	/**添加轮次
	 * @param tenderId
	 * @param round
	 * @return
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public boolean moveHisBidPrice(String tenderId, List<Supplier> supplierList, int round) {
		int count = 0;
		List<BidPriceResult> bidPriceResultList = bidPriceResultMapper.getSingBidPriceResultList(tenderId);
		String roundName = reBidPriceRoundMapper.getRoundName(tenderId,round-1);
		
		List<BidPriceHisResult> bidPriceHisResultList = new ArrayList<BidPriceHisResult>();
		List<String> bidPriceIdList = new ArrayList<String>();
		if(null != bidPriceResultList && bidPriceResultList.size() > 0){
			for(BidPriceResult bidPriceResult : bidPriceResultList){
				BidPriceHisResult bidPriceHisResult = new BidPriceHisResult();
				bidPriceHisResult.setId(IDUtils.getId());
				bidPriceHisResult.setTenderId(tenderId);
				bidPriceHisResult.setBidPrice(bidPriceResult.getBidPrice());
				bidPriceHisResult.setBidPriceId(bidPriceResult.getBidPriceId());
				bidPriceHisResult.setSupplierId(bidPriceResult.getSupplierId());
				bidPriceHisResult.setPriceUnit(bidPriceResult.getPriceUnit());
				bidPriceHisResult.setBidPriceRound(round);
				if(round-1 == 0){
					bidPriceHisResult.setRoundName("初始报价");
				}else{
					bidPriceHisResult.setRoundName(roundName);
				}
				bidPriceHisResult.setCurrencyCode(bidPriceResult.getCurrencyCode());
				bidPriceHisResult.setEvaluationPrice(bidPriceResult.getEvaluationPrice());
				
				bidPriceHisResultList.add(bidPriceHisResult);
				
				if(!bidPriceIdList.contains(bidPriceResult.getBidPriceId())){
					bidPriceIdList.add(bidPriceResult.getBidPriceId());
					List<BidPriceItemResult> bidPriceItemResultList = bidPriceItemResultMapper.getBidPriceItemResultListByBidPriceId(bidPriceResult.getBidPriceId());
					List<BidPriceItemHisResult> bidPriceItemHisResultList = new ArrayList<BidPriceItemHisResult>();
					if(null != bidPriceItemResultList && bidPriceItemResultList.size() > 0){
						for(BidPriceItemResult bidPriceItemResult : bidPriceItemResultList){
							BidPriceItemHisResult bidPriceItemHisResult= new BidPriceItemHisResult();
							bidPriceItemHisResult.setBidPriceItemResultId(IDUtils.getId());
							bidPriceItemHisResult.setTenderId(tenderId);
							bidPriceItemHisResult.setBidPriceId(bidPriceItemResult.getBidPriceId());
							bidPriceItemHisResult.setBidPriceItemId(bidPriceItemResult.getBidPriceItemId());
							bidPriceItemHisResult.setItemCode(bidPriceItemResult.getItemCode());
							bidPriceItemHisResult.setSupplierId(bidPriceItemResult.getSupplierId());
							bidPriceItemHisResult.setItemName(bidPriceItemResult.getItemName());
							bidPriceItemHisResult.setItemValue(bidPriceItemResult.getItemValue());
							bidPriceItemHisResult.setItemContent(bidPriceItemResult.getItemContent());
							bidPriceItemHisResult.setItemUnit(bidPriceItemResult.getItemUnit());
							bidPriceItemHisResult.setRound(round);
							bidPriceItemResult.setItemValue(null);
							
							bidPriceItemHisResultList.add(bidPriceItemHisResult);
						}
					}
					
					if(bidPriceItemHisResultMapper.insertByBatch(bidPriceItemHisResultList) > 0){
						//清空BidPriceItemResult的value
						if(bidPriceItemResultMapper.updateObeBidPriceItemResultBatchList(bidPriceItemResultList) > 0){
							count++;
						}
					}
				}
			}
			if(bidPriceHisResultMapper.insertByBatch(bidPriceHisResultList) > 0){
					count++;
			}
		}
		return count == 2;
	}

	/**
	 * @Description: 获得项目的最大伦次
	 * @author wangxiang
	 * @param tenderId
	 * @return 
	 */
	@Override
	public int getMaxRound(String tenderId) {
		Integer round = reBidPriceRoundMapper.getMaxRound(tenderId);
		return null != round ? round : 0;
	}

	/**
	 * @Description: 复原初始报价
	 * @author wangxiang
	 * @return 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public boolean moveInitBidPrice(String tenderId) {
		List<SupplierOpeningResult>  supplierOpeningResultList = supplierOpeningResultMapper.getSupplierOpeningResultByTenderId(tenderId);
		List<BidPriceResult> bidPriceResultList = bidPriceResultMapper.getSingBidPriceResultList(tenderId);
		List<BidPriceHisResult> bidPriceResultHisList = bidPriceHisResultMapper.getBidPriceHisResultListByRound(tenderId,bidPriceResultList.get(0).getBidPriceId(),0);
		Map<String, BigDecimal> hisMap = new HashMap<String, BigDecimal>();
		for (BidPriceHisResult bidPriceHisResult : bidPriceResultHisList) {
			hisMap.put(bidPriceHisResult.getSupplierId(), bidPriceHisResult.getEvaluationPrice());
		}
		for (BidPriceResult bidPriceResult : bidPriceResultList) {
			bidPriceResult.setEvaluationPrice(hisMap.get(bidPriceResult.getSupplierId()));
		}
		for (SupplierOpeningResult supplierOpeningResult : supplierOpeningResultList) {
			supplierOpeningResult.setEvaluationPrice(hisMap.get(supplierOpeningResult.getSupplierId()));
		}
		
		bidPriceResultMapper.updateObeBidPriceResultBatchList(getUpdateList(bidPriceResultList,new String[]{"evaluationPrice"}));
		
		//复原supplierOpeningResult
		supplierOpeningResultMapper.updateObeSupplierOpeningResultBatchList(getUpdateOpenResultList(supplierOpeningResultList, new String[]{"evaluationPrice"}));
		
		List<BidPriceItemHisResult>  bidPriceItemHisResultList = bidPriceItemHisResultMapper.getItemHisResultBytenderIdpriceIdRoud(tenderId,bidPriceResultList.get(0).getBidPriceId(),0);
		
		List<BidPriceItemResult> bidPriceItemResultList = bidPriceItemResultMapper.getBidPriceItemResultListByBidPriceId(bidPriceResultList.get(0).getBidPriceId());
		
		Map<String, String> hisItemMap = new HashMap<String, String>();
		for (BidPriceItemHisResult bidPriceItemHisResult : bidPriceItemHisResultList) {
			hisItemMap.put(bidPriceItemHisResult.getSupplierId().concat(null != bidPriceItemHisResult.getBidPriceItemId() ? bidPriceItemHisResult.getBidPriceItemId() : ""), bidPriceItemHisResult.getItemValue());
		}
		for (BidPriceItemResult bidPriceItemResult : bidPriceItemResultList) {
			bidPriceItemResult.setItemValue(hisItemMap.get(bidPriceItemResult.getSupplierId().concat(null != bidPriceItemResult.getBidPriceItemId() ? bidPriceItemResult.getBidPriceItemId() : "")));
		}
		
		bidPriceItemResultMapper.updateObeBidPriceItemResultBatchList(getBidPriceItemResultList(bidPriceItemResultList, new String[]{"itemValue"}));
		
		reBidPriceRoundMapper.deleteByTenderId(tenderId);
		bidPriceHisResultMapper.deleteByTenderId(tenderId);
		bidPriceItemHisResultMapper.deleteByTenderId(tenderId);
		return true;
	}
	
	/**
	 * @Description: 批量更新时修改List
	 * @author wangxiang
	 * @param bidPriceResultList
	 * @param attributes
	 * @return 
	 */
	public List<BidPriceResult>  getUpdateList(List<BidPriceResult> bidPriceResultList,String[] attributes){
		List<BidPriceResult> newBidPriceResultList;
		if(null != attributes && attributes.length > 0){
			newBidPriceResultList  = new ArrayList<BidPriceResult>();
			for(BidPriceResult bidPriceResult : bidPriceResultList){
				BidPriceResult ruleEntity = ReflecUtils.reflex(bidPriceResult, attributes,BidPriceResult.class);
				if(null != ruleEntity){
					newBidPriceResultList.add(ruleEntity);
				}
			}
 		}else{
 			newBidPriceResultList = bidPriceResultList;
		}	
		return newBidPriceResultList;
	}
	
	/**
	 * @Description: 批量更新时修改List
	 * @author wangxiang
	 * @param bidPriceResultList
	 * @param attributes
	 * @return 
	 */
	public List<SupplierOpeningResult>  getUpdateOpenResultList(List<SupplierOpeningResult> supplierOpeningResultList,String[] attributes){
		List<SupplierOpeningResult> newBidPriceResultList;
		if(null != attributes && attributes.length > 0){
			newBidPriceResultList  = new ArrayList<SupplierOpeningResult>();
			for(SupplierOpeningResult supplierOpeningResult : supplierOpeningResultList){
				SupplierOpeningResult ruleEntity = ReflecUtils.reflex(supplierOpeningResult, attributes,SupplierOpeningResult.class);
				if(null != ruleEntity){
					newBidPriceResultList.add(ruleEntity);
				}
			}
 		}else{
 			newBidPriceResultList = supplierOpeningResultList;
		}	
		return newBidPriceResultList;
	}
	
	
	/**
	 * @Description: 批量更新时修改List
	 * @author wangxiang
	 * @param bidPriceResultList
	 * @param attributes
	 * @return 
	 */
	public List<BidPriceItemResult>  getBidPriceItemResultList(List<BidPriceItemResult> bidPriceItemResultList,String[] attributes){
		List<BidPriceItemResult> newbidPriceItemResultList;
		if(null != attributes && attributes.length > 0){
			newbidPriceItemResultList  = new ArrayList<BidPriceItemResult>();
			for(BidPriceItemResult supplierOpeningResult : bidPriceItemResultList){
				BidPriceItemResult ruleEntity = ReflecUtils.reflex(supplierOpeningResult, attributes,BidPriceItemResult.class);
				if(null != ruleEntity){
					newbidPriceItemResultList.add(ruleEntity);
				}
			}
 		}else{
 			newbidPriceItemResultList = bidPriceItemResultList;
		}	
		return newbidPriceItemResultList;
	}

	@Override
	public ReBidPriceRound getBidPriceRoundByIndex(String tenderId, int round) {
		QueryWrapper<ReBidPriceRound> query = new QueryWrapper<ReBidPriceRound>();
		query.lambda().eq(ReBidPriceRound::getTenderId, tenderId).eq(ReBidPriceRound::getRoundNo,round);
		return reBidPriceRoundMapper.selectOne(query);
	}

	@Override
	public List<ReBidPriceRound> getBidPriceRoundList(String tenderId) {
		QueryWrapper<ReBidPriceRound> query = new QueryWrapper<ReBidPriceRound>();
		query.lambda().eq(ReBidPriceRound::getTenderId, tenderId).orderByAsc(ReBidPriceRound::getRoundNo);
		return reBidPriceRoundMapper.selectList(query);
	}

	@Override
	public boolean updateAssignProperty(ReBidPriceRound bidPriceRound) {
		return reBidPriceRoundMapper.updateById(bidPriceRound) > 0;
	}
	
	/**
	 * @Description: 开启新一轮报价
	 * @author guoyr
	 * @param bidPriceRound
	 * @param tenderId
	 * @param supplierIds
	 * @param useLastBidPrice
	 * @return 
	 */
	@Transactional(rollbackFor = Exception.class)
	public boolean insertNewBidpriceRound(ReBidPriceRound bidPriceRound , String tenderId, String[] supplierIds, String[] evaluationStepSupplierIds) {
		if(null == bidPriceRound) {
			return false;
		}
		if(null == bidPriceRound.getId()) {
			bidPriceRound.setId(IDUtils.getId());
		}
		List<BidPriceHisResult> bidPriceHisResultList = new ArrayList<BidPriceHisResult>();
		List<BidPriceResult> bidPriceResultList = bidPriceResultMapper.getBidPriceResultTableList(tenderId, null);
		// 临时存储要清空投标报价时间属性
		List<BidPriceResult> bidPriceResultId = new ArrayList<BidPriceResult>();
		List<BidPriceRoundStatus> bidPriceRoundStatusList = new ArrayList<BidPriceRoundStatus>();
		String[] tempSupplierIds = null;
		if(null != bidPriceResultList) {
			// 如果当前是第二轮报价，则先创建一个原始报价
			if(bidPriceRound.getRoundNo() -1 <= 1) {
				ReBidPriceRound originalBidPriceRound = new ReBidPriceRound();
				originalBidPriceRound.setId(IDUtils.getId());
				originalBidPriceRound.setRoundNo(1);
				originalBidPriceRound.setRoundStatus("01");
				originalBidPriceRound.setRoundName("初始报价");
				originalBidPriceRound.setTenderId(tenderId);
				reBidPriceRoundMapper.insert(originalBidPriceRound);
				
				tempSupplierIds = evaluationStepSupplierIds;
				
				for(String supplierId : tempSupplierIds) {
					if(null != bidPriceResultList && bidPriceResultList.size() > 0){
						for(BidPriceResult bidPriceResult : bidPriceResultList){
							if(supplierId.equals(bidPriceResult.getSupplierId())) {
								BidPriceHisResult bidPriceHisResult = new BidPriceHisResult();
								bidPriceHisResult.setId(IDUtils.getId());
								bidPriceHisResult.setTenderId(tenderId);
								bidPriceHisResult.setBidPriceName(bidPriceResult.getBidPriceName());
								bidPriceHisResult.setPriceContent(bidPriceResult.getPriceContent());
								bidPriceHisResult.setFinalBidPrice(bidPriceResult.getFinalBidPrice());
								bidPriceHisResult.setBidPrice(bidPriceResult.getBidPrice());
								bidPriceHisResult.setBidPriceId(bidPriceResult.getBidPriceId());
								bidPriceHisResult.setSupplierId(bidPriceResult.getSupplierId());
								bidPriceHisResult.setPriceUnit(bidPriceResult.getPriceUnit());
								bidPriceHisResult.setCurrencyCode(bidPriceResult.getCurrencyCode());
								bidPriceHisResult.setCurrencyName(bidPriceResult.getCurrencyName());
								bidPriceHisResult.setCreateTime(bidPriceResult.getCreateTime());
								bidPriceHisResult.setRoundName(originalBidPriceRound.getRoundName());
								bidPriceHisResult.setBidPriceRound(Math.max(bidPriceRound.getRoundNo() -1, 1));
								bidPriceHisResult.setModifyReason(bidPriceResult.getModifyReason());
								bidPriceHisResult.setCurrencyCode(bidPriceResult.getCurrencyCode());
								bidPriceHisResult.setEvaluationPrice(bidPriceResult.getEvaluationPrice());
								
								bidPriceHisResultList.add(bidPriceHisResult);
								bidPriceResult.setCreateTime(null);
								bidPriceResultId.add(bidPriceResult);
							}
						}
					}
				}
				
				
				
				
			}else {
				tempSupplierIds = supplierIds;
			}
			
			
			for(String id : supplierIds) {
				BidPriceRoundStatus bidPriceRoundStatus = new BidPriceRoundStatus();
				bidPriceRoundStatus.setId(IDUtils.getId());
				bidPriceRoundStatus.setPriceRoundId(bidPriceRound.getId());
				bidPriceRoundStatus.setPriceStatus(BidPriceEnum.PRICE_STATUS_UNDONE);
				bidPriceRoundStatus.setSupplierId(id);
				bidPriceRoundStatusList.add(bidPriceRoundStatus);
			}
		}
		if(bidPriceRoundStatusList.size() > 0) {
			//插入选择供应商报价状态数据
			bidPriceRoundStatusMapper.insertByBatch(bidPriceRoundStatusList);
		}	
		if(reBidPriceRoundMapper.insert(bidPriceRound) > 0) {
			if(bidPriceHisResultList.size() > 0) {
				if(bidPriceHisResultMapper.insertByBatch(bidPriceHisResultList) > 0) {
					//清空报价时间
//					if(bidPriceResultId.size() > 0) {
//						bidPriceResultMapper.batchUpdateProperty(bidPriceResultList, new String[] {"createTime"});
//					}			
					return true;
				}
			}
		}
		return true;
	}


	
	@Override
	public Integer updateAssignProperty(ReBidPriceRound ObeBidpriceRound, String[] attributes) {
		return reBidPriceRoundMapper.updateAssignProperty(ObeBidpriceRound, attributes);
	}

	@Override
	public Integer batchUpdateProperty(List<ReBidPriceRound> ObeBidpriceRoundList, String[] attributes) {
		return reBidPriceRoundMapper.batchUpdateProperty(ObeBidpriceRoundList, attributes);
	}
	
	@Override
	@Transactional(rollbackFor = Exception.class)
	public boolean completeCurrentRound(ReBidPriceRound bidPriceRound) {
		
		 List<BidPriceResult> updateBidPriceResultList = new ArrayList<BidPriceResult>();
		 List<BidPriceResult> insertBidPriceResultList = new ArrayList<BidPriceResult>();
		 
		//获取所有供应商 id 
		//将当前轮次已报价的供应商的报价 更新priceresult表中
		List<Supplier> supperList = supplierMapper.getPrivceRoundSupplierList(bidPriceRound.getId()); //获得当前轮次的供应商
	
		//获取历史数据最后一轮历史数据
		List<BidPriceHisResult> oldBidPriceHisResult = bidPriceHisResultMapper.getMaxRoundByTenderId(bidPriceRound.getTenderId());
		 
		//查询出来当前项目所有供应商的报价
		QueryWrapper<BidPriceResult> queryWrapperOldPriceResul = new QueryWrapper<BidPriceResult>();
		queryWrapperOldPriceResul.lambda().eq(BidPriceResult::getTenderId, bidPriceRound.getTenderId());
		List<BidPriceResult> oldBidPriceResultList =bidPriceResultMapper.selectList(queryWrapperOldPriceResul);
		
		Map<String, BidPriceResult> bidPriceResultMap = new HashMap<String, BidPriceResult>();
		oldBidPriceResultList.forEach(bidPriceResult -> {
			String key = bidPriceResult.getSupplierId() + bidPriceResult.getBidPriceId();
			bidPriceResultMap.put(key, bidPriceResult);
		});
		//循环当前轮次指定供应商
		for(Supplier supplier : supperList) {
			oldBidPriceHisResult.forEach(hisResult->{
				if(supplier.getId().equals(hisResult.getSupplierId())) {
					 String key = supplier.getId() + hisResult.getBidPriceId();
					 BidPriceResult bidPriceResultTemp = bidPriceResultMap.get(key);	
					 if(null == bidPriceResultTemp) {
						 bidPriceResultTemp = new BidPriceResult();
						 bidPriceResultTemp.setId(IDUtils.getId());
						 bidPriceResultTemp.setTenderId(hisResult.getTenderId());
						 bidPriceResultTemp.setSupplierId(hisResult.getSupplierId());
						 bidPriceResultTemp.setBidPriceId(hisResult.getBidPriceId());
						 bidPriceResultTemp.setBidPriceName(hisResult.getBidPriceName());
						 bidPriceResultTemp.setBidPriceType(hisResult.getBidPriceType());
						 bidPriceResultTemp.setPriceUnit(hisResult.getPriceUnit());
						 bidPriceResultTemp.setBidPriceCode(hisResult.getBidPriceCode());
						 bidPriceResultTemp.setPriceContent(hisResult.getPriceContent());
						 bidPriceResultTemp.setBidPrice(hisResult.getBidPrice());
						 bidPriceResultTemp.setFinalBidPrice(hisResult.getFinalBidPrice());
						 bidPriceResultTemp.setEvaluationPrice(hisResult.getEvaluationPrice());
						 bidPriceResultTemp.setCreateTime(new Date());
						 insertBidPriceResultList.add(bidPriceResultTemp);
					 }else {
						 bidPriceResultTemp.setBidPriceName(hisResult.getBidPriceName());
						 bidPriceResultTemp.setBidPriceType(hisResult.getBidPriceType());
						 bidPriceResultTemp.setPriceUnit(hisResult.getPriceUnit());
						 bidPriceResultTemp.setBidPriceCode(hisResult.getBidPriceCode());
						 bidPriceResultTemp.setPriceContent(hisResult.getPriceContent());
						 bidPriceResultTemp.setBidPrice(hisResult.getBidPrice());
						 bidPriceResultTemp.setFinalBidPrice(hisResult.getFinalBidPrice());
						 bidPriceResultTemp.setEvaluationPrice(hisResult.getEvaluationPrice());
						 bidPriceResultTemp.setCreateTime(new Date());
						 updateBidPriceResultList.add(bidPriceResultTemp);
					 }
				}
			});
		}
		 if(insertBidPriceResultList.size() > 0) {
			 bidPriceResultMapper.insertByBatch(insertBidPriceResultList);			
		 }
		 
		 if(updateBidPriceResultList.size() > 0) {
			 bidPriceResultMapper.updateBatchList(updateBidPriceResultList);				
		 }
		 //计算供应商的总价
		 supplierOpeningResultService.updateSupplierSumPrice(bidPriceRound.getTenderId(), null, true, true);
		 // 更新轮次状态
		 return  reBidPriceRoundMapper.updateAssignProperty(bidPriceRound, new String[]{"id","roundStatus","roundOpinion"}) > 0;
 	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public boolean insertNewLocalBidpriceRound(ReBidPriceRound bidPriceRound, String tenderId, String[] supplierIds) {
		if(null == bidPriceRound) {
			return false;
		}
		if(null == bidPriceRound.getId()) {
			bidPriceRound.setId(IDUtils.getId());
		}
		List<BidPriceHisResult> bidPriceHisResultList = new ArrayList<BidPriceHisResult>();
		List<BidPriceResult> bidPriceResultList = bidPriceResultMapper.getBidPriceResultTableList(tenderId, null);
		String roundName = (bidPriceRound.getRoundNo() -1 <= 1) ? "初始报价" : reBidPriceRoundMapper.getRoundName(tenderId,bidPriceRound.getRoundNo()-1);
		List<BidPriceRoundStatus> bidPriceRoundStatusList = new ArrayList<BidPriceRoundStatus>();
 		if(CollectionUtils.isNull(bidPriceResultList)) {
 			return false;
 		}
 		

		// 如果当前是第二轮报价，则先创建一个原始报价
		if(bidPriceRound.getRoundNo() -1 <= 1) {
			ReBidPriceRound originalBidPriceRound = new ReBidPriceRound();
			originalBidPriceRound.setId(IDUtils.getId());
			originalBidPriceRound.setRoundNo(1);
			originalBidPriceRound.setRoundStatus("01");
			originalBidPriceRound.setRoundName(roundName);
			originalBidPriceRound.setTenderId(tenderId);
			reBidPriceRoundMapper.insert(originalBidPriceRound);
				 
			Map<String, List<BidPriceResult>> map = bidPriceResultList.stream().collect(Collectors.groupingBy(BidPriceResult::getSupplierId));
			
			for (String supplierId : supplierIds) {
				if (map.containsKey(supplierId)) {
					for (BidPriceResult bidPriceResult : map.get(supplierId)) {
						BidPriceHisResult bidPriceHisResult = new BidPriceHisResult();
						bidPriceHisResult.setId(IDUtils.getId());
						bidPriceHisResult.setRoundName(roundName);
						bidPriceHisResult.setBidPriceRound(Math.max(bidPriceRound.getRoundNo() - 1, 1));
						
						initBidPriceHisResult(bidPriceHisResult, tenderId, bidPriceResult, bidPriceHisResultList);
						
						BidPriceHisResult bidPriceHisResult1 = new BidPriceHisResult();
						bidPriceHisResult1.setId(IDUtils.getId());
						bidPriceHisResult1.setRoundName(bidPriceRound.getRoundName());
						bidPriceHisResult1.setBidPriceRound(Math.max(bidPriceRound.getRoundNo(), 1));
						initBidPriceHisResult(bidPriceHisResult1, tenderId, bidPriceResult, bidPriceHisResultList);
					}
				}
			}
			
		}else {
			//获取历史数据最后一轮历史数据	bidPriceRound			
			bidPriceHisResultList = bidPriceHisResultMapper.getMaxRoundByTenderId(bidPriceRound.getTenderId());
			bidPriceHisResultList.forEach(hisResult->{
				hisResult.setId(IDUtils.getId());
				hisResult.setRoundName(bidPriceRound.getRoundName());
				hisResult.setBidPriceRound(Math.max(bidPriceRound.getRoundNo(), 1));
			});
		}
		
		for(String id : supplierIds) {
			BidPriceRoundStatus bidPriceRoundStatus = new BidPriceRoundStatus();
			bidPriceRoundStatus.setId(IDUtils.getId());
			bidPriceRoundStatus.setPriceRoundId(bidPriceRound.getId());
			bidPriceRoundStatus.setPriceStatus(BidPriceEnum.PRICE_STATUS_UNDONE);
			bidPriceRoundStatus.setSupplierId(id);
			bidPriceRoundStatusList.add(bidPriceRoundStatus);
		}
	
		if(bidPriceRoundStatusList.size() > 0) {
			//插入选择供应商报价状态数据
			bidPriceRoundStatusMapper.insertByBatch(bidPriceRoundStatusList);
		}	
		if(reBidPriceRoundMapper.insert(bidPriceRound) > 0) {
			if(bidPriceHisResultList.size() > 0) {
				bidPriceHisResultMapper.insertByBatch(bidPriceHisResultList);
			}
		}
		return true;
	}

	private void initBidPriceHisResult(BidPriceHisResult bidPriceHisResult, String tenderId, BidPriceResult bidPriceResult, List<BidPriceHisResult> bidPriceHisResultList) {
		bidPriceHisResult.setTenderId(tenderId);
		bidPriceHisResult.setFinalBidPrice(bidPriceResult.getFinalBidPrice());
		bidPriceHisResult.setBidPriceName(bidPriceResult.getBidPriceName());
		bidPriceHisResult.setPriceContent(bidPriceResult.getPriceContent());
		bidPriceHisResult.setBidPrice(bidPriceResult.getBidPrice());
		bidPriceHisResult.setBidPriceId(bidPriceResult.getBidPriceId());
		bidPriceHisResult.setSupplierId(bidPriceResult.getSupplierId());
		bidPriceHisResult.setPriceUnit(bidPriceResult.getPriceUnit());
		bidPriceHisResult.setCurrencyCode(bidPriceResult.getCurrencyCode());
		bidPriceHisResult.setCurrencyName(bidPriceResult.getCurrencyName());
		bidPriceHisResult.setCreateTime(bidPriceResult.getCreateTime());
		bidPriceHisResult.setModifyReason(bidPriceResult.getModifyReason());
		bidPriceHisResult.setCurrencyCode(bidPriceResult.getCurrencyCode());
		bidPriceHisResult.setEvaluationPrice(bidPriceResult.getEvaluationPrice());
		
		bidPriceHisResultList.add(bidPriceHisResult);
	}
	
}
