package com.gx.obe.server.management.project.service.impl;

import java.util.*;
import java.util.stream.Collectors;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.gx.obe.server.common.utils.UuidUtils;
import com.gx.obe.server.management.check.entity.CheckResult;
import com.gx.obe.server.management.check.service.CheckResultService;
import com.gx.obe.server.management.dictionary.entity.DictLink;
import com.gx.obe.server.management.dictionary.service.DictLinkService;
import com.gx.obe.server.management.evaluation.entity.*;
import com.gx.obe.server.management.evaluation.service.*;
import com.gx.obe.server.management.expert.entity.EvaluationStepExpert;
import com.gx.obe.server.management.expert.entity.ExpertEntity;
import com.gx.obe.server.management.expert.entity.ExpertEvaluationResult;
import com.gx.obe.server.management.expert.entity.ExpertEvaluationStep;
import com.gx.obe.server.management.expert.service.EvaluationStepExpertService;
import com.gx.obe.server.management.expert.service.ExpertEvaluationResultService;
import com.gx.obe.server.management.expert.service.ExpertEvaluationStepService;
import com.gx.obe.server.management.expert.service.ExpertService;
import com.gx.obe.server.management.supplier.entity.SupplierEvaluationResult;
import com.gx.obe.server.management.supplier.entity.SupplierEvaluationStep;
import com.gx.obe.server.management.supplier.service.SupplierEvaluationResultService;
import com.gx.obe.server.management.supplier.service.SupplierEvaluationStepService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.enumeration.CommonEnum;
import com.gx.obe.server.common.enumeration.TenderProjectEnum;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.ObjectUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.config.error.BizException;
import com.gx.obe.server.config.error.ResultBody;
import com.gx.obe.server.management.check.dao.CheckResultMapper;
import com.gx.obe.server.management.evaluation.dao.BidPriceHisResultMapper;
import com.gx.obe.server.management.evaluation.dao.BidPriceItemMapper;
import com.gx.obe.server.management.evaluation.dao.BidPriceItemResultMapper;
import com.gx.obe.server.management.evaluation.dao.BidPriceMapper;
import com.gx.obe.server.management.evaluation.dao.BidPriceResultMapper;
import com.gx.obe.server.management.evaluation.dao.EvaluationFactorResultMapper;
import com.gx.obe.server.management.evaluation.dao.EvaluationStepMapper;
import com.gx.obe.server.management.expert.dao.ExpertEvaluationResultMapper;
import com.gx.obe.server.management.expert.dao.ExpertEvaluationStepMapper;
import com.gx.obe.server.management.expert.dao.ExpertMapper;
import com.gx.obe.server.management.price.dao.ReBidPriceRoundMapper;
import com.gx.obe.server.management.project.dao.ProjectRuleMapper;
import com.gx.obe.server.management.project.dao.TenderProjectMapper;
import com.gx.obe.server.management.project.entity.ProjectRule;
import com.gx.obe.server.management.project.entity.TenderProject;
import com.gx.obe.server.management.project.service.ProjectRuleService;
import com.gx.obe.server.management.project.service.TenderProjectService;
import com.gx.obe.server.management.supplier.dao.SupplierEvaluationResultMapper;
import com.gx.obe.server.management.supplier.dao.SupplierEvaluationStepMapper;
import com.gx.obe.server.management.supplier.dao.SupplierMapper;
import com.gx.obe.server.management.supplier.dao.SupplierOpeningResultMapper;
import com.gx.obe.server.management.supplier.entity.Supplier;
import com.gx.obe.server.management.supplier.entity.SupplierOpeningResult;
import com.gx.obe.server.management.supplier.service.SupplierOpeningResultService;
import com.gx.obe.server.management.supplier.service.SupplierService;
import com.gx.obe.server.management.vo.TenderPackOpeningInfo;
import com.gx.obe.server.management.vo.TenderPackPriceFactorVo;
import com.gx.obe.server.management.vo.TenderProjectVo;

  /**  
 * 
 * @author 
 * @Title: IObeTenderProjectService
 * @Package com.gx.obe.server.management.user.service  
 * @Description:     
 * @author: XiaoMa Pro - myzhichao@163.com
 * @date:   2019-04-29   
 * @version V1.0 
 * @Copyright: 2019 www.msypro.com Inc. All rights reserved. 
 *  
 */
@Service
public class TenderProjectServiceImpl extends ServiceImpl<TenderProjectMapper, TenderProject> implements TenderProjectService {

	@Autowired
	private TenderProjectMapper tenderProjectMapper;
	
	@Autowired
	private ProjectRuleMapper projectRuleMapper;
	
	@Autowired
	private ProjectRuleService projectRuleService;
	
	@Autowired
	private SupplierMapper supplierMapper;
	
	@Autowired
	private SupplierService supplierService;
	
	@Autowired
	private SupplierOpeningResultMapper supplierOpeningResultMapper;
	
	@Autowired
	private BidPriceMapper bidPriceMapper;
	
	@Autowired
	private BidPriceItemMapper bidPriceItemMapper;
	
	@Autowired
	private BidPriceItemResultMapper bidPriceItemResultMapper;
	
	@Autowired
	private BidPriceResultMapper bidPriceResultMapper;
	
	@Autowired
	private SupplierOpeningResultService supplierOpeningResultService;
	
	@Autowired
	private EvaluationStepMapper evaluationStepMapper;
	
	@Autowired
	private ExpertEvaluationResultMapper expertEvaluationResultMapper;
	
	@Autowired
	private SupplierEvaluationResultMapper supplierEvaluationResultMapper;
	
	@Autowired
	private ExpertMapper expertMapper;
	@Autowired
	private SupplierEvaluationStepMapper supplierEvaluationStepMapper;
 	
	@Autowired
	private ExpertEvaluationStepMapper expertEvaluationStepMapper;
	
	@Autowired
	private EvaluationFactorResultMapper evaluationFactorResultMapper;

	@Autowired
	private CheckResultMapper checkResultMapper;
	
	@Autowired
	private BidPriceHisResultMapper bidPriceHisResultMapper;
	@Autowired
	private ReBidPriceRoundMapper reBidPriceRoundMapper;

	@Autowired
	private EvaluationStepService evaluationStepService;

	@Autowired
	private ExpertService expertService;

	  @Autowired
	  EvaluationFactorService evaluationFactorService;

	  @Autowired
	  ExpertEvaluationStepService expertEvaluationStepService;

	  @Autowired
	  EvaluationStepExpertService evaluationStepExpertService;

	  @Autowired
	  EvaluationStepFactorService evaluationStepFactorService;

	  @Autowired
	  EvaluationFactorResultService evaluationFactorResultService;

	  @Autowired
	  ExpertEvaluationResultService expertEvaluationResultService;

	  @Autowired
	  SupplierEvaluationFactorService supplierEvaluationFactorService;

	  @Autowired
	  CheckResultService checkResultService;

	  @Autowired
	  BidRegulationsService bidRegulationsService;

	  @Autowired
	  SupplierEvaluationStepService supplierEvaluationStepService;

	  @Autowired
	  SupplierEvaluationResultService supplierEvaluationResultService;

	  @Autowired
	  DictLinkService dictLinkService;

	  @Autowired
	  BidPriceService bidPriceService;

	  @Autowired
	  BidPriceResultService bidPriceResultService;
 	
	/**
	 * @Description: 批量更新
	 * @author mazc
	 * @param tenderProjectList
	 */
	 @Override
	 @Transactional(rollbackFor = Exception.class)
	public Integer updateByBatch (List<TenderProject> tenderProjectList){
		return tenderProjectMapper.updateBatchList(tenderProjectList);
	}
	 /**
	  * @Description: 批量指定字段修改
	  * @author chenxw
	  * @param tenderProjectList
	  * @param attributes
	  */
	 public Integer batchUpdateProperty(List<TenderProject> tenderProjectList, String[] attributes) {
		 return tenderProjectMapper.updateBatchList(tenderProjectList);
	 }
	
	/** 
	 * @Description:获得分包项目
	 * @author mazc
	 * @param parentId
	 * @return 
	 */
	@Override
	public List<TenderProject> getSubPackageList(String parentId) {
		return tenderProjectMapper.selectSubPackageList(parentId);
	}
	 
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param tenderProjectList
	 */
	 @Override
	 @Transactional(rollbackFor = Exception.class)
	public Integer insertByBatch (List<TenderProject> tenderProjectList){
		return tenderProjectMapper.insertByBatch(tenderProjectList);
	}
	
	
	 /**
	  * @Description: 获得开标项目分页列表
	  * @author guoyr
	  * @param page
	  * @param userId
	  * @param userType
	  * @param openingStatus
	  * @param searchKey
	  * @param startTime
	  * @param endTime
	  * @return 
	  */
	@Transactional(readOnly=true,rollbackFor = Exception.class) 
    public IPage<TenderProjectVo> getOpeningProjectPageList(Page<TenderProjectVo> page, String userId, String userType, String openingStatus, String searchKey, Date startTime, Date endTime) {
        return tenderProjectMapper.getTenderProjectPageList(page, userId, userType, true, openingStatus, false, null, searchKey, startTime, endTime);
    }
	
	/**
	 * @Description: 获取开标项目的标段分页列表
	 * @author guoyr
	 * @param page
	 * @param parentId
	 * @param userType
	 * @param openingStatus
	 * @param searchKey
	 * @param startTime
	 * @param endTime
	 * @return 
	 */
	@Transactional(readOnly=true,rollbackFor = Exception.class) 
	public IPage<TenderProjectVo> getOpeningTenderPackPageList(Page<TenderProjectVo> page, String parentId, String userType, String openingStatus, String searchKey, Date startTime, Date endTime) {
		return tenderProjectMapper.getTenderPackPageList(page, parentId, userType, null, openingStatus, null, searchKey, startTime, endTime);
	}
	/**
	 * @Description: 获取开标项目的标段分页列表
	 * @author guoyr
	 * @param page
	 * @param parentId
	 * @param userType
	 * @param evaluationStatus
	 * @param searchKey
	 * @param startTime
	 * @param endTime
	 * @return 
	 */
	@Transactional(readOnly=true,rollbackFor = Exception.class) 
	public IPage<TenderProjectVo> getTenderPackPageList(Page<TenderProjectVo> page, String parentId, String userType, String userId, String openingStatus, String evaluationStatus, String searchKey, Date startTime, Date endTime) {
		return tenderProjectMapper.getTenderPackPageList(page, parentId, userType, userId, openingStatus, evaluationStatus, searchKey, startTime, endTime);
	}
	
	@Override
	@Transactional(readOnly=true,rollbackFor = Exception.class) 
	public IPage<TenderProjectVo> getMyProjectPageList(Page<TenderProjectVo> page, String userId, String userType, String evaluationStatus, String searchKey, Date startTime, Date endTime) {
		return tenderProjectMapper.getTenderProjectPageList(page, userId, userType, true, null, true, evaluationStatus, searchKey, startTime, endTime);
	}

	@Override
	@Transactional(readOnly=true,rollbackFor = Exception.class) 
	public IPage<TenderProjectVo> selectSuperviseProjectPage(Page<TenderProjectVo> page, String userId, String evaluationStatus, String searchKey, Date startTime, Date endTime) {
		return tenderProjectMapper.selectSuperviseProjectPage(page, userId, true, null, true, evaluationStatus, searchKey, startTime, endTime);
	}

	@Override
	public IPage<TenderProjectVo> selectSubPackagePage(Page<TenderProjectVo> page, String parentId) {
		return tenderProjectMapper.selectSubPackagePage(page, parentId);
	}

	@Override
	public boolean isUniqueTenderProject(String id, String useStatus, String tenderNO, String phaseType, String parentId) {
		Integer	count = tenderProjectMapper.selectUniqueTedderProjectCount( id,  useStatus,  tenderNO,  phaseType,  parentId);
		return count > 0 ? false : true;
	}

	@Override
	public List<String> getTenderIdList(String parentId) {
		return tenderProjectMapper.getTenderIdList(parentId);
	}
	
	@Override
	public boolean isInEvaluation(String tenderId, String stepType) {
		return tenderProjectMapper.isInEvaluation(tenderId, stepType) > 0;
	}

	@Override
	public List<TenderPackPriceFactorVo> selectTenderPackPriceFactor(String tenderParentId, String auditType) {
 		return tenderProjectMapper.selectTenderPackPriceFactor(tenderParentId, auditType);
	}

	@Override
	@Transactional(rollbackFor = Exception.class) 
	public boolean updateDownLoadProject(TenderProject tenderProject ,ProjectRule projectRule) {
		boolean isOk = false;
		if(StringUtils.isEmpty(tenderProject.getId())) {
			tenderProject.setId(IDUtils.getId());
			isOk = tenderProjectMapper.insert(tenderProject)>0;
		}else {
			
//			isOk = tenderProjectMapper.updateById(tenderProject)>0;
			isOk = tenderProjectMapper.updateAssignProperty(tenderProject, new String[]{"tenderMethod", "tenderType", "phaseType", "isPack", "remoteStatus"}) > 0;
		}
		if(StringUtils.isEmpty(projectRule.getId())) {
			tenderProject.setId(IDUtils.getId());
			isOk = projectRuleMapper.insert(projectRule)>0;
		}else {
//			isOk = projectRuleMapper.updateById(projectRule)>0;
			isOk = projectRuleMapper.updateAssignProperty(projectRule, new String[]{"bidOpeningStage", "bidOpeningStartTime", "priceBidOpeningTime"}) > 0;
		}
		return isOk;
	}

	
	/** 
	 * @Description: 获得父级项目的开标信息
	 * @author mazc
	 * @param tenderProject
	 * @param bidopeningStage
	 * @return 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class) 
	public TenderPackOpeningInfo getTenderProjectOpeningInfo(TenderProject tenderProject, String bidopeningStage) {
		TenderPackOpeningInfo tenderProjectOpeningInfo = null;
		boolean isPriceStage = false;
		boolean isBusAndTecStage = false;
		if(null != bidopeningStage && bidopeningStage.equals(TenderProjectEnum.STAGE_PRICE)){
			isPriceStage = true;
		}else if(null != bidopeningStage && bidopeningStage.equals(TenderProjectEnum.STAGE_BUS_TEC)){
			isBusAndTecStage = true;
		}
		
		tenderProject = tenderProjectMapper.selectTenderProjectBidOpeningInfo(tenderProject.getId());
		List<Map<String, Object>> tenderPackOpeningInfoMapList = tenderProjectMapper.selectTenderPackOpeningInfoList(tenderProject.getId(), true, bidopeningStage);
		List<TenderPackOpeningInfo> tenderPackOpeningInfoList = getTenderPackOpeningInfoListByMap(tenderPackOpeningInfoMapList, isPriceStage, isBusAndTecStage);
		if(null != tenderPackOpeningInfoList){
			int signCount = 0;
			int tenderCount = 0;
			int threeTenderCount = 0;
			int openingCount = 0;
			int decCount = 0;
			int decFailCount = 0;
			int confirmCount = 0;
			int confirmQuestionCount = 0;
			int errorsCount = 0;
			int complateCount = 0;
			int complateSupplierCount = 0;
			boolean projectIsStartSign = false;
			boolean projectIsStartDecrypt = false;
			List<TenderProject> tenderPackList = new ArrayList<TenderProject>();
			for(TenderPackOpeningInfo tenderPackOpeningInfo : tenderPackOpeningInfoList){
				TenderProject tenderPack = tenderPackOpeningInfo.getTenderProject();
				if(null != tenderPack.getEffectiveType() && tenderPack.getEffectiveType().equals(TenderProjectEnum.EFFECTIVE_NO)){
					continue;
				}
				tenderPackList.add(tenderPack);
				
				tenderCount += tenderPackOpeningInfo.getTenderCount();
				if(tenderPackOpeningInfo.getTenderCount() > 2){
					threeTenderCount += tenderPackOpeningInfo.getTenderCount();
				}
				signCount += tenderPackOpeningInfo.getSignCount();
				decCount += tenderPackOpeningInfo.getDecCount();
				decFailCount += tenderPackOpeningInfo.getDecFailCount();
				confirmCount += tenderPackOpeningInfo.getConfirmCount();
				confirmQuestionCount += tenderPackOpeningInfo.getConfirmQuestionCount();
				errorsCount += tenderPackOpeningInfo.getNoPriceCount();
				if(tenderPackOpeningInfo.isComplate()){
					complateCount += 1;
					complateSupplierCount += tenderPackOpeningInfo.getComplateSupplierCount();
				}
				if(!projectIsStartSign){
					if(signCount > 0){
						projectIsStartSign = true;
					}else {
						String startBidopening = tenderPackOpeningInfo.getTenderProject().getStartBidOpening();
						if(isPriceStage){
							startBidopening = tenderPackOpeningInfo.getTenderProject().getStartPriceBidOpening();
						}
						if(null != startBidopening && startBidopening.equals(CommonEnum.YES)){
							projectIsStartSign = true;
						}
					}
				}
				
				openingCount += tenderPackOpeningInfo.getOpeningCount();
				if(!projectIsStartDecrypt){

					if(decCount > 0){
						projectIsStartDecrypt = true;
					}else {
						// 是否已经启动解密
						boolean isStartDecrypt = true;
						if(isPriceStage){
							if(StringUtils.isEmpty(tenderPackOpeningInfo.getTenderProject().getPriceBidStatus()) || tenderPackOpeningInfo.getTenderProject().getPriceBidStatus().equals(CommonEnum.NOT_START)){
								isStartDecrypt = false;
							}
							
						}else {
							if(StringUtils.isEmpty(tenderPackOpeningInfo.getTenderProject().getBusinessBidStatus()) || tenderPackOpeningInfo.getTenderProject().getBusinessBidStatus().equals(CommonEnum.NOT_START)){
								isStartDecrypt = false;
							}
						}
						if(isStartDecrypt){
							projectIsStartDecrypt = true;
						}
					}
				}
			
			}
			
			if(isPriceStage){
				tenderProject.setStartPriceBidOpening(projectIsStartSign ? CommonEnum.YES : CommonEnum.NO);
				tenderProject.setPriceBidStatus(projectIsStartDecrypt ? CommonEnum.IN_PROGRESS : CommonEnum.NOT_START);
			}else {
				tenderProject.setStartBidOpening(projectIsStartSign ? CommonEnum.YES : CommonEnum.NO);
				if(projectIsStartDecrypt){
					tenderProject.setBusinessBidStatus(CommonEnum.IN_PROGRESS);
					tenderProject.setTechnologyBidStatus(CommonEnum.IN_PROGRESS);
					if(!isBusAndTecStage){
						tenderProject.setPriceBidStatus(CommonEnum.IN_PROGRESS);
					}
				}else {
					tenderProject.setBusinessBidStatus(CommonEnum.NOT_START);
					tenderProject.setTechnologyBidStatus(CommonEnum.NOT_START);
					if(!isBusAndTecStage){
						tenderProject.setPriceBidStatus(CommonEnum.NOT_START);
					}
				}
			}
			
			tenderProject.setChildList(tenderPackList);
			tenderProjectOpeningInfo = new TenderPackOpeningInfo(tenderProject, tenderPackList);
			tenderProjectOpeningInfo.setTenderCount(tenderCount);
			tenderProjectOpeningInfo.setThreeTenderCount(threeTenderCount);
			tenderProjectOpeningInfo.setOpeningCount(openingCount);
			tenderProjectOpeningInfo.setSignCount(signCount);
			tenderProjectOpeningInfo.setDecCount(decCount);
			tenderProjectOpeningInfo.setDecFailCount(decFailCount);
			tenderProjectOpeningInfo.setConfirmCount(confirmCount);
			tenderProjectOpeningInfo.setConfirmQuestionCount(confirmQuestionCount);
			tenderProjectOpeningInfo.setNoPriceCount(errorsCount);
			tenderProjectOpeningInfo.setComplate(complateCount == tenderPackList.size());
			tenderProjectOpeningInfo.setComplatePackCount(complateCount);
			tenderProjectOpeningInfo.setComplateSupplierCount(complateSupplierCount);
		}
		
		
		
		
		
		
		
		return tenderProjectOpeningInfo;
	}
	
	
	/**
	 * @Description: 按分页查询评标项目
	 * @author mazc
	 * @param page
	 * @param user
	 * @param useSatus
	 * @param searchKey
	 * @return 
	 */
	@Override
    public IPage<TenderProjectVo> getEvaluationProjectPageList(Page<TenderProjectVo> page, String userId, String userType, String evaluationStatus, String searchKey, Date startTime, Date endTime) {
		return tenderProjectMapper.getTenderProjectPageList(page, userId, userType, true, CommonEnum.IN_PROGRESS, true, evaluationStatus, searchKey, startTime, endTime);
    }
	
	/**
	 * @Description: 按分页查询评标项目的分包
	 * @author mazc
	 * @param userId
	 * @param projectId
	 * @param evaluatoinStatus
	 * @return 
	 */
	@Override
	public IPage<TenderProjectVo> getExpertEvaluationTenderPackPageList(Page<TenderProjectVo> page,String userId, String projectId, String evaluatoinStatus) {
 		return tenderProjectMapper.getExpertEvaluationTenderPackPageList(page, userId, projectId, evaluatoinStatus);
	}

	/** 
	 * @Description: 获取开标不足三家供应商的标段
	 * @author mazc
	 * @param parentId
	 * @return 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class) 
	public List<TenderProject> findUnEnoughSupplierList(String parentId) {
		List<TenderProject> tenderProjectList = new ArrayList<TenderProject>();
		List<TenderProject> tenderkList = tenderProjectMapper.findSubPackageList(parentId);
		for(TenderProject tenderProject : tenderkList){
			int count = supplierService.findSupplierOpeningCount(tenderProject.getId());
			if( count < 3){//开标不足三家
				tenderProjectList.add(tenderProject);
			}
		}
		return tenderProjectList;
	}

	/** 
	 * @Description: 获得分包项目
	 * @author mazc
	 * @param page
	 * @param tenderId
	 * @return 
	 */
	@Override
	public IPage<TenderProject> findTenderPack(Page<TenderProject> page, String parentId) {
		return tenderProjectMapper.selectTenderProjectPack(page, parentId);
	}

	/** 
	 * @Description: selectExpertTenderProjectList
	 * @author mazc
	 * @param userId
	 * @param searchKey
	 * @param macName
	 * @return 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class) 
	public List<TenderProject> getExpertTenderProjectList(String userId, String searchKey, String macName) {
		List<TenderProject> tenderProjcetList=null;
		if(StringUtils.isEmpty(searchKey)&&StringUtils.isEmpty(macName)){
			LambdaQueryWrapper<TenderProject> wrapper =new LambdaQueryWrapper<>();
			wrapper.eq(TenderProject::getUserId,userId);
			tenderProjcetList = tenderProjectMapper.selectList(wrapper);
		}else{
			tenderProjcetList = tenderProjectMapper.selectExpertTenderProjectList(userId, searchKey, macName);
		}


		List<String> tenderIdList = new ArrayList<String>();
		
		if(null != tenderProjcetList && tenderProjcetList.size() > 0){
			StringBuilder parentId = new StringBuilder();
			int index = 0;
			for(TenderProject tenderProject : tenderProjcetList){
				if(StringUtils.isNotEmpty(tenderProject.getParentId())){
					if(!parentId.toString().contains(tenderProject.getParentId())){
						tenderIdList.add(tenderProject.getParentId());
					}
				}
			}
			if(index > 0){
				QueryWrapper<TenderProject> queryWrapper = new QueryWrapper<TenderProject>();
				queryWrapper.in(TenderProject.TENDER_ID, tenderIdList.toArray());
				List<TenderProject> parentList = tenderProjectMapper.selectList(queryWrapper);
				if(null != parentList && parentList.size() > 0){
					tenderProjcetList.addAll(parentList);
				}
			}
		}
		
		return tenderProjcetList;
	}

	/** 
	 * @Description: 查找和当前品目相同的相同分包
	 * @author mazc
	 * @param parentId
	 * @param purcategoryNames
	 * @param id
	 * @return 
	 */
	@Override
	public List<TenderProject> getPurcategoryNamesAndParentId(String parentId, String purcategoryNames, String id) {
		QueryWrapper<TenderProject> queryWrapper = new QueryWrapper<TenderProject>();
		queryWrapper.lambda().eq(TenderProject::getParentId, parentId).eq(TenderProject::getPurcategoryNames, purcategoryNames)
		.ne(TenderProject::getId, id);
		return tenderProjectMapper.selectList(queryWrapper);
	}

	/** 
	 * @Description: 修改开标状态
	 * @author mazc
	 * @param tenderId
	 * @param parentId
	 * @param bidOpeningStatus
	 * @return 
	 */
	@Override
	public int updateBidOpeningStatus(String tenderId, String parentId, String bidOpeningStatus) {
		return tenderProjectMapper.updateBidopeningStatus(tenderId, parentId, bidOpeningStatus);
	}

	@Override
	@Transactional(rollbackFor = Exception.class) 
	public int updateBidOpeningStatus(TenderProject tenderProject, String bidopeningStage, String status) {
		boolean isPriceStage = false;
		boolean isBusAndTecStage = false;
		if(null != bidopeningStage && bidopeningStage.equals(TenderProjectEnum.STAGE_PRICE)){
			isPriceStage = true;
		}else if(null != bidopeningStage && bidopeningStage.equals(TenderProjectEnum.STAGE_BUS_TEC)){
			isBusAndTecStage = true;
		}
		if(null == status){
			status = CommonEnum.NOT_START;
		}
		List<String> updateProperty = new ArrayList<String>();
		if(null == tenderProject.getBidOpeningStatus() || !tenderProject.getBidOpeningStatus().equals(CommonEnum.COMPLETE)){
			updateProperty.add("bidOpeningStatus");
			tenderProject.setBidOpeningStatus(status);
		}
		if(isPriceStage){
			if(null == tenderProject.getPriceBidStatus() || !tenderProject.getPriceBidStatus().equals(status)){
				updateProperty.add("priceBidStatus");
				tenderProject.setPriceBidStatus(status);
			}
		}else if(isBusAndTecStage){
			if(null == tenderProject.getBusinessBidStatus() || !tenderProject.getBusinessBidStatus().equals(status)){
				updateProperty.add("businessBidStatus");
				tenderProject.setBusinessBidStatus(status);
			}
			if(null == tenderProject.getTechnologyBidStatus() || !tenderProject.getTechnologyBidStatus().equals(status)){
				updateProperty.add("technologyBidStatus");
				tenderProject.setTechnologyBidStatus(status);
			}
		}else{
			if(null == tenderProject.getBusinessBidStatus() || !tenderProject.getBusinessBidStatus().equals(status)){
				updateProperty.add("businessBidStatus");
				tenderProject.setBusinessBidStatus(status);
			}
			if(null == tenderProject.getTechnologyBidStatus() || !tenderProject.getTechnologyBidStatus().equals(status)){
				updateProperty.add("technologyBidStatus");
				tenderProject.setTechnologyBidStatus(status);
			}
			if(null == tenderProject.getPriceBidStatus() || !tenderProject.getPriceBidStatus().equals(status)){
				updateProperty.add("priceBidStatus");
				tenderProject.setPriceBidStatus(status);
			}
		}
		boolean isUpdate = false; 	
		int iCount = 0;
		if(updateProperty.size() > 0){
			isUpdate = updateAssignProperty(tenderProject, updateProperty.toArray(new String[updateProperty.size()]));
		}
		if(status.equals(CommonEnum.COMPLETE) && null != tenderProject.getParentId()){
			iCount = updateBidOpeningStatus(tenderProject.getId(), tenderProject.getParentId(), CommonEnum.COMPLETE);
		}
		if(isUpdate == true || iCount > 0) {
			return iCount > 0 ? iCount: 1;
		}
		return 0;
	}
	
	/** 
	 * @Description: 指定字段修改
	 * @author chenxw
	 * @param tenderProject
	 * @param attributes
	 * @return 
	 */
	@Override
	public boolean updateAssignProperty(TenderProject tenderProject, String[] attributes) {
		return tenderProjectMapper.updateAssignProperty(tenderProject, attributes) > 0;
	}

	/** 
	 * @Description:  获得项目下的所有包件的开标信息
	 * @author mazc
	 * @param projectId
	 * @param bidopeningStage
	 * @return 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class) 
	public List<TenderPackOpeningInfo> getTenderPackOpeningInfoList(String projectId, String bidopeningStage) {
		List<TenderPackOpeningInfo> tenderPackOpeningInfoList = null;
		boolean isPriceStage = false;
		boolean isBusAndTecStage = false;
		if(null != bidopeningStage && bidopeningStage.equals(TenderProjectEnum.STAGE_PRICE)){
			isPriceStage = true;
		}else if(null != bidopeningStage && bidopeningStage.equals(TenderProjectEnum.STAGE_BUS_TEC)){
			isBusAndTecStage = true;
		}
		List<Map<String, Object>> tenderPackOpeningInfoMapList = tenderProjectMapper.selectTenderPackOpeningInfoList(projectId, true, bidopeningStage);
		tenderPackOpeningInfoList = getTenderPackOpeningInfoListByMap(tenderPackOpeningInfoMapList, isPriceStage, isBusAndTecStage);
		return tenderPackOpeningInfoList;
	}
	
	
	/**
	 * @Description:  获得包件的开标信息
	 * @author mazc
	 * @param tenderId
	 * @param bidopeningStage
	 * @return 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class) 
	public TenderPackOpeningInfo getTenderPackOpeningInfo(String tenderId, String bidopeningStage) {
		TenderPackOpeningInfo tenderPackOpeningInfo = null;
		Map<String, Object> tenderPackOpeningInfoMap = getTenderPackOpeningMapInfo(tenderId, bidopeningStage);
		boolean isPriceStage = false;
		boolean isBusAndTecStage = false;
		if(null != bidopeningStage && bidopeningStage.equals(TenderProjectEnum.STAGE_PRICE)){
			isPriceStage = true;
		}else if(null != bidopeningStage && bidopeningStage.equals(TenderProjectEnum.STAGE_BUS_TEC)){
			isBusAndTecStage = true;
		}
		tenderPackOpeningInfo = getTenderPackOpeningInfo(tenderPackOpeningInfoMap, isPriceStage, isBusAndTecStage);
		return tenderPackOpeningInfo;
	}
	
	
	
	
	
	

	
	
	/**
	 * @Description: 组装包件列表信息
	 * @author mazc
	 * @param queryList
	 * @param isPriceStage
	 * @param isBusAndTecStage
	 * @return 
	 */
	private List<TenderPackOpeningInfo> getTenderPackOpeningInfoListByMap(List<Map<String, Object>> queryList, boolean isPriceStage, boolean isBusAndTecStage){
		List<TenderPackOpeningInfo> tenderPackOpeningInfoList = new ArrayList<TenderPackOpeningInfo>();
		if(null != queryList && queryList.size() > 0){
			tenderPackOpeningInfoList = new ArrayList<TenderPackOpeningInfo>();
			for(Map<String, Object> tenderPackMap : queryList){
				tenderPackOpeningInfoList.add(getTenderPackOpeningInfo(tenderPackMap, isPriceStage, isBusAndTecStage));
			}
		}
		return tenderPackOpeningInfoList;
	}
	
	/**
	 * @Description: 组装包件信息
	 * @author guoyr
	 * @param tenderPackMap
	 * @param isPriceStage
	 * @param isBusAndTecStage
	 * @return 
	 */
	private TenderPackOpeningInfo getTenderPackOpeningInfo(Map<String, Object> tenderPackMap, boolean isPriceStage, boolean isBusAndTecStage){
		TenderProject tenderPack = new TenderProject();
		tenderPack.setId((String)tenderPackMap.get("TENDER_ID"));
		tenderPack.setTenderNo((String)tenderPackMap.get("TENDER_NO"));
		tenderPack.setParentId((String)tenderPackMap.get("TENDER_PARENT_ID"));
		tenderPack.setTenderName((String)tenderPackMap.get("TENDER_NAME"));
		tenderPack.setPhaseType((String)tenderPackMap.get("PHASE_TYPE"));
		tenderPack.setTenderType((String)tenderPackMap.get("TENDER_TYPE"));
		tenderPack.setTenderNumber((String)tenderPackMap.get("TENDERNO_NUMBER"));
		tenderPack.setStageId((String)tenderPackMap.get("STAGE_ID"));
		tenderPack.setTenderMethod((String)tenderPackMap.get("TENDER_METHOD"));
		tenderPack.setStartBidOpening((String)tenderPackMap.get("START_BID_OPENING"));
		tenderPack.setStartPriceBidOpening((String)tenderPackMap.get("START_PRICE_BID_OPENING"));
		tenderPack.setBidOpeningStage((String)tenderPackMap.get("BID_OPENING_STAGE"));
		tenderPack.setBidOpeningStatus((String)tenderPackMap.get("BID_OPENING_STATUS"));
		tenderPack.setBusinessBidStatus((String)tenderPackMap.get("BUSINESS_BID_STATUS"));
		tenderPack.setTechnologyBidStatus((String)tenderPackMap.get("TECHNOLOGY_BID_STATUS"));
		tenderPack.setPriceBidStatus((String)tenderPackMap.get("PRICE_BID_STATUS"));
		tenderPack.setUploadOpeningResultStatus((String)tenderPackMap.get("UPLOAD_OPENING_RESULT_STATUS"));
		tenderPack.setUploadPriceOpeningResultStatus((String)tenderPackMap.get("UPLOAD_PRICE_OPENING_RESULT_STATUS"));
		tenderPack.setIsPack((String)tenderPackMap.get("IS_PACK"));
		tenderPack.setTransactionServerId((String)tenderPackMap.get("TRANSACTION_SERVER_ID"));
		tenderPack.setServerAccount((String)tenderPackMap.get("SERVER_ACCOUNT"));
		tenderPack.setServerPassword((String)tenderPackMap.get("SERVER_PASSWORD"));
		tenderPack.setLatestBidinfo((String)tenderPackMap.get("MODIFY_SORT_REASON"));//// 标记最后一次同步投标信息状态
		TenderPackOpeningInfo tenderPackOpeningInfo = new TenderPackOpeningInfo(tenderPack, false);
		boolean isPreEvaluation = false;
		if(null != tenderPack.getPhaseType() && tenderPack.getPhaseType().equals(TenderProjectEnum.PHASE_TYPE_PRE)){
			isPreEvaluation = true;
		}
		int signCount = 0;//签到数量
		int tenderCount = 0;//包件数量
		int decCount = 0;//解密数量
		int decFailCount = 0;//解密失败数量
		int confirmCount = 0;//确认数量
		int confirmQuestionCount = 0;//提出质疑数量
		int noPriceCount = 0;//开标成功但无报价标数量
		
		// 是否已经启动解密
		boolean isStartDecrypt = true;
		if(isPriceStage){
			if(StringUtils.isEmpty(tenderPackOpeningInfo.getTenderProject().getPriceBidStatus()) || tenderPackOpeningInfo.getTenderProject().getPriceBidStatus().equals(CommonEnum.NOT_START)){
				isStartDecrypt = false;
			}
			
		}else {
			if(StringUtils.isEmpty(tenderPackOpeningInfo.getTenderProject().getBusinessBidStatus()) || tenderPackOpeningInfo.getTenderProject().getBusinessBidStatus().equals(CommonEnum.NOT_START)){
				isStartDecrypt = false;
			}
		}
		signCount = ObjectUtils.getIntValue(tenderPackMap.get("SIGN_COUNT"), 0);
		if(isPriceStage){
			signCount =  ObjectUtils.getIntValue(tenderPackMap.get("PRICE_SIGN_COUNT"), 0);
		}		
		tenderCount =  ObjectUtils.getIntValue(tenderPackMap.get("TENDER_COUNT"), 0);
		tenderPackOpeningInfo.setTenderCount(tenderCount);
		if(tenderCount > 2){
			tenderPackOpeningInfo.setThreeTenderCount(tenderCount);
		}
		if(isPriceStage){
			decCount = ObjectUtils.getIntValue(tenderPackMap.get("PRICE_BID_STATUS_SUCCESS_COUNT"), 0);//PRICE_BID_STATUS_SUCCESS
			decFailCount = ObjectUtils.getIntValue(tenderPackMap.get("PRICE_BID_STATUS_FIALE_COUNT"), 0);
			confirmCount = ObjectUtils.getIntValue(tenderPackMap.get("PRICE_CONFIRM_STATUS_COUNT"), 0);
			confirmQuestionCount = ObjectUtils.getIntValue(tenderPackMap.get("PRICE_CONFIRM_STATUS_QUESTION_COUNT"), 0);
			tenderPackOpeningInfo.setBidOpeningTime((Date)tenderPackMap.get("PRICE_BID_OPENING_TIME"));
		}else {
			decCount = ObjectUtils.getIntValue(tenderPackMap.get("BUSINESS_BID_STATUS_COUNT"), 0);
			decFailCount = ObjectUtils.getIntValue(tenderPackMap.get("BUSINESS_BID_STATUS_FIALE_COUNT"), 0);
			confirmCount = ObjectUtils.getIntValue(tenderPackMap.get("CONFIRM_STATUS_COUNT"), 0);
			confirmQuestionCount = ObjectUtils.getIntValue(tenderPackMap.get("CONFIRM_STATUS_QUESTION_COUNT"), 0);
			tenderPackOpeningInfo.setBidOpeningTime((Date)tenderPackMap.get("BID_OPENING_START_TIME"));
		}
		if(!isBusAndTecStage && !isPreEvaluation){
			noPriceCount = ObjectUtils.getIntValue(tenderPackMap.get("NO_PRICE_COUNT"), 0);
		}
		if(tenderCount > 2){
			tenderPackOpeningInfo.setThreeTenderCount(tenderCount);
		}
		if(isStartDecrypt){
			tenderPackOpeningInfo.setOpeningCount(tenderCount);
		}else if(decCount > 0){
			// 存在未启动开标，但实际已经完成开标了（现场开标数据和下载历史开标数据会出现统计数字对不上）
			tenderPackOpeningInfo.setOpeningCount(tenderCount);
			
		}
		// 如果是经济标开标，则经济标的可开标家次设置为商务标的已开标家次（已开商务技术后的才能开经济）
		if(isPriceStage){
			boolean busDecComplate = true;
			if(StringUtils.isEmpty(tenderPackOpeningInfo.getTenderProject().getBusinessBidStatus()) || tenderPackOpeningInfo.getTenderProject().getBusinessBidStatus().equals(CommonEnum.NOT_START)){
//			if(null != tenderPackOpeningInfo.getTenderProject().getBusinessBidStatus() && tenderPackOpeningInfo.getTenderProject().getBusinessBidStatus().equals(CommonErrorEnum.COMPLETE)){
				busDecComplate = false;
			}
			if(busDecComplate){
				int busDecCount = ObjectUtils.getIntValue(tenderPackMap.get("BUSINESS_BID_STATUS_COUNT"), 0);
				tenderPackOpeningInfo.setOpeningCount(busDecCount);
			}
		
		}
		
		tenderPackOpeningInfo.setSignCount(signCount);
		tenderPackOpeningInfo.setDecCount(decCount);
		tenderPackOpeningInfo.setDecFailCount(decFailCount);
//		tenderPackOpeningInfo.setErrorsCount(decFailCount + noPriceCount);
		tenderPackOpeningInfo.setNoPriceCount(noPriceCount);
		tenderPackOpeningInfo.setConfirmCount(confirmCount);
		tenderPackOpeningInfo.setConfirmQuestionCount(confirmQuestionCount);
		if(isPriceStage){
			tenderPackOpeningInfo.setComplate(null != tenderPack.getPriceBidStatus() && tenderPack.getPriceBidStatus().equals(CommonEnum.COMPLETE));
		}else {
			tenderPackOpeningInfo.setComplate(null != tenderPack.getBusinessBidStatus() && tenderPack.getBusinessBidStatus().equals(CommonEnum.COMPLETE));
		}
		if(tenderPackOpeningInfo.isComplate()){
			tenderPackOpeningInfo.setComplatePackCount(1);
			tenderPackOpeningInfo.setComplateSupplierCount(tenderPackOpeningInfo.getDecCount());
		}
		tenderPackOpeningInfo.setTenderProject(tenderPack);
		return tenderPackOpeningInfo;
	}
	/**
	 * @Description: 获得包件的开标信息
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	public Map<String, Object> getTenderPackOpeningMapInfo(String tenderId, String bidopeningStage) {
		List<Map<String, Object>> projectList = tenderProjectMapper.selectTenderPackOpeningInfoList(tenderId, false, bidopeningStage);
		if(null != projectList && projectList.size() > 0){
			return projectList.get(0);
		}
		return null;
	}

	/** 
	 * @Description:  获得包件的开标信息
	 * @author mazc
	 * @param projectId
	 * @return 
	 */
	@Override
	public List<TenderProject> getTenderPackBidOpeningInfoList(String projectId) {
		return  tenderProjectMapper.selectgetTenderPackBidOpeningInfoList(projectId);
	}

	/** 
	 * @Description: 获得可唱标的包件
	 * @author mazc
	 * @param page
	 * @param parentId
	 * @return 
	 */
	@Override
	public IPage<TenderProject> getSignBidTenderPackagePageList(Page<TenderProject> page, String parentId) {
		return tenderProjectMapper.selectTenderProjectPack(page, parentId);
	}

	@Override
	public int  updateByParendId(TenderProject tenderProject, String parendId) {
			QueryWrapper<TenderProject> queryWrapper = new QueryWrapper<TenderProject>();
			queryWrapper.lambda().eq(TenderProject::getParentId, parendId);
		return tenderProjectMapper.update(tenderProject, queryWrapper);
	}

	/** 
	 * @Description: 是否获取了最新的投标信息
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	public boolean isGetLatestBidinfo(String tenderId) {
		String latestBidinfo = tenderProjectMapper.getLatestBidinfo(tenderId);
		if(CommonEnum.YES.equals(latestBidinfo)) {
			return true;
		}
		return false;
	}
	/** 
	 * @Description: 是否完成开标
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	@Override
	public boolean isBidOpeningComplate(String tenderId) {
		String bidOpeningStatus = tenderProjectMapper.getBidOpeningStatus(tenderId);
		if(CommonEnum.COMPLETE.equals(bidOpeningStatus)){
			return true;
		}
		return false;
	}
	/** 
	 * @Description: 评标是否结束
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	@Override
	public boolean isEvaluationComplate(String tenderId) {
		String strComplateStatus = tenderProjectMapper.getComplateStatus(tenderId);
		if(CommonEnum.COMPLETE.equals(strComplateStatus)){
			return true;
		}
		return false;
	}

	/** 
	 * @Description: 通过 tenderNo、phaseType、tenderNumber 查询项目信息
	 * @author mazc
	 * @param tenderNo
	 * @param phaseType
	 * @param tenderNumber
	 * @return 
	 */
	@Override
	public TenderProject getTenderProjectByPhaseTypeTenderNumber(String tenderNo, String phaseType,String tenderNumber) {
		
		QueryWrapper<TenderProject> queryWrapper = new QueryWrapper<TenderProject>();
		if(StringUtils.isNotEmpty(tenderNumber)) {
			queryWrapper.lambda().eq(TenderProject::getTenderNumber, tenderNumber);
		}
		queryWrapper.lambda().eq(TenderProject::getTenderNo, tenderNo);
		if(StringUtils.isNotEmpty(phaseType)) {
			queryWrapper.lambda().eq(TenderProject::getPhaseType, phaseType);
		}
		queryWrapper.lambda().isNull(TenderProject::getParentId);
		return tenderProjectMapper.selectOne(queryWrapper);
	}

	/** 
	 * @Description: 
	 * @author mazc
	 * @param tenderNo
	 * @param parentId
	 * @return 
	 */
	@Override
	public List<String> queryTenderIdListByTenderNoAndparentId(String tenderNo, String parentId) {
 		return tenderProjectMapper.queryTenderIdListByTenderNoAndparentId(tenderNo, parentId);
	}

	/** 
	 * @Description:  导入开标结果
	 * @author mazc
	 * @param tenderProject
	 * @param supplierList
	 * @param bidPriceList
	 * @param bidOpeningStage
	 * @return 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class) 
	public boolean importBidOpenResult(TenderProject tenderProject, List<Supplier> supplierList,List<BidPrice> bidPriceList, String bidOpeningStage) {
		List<BidPriceResult> bidPriceResultList = new ArrayList<BidPriceResult>();
		List<BidPriceItemResult> bidPriceItemResultList = new ArrayList<BidPriceItemResult>();
		List<SupplierOpeningResult> SupplierOpeningResultList = new ArrayList<SupplierOpeningResult>();
		for(Supplier supplier : supplierList){
			QueryWrapper<Supplier> queryWrapper = new QueryWrapper<Supplier>();
			queryWrapper.select(Supplier.SUPPLIER_ID).lambda().eq(Supplier::getTenderId, tenderProject.getId()).eq(Supplier::getOrgCode, supplier.getOrgCode());		
			Supplier tempSupplier = supplierMapper.selectOne(queryWrapper);
			
			if(null != tempSupplier && null != tempSupplier.getId()){
				supplier.setId(tempSupplier.getId());
				if(null != supplier.getSupplierOpeningResult()){
					SupplierOpeningResult supplierOpeningResult = supplier.getSupplierOpeningResult();
					QueryWrapper<SupplierOpeningResult> resultWrapper = new QueryWrapper<SupplierOpeningResult>();
					resultWrapper.select(SupplierOpeningResult.OPENING_RESULT_ID).lambda().eq(SupplierOpeningResult::getTenderId, tenderProject.getId())
															.eq(SupplierOpeningResult::getSupplierId, supplier.getId());
					SupplierOpeningResult tempSupplierOpeningResult = supplierOpeningResultMapper.selectOne(resultWrapper);
					
					if(null != tempSupplierOpeningResult){
						supplierOpeningResult.setId(tempSupplierOpeningResult.getId());
					}
					if(bidOpeningStage.equals(TenderProjectEnum.STAGE_BUS_TEC)){
						supplierOpeningResult.setBidPrice(null);
						supplierOpeningResult.setEvaluationPrice(null);
					}
					supplierOpeningResult.setSupplierId(tempSupplier.getId());
					SupplierOpeningResultList.add(supplierOpeningResult);
				}
				List<BidPriceResult> bidPriceResultListTemp = supplier.getBidPriceResultList();
				if(null != bidPriceResultListTemp && bidPriceResultListTemp.size() > 0){
					for(BidPriceResult bidPriceResult : bidPriceResultListTemp){
						bidPriceResult.setSupplierId(tempSupplier.getId());
						bidPriceResultList.add(bidPriceResult);
						List<BidPriceItemResult> tempBidPriceItemResultList = bidPriceResult.getBidPriceItemResultList();
						if(null != tempBidPriceItemResultList && tempBidPriceItemResultList.size() > 0){
							for(BidPriceItemResult bidPriceItemResult : tempBidPriceItemResultList){
								bidPriceItemResult.setSupplierId(tempSupplier.getId());
								bidPriceItemResultList.add(bidPriceItemResult);
							}
						}
					}
				}
			}
		}
		
		if(null == tenderProject.getBidOpeningStatus() || !tenderProject.getBidOpeningStatus().endsWith(CommonEnum.COMPLETE)){
			tenderProject.setBidOpeningStatus(CommonEnum.COMPLETE);
		}
		if(bidOpeningStage.equals(TenderProjectEnum.STAGE_PRICE)){
			//导入价格开标结果
			deleteBidPriceResult(tenderProject.getId());
			supplierService.batchUpdateProperty(supplierList, new String[]{"id","priceBidStatus", "priceConfirmTime", "priceConfirmStatus", "priceBidOpeningTime", "evaluationPrice", "bidPrice", "remoteDecrypt"});
			
			supplierOpeningResultService.batchSaveOrUpdate(SupplierOpeningResultList, new String[]{"id","bidOpeningStatus", "priceSignPerson", "priceSignTime", "priceSignPersonTel", "priceSignPersonCard"});
			if(null != bidPriceList && bidPriceList.size() > 0){
				for(BidPrice bidPrice : bidPriceList){
					bidPriceMapper.insert(bidPrice);
					List<BidPriceItem> bidPriceItemList = bidPrice.getBidPriceItemList();
					if(null != bidPriceItemList && bidPriceItemList.size() > 0){
						for(BidPriceItem bidPriceItem :bidPriceItemList) {
							bidPriceItem.setId(IDUtils.getID(bidPriceItem.getId()));
						}
						bidPriceItemMapper.insertByBatch(bidPriceItemList);
					}
				}
				
			}
			bidPriceResultMapper.insertByBatch(bidPriceResultList);
			bidPriceItemResultMapper.insertByBatch(bidPriceItemResultList);
			tenderProject.setPriceBidStatus(CommonEnum.COMPLETE);
			updateAssignProperty(tenderProject, new String[] {"id","bidOpeningStatus", "priceBidStatus"});
		}else if(bidOpeningStage.equals(TenderProjectEnum.STAGE_BUS_TEC)){
			//导入商务技术开标结果
 			supplierService.batchUpdateProperty(supplierList, new String[]{"id","signPerson", "signTime", "signPersonTel", "signPersonCard"});
 			supplierOpeningResultService.batchSaveOrUpdate(SupplierOpeningResultList, new String[] {"id","bidOpeningStatus", "businessBidStatus", "technologyBidStatus", "confirmTime", "confirmStatus", "remoteDecrypt"});
			tenderProject.setBusinessBidStatus(CommonEnum.COMPLETE);
			tenderProject.setTechnologyBidStatus(CommonEnum.COMPLETE);
			updateAssignProperty(tenderProject, new String[] {"id","bidOpeningStatus", "businessBidStatus",  "technologyBidStatus"});
		}else{
			//导入所有开标结果
			deleteBidPriceResult(tenderProject.getId());
			supplierMapper.updateObeSupplierBatchList(supplierList);
			supplierOpeningResultService.batchSaveOrUpdate(SupplierOpeningResultList,null);
			if(null != bidPriceList && bidPriceList.size() > 0){
				for(BidPrice bidPrice : bidPriceList){
					bidPriceMapper.insert(bidPrice);
					List<BidPriceItem> bidPriceItemList = bidPrice.getBidPriceItemList();
					if(null != bidPriceItemList && bidPriceItemList.size() > 0){
						for(BidPriceItem bidPriceItem :bidPriceItemList) {
							bidPriceItem.setId(IDUtils.getID(bidPriceItem.getId()));
						}
						bidPriceItemMapper.insertByBatch(bidPriceItemList);
					}
				}
				
			}
			
			bidPriceResultMapper.insertByBatch(bidPriceResultList);
			bidPriceItemResultMapper.insertByBatch(bidPriceItemResultList);
			
			tenderProject.setBusinessBidStatus(CommonEnum.COMPLETE);
			tenderProject.setTechnologyBidStatus(CommonEnum.COMPLETE);
			tenderProject.setPriceBidStatus(CommonEnum.COMPLETE);
			updateAssignProperty(tenderProject, new String[] {"id","bidOpeningStatus", "businessBidStatus", "technologyBidStatus", "priceBidStatus"});
			
		}
		
		
		
		if(null != tenderProject.getParentId() && !"".equals(tenderProject.getParentId().trim())){
			UpdateWrapper<TenderProject> queryWrapper = new UpdateWrapper<TenderProject>();
			queryWrapper.set(TenderProject.BID_OPENING_STATUS, CommonEnum.COMPLETE).lambda().eq(TenderProject::getId, tenderProject.getParentId());
			tenderProjectMapper.update(null, queryWrapper);
		}
		return true;
	}

    private void deleteBidPriceResult(String id) {
    	bidPriceItemMapper.deleteByTenderId(id);
		bidPriceItemResultMapper.deleteByTenderId(id);
		bidPriceResultMapper.deleteByTenderId(id);
		bidPriceMapper.deleteByTenderId(id);
    }

	@Override
	@Transactional(rollbackFor = Exception.class) 
	public TenderProject saveDownLoadProject(TenderProject tenderProject, TenderProject childTenderProject,ProjectRule parentRule, ProjectRule childRule) {
 		if(null != tenderProject){
 			if(StringUtils.isEmpty(tenderProject.getId())){
//					保存项目信息：先根据编号查询当前项目编号是否已经存在，如果不存在，则新增，如果存在，不改动
 				QueryWrapper<TenderProject> tenderProjectQuery = new QueryWrapper<TenderProject>();
 				tenderProjectQuery.lambda().isNull(TenderProject::getParentId)
 				.eq(TenderProject::getPhaseType, tenderProject.getPhaseType());
 				if(StringUtils.isNotEmpty(tenderProject.getTenderNumber())) {
 					tenderProjectQuery.lambda().eq(TenderProject::getTenderNumber, tenderProject.getTenderNumber());
 				}else {
 					tenderProjectQuery.lambda().eq(TenderProject::getTenderNo, tenderProject.getTenderNo());
 				}
 				TenderProject queryTenderProject = tenderProjectMapper.selectOne(tenderProjectQuery);
 				if(null != queryTenderProject){
 					tenderProject = queryTenderProject;
 				}
 			}
 			if(StringUtils.isNotEmpty(tenderProject.getId())){
 				ProjectRule queryParentRule = projectRuleMapper.getProjectRuleByTenderId(tenderProject.getId());
 				if(null != queryParentRule){
 					parentRule.setId(queryParentRule.getId());
 					if(null == parentRule.getBidOpeningStartTime()){
 						parentRule.setBidOpeningStartTime(queryParentRule.getBidOpeningStartTime());
 					}
 					if(null == parentRule.getPriceBidOpeningTime()){
 						parentRule.setPriceBidOpeningTime(queryParentRule.getPriceBidOpeningTime());
 					}
 					if(null == parentRule.getBidOpeningStage()){
 						parentRule.setBidOpeningStage(queryParentRule.getBidOpeningStage());
 					}
 					if(null == parentRule.getBidOpeningType()){
 						parentRule.setBidOpeningType(queryParentRule.getBidOpeningType());
 					}
 					if(null == parentRule.getDecodeType()){
 						parentRule.setDecodeType(queryParentRule.getDecodeType());
 					}
 				}
// 				tenderProjectMapper.updateAssignProperty(tenderProject, new String[] {"userId", "managerName","serverAccount", "serverPassword"});
 			}else {
 				tenderProject.setId(IDUtils.getId());
 				tenderProjectMapper.insert(tenderProject);
 			}
 			if(null == parentRule.getBidOpeningStage() || (null != childRule.getBidOpeningStage() && childRule.getBidOpeningStage().equals(TenderProjectEnum.BID_OPENING_TWO_STAGE))){
 				parentRule.setBidOpeningStage(childRule.getBidOpeningStage());
 			}
 			if(null == parentRule.getBidOpeningType() || (null != childRule.getBidOpeningType() && childRule.getBidOpeningType().equals(TenderProjectEnum.BIDOPENING_REMOTE))){
 				parentRule.setBidOpeningType(childRule.getBidOpeningType());
 			}
 			if(null != childRule.getBidOpeningStartTime()){
 				if(null == parentRule.getBidOpeningStartTime() || childRule.getBidOpeningStartTime().before(parentRule.getBidOpeningStartTime())){
 					parentRule.setBidOpeningStartTime(childRule.getBidOpeningStartTime());
 				}
 			}
 			if(null != childRule.getPriceBidOpeningTime()){
 				if(null == parentRule.getPriceBidOpeningTime() || childRule.getPriceBidOpeningTime().before(parentRule.getPriceBidOpeningTime())){
 					parentRule.setPriceBidOpeningTime(childRule.getPriceBidOpeningTime());
 				}
 			}
 			if(StringUtils.isNotEmpty(parentRule.getId())){
 				projectRuleService.updateAssignProperty(parentRule, new String[]{"id","bidOpeningStartTime", "priceBidOpeningTime", "bidOpeningStage", "bidOpeningType","decodeType"});
 			}else {
 				if(null == parentRule.getScoreRound() || parentRule.getScoreRound() == 0) {
 					parentRule.setScoreRound(2);
 				}
 				parentRule.setTenderId(tenderProject.getId());
 				parentRule.setId(IDUtils.getId());
 				projectRuleMapper.insert(parentRule);
 			}
 		}
 		if(null != childTenderProject){
 			if(StringUtils.isEmpty(childTenderProject.getId())){
 				QueryWrapper<TenderProject> tenderPackQuery = new QueryWrapper<TenderProject>();
 				if(null != tenderProject){
 					childTenderProject.setParentId(tenderProject.getId());
 					if(null != tenderProject.getId()) {
 						tenderPackQuery.lambda().eq(TenderProject::getParentId, tenderProject.getId());
 					}
 				}
 				
 				if(null != childTenderProject.getStageId()) {
 					tenderPackQuery.lambda().eq(TenderProject::getStageId, childTenderProject.getStageId());
 				} else {
 					tenderPackQuery.lambda().eq(TenderProject::getTenderNo, childTenderProject.getTenderNo());
 				}
 				if(null != childTenderProject.getPhaseType()) {
 					tenderPackQuery.lambda().eq(TenderProject::getPhaseType, childTenderProject.getPhaseType());
 				}
 				TenderProject queryTenderPack = tenderProjectMapper.selectOne(tenderPackQuery);	
 				if(null != queryTenderPack){
 					childTenderProject = queryTenderPack;
 				}
 			}
 			if(StringUtils.isNotEmpty(childTenderProject.getId())){
 				if(StringUtils.isEmpty(childRule.getId())) {
 					ProjectRule queryPackRule = projectRuleMapper.getProjectRuleByTenderId(childTenderProject.getId());
 					if(null != queryPackRule) {
 						childRule.setId(queryPackRule.getId());
 					}
 				}
			}else {
				childTenderProject.setId(IDUtils.getId());
				tenderProjectMapper.insert(childTenderProject);
			}
 			
 			if(StringUtils.isNotEmpty(childRule.getId())){
 				projectRuleService.updateAssignProperty(childRule, new String[]{"id","bidOpeningStartTime", "priceBidOpeningTime", "bidOpeningStage", "bidOpeningType","decodeType"});
 			}else {
 				if(null == childRule.getScoreRound() || childRule.getScoreRound() == 0) {
 					childRule.setScoreRound(2);
 				}
 				childRule.setTenderId(childTenderProject.getId());
 				childRule.setId(IDUtils.getId());
 				projectRuleMapper.insert(childRule);
 			}
 			return childTenderProject;
 		}
 		return tenderProject;
	}

	/** 
	 * @Description: 删除开标结果
	 * @author mazc
	 * @param id
	 * @return 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class) 
	public int deleteBidOpeningResult(String id) {
		int count = 0;
		/**
		 * bidPriceItemMapper与  bidPriceItemResultMapper已经抛弃 暂时不进行使用   
		 * 这两个 deletebytenderId目前还会保留一段时间 为了可以清理历史数据
		 * 在面删除表结构时候进行删除
		 **/
		count += bidPriceItemMapper.deleteByTenderId(id);
		count += bidPriceItemResultMapper.deleteByTenderId(id);
		
		count += bidPriceResultMapper.deleteByTenderId(id);
		//删除历史报价表
		count += bidPriceHisResultMapper.deleteByTenderId(id);
		//删除轮次表
		count += reBidPriceRoundMapper.deleteByTenderId(id);
		
		count += bidPriceMapper.deleteByTenderId(id);
		count += supplierOpeningResultMapper.deleteByTenderId(id);
		return count;
	}

	@Override
	public List<TenderProject> getTenderProjectList(String userId) {
		List<TenderProject> tenderProjectList = null;
		if(StringUtils.isNotEmpty(userId)){
			// 因为为某个包件指标项目经理，在显示时需要将包件的项目也查询出来
			tenderProjectList = queryTenderProjectAndChildrenList(null,userId);
		}else {
			tenderProjectList = tenderProjectMapper.getTenderProjectList(null);
		}
 		return tenderProjectList;
	}
	
	
	private List<TenderProject> queryTenderProjectAndChildrenList(String searchKey, String userId){
		List<Map<String, Object>> queryResultList = tenderProjectMapper.queryTenderProjectIdsList(searchKey, userId);
		Set<String> objIdsSet = new HashSet<String>();
		if(null != queryResultList && queryResultList.size() > 0){
			for(Map<String, Object> tenderMap : queryResultList){
				if(StringUtils.isNotEmpty(ObjectUtils.getObjString(tenderMap.get("TENDER_PARENT_ID")))){
					objIdsSet.add(ObjectUtils.getObjString(tenderMap.get("TENDER_PARENT_ID")));
				}
				objIdsSet.add(ObjectUtils.getObjString(tenderMap.get("TENDER_ID")));
			}
			List<TenderProject> resultList = tenderProjectMapper.getTenderProjectList(new ArrayList<String>(objIdsSet));
			return resultList;
		}	
		return null;
	}

	@Override
	public List<Map<String, Object>> getTenderPackEvaluationInfoList(String projectId, String withoutTenderId) {
 		return tenderProjectMapper.getTenderPackEvaluationInfoList(projectId, withoutTenderId);
	}

	/** 
	 * @Description: 获得可生成报表的包件信息
	 * @author mazc
	 * @param projectId
	 * @param withoutTenderId
	 * @return 
	 */
	@Override
	public List<Map<String, Object>> getReportTenderPackInfoList(String projectId, String withoutTenderId) {
		return tenderProjectMapper.getReportTenderPackInfoList(projectId, withoutTenderId);
	}

	/** 
	 * @Description: 获得当前项目下所有标段专家信息
	 * @author mazc
	 * @param projectId
	 * @param withoutTenderId
	 * @return 
	 */
	@Override
	public List<Map<String, Object>> getTenderPackExpertInfoList(String projectId, String withoutTenderId) {
		return tenderProjectMapper.getTenderPackExpertInfoList(projectId,withoutTenderId);
	}

	@Override
	public List<TenderProject> getTenderProjectByExpert() {
		return tenderProjectMapper.getTenderProjectByExpert();
	}

	/** 
	 * @Description:重新开始评审 
	 * @author mazc
	 * @param tenderId
	 * @param cleanData 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class) 
	public void restartEvaluation(String tenderId, boolean cleanData) {
		// 将所有评标步骤的状态改为未评审
		evaluationStepMapper.updateRestartEvaluation(tenderId);
		//修改专家的签到状态
		expertMapper.updateRestartEvaluation(tenderId);
		// 删除专家对投标人的评标结果
		expertEvaluationResultMapper.deleteRestartEvaluation(tenderId);
		// 删除投标人的评标结果
		supplierEvaluationResultMapper.deleteRestartEvaluation(tenderId);
		// 删除投标人的评标步骤
		supplierEvaluationStepMapper.deleteRestartEvaluation(tenderId);
		if(cleanData){
			// 删除专家的评标步骤
			expertEvaluationStepMapper.deleteRestartEvaluation(tenderId);
			// 删除评审指标的评标结果
			evaluationFactorResultMapper.deleteRestartEvaluation(tenderId);
			// 删除清标检查结果
			checkResultMapper.deleteRestartEvaluation(tenderId);
 		}else {		
			// 修改专家的评标步骤为未开始
 			expertEvaluationStepMapper.updateRestartEvaluation(tenderId);
 		}
	}

	/** 
	 * @Description: 所有包件的项目信息来源都进行修改
	 * @author mazc
	 * @param serverId
	 * @param serverAccount
	 * @param serverPassword
	 * @param parentId
	 * @return 
	 */
	@Override
	public int updateServerContent(String serverId, String serverAccount, String serverPassword, String parentId) {
		return tenderProjectMapper.updateServerContent(serverId,serverAccount,serverPassword,parentId);
	}

	@Override
	public TenderProject valProjectExist(String tenderNo, String parentId, String phaseType, String tenderNumber) {
		QueryWrapper<TenderProject> queryWrapper = new QueryWrapper<TenderProject>();
		if(StringUtils.isNotEmpty(tenderNumber)) {
			queryWrapper.lambda().eq(TenderProject::getTenderNumber, tenderNumber);
		}
		queryWrapper.lambda().eq(TenderProject::getTenderNo, tenderNo);
		if(StringUtils.isNotEmpty(parentId)) {
			queryWrapper.lambda().eq(TenderProject::getParentId, parentId);
		}else {
			queryWrapper.lambda().isNull(TenderProject::getParentId);
		}
		if(StringUtils.isNotEmpty(phaseType)) {
			queryWrapper.lambda().eq(TenderProject::getPhaseType, phaseType);
		}
		return tenderProjectMapper.selectOne(queryWrapper);
	}

	/** 
	 * @Description: 修改项目的进度状态
	 * @author mazc
	 * @param tenderId
	 * @param status
	 * @return 
	 */
	@Override
	public int updateEvaluateStatus(String tenderId, String status) {
		return tenderProjectMapper.updateEvaluateStatus(tenderId, status);
	}
	
	/**
	 * @Description: 验证项目是否已经完成指定阶段的开标
	 * @param tenderId
	 * @param bidopeningStage
	 * @return
	 */
	public boolean isComplateProjectOpeningByStage(String projectId, String bidopeningStage) {
		// 如果项目下所有的标段都完成开标，则认为项目完成开标
		return tenderProjectMapper.isComplateProjectOpeningByStage(projectId, bidopeningStage) <= 0;
	}
	
	/**
	 *删除项目详细数据
	 */
	@Override
	public Integer deleteDetailByTenderId(String tenderId) {
		return tenderProjectMapper.deleteDetailByTenderId(tenderId);
	}
	
	/** 
	 * @Description: 
	 * 				 
 	 * @author mazc
	 * @param tenderCode
	 * @param tenderNumber
	 * @param stageType
	 * @param stageId
	 * @return 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class) 
	public ResultBody deleteProjectInfo(String tenderCode,String packCode,String stageType ,String tenderNumber,String stageId) {
		
		if(StringUtils.isEmpty(tenderCode) && StringUtils.isEmpty(tenderNumber)) {
			throw  new BizException("-1","缺少参数[tenderCode]!");
 		}
		if(StringUtils.isEmpty(stageType)) {
			throw  new BizException("-1","缺少参数[stageType]!");
 		}
		
		QueryWrapper<TenderProject> queryWrapper = new QueryWrapper<TenderProject>();
		if(StringUtils.isNotEmpty(tenderNumber) ) {
			queryWrapper.lambda().eq(TenderProject::getTenderNumber, tenderNumber);
		}else {
			queryWrapper.lambda().eq(TenderProject::getTenderNo, tenderCode);
		}
		queryWrapper.lambda().eq(TenderProject::getPhaseType, stageType);
		int deleteSize = 0;
		TenderProject tenderProjcet = tenderProjectMapper.selectOne(queryWrapper);
		if(null != tenderProjcet) {
			// 判断是否为包件
			if(CommonEnum.YES.equals(tenderProjcet.getIsPack())){
				QueryWrapper<TenderProject> deletePackWrapper = new QueryWrapper<TenderProject>();
				deletePackWrapper.lambda().eq(TenderProject::getParentId, tenderProjcet.getId());
				if(StringUtils.isNotEmpty(stageId)) {
					deletePackWrapper.lambda().eq(TenderProject::getStageId, stageId);
				}else if(StringUtils.isNotEmpty(packCode)) {
					deletePackWrapper.lambda().eq(TenderProject::getTenderNo, packCode);
				}else {
					throw  new BizException("-1","缺少参数[packCode]!");
				}
				deleteSize = tenderProjectMapper.delete(deletePackWrapper);
				if(deleteSize > 0) {
					queryWrapper.lambda().eq(TenderProject::getParentId, tenderProjcet.getId());
					int sum = tenderProjectMapper.selectCount(queryWrapper);
					// 如果包件删除成功，判断是否需要删除项目
					if(sum <= 0) {
						deleteDetailByTenderId(tenderProjcet.getId());
					}
				}
				
			}else {
				deleteSize = deleteDetailByTenderId(tenderProjcet.getId());
			}
		}
		return ResultBody.success(deleteSize>0);
		
		
   	}

	  /**
	   * @Description: 评标是否结束
	   * @author mazc
	   * @param tenderId
	   * @return
	   */
	  @Override
	  public boolean isUploadEvalResult(String tenderId) {
		  String strComplateStatus = tenderProjectMapper.getUploadEvalResultStatus(tenderId);
		  if(CommonEnum.COMPLETE.equals(strComplateStatus)){
			  return true;
		  }
		  return false;
	  }
	@Override
	public boolean updateEvaluationComplateStatus(String tenderId, String parentId, String archiveStatus, Date date) {
		TenderProject tenderProject = tenderProjectMapper.selectById(tenderId);
		tenderProject.setUseStatus(archiveStatus);
		tenderProject.setModifyTime(date);
		return tenderProjectMapper.updateAssignProperty(tenderProject, new String[] {"modifyTime"})> 0;
//		return false;
	}

	  public String copyProject(String tenderNo, String stageId, String tenderNonumber, String username, String password, String copyStatus) {
		  String returnFlag = "{\n\"suc\":\"0\"\n}";
		  System.out.println("开始操作" + new Date());

		  try {
			  String tenderId = this.tenderProjectMapper.getTenderId(tenderNo, stageId, tenderNonumber);
			  TenderProject tenderproject = (TenderProject)this.getById(tenderId);
			  TenderProject parentProject = (TenderProject)this.getById(tenderproject.getParentId());
			  ProjectRule projectRule = this.projectRuleMapper.getProjectRuleByTenderId(tenderId);
			  ProjectRule parentProjectRule = this.projectRuleMapper.getProjectRuleByTenderId(parentProject.getId());
			  QueryWrapper<Supplier> SupplierQueryWrapper = new QueryWrapper();
			  SupplierQueryWrapper.lambda().eq(Supplier::getTenderId, tenderId);
			  List<Supplier> supplierList = this.supplierService.list(SupplierQueryWrapper);
			  Map<String, String> tendrProjectIdMap = new HashMap();
			  String tenderProjectUUid = UuidUtils.getUUID();
			  String tenderParenProjectUUid = UuidUtils.getUUID();
			  tendrProjectIdMap.put(tenderproject.getId(), tenderProjectUUid);
			  tendrProjectIdMap.put(parentProject.getId(), tenderParenProjectUUid);
			  parentProject.setTenderSourceId(parentProject.getId());
			  tenderproject.setTenderSourceId(tenderId);
			  tenderproject.setId(tenderProjectUUid);
			  parentProject.setId(tenderParenProjectUUid);
			  tenderproject.setParentId(tenderParenProjectUUid);
			  projectRule.setTenderId(tenderProjectUUid);
			  parentProjectRule.setTenderId(tenderParenProjectUUid);
			  projectRule.setId(UuidUtils.getUUID());
			  parentProjectRule.setId(UuidUtils.getUUID());
			  tenderproject.setComplateStatus("00");
			  tenderproject.setUploadEvaluationResultStatus("00");
			  this.tenderProjectMapper.insert(tenderproject);
			  this.tenderProjectMapper.insert(parentProject);
			  this.projectRuleMapper.insert(projectRule);
			  this.projectRuleMapper.insert(parentProjectRule);
			  System.out.println("项目信息复制完毕" + new Date());
			  Map<String, String> supplierIdMap = new HashMap();
			  Iterator var19 = supplierList.iterator();

			  while(var19.hasNext()) {
				  Supplier supplier = (Supplier)var19.next();
				  String supplierUUid = UuidUtils.getUUID();
				  supplierIdMap.put(supplier.getId(), supplierUUid);
				  supplier.setSourceId(supplier.getId());
				  supplier.setId(supplierUUid);
				  supplier.setTenderId(tenderProjectUUid);
			  }

			  this.supplierService.saveBatch(supplierList);
			  System.out.println("供应商信息复制完毕" + new Date());
			  List<EvaluationStep> evaluationStepList = this.evaluationStepService.getEvaluationStepList(tenderId);
			  Map<String, String> stepIdMap = new HashMap();
			  Iterator var69 = evaluationStepList.iterator();

			  while(var69.hasNext()) {
				  EvaluationStep evaluationStep = (EvaluationStep)var69.next();
				  String stepUUid = UuidUtils.getUUID();
				  stepIdMap.put(evaluationStep.getId(), stepUUid);
				  evaluationStep.setId(stepUUid);
				  evaluationStep.setTenderId(tenderProjectUUid);
			  }

			  this.evaluationStepService.saveBatch(evaluationStepList);
			  System.out.println("评审步骤信息复制完毕" + new Date());
			  QueryWrapper<ExpertEntity> expertEntityQueryWrapper = new QueryWrapper();
			  expertEntityQueryWrapper.lambda().eq(ExpertEntity::getTenderId, tenderId);
			  List<ExpertEntity> expertList = this.expertService.list(expertEntityQueryWrapper);
			  Map<String, String> expertIdMap = new HashMap();
			  Iterator var24 = expertList.iterator();

			  while(var24.hasNext()) {
				  ExpertEntity expertEntity = (ExpertEntity)var24.next();
				  String expertUUid = UuidUtils.getUUID();
				  expertIdMap.put(expertEntity.getId(), expertUUid);
				  expertEntity.setId(expertUUid);
				  expertEntity.setTenderId(tenderProjectUUid);
				  expertEntity.setProjectId(tenderParenProjectUUid);
			  }

			  this.expertService.saveBatch(expertList);
			  System.out.println("专家信息复制完毕" + new Date());
			  QueryWrapper<EvaluationFactor> evaluationFactorQueryWrapper = new QueryWrapper();
			  evaluationFactorQueryWrapper.lambda().eq(EvaluationFactor::getTenderId, tenderId);
			  List<EvaluationFactor> evaluationFactorList = this.evaluationFactorService.list(evaluationFactorQueryWrapper);
			  Map<String, String> factorIdMap = new HashMap();
			  Iterator var27 = evaluationFactorList.iterator();

			  EvaluationFactor factor;
			  while(var27.hasNext()) {
				  factor = (EvaluationFactor)var27.next();
				  String factorUUid = UuidUtils.getUUID();
				  factorIdMap.put(factor.getId(), factorUUid);
				  factor.setId(factorUUid);
				  factor.setTenderId(tenderProjectUUid);
			  }

			  var27 = evaluationFactorList.iterator();

			  while(var27.hasNext()) {
				  factor = (EvaluationFactor)var27.next();
				  factor.setParentId((String)factorIdMap.get(factor.getParentId()));
			  }

			  this.evaluationFactorService.saveBatch(evaluationFactorList);
			  System.out.println("指标信息复制完毕" + new Date());
			  QueryWrapper<ExpertEvaluationStep> queryWrapper = new QueryWrapper();
			  queryWrapper.lambda().eq(ExpertEvaluationStep::getTenderId, tenderId);
			  List<ExpertEvaluationStep> expertEvaluationStepList = this.expertEvaluationStepService.list(queryWrapper);
			  Iterator var78 = expertEvaluationStepList.iterator();

			  while(var78.hasNext()) {
				  ExpertEvaluationStep expertEvaluationStep = (ExpertEvaluationStep)var78.next();
				  String expertEvaluationStepUUid = UuidUtils.getUUID();
				  expertEvaluationStep.setId(expertEvaluationStepUUid);
				  expertEvaluationStep.setTenderId(tenderProjectUUid);
				  expertEvaluationStep.setStepId((String)stepIdMap.get(expertEvaluationStep.getStepId()));
			  }

			  this.expertEvaluationStepService.saveBatch(expertEvaluationStepList);
			  System.out.println("专家步骤信息复制完毕" + new Date());
			  QueryWrapper<EvaluationStepExpert> evaluationStepExpertQueryWrapper = new QueryWrapper();
			  evaluationStepExpertQueryWrapper.lambda().eq(EvaluationStepExpert::getTenderId, tenderId);
			  List<EvaluationStepExpert> evaluationStepExpertList = this.evaluationStepExpertService.list(evaluationStepExpertQueryWrapper);
			  Iterator var81 = evaluationStepExpertList.iterator();

			  while(var81.hasNext()) {
				  EvaluationStepExpert expertEvaluationStep = (EvaluationStepExpert)var81.next();
				  String expertEvaluationStepUUid = UuidUtils.getUUID();
				  expertEvaluationStep.setId(expertEvaluationStepUUid);
				  expertEvaluationStep.setTenderId(tenderProjectUUid);
				  expertEvaluationStep.setStepId((String)stepIdMap.get(expertEvaluationStep.getStepId()));
			  }

			  this.evaluationStepExpertService.saveBatch(evaluationStepExpertList);
			  QueryWrapper<EvaluationStepFactor> evaluationStepFactorQueryWrapper = new QueryWrapper();
			  evaluationStepFactorQueryWrapper.lambda().eq(EvaluationStepFactor::getTenderId, tenderId);
			  List<EvaluationStepFactor> evaluationStepFactorList = this.evaluationStepFactorService.list(evaluationStepFactorQueryWrapper);
			  Iterator var84 = evaluationStepFactorList.iterator();

			  while(var84.hasNext()) {
				  EvaluationStepFactor expertEvaluationStep = (EvaluationStepFactor)var84.next();
				  String expertEvaluationStepUUid = UuidUtils.getUUID();
				  expertEvaluationStep.setId(expertEvaluationStepUUid);
				  expertEvaluationStep.setTenderId(tenderProjectUUid);
				  expertEvaluationStep.setFactorId((String)factorIdMap.get(expertEvaluationStep.getFactorId()));
				  expertEvaluationStep.setStepId((String)stepIdMap.get(expertEvaluationStep.getStepId()));
			  }

			  this.evaluationStepFactorService.saveBatch(evaluationStepFactorList);
			  System.out.println("项目指标信息复制完毕" + new Date());
			  QueryWrapper<DictLink> dictLinkQueryWrapper = new QueryWrapper();
			  dictLinkQueryWrapper.lambda().eq(DictLink::getTenderId, tenderId);
			  List<DictLink> dictLinkList = this.dictLinkService.list(dictLinkQueryWrapper);
			  Iterator var87 = dictLinkList.iterator();

			  while(var87.hasNext()) {
				  DictLink dictLink = (DictLink)var87.next();
				  String uuid = UuidUtils.getUUID();
				  dictLink.setId(uuid);
				  dictLink.setTenderId(tenderProjectUUid);
				  dictLink.setStepId((String)stepIdMap.get(dictLink.getStepId()));
				  dictLink.setSupplierId((String)supplierIdMap.get(dictLink.getSupplierId()));
			  }

			  this.dictLinkService.saveBatch(dictLinkList);
			  QueryWrapper<EvaluationFactorResult> evaluationFactorResultQueryWrapper = new QueryWrapper();
			  evaluationFactorResultQueryWrapper.lambda().eq(EvaluationFactorResult::getTenderId, tenderId);
			  List<EvaluationFactorResult> evaluationFactorResultList = this.evaluationFactorResultService.list(evaluationFactorResultQueryWrapper);
			  Iterator var90 = evaluationFactorResultList.iterator();

			  while(var90.hasNext()) {
				  EvaluationFactorResult evaluationFactorResult = (EvaluationFactorResult)var90.next();
				  evaluationFactorResult.setId(UuidUtils.getUUID());
				  evaluationFactorResult.setTenderId(tenderProjectUUid);
				  evaluationFactorResult.setFactorId((String)factorIdMap.get(evaluationFactorResult.getFactorId()));
				  evaluationFactorResult.setStepId((String)stepIdMap.get(evaluationFactorResult.getStepId()));
				  evaluationFactorResult.setSupplierId((String)supplierIdMap.get(evaluationFactorResult.getSupplierId()));
			  }

			  this.evaluationFactorResultService.saveBatch(evaluationFactorResultList);
			  QueryWrapper<ExpertEvaluationResult> expertEvaluationResultQueryWrapper = new QueryWrapper();
			  expertEvaluationResultQueryWrapper.lambda().eq(ExpertEvaluationResult::getTenderId, tenderId);
			  List<ExpertEvaluationResult> expertEvaluationResultList = this.expertEvaluationResultService.list(expertEvaluationResultQueryWrapper);
			  Iterator var39 = expertEvaluationResultList.iterator();

			  while(var39.hasNext()) {
				  ExpertEvaluationResult expertEvaluationResult = (ExpertEvaluationResult)var39.next();
				  expertEvaluationResult.setId(UuidUtils.getUUID());
				  expertEvaluationResult.setTenderId(tenderProjectUUid);
				  expertEvaluationResult.setStepId((String)stepIdMap.get(expertEvaluationResult.getStepId()));
				  expertEvaluationResult.setSupplierId((String)supplierIdMap.get(expertEvaluationResult.getSupplierId()));
			  }

			  System.out.println("项目信息复制完毕6" + new Date());
			  this.expertEvaluationResultService.saveBatch(expertEvaluationResultList);
			  QueryWrapper<SupplierEvaluationFactor> supplierEvaluationFactorQueryWrapper = new QueryWrapper();
			  supplierEvaluationFactorQueryWrapper.lambda().eq(SupplierEvaluationFactor::getTenderId, tenderId);
			  List<SupplierEvaluationFactor> supplierEvaluationFactorList = this.supplierEvaluationFactorService.list(supplierEvaluationFactorQueryWrapper);
			  Iterator var41 = supplierEvaluationFactorList.iterator();

			  while(var41.hasNext()) {
				  SupplierEvaluationFactor expertEvaluationResult = (SupplierEvaluationFactor)var41.next();
				  expertEvaluationResult.setId(UuidUtils.getUUID());
				  expertEvaluationResult.setTenderId(tenderProjectUUid);
				  expertEvaluationResult.setParentId((String)factorIdMap.get(expertEvaluationResult.getParentId()));
				  expertEvaluationResult.setSupplierId((String)supplierIdMap.get(expertEvaluationResult.getSupplierId()));
			  }

			  this.supplierEvaluationFactorService.saveBatch(supplierEvaluationFactorList);
			  QueryWrapper<CheckResult> checkResultQueryWrapper = new QueryWrapper();
			  checkResultQueryWrapper.lambda().eq(CheckResult::getTenderId, tenderId);
			  List<CheckResult> checkResultList = this.checkResultService.list(checkResultQueryWrapper);
			  Iterator var43 = checkResultList.iterator();

			  while(var43.hasNext()) {
				  CheckResult checkResult = (CheckResult)var43.next();
				  checkResult.setId(UuidUtils.getUUID());
				  checkResult.setTenderId(tenderProjectUUid);
				  checkResult.setStepId((String)stepIdMap.get(checkResult.getStepId()));
				  checkResult.setSupplierId((String)supplierIdMap.get(checkResult.getSupplierId()));
			  }

			  this.checkResultService.saveBatch(checkResultList);
			  System.out.println("项目信息复制完毕4" + new Date());
			  QueryWrapper<BidRegulations> bidRegulationsQueryWrapper = new QueryWrapper();
			  bidRegulationsQueryWrapper.lambda().eq(BidRegulations::getTenderId, tenderId);
			  List<BidRegulations> bidRegulationsList = this.bidRegulationsService.list(bidRegulationsQueryWrapper);
			  Iterator var45 = bidRegulationsList.iterator();

			  while(var45.hasNext()) {
				  BidRegulations bidRegulations = (BidRegulations)var45.next();
				  bidRegulations.setId(UuidUtils.getUUID());
				  bidRegulations.setTenderId(tenderProjectUUid);
			  }

			  this.bidRegulationsService.saveBatch(bidRegulationsList);
			  QueryWrapper<SupplierEvaluationStep> supplierEvaluationStepQueryWrapper = new QueryWrapper();
			  supplierEvaluationStepQueryWrapper.lambda().eq(SupplierEvaluationStep::getTenderId, tenderId);
			  List<SupplierEvaluationStep> supplierEvaluationStepList = this.supplierEvaluationStepService.list(supplierEvaluationStepQueryWrapper);
			  Iterator var47 = supplierEvaluationStepList.iterator();

			  while(var47.hasNext()) {
				  SupplierEvaluationStep supplierEvaluationStep = (SupplierEvaluationStep)var47.next();
				  String uuid = UuidUtils.getUUID();
				  supplierEvaluationStep.setId(uuid);
				  supplierEvaluationStep.setTenderId(tenderProjectUUid);
				  supplierEvaluationStep.setStepId((String)stepIdMap.get(supplierEvaluationStep.getStepId()));
				  supplierEvaluationStep.setSupplierId((String)supplierIdMap.get(supplierEvaluationStep.getSupplierId()));
			  }

			  this.supplierEvaluationStepService.saveBatch(supplierEvaluationStepList);
			  QueryWrapper<SupplierOpeningResult> supplierOpeningResultQueryWrapper = new QueryWrapper();
			  supplierOpeningResultQueryWrapper.lambda().eq(SupplierOpeningResult::getTenderId, tenderId);
			  List<SupplierOpeningResult> supplierOpeningResultList = this.supplierOpeningResultService.list(supplierOpeningResultQueryWrapper);
			  Iterator var103 = supplierOpeningResultList.iterator();

			  while(var103.hasNext()) {
				  SupplierOpeningResult supplierEvaluationStep = (SupplierOpeningResult)var103.next();
				  String uuid = UuidUtils.getUUID();
				  supplierEvaluationStep.setId(uuid);
				  supplierEvaluationStep.setTenderId(tenderProjectUUid);
				  supplierEvaluationStep.setSupplierId((String)supplierIdMap.get(supplierEvaluationStep.getSupplierId()));
			  }

			  this.supplierOpeningResultService.saveBatch(supplierOpeningResultList);
			  QueryWrapper<BidPrice> bidPriceQueryWrapper = new QueryWrapper();
			  bidPriceQueryWrapper.lambda().eq(BidPrice::getTenderId, tenderId);
			  List<BidPrice> oldBidPriceList = this.bidPriceService.list(bidPriceQueryWrapper);
			  List<BidPrice> newBidPriceList = (List)oldBidPriceList.stream().map((b) -> {
				  new BidPrice();
				  b.setTenderId(tenderProjectUUid);
				  b.setId(UuidUtils.getUUID());
				  return b;
			  }).collect(Collectors.toList());
			  this.bidPriceService.batchSaveResolverBidPriceTable(newBidPriceList);
			  System.out.println("分项报价信息复制完毕" + new Date());
			  List<BidPriceResult> oldBidPriceResultList = this.bidPriceResultService.getBidPriceResultList(tenderId);
			  List<BidPriceResult> newBidPriceResultList = new ArrayList();
			  QueryWrapper<Supplier> NewSupplierQueryWrapper = new QueryWrapper();
			  NewSupplierQueryWrapper.lambda().eq(Supplier::getTenderId, tenderProjectUUid);
			  List<Supplier> newsupplierList = this.supplierService.list(NewSupplierQueryWrapper);
			  Iterator var56 = newsupplierList.iterator();

			  Iterator var58;
			  while(var56.hasNext()) {
				  Supplier newSupplier = (Supplier)var56.next();
				  var58 = newBidPriceList.iterator();

				  while(var58.hasNext()) {
					  BidPrice newBidPrice = (BidPrice)var58.next();
					  Iterator var60 = oldBidPriceResultList.iterator();

					  while(var60.hasNext()) {
						  BidPriceResult bidPriceResult = (BidPriceResult)var60.next();
						  Supplier oldSupplier = (Supplier)this.supplierService.getById(bidPriceResult.getSupplierId());
						  if (bidPriceResult.getBidPriceName().equals(newBidPrice.getBidPriceName()) && oldSupplier.getSupplierName().equals(newSupplier.getSupplierName())) {
							  bidPriceResult.setBidPriceId(newBidPrice.getId());
							  bidPriceResult.setSupplierId(newSupplier.getId());
							  bidPriceResult.setTenderId(tenderProjectUUid);
							  bidPriceResult.setId("");
							  newBidPriceResultList.add(bidPriceResult);
						  }
					  }
				  }
			  }

			  this.bidPriceResultService.batchSaveOrUpdate(newBidPriceResultList, (String[])null);
			  System.out.println("投标报价表复制完毕" + new Date());
			  QueryWrapper<SupplierEvaluationResult> supplierEvaluationResultQueryWrapper = new QueryWrapper();
			  supplierEvaluationResultQueryWrapper.lambda().eq(SupplierEvaluationResult::getTenderId, tenderId);
			  List<SupplierEvaluationResult> supplierEvaluationResultList = this.supplierEvaluationResultService.list(supplierEvaluationResultQueryWrapper);
			  var58 = supplierEvaluationResultList.iterator();

			  while(var58.hasNext()) {
				  SupplierEvaluationResult supplierEvaluationResult = (SupplierEvaluationResult)var58.next();
				  String uuid = UuidUtils.getUUID();
				  supplierEvaluationResult.setId(uuid);
				  supplierEvaluationResult.setTenderId(tenderProjectUUid);
				  supplierEvaluationResult.setSupplierId((String)supplierIdMap.get(supplierEvaluationResult.getSupplierId()));
			  }

			  this.supplierEvaluationResultService.saveBatch(supplierEvaluationResultList);
			  TenderProject ziproject = (TenderProject)this.getById(tenderId);
			  ziproject.setUserId("");
			  String ztenderNumber = ziproject.getTenderNumber();
			  ziproject.setTenderNumber(ztenderNumber + "（复议）");
			  this.tenderProjectMapper.updateById(ziproject);
			  TenderProject fproject = (TenderProject)this.getById(ziproject.getParentId());
			  fproject.setUserId("");
			  String ftenderNumber = fproject.getTenderNumber();
			  fproject.setTenderNumber(ftenderNumber + "（复议）");
			  this.tenderProjectMapper.updateById(fproject);
			  QueryWrapper<ExpertEntity> yexpert = new QueryWrapper();
			  yexpert.lambda().eq(ExpertEntity::getTenderId, tenderId);
			  List<ExpertEntity> yexpertList = this.expertService.list(yexpert);
			  Iterator var64 = yexpertList.iterator();

			  while(var64.hasNext()) {
				  ExpertEntity expertEntity = (ExpertEntity)var64.next();
				  expertEntity.setUserId("");
				  this.expertService.updateById(expertEntity);
			  }

			  System.out.println("项目信息复制完毕" + new Date());
			  returnFlag = "{\n\"suc\":\"1\"\n}";
		  } catch (Exception var66) {
			  var66.printStackTrace();
		  }

		  return returnFlag;
	  }

  }
