package com.gx.obe.server.management.revokerecord.controller;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gx.obe.server.common.base.BaseController;
import com.gx.obe.server.management.revokerecord.entity.RevokeRecord;
import com.gx.obe.server.management.revokerecord.service.RevokeRecordService;

import io.swagger.annotations.Api;
import lombok.AllArgsConstructor;
/** 
 * @Description: 
 * @author mazc 
 */
@Api(tags = "废标记录")
@RestController
@AllArgsConstructor
@RequestMapping("/revokeRecord")
public class RevokeRecordController extends BaseController<RevokeRecordService,RevokeRecord>{
	@Autowired
	public RevokeRecordService RevokeRecordServices;
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param ObeRevokeRecordList 
	 */
	@PostMapping("/insertByBatch")
	public int insertByBatch(@RequestBody List<RevokeRecord> ObeRevokeRecordList) {
		return RevokeRecordServices.insertByBatch(ObeRevokeRecordList);
	}
	/**
	 * @Description: 批量更新
	 * @author mazc
	 * @param ObeRevokeRecordList 
	 */
	@PostMapping("/updateByBatch")
	public int updateByBatch(@RequestBody List<RevokeRecord> ObeRevokeRecordList) {
		return RevokeRecordServices.updateByBatch(ObeRevokeRecordList);
	}
	
	
	/**
	 * @Description: 是否发起项目废标
	 * @author mazc
	 * @param tenderId 项目ID
	 * @param revokeType 废标状态(01：废标；02：取消废标)
	 * @return boolean
	 */
	@GetMapping("/isStartRevoke")
	public boolean isStartRevoke(String tenderId, String revokeType){
		return RevokeRecordServices.isStartRevoke(tenderId, revokeType);
	}
	
	/**
	 * @Description: 根据废标状态，获得废标记录集合。 
	 * @author mazc
	 * @param tenderId 项目ID
	 * @param revokeType 废标状态(01：废标；02：取消废标)
	 * @return List<RevokeRecord>
	 */
	@GetMapping("/getRevokeRecordList")
	public List<RevokeRecord> getRevokeRecordList(String tenderId, String revokeType){
		return RevokeRecordServices.getRevokeRecordList(tenderId, revokeType);
	}
	
	/**
	 * @Description: 根据废标状态，删除废标记录。 
	 * @author mazc
	 * @param tenderId 项目ID
	 * @param revokeType 废标状态(01：废标；02：取消废标)
	 * @return int
	 */
	@GetMapping("/delRevokeRecord")
	public int delRevokeRecord(String tenderId, String revokeType){
		return RevokeRecordServices.delRevokeRecord(tenderId, revokeType);
	}

}