package com.gx.obe.server.management.struct_new.handler;

import com.gx.obe.server.common.enumeration.CommonEnum;
import com.gx.obe.server.common.utils.BigDecimalUtils;
import com.gx.obe.server.common.utils.CollectionUtils;
import com.gx.obe.server.common.utils.NumberFormatUtils;
import com.gx.obe.server.common.utils.SpringUtil;
import com.gx.obe.server.management.evaluation.entity.EvaluationFactor;
import com.gx.obe.server.management.struct_new.entity.*;
import com.gx.obe.server.management.struct_new.enums.DataCategoryEnum;
import com.gx.obe.server.management.struct_new.enums.ModelDataTypeEnum;
import com.gx.obe.server.management.struct_new.enums.PerformanceStatus;
import com.gx.obe.server.management.struct_new.service.*;

import java.math.BigDecimal;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.function.BiFunction;
import java.util.function.Function;
import java.util.stream.Collectors;

public class FactorClearBidHandler {

    private final String tenderId;

    private final ObeEvaluationContentService obeEvaluationContentService = SpringUtil.getBean("obeEvaluationContentServiceImpl", ObeEvaluationContentService.class);
    private final ObeTemplateDataItemService obeTemplateDataItemService = SpringUtil.getBean("obeTemplateDataItemServiceImpl", ObeTemplateDataItemService.class);
    private final ObeModelDataService obeModelDataService = SpringUtil.getBean("obeModelDataServiceImpl", ObeModelDataService.class);
    private final ObeTemplateTableService obeTemplateTableService = SpringUtil.getBean("obeTemplateTableServiceImpl", ObeTemplateTableService.class);
    private final ObeQualificationService obeQualificationService = SpringUtil.getBean("obeQualificationServiceImpl", ObeQualificationService.class);
    private final ObeFinanceService obeFinanceService = SpringUtil.getBean("obeFinanceServiceImpl", ObeFinanceService.class);
    private final ObePerformanceService obePerformanceService = SpringUtil.getBean("obePerformanceServiceImpl", ObePerformanceService.class);
    private final ObeProjectLeaderService obeProjectLeaderService = SpringUtil.getBean("obeProjectLeaderServiceImpl", ObeProjectLeaderService.class);

    private final Function<BigDecimal, String> toString = t -> Optional.ofNullable(t).map(NumberFormatUtils::format).orElse("");

    private final List<ObeEvaluationContent> evaluationContentList;

    public FactorClearBidHandler(EvaluationFactor evaluationFactor) {
        this.tenderId = evaluationFactor.getTenderId();
        this.evaluationContentList = getEvaluationContentList(evaluationFactor.getFactorCode());
    }

    private List<ObeEvaluationContent> getEvaluationContentList(String factorCode) {
        return obeEvaluationContentService.getEvaluationContentListByFactorCode(tenderId, factorCode);
    }

    public String calculateOpinion(String supplierId) {
        if (CollectionUtils.isNull(evaluationContentList)) return "";
        List<String> opinionList = evaluationContentList.stream().map(createOpinion(supplierId)).filter(Objects::nonNull).collect(Collectors.toList());
        if (opinionList.isEmpty()) {
            boolean exist = evaluationContentList.stream().allMatch(t -> {
                switch (DataCategoryEnum.MAP.getOrDefault(t.getDataCategory(), DataCategoryEnum.NULL)) {
                    case MD:
                        return Objects.nonNull(obeModelDataService.getModelData(tenderId, supplierId, t.getRelChapterType(), t.getDataCode()));
                    case TT:
                        return Objects.nonNull(obeTemplateTableService.getTemplateTable(tenderId, supplierId, t.getRelChapterType(), t.getDataCode()));
                    case TDI:
                        return Objects.nonNull(obeTemplateDataItemService.getTemplateDataItemList(tenderId, supplierId, t.getFactorCode()));
                }
                return true;
            });
            return exist ? "有" : "无";
        }
        return String.join("\r\n", opinionList);
    }

    private Function<ObeEvaluationContent, String> createOpinion(String supplierId) {
        return evaluationContent -> {
            if (DataCategoryEnum.MD.getKey().equals(evaluationContent.getDataCategory())) {
                RelChapterTypeHandler relChapterTypeHandler = new RelChapterTypeHandler(evaluationContent.getRelChapterType());
                ModelDataTypeEnum modelDataTypeEnum = ModelDataTypeEnum.MAP.get(relChapterTypeHandler.getRelChapterType());
                if (modelDataTypeEnum != null) {
                    switch (modelDataTypeEnum) {
                        case EQ_Qualification_SJZZ:
                        case EQ_Qualification_SGZZ:
                            return evaluationContent.getEvalPointName() + "：" + createModelDataOpinion(supplierId, relChapterTypeHandler.getRelChapterType(), obeQualificationService::getQualificationList,
                                    l -> l.stream().map(ObeQualification::getCertificateName).collect(Collectors.joining("、")));
                        case EQ_Qualification_ZXDJ:
                            return evaluationContent.getEvalPointName() + "：" + createModelDataOpinion(supplierId, relChapterTypeHandler.getRelChapterType(), obeQualificationService::getQualificationList, mapJoin(
                                    t -> t.map(ObeQualification::getCertificateName).orElse("未知证书"),
                                    t -> t.map(ObeQualification::getAwardLevel).orElse("未知等级")));
                    }
                    switch (modelDataTypeEnum.getClassEnum()) {
                        case Finance:
                            return "企业净资产：" + createModelDataOpinion(supplierId, relChapterTypeHandler.getRelChapterType(), obeFinanceService::getFinanceList, mapJoin(
                                    t -> t.map(ObeFinance::getAnnual).map(s -> s + "年").orElse("未知年度"),
                                    t -> t.map(f -> BigDecimalUtils.sub(f.getTotal(), f.getTotalliabilities())).map(toString).map(s -> s + "万元").orElse("未知金额")));
//                        case Performance:
//                            if (relChapterTypeHandler.isHasMark()) {
//                                return PerformanceStatus.keyToValueFun.apply(relChapterTypeHandler.getMark()) + "业绩数量：" + createModelDataOpinion(supplierId, relChapterTypeHandler.getRelChapterType(), obePerformanceService::getPerformanceList,
//                                        l -> l.stream().filter(obePerformance -> relChapterTypeHandler.getMark().equals(obePerformance.getStatus())).count() + "个");
//                            } else {
//                                return "业绩数量：" + createModelDataOpinion(supplierId, relChapterTypeHandler.getRelChapterType(), , obePerformanceService::getPerformanceList, l -> l.size() + "个");
//                            }

                        case ProjectLeader:
                            String message = createModelDataOpinion(supplierId, relChapterTypeHandler.getRelChapterType(), obeProjectLeaderService::getProjectLeaderListAll, l -> {
                                Optional<ObeProjectLeader> projectLeader = l.stream().filter(t -> CommonEnum.YES.equals(t.getIsLeaderFlag())).findAny();
                                if (!projectLeader.isPresent()) return "未找到项目主要负责人";
                                return "项目经理姓名" + projectLeader.map(ObeProjectLeader::getLeaderName).orElse("未知名称") + "，" +
                                        projectLeader.map(ObeProjectLeader::getCertificateList)
                                                .map(mapJoin(t -> t.map(ObeCertificate::getCertificateName).orElse("未知证书"), t -> t.map(ObeCertificate::getCertificateLevel).orElse("未知等级")))
                                                .orElse("无证书") + "，" +
                                        "业绩数量：" + projectLeader.map(ObeProjectLeader::getWorkExperienceList).map(List::size).orElse(0) + "个";
                            });
                            return Optional.of(message).filter(t -> !t.isEmpty()).orElse("未找到项目负责人");
                    }
                } else {
                    return "未找到所关联的资质类型";
                }
            }
            if (DataCategoryEnum.TDI.getKey().equals(evaluationContent.getDataCategory())) {
                if ("ConsortiumAgreement".equals(evaluationContent.getRelChapterType())) {
                    if ("consortiumName".equals(evaluationContent.getDataCode())) {
                        return createTemplateDataItemOpinion(supplierId, evaluationContent.getId());
                    }
                }
                if ("TenderLetter".equals(evaluationContent.getRelChapterType())) {
                    if ("workTime".equals(evaluationContent.getDataCode())) {
                        return createTemplateDataItemOpinion(supplierId, evaluationContent.getId());
                    }
                }
            }
            if ("CompletedAchievements".equals(evaluationContent.getRelChapterType())) {
                if ("CompletedAchievements".equals(evaluationContent.getDataCode())) {
                    return createTemplateDataItemOpinion(supplierId, evaluationContent.getId());
                }
            }
            if ("OngoingAchievements".equals(evaluationContent.getRelChapterType())) {
                if ("OngoingAchievements".equals(evaluationContent.getDataCode())) {
                    return createTemplateDataItemOpinion(supplierId, evaluationContent.getId());
                }
            }
            return null;
        };
    }

    private <T> Function<List<T>, String> mapJoin(Function<Optional<T>, String> keyFunction, Function<Optional<T>, String> valueFunction) {
        return l -> l.stream().map(Optional::of).map(c -> keyFunction.apply(c) + " " + valueFunction.apply(c)).collect(Collectors.joining("、"));
    }

    private <T> String createModelDataOpinion(String supplierId, String relChapterType, BiFunction<String, String, List<T>> tenderIdModelDataIdListBiFunction, Function<List<T>, String> getNameFunction) {
        ObeModelData modelData = obeModelDataService.getModelData(tenderId, supplierId, relChapterType, relChapterType);
        if (modelData == null) return "";
        List<T> tList = tenderIdModelDataIdListBiFunction.apply(tenderId, modelData.getId());
        if (CollectionUtils.isNull(tList)) return "";
        return getNameFunction.apply(tList);
    }

    private String createTemplateDataItemOpinion(String supplierId, String contentId) {
        List<ObeTemplateDataItem> templateDataItemList = obeTemplateDataItemService.getListByContentId(tenderId, supplierId, contentId);
        if (CollectionUtils.isNull(templateDataItemList)) return "";
        return templateDataItemList.stream().findFirst().map(t -> t.getName() + "：" + t.getValue()).orElse("");
    }

}
