package com.gx.obe.server.management.struct_new.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.struct_new.dao.ObeFinanceMapper;
import com.gx.obe.server.management.struct_new.entity.ObeFinance;
import com.gx.obe.server.management.struct_new.service.ObeFinanceService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * @author mazc
 * @Description:
 */
@Service
@Transactional
public class ObeFinanceServiceImpl extends ServiceImpl<ObeFinanceMapper, ObeFinance> implements ObeFinanceService {

    @Autowired
    private ObeFinanceMapper obeFinanceMapper;

    /**
     * @param ObeFinanceList
     * @Description: 批量更新
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer updateByBatch(List<ObeFinance> ObeFinanceList) {
        return obeFinanceMapper.updateBatchList(ObeFinanceList);
    }

    /**
     * @param ObeFinanceList
     * @Description: 批量插入
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer insertByBatch(List<ObeFinance> ObeFinanceList) {
        return obeFinanceMapper.insertByBatch(ObeFinanceList);
    }

    /**
     * @param ObeFinance
     * @param attributes
     * @Description: 指定字段修改
     * @author chenxw
     */
    @Override
    public boolean updateAssignProperty(ObeFinance ObeFinance, String[] attributes) {
        return obeFinanceMapper.updateAssignProperty(ObeFinance, attributes) > 0;
    }

    /**
     * @param ObeFinanceList
     * @param attributes
     * @Description: 批量指定字段修改
     * @author chenxw
     */
    @Override
    public Integer batchUpdateProperty(List<ObeFinance> ObeFinanceList, String[] attributes) {
        return obeFinanceMapper.batchUpdateProperty(ObeFinanceList, attributes);
    }

    /**
     * @param ObeFinanceList
     * @param attributes
     * @return
     * @Description: 批量添加或跟新
     * @author chenxw
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer batchSaveOrUpdate(List<ObeFinance> ObeFinanceList, String[] attributes) {
        List<ObeFinance> insertObeFinanceList = new ArrayList<>();
        List<ObeFinance> updateObeFinanceList = new ArrayList<>();
        for (ObeFinance ObeFinance : ObeFinanceList) {
            if (StringUtils.isEmpty(ObeFinance.getId())) {
                ObeFinance.setId(IDUtils.getId());
                insertObeFinanceList.add(ObeFinance);
            } else {
                updateObeFinanceList.add(ObeFinance);
            }
        }
        int count = 0;
        if (insertObeFinanceList.size() > 0) {
            count += obeFinanceMapper.insertByBatch(insertObeFinanceList);
        }
        if (updateObeFinanceList.size() > 0) {
            count += obeFinanceMapper.batchUpdateProperty(updateObeFinanceList, attributes);
        }
        return count;
    }

    /**
     * @param tenderId
     * @param modelDataId
     * @return
     * @Description: 获取财务状况列表
     * @author chenxw
     */
    @Override
    public List<ObeFinance> getFinanceList(String tenderId, String modelDataId) {
        return obeFinanceMapper.getFinanceList(tenderId, modelDataId);
    }

}
