package com.gx.obe.server.management.struct_new.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.struct_new.dao.ObeQualificationMapper;
import com.gx.obe.server.management.struct_new.entity.ObeQualification;
import com.gx.obe.server.management.struct_new.service.ObeQualificationService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * @author mazc
 * @Description:
 */
@Service
@Transactional
public class ObeQualificationServiceImpl extends ServiceImpl<ObeQualificationMapper, ObeQualification> implements ObeQualificationService {

    @Autowired
    private ObeQualificationMapper obeQualificationMapper;

    /**
     * @param ObeQualificationList
     * @Description: 批量更新
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer updateByBatch(List<ObeQualification> ObeQualificationList) {
        return obeQualificationMapper.updateBatchList(ObeQualificationList);
    }

    /**
     * @param ObeQualificationList
     * @Description: 批量插入
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer insertByBatch(List<ObeQualification> ObeQualificationList) {
        return obeQualificationMapper.insertByBatch(ObeQualificationList);
    }

    /**
     * @param ObeQualification
     * @param attributes
     * @Description: 指定字段修改
     * @author chenxw
     */
    @Override
    public boolean updateAssignProperty(ObeQualification ObeQualification, String[] attributes) {
        return obeQualificationMapper.updateAssignProperty(ObeQualification, attributes) > 0;
    }

    /**
     * @param ObeQualificationList
     * @param attributes
     * @Description: 批量指定字段修改
     * @author chenxw
     */
    @Override
    public Integer batchUpdateProperty(List<ObeQualification> ObeQualificationList, String[] attributes) {
        return obeQualificationMapper.batchUpdateProperty(ObeQualificationList, attributes);
    }

    /**
     * @param ObeQualificationList
     * @param attributes
     * @return
     * @Description: 批量添加或跟新
     * @author chenxw
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer batchSaveOrUpdate(List<ObeQualification> ObeQualificationList, String[] attributes) {
        List<ObeQualification> insertObeQualificationList = new ArrayList<>();
        List<ObeQualification> updateObeQualificationList = new ArrayList<>();
        for (ObeQualification ObeQualification : ObeQualificationList) {
            if (StringUtils.isEmpty(ObeQualification.getId())) {
                ObeQualification.setId(IDUtils.getId());
                insertObeQualificationList.add(ObeQualification);
            } else {
                updateObeQualificationList.add(ObeQualification);
            }
        }
        int count = 0;
        if (insertObeQualificationList.size() > 0) {
            count += obeQualificationMapper.insertByBatch(insertObeQualificationList);
        }
        if (updateObeQualificationList.size() > 0) {
            count += obeQualificationMapper.batchUpdateProperty(updateObeQualificationList, attributes);
        }
        return count;
    }

    /**
     * @param tenderId
     * @param modelDataId
     * @return
     * @Description: 获取证书列表
     * @author chenxw
     */
    public List<ObeQualification> getQualificationList(String tenderId, String modelDataId) {
        QueryWrapper<ObeQualification> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ObeQualification::getModelDataId, modelDataId);
        return obeQualificationMapper.selectList(queryWrapper);
    }

}
