package com.gx.obe.server.management.supplier.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.gx.obe.server.common.base.BaseController;
import com.gx.obe.server.common.enumeration.CommonEnum;
import com.gx.obe.server.common.enumeration.EvaluationFactorEnum;
import com.gx.obe.server.common.enumeration.SupplierEnum;
import com.gx.obe.server.common.log.OperationType;
import com.gx.obe.server.common.log.SysLogAnnotation;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.management.supplier.entity.Supplier;
import com.gx.obe.server.management.supplier.entity.SupplierOpeningResult;
import com.gx.obe.server.management.supplier.service.SupplierOpeningResultService;
import com.gx.obe.server.management.supplier.service.SupplierService;
import com.gx.obe.server.management.vo.*;
import io.swagger.annotations.Api;
import lombok.AllArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;


/** 
 * @Description: 供应商模块
 * @author wangxiang 
 */
@Api(tags = "供应商模块")
@RestController
@AllArgsConstructor
@RequestMapping("/supplier")
public class SupplierController extends BaseController<SupplierService,Supplier>{

	@Autowired
	public SupplierService supplierService;
	
	@Autowired
	public SupplierOpeningResultService supplierOpeningResultService;
	
	@PostMapping("/batchSaveOrUpdate")
	public int batchSaveOrUpdate(@RequestBody BatchSaveOrUpdateVo<Supplier> batchSaveOrUpdateVo) {
	    List<Supplier> supplierList = batchSaveOrUpdateVo.getEntityList();
	    String[] attributes = batchSaveOrUpdateVo.getAttributes();
		return supplierService.batchSaveOrUpdate(supplierList, attributes);
	}
	
	/**
	 * @Description: 批量指定字段修改
	 * @author chenxw
	 * @param batchUpdatePropertyVo
	 * @return 
	 */
	@PostMapping("/batchUpdateProperty")
	public int batchUpdateProperty(@RequestBody BatchUpdatePropertyVo<Supplier> batchUpdatePropertyVo) {
	    List<Supplier> supplierList = batchUpdatePropertyVo.getEntityList();
	    String[] attributes = batchUpdatePropertyVo.getAttributes();
	    return supplierService.batchUpdateProperty(supplierList, attributes);
	}
	
	@PostMapping("/insertByBatch")
	public int insertByBatch(@RequestBody List<Supplier> supplierList) {
		return supplierService.insertByBatch(supplierList);
	}
	
	@PostMapping("/updateByBatch")
	public int updateByBatch(@RequestBody List<Supplier> supplierList) {
		return supplierService.updateByBatch(supplierList);
	}
	
	@PostMapping("/updateSupplier")
	@SysLogAnnotation(detail="删除供应商",level = 0,operationType = OperationType.DELETE)
	public boolean  updateSupplierInfo(@RequestBody Supplier supplier) {
		UpdateWrapper<Supplier> updateWrapper = new UpdateWrapper<Supplier>();
		updateWrapper.lambda().eq(Supplier::getId, supplier.getId());
		return supplierService.update(supplier,updateWrapper);	
	}
	
	@GetMapping("/delete")
	@SysLogAnnotation(detail="删除供应商",level = 0,operationType = OperationType.DELETE)
	public boolean delete(String tenderId,String supplierId){
		boolean  delete = supplierService.removeSupplier(tenderId,supplierId);
		return delete;
	}
	
	@GetMapping("/getSupplierList")
	public List<Supplier> getSupplierList(String tenderId){
		return supplierService.getSupplierList(tenderId,CommonEnum.REAL_STATUS);
	}
	
	@GetMapping("/getSupplierByStatusList")
	public List<Supplier> getSupplierByStatusList(String tenderId,String status){
		return supplierService.getSupplierList(tenderId,status);
	}
	
	@GetMapping("/getSupplierListWithOpeningResult")
	public List<Supplier> getSupplierOpenBidList(String tenderId){
		return supplierService.getSupplierOpenBidList(tenderId);
	}
	
	@GetMapping("/getOpeningSuccessSupplierList")
	public List<Supplier> getOpeningSuccessSupplierList(String tenderId, String bidFileType){
		return supplierService.getOpenBidSuccessSupplier(tenderId, bidFileType);
	}

	/**
	 * @Description: 按照签到顺序排序
	 * @author wangxiang
	 * @param id
	 * @param b 
	 */
	@GetMapping("/sortSupplierBySignTime")
	public boolean sortSupplierBySignTime(String tenderProjectId, boolean order) {
		return supplierService.sortSupplierBySignTime(tenderProjectId,order);
		
	}
	
	/**
	 * @Description: 批量删除供应商
	 * @author wangxiang
	 * @param supplierIdList
	 * @return 
	 */
	@GetMapping("/batchDeleteSupplierList")
	@SysLogAnnotation(detail="批量删除供应商",level = 0,operationType = OperationType.DELETE)
	public boolean batchDeleteSupplierList(@RequestParam(value="supplierIdList",required=true)List<String> supplierIdList) {
		if(null != supplierIdList && supplierIdList.size() > 0) {
			return supplierService.removeByIds(supplierIdList);
		}else {
			return false;
		}
		
	}
	@GetMapping("/deleteAllSupplierByTenderId")
	@SysLogAnnotation(detail="删除所有供应商",level = 0,operationType = OperationType.DELETE)
	public boolean deleteAllSupplierByTenderId(String tenderId) {
		return supplierService.deleteAllSupplierByTenderId(tenderId);
	}
	
	
	/**
	 * @Description: 获得所有未投标的投标人
	 * @author wangxiang
	 * @param tenderId
	 * @return 
	 */
	@GetMapping("/getAllUnTenderSupplierList")
	public List<Supplier> getAllUnTenderSupplierList(String tenderId) {
		return supplierService.getAllUnTenderSupplierList(tenderId);
	}
	
	/**
	 * @Description: 指定字段修改
	 * @author chenxw
	 * @param updateAssignPropertyVo
	 * @return 
	 */
	@PostMapping("/updateAssignProperty")
    public boolean updateAssignProperty(@RequestBody UpdateAssignPropertyVo<Supplier> updateAssignPropertyVo) {
        Supplier supplier = updateAssignPropertyVo.getEntity();
        String[] attributes = updateAssignPropertyVo.getAttributes();
        return supplierService.updateAssignProperty(supplier, attributes);
    }
	
	/**
	 * @Description: 验证供应商属性是否唯一
	 * @author wangxiang
	 * @param tenderId 项目 
	 * @param supplierId 主键
	 * @param column 数据库字段
	 * @param columnValue 字段值
	 * @return 
	 */
	@GetMapping("/isUniqueSupplier")
	public boolean isUniqueSupplier(String tenderId, String supplierId, String column, String columnValue) {
		return supplierService.isUniqueSupplier(tenderId,supplierId,column,columnValue);
	}
	
	/**
	 * @Description: 验证投标人名称是否唯一
	 * @author wangxiang
	 * @param tenderId
	 * @param supplierId
	 * @param supplierName
	 * @return 
	 */
	@GetMapping("/isUniqueSupplierByName")
	public boolean isUniqueSupplierByName(String tenderId, String supplierId, String supplierName) {
		Integer count = supplierService.isUniqueSupplierByName(tenderId,supplierId,supplierName);
		return null != count ? count <= 0 : false;
	}
	
	/**
	 * @Description: 获得指定评标步骤下的供应商列表
	 * @author wangxiang
	 * @param tenderId
	 * @param stepIndex
	 * @return 
	 */
	@PostMapping("/getEvaluationStepSupplierList")
	public List<Supplier> getEvaluationStepSupplierList(@RequestBody EvaluationSupplierVo evaluationSupplierVo) {
		 return supplierService.getEvaluationStepSupplierList(evaluationSupplierVo);
	}

	/**
	 * @Description: 获得投标人的开标结果和评标结果
	 * @author wangxiang
	 * @param tenderId
	 * @returnsupplierIdListsupplierIdList
	 */
	@GetMapping("/getSupplierListWithOpeningEvaluationResult")
	public List<Supplier> getSupplierOpenbidAndEvaluateResultList(String tenderId){
		 return supplierService.getSupplierOpenbidAndEvaluateResultList(tenderId);
	}
	
	/**
	 * @Description: 获得投标人的最大序号
	 * @author wangxiang
	 * @param tenderId
	 * @return
	 */
	@GetMapping("/getMaxSupplierSortNo")
	public int getMaxSupplierSortNo(String tenderId) {
		return supplierService.getMaxSupplierSortNo(tenderId);
	}
	
	/**
	 * @Description: 获得指定步骤评审不通过的投标人
	 * @author wangxiang
	 * @param tenderId
	 * @param stepId 指定的评标步骤
	 * @return
	 */
	@GetMapping("/getDisAccessmentSupplier")
	public List<Supplier> getDisAccessmentSupplier(String tenderId, String stepId){
		return supplierService.getDisAccessmentSupplier(tenderId,stepId);
	}
	
	/**
	 * @Description: 根据项目Id，获得项目下所有分包的投标人集合(去重复)
	 * @author wangxiang
	 * @param tenderParentId 项目Id
	 * @return List<Supplier>
	 */
	@GetMapping("/getSupplierListByParentProjectId")
	public List<Supplier> getSupplierListByTenderProject(String tenderParentId){
		return supplierService.getSupplierListByTenderProject(tenderParentId);
	}
	
	/**
	 * @Description: 模糊查询投标人名称或编码，获得项目下的投标人列表。
	 * @author wangxiang
	 * @param tenderId 项目ID
	 * @param content 查询条件(投标人名称和编码)
	 * @return List<Supplier>
	 */
	@GetMapping("/getSupplierListByName")
	public List<Supplier> getSupplierListByName(String tenderId, String content){
		return supplierService.getSupplierListByName(tenderId,content);
	}
	
	/**
	 * @Description: 根据父项目id查询所有包件下的投标人信息
	 * @author wangxiang
	 * @param parentId
	 * @return 
	 */
	@GetMapping("/getParentProjectSupplieList")
	public List<Map<String,Object>> getparentSupplierByParentTenderId(String tenderId){
		return supplierService.getparentSupplierByParentTenderId(tenderId);
	}
	
	/**
	 * @Description: 修改或者更新
	 * @author wangxiang
	 * @param supplier
	 * @return 
	 */
	@PostMapping("/saveOrUpdate")
	@SysLogAnnotation(detail="修改供应商",level = 0,operationType = OperationType.UPDATE)
	public boolean saveOrUpdate(@RequestBody Supplier supplier) {
		return supplierService.saveOrUpdate(supplier);
	}
	
	
	/**
	 * @Description: 获得报名供应商的个数
	 * @author wangxiang
	 * @param tenderId
	 * @return 
	 */
	@GetMapping("/getSupplierCount")
	public Integer getSupplierCount(String tenderId) {
		return supplierService.getSupplierCount(tenderId);
	}
	
	/**
	 * @Description: 获得未废标且已签到的供应商家数
	 * @author wangxiang
	 * @param tenderId
	 * @return 
	 */
	@GetMapping("/getSupplierSignCount")
	public Integer getSupplierSignCount(String tenderId){
		return supplierService.getSupplierSignCount(tenderId);
	}
	
	/**
	 * @Description: 获得推荐的供应商列表
	 * @author wangxiang
	 * @param tenderId
	 * @param stepIndex
	 * @return 
	 */
	@GetMapping("/getRecommendSupplierList")
	public List<Supplier> getRecommendSupplierList(String tenderId, int stepIndex){
		return supplierService.getRecommendSupplierList(tenderId,stepIndex);
	}
	
	/**
	 * @Description: 初始化供应商开标结果
	 * @author wangxiang
	 * @param tenderId 
	 */
	@GetMapping("/initSupplierBidOpeningResult")
    public void initSupplierBidOpeningResult(String tenderId) {
        QueryWrapper<Supplier> queryWrapper = new QueryWrapper<Supplier>();
        queryWrapper.lambda().select(Supplier::getId).eq(Supplier::getTenderId, tenderId);
        List<Supplier> supplierList = supplierService.list(queryWrapper);
        if (null != supplierList && supplierList.size() > 0) {
            QueryWrapper<SupplierOpeningResult> resultQeryWrapper = new QueryWrapper<SupplierOpeningResult>();
            resultQeryWrapper.lambda().select(SupplierOpeningResult::getId, SupplierOpeningResult::getTenderId, SupplierOpeningResult::getSupplierId)
                                      .eq(SupplierOpeningResult::getTenderId, tenderId);
            List<SupplierOpeningResult> resultList = supplierOpeningResultService.list(resultQeryWrapper);
            Map<String, String> supplierBidOpeningResultIdMap = new HashMap<String, String>();
            if (null != resultList && resultList.size() > 0) {
                for (SupplierOpeningResult supplierOpeningResult : resultList) {
                    supplierBidOpeningResultIdMap.put(supplierOpeningResult.getSupplierId(), supplierOpeningResult.getId());
                }
            }
            List<SupplierOpeningResult> supplierOpeningResultList = new ArrayList<SupplierOpeningResult>();
            for (Supplier supplier : supplierList) {
                String supplierId = supplier.getId();
                if (null == supplierBidOpeningResultIdMap.get(supplierId)) {
                    SupplierOpeningResult supplierOpeningResult = new SupplierOpeningResult();
                    supplierOpeningResult.setId(IDUtils.getId());
                    supplierOpeningResult.setSupplierId(supplierId);
                    supplierOpeningResult.setTenderId(tenderId);
                    supplierOpeningResultList.add(supplierOpeningResult);
                }
            }
            if (supplierOpeningResultList.size() > 0) {
                supplierOpeningResultService.insertByBatch(supplierOpeningResultList);
            }
        }
    }
	
	/**
	 * @Description: 获得按项目开标的供应商
	 * @author wangxiang
	 * @param projectId
	 * @return 
	 */
	@Deprecated
	@GetMapping("/getDecryptSupplierList")
	public List<DecryptSupplier> getDecryptSupplierList(String projectId, String tenderId, boolean onlyTenderBid){
		return supplierService.getDecryptSupplierList(projectId,tenderId,onlyTenderBid);
	}
	
	/**
	 * @Description: 通过项目id查找项目下所有包件中资审为通过供应商数量
	 * @author wangxiang
	 * @param tenderParentId
	 * @param type [01：通过；02：不通过]
	 * @return 
	 */
	@GetMapping("/getComplianceFailSupplierCount")
	public List<Supplier> getComplianceFailSupplierCount(String tenderParentId, String accessment){
		return supplierService.getComplianceFailSupplierCount(tenderParentId,accessment);
		
	}
	
	/**
	 * @Description: 获得整个项目下开标成功的投标人列表
	 * @author wangxiang
	 * @param tenderParentId
	 * @return
	 */
	@GetMapping("/getOpenBidSuccessByParentId")
	public List<Supplier> getOpenBidSuccessByParentId(String tenderParentId){
		return supplierService.getOpenBidSuccessByParentId(tenderParentId);
	}
	
	/**
	 * @Description: 获得供应商开标信息
	 * @author wangxaing
	 * @param tenderId
	 * @return 
	 */
	@GetMapping("/getSupplierBidOpeningInfoList")
	public List<Map<String, Object>> getSupplierBidOpeningInfoList(String tenderId){
		return supplierService.getSupplierBidOpeningInfoList(tenderId);
	}
	
	/**
	 * @Description: 根据包件id查询该包件下，供应商投标人数
	 * @author wangxiang
	 * @param tenderId
	 * @return 
	 */
	@GetMapping("/getBidSupplierCount")
	public Integer getBidSupplierCount(String tenderId){
		return supplierService.getBidSupplierCount(tenderId);
	}
	
	/**
	 * @Description: 根据包件id查询该包件下，供应商商务技术解密人数
	 * @author wangxiang
	 * @param tenderId
	 * @param economicsFlag
	 * @return 
	 */
	@GetMapping("/getSupplierBusinessDecryptCount")
	public Integer getSupplierBusinessDecryptCount(String tenderId, boolean priceBidStatus){
		return supplierService.getSupplierBusinessDecryptCount(tenderId,priceBidStatus);
	}
	
	/**
	 * @Description: 根据项目ID orgCode 查询
	 * @author wangxiang
	 * @param tenderId
	 * @param orgCode
	 * @return 
	 */
	@GetMapping("/getSupplierByOrgCode")
	public Supplier getSupplierByOrgCode(String tenderId, String orgCode) {
		QueryWrapper<Supplier> wrapper = new QueryWrapper<Supplier>();
		wrapper.lambda().eq(Supplier::getTenderId, tenderId)
						.eq(Supplier::getOrgCode, orgCode);
		return supplierService.getOne(wrapper);
	}
	
	@GetMapping("/queryToMap")
	public List<Map<String, Object>> queryToMap(String tenderId,boolean hasParent) {
		return supplierService.queryToMap(tenderId,hasParent);
	}
	
	/**
	 * @Description: 返回商务标开标成功数量
	 * @author wangxiang
	 * @param tenderId
	 * @return 
	 */
	@GetMapping("/queryBidOpenSuccessCount")
	public int queryBidOpenSuccessCount(String tenderId,String bidType) {
		QueryWrapper<SupplierOpeningResult> wrapper = new QueryWrapper<SupplierOpeningResult>();
		wrapper.lambda().eq(SupplierOpeningResult::getTenderId, tenderId);
		if(EvaluationFactorEnum.EVAL_BID_BUS.equals(bidType)) {
			wrapper.lambda().eq(SupplierOpeningResult::getBusinessBidStatus, SupplierEnum.OPENBID_SUCCESS);
		}else {
			wrapper.lambda().eq(SupplierOpeningResult::getPriceBidStatus, SupplierEnum.OPENBID_SUCCESS);
		}
		wrapper.select("OPENING_RESULT_ID");
		return supplierOpeningResultService.count(wrapper);
	}
	
	
	
	/**
	 * @Description: 通过轮次id获取供应商信息
	 * @author mazc
	 * @param roundId
	 * @return 
	 */
	@GetMapping("/getPrivceRoundSupplierList")
	public List<Supplier> getPrivceRoundSupplierList(String roundId) {
		return supplierService.getPrivceRoundSupplierList(roundId);
	}
	
	
	
}

/**
 * @Description: 
 * @author guoyr
 */
class SupplierComparator implements Comparator<Supplier>{

	public int compare(Supplier supplier1, Supplier supplier2) {
		if(null == supplier1.getSealedBidCode() || null == supplier2.getSealedBidCode()){
			return 0;
		}
		int comp = supplier1.getSealedBidCode().compareToIgnoreCase(supplier2.getSealedBidCode());
		if (comp < 0) {
			return -1;
		}else if(comp > 0){
			return 1;
		}
		return 0;
	}
	
	
}