package com.gx.obe.server.management.supplier.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gx.obe.server.common.base.BaseController;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.management.supplier.entity.SupplierEvaluationResult;
import com.gx.obe.server.management.supplier.service.SupplierEvaluationResultService;

import io.swagger.annotations.Api;
import lombok.AllArgsConstructor;
/** 
 * @Description: 
 * @author mazc 
 */
@Api(tags = "")
@RestController
@AllArgsConstructor
@RequestMapping("/supplierEvaluationResult")
public class SupplierEvaluationResultController extends BaseController<SupplierEvaluationResultService,SupplierEvaluationResult>{

	@Autowired
	public SupplierEvaluationResultService supplierEvaluationResultService;
	
	/**
	 * @Description: 申请编辑供应商的评审意见
	 * @author wangxiang
	 * @param applyUserId
	 * @param tenderId
	 * @param supplierId
	 * @return 
	 */
	@GetMapping("/applyEditSupplierEvaluationResult")
	public Integer applyEditSupplierEvaluationResult(String tenderId, String supplierId, String applyUserId, String opinionEditor){
		return supplierEvaluationResultService.applyEditSupplierEvaluationResult(tenderId,supplierId,applyUserId,opinionEditor);
	}
	
	/**
	 * @Description: 统计初审通过（不通过）的供应商数量
	 * @author wangxaing
	 * @param tenderId
	 * @param type
	 * @return
	 */
	@GetMapping("/getSupplierQualifiedCount")
	public Integer getSupplierQualifiedCount(String tenderId, String accessment){
		return supplierEvaluationResultService.getSupplierQualifiedCount(tenderId,accessment);
	}
	
	/**
	 * @Description: 释放编辑供应商评审意见权限
	 * @author wangxiang
	 * @param tenderId
	 * @param supplierId
	 * @param applyUserId
	 * @return 
	 */
	@GetMapping("/releaseEditSupplierEvaluationResult")
	public boolean releaseEditSupplierEvaluationResult(String tenderId, String supplierId, String applyUserId) {
		return supplierEvaluationResultService.releaseEditSupplierEvaluationResult(tenderId,supplierId,applyUserId);
	}
	
	/**
	 * @Description: 保存供应商的评审意见
	 * @author wangxiang
	 * @param tenderId
	 * @param supplierId
	 * @param biddingRequest
	 * @param evaluationOpinion
	 * @return 
	 */
	@PostMapping("/saveSupplierEvaluationOpinion")
	public boolean saveSupplierEvaluationOpinion(boolean isSubmit, String tenderId, String supplierId, String biddingRequest, String evaluationOpinion, String vetoType){
		return supplierEvaluationResultService.saveSupplierEvaluationOpinion(isSubmit,tenderId,supplierId,biddingRequest,evaluationOpinion, vetoType);
	}

	/**
	 * @Description: 修改供应商的废标状态
	 * @author wangxiang
	 * @param tenderId
	 * @param stepIndex 
	 */
	@GetMapping("/updateSupplierEvaluationAccessmentStatus")
	public boolean updateSupplierEvaluationAccessmentStatus(String tenderId, int stepIndex){
		return supplierEvaluationResultService.updateSupplierEvaluationAccessmentStatus(tenderId,stepIndex);
	}
	
	
	
	@PostMapping("/saveOrUpdate")
	public boolean saveOrUpdate(@RequestBody SupplierEvaluationResult supplierEvaluationResult){
		if(null == supplierEvaluationResult) {
			return false;
		}
		supplierEvaluationResult.setId(IDUtils.getID(supplierEvaluationResult.getId()));
		return supplierEvaluationResultService.saveOrUpdate(supplierEvaluationResult);
	}
	/**
	 * @Description: id批量更新
	 * @author mazc
 	 * @param attributes
	 * @return 
	 */
	@PostMapping("/batchUpdateProperty")
	public int batchUpdateProperty(@RequestBody List<SupplierEvaluationResult> supplierEvaluationResultList){ 
		return supplierEvaluationResultService.batchUpdateProperty(supplierEvaluationResultList);
	}
	
	@GetMapping("/getSupplierEvaluationResultByTenderIdAndSupplierId")
	public SupplierEvaluationResult getSupplierEvaluationResultByTenderIdAndSupplierId(String tenderId,String supplierId){ 
		return supplierEvaluationResultService.getSupplierEvaluationResultByTenderIdAndSupplierId(tenderId, supplierId);
	}
	
	@GetMapping("/getSupplierEvaluationResultBySupplierId")
	public SupplierEvaluationResult getSupplierEvaluationResultBySupplierId( String supplierId){ 
		return supplierEvaluationResultService.getSupplierEvaluationResultBySupplierId(supplierId);
	}
	
	@GetMapping("/getSupplierEvaluationResultListByTenderId")
	public List<SupplierEvaluationResult> getSupplierEvaluationResultListByTenderId( String tenderId){
		return supplierEvaluationResultService.getSupplierEvaluationResultListByTenderId(tenderId);
	}
	
	@GetMapping("/getSupplierEvaluationResultListByTenderIdAndAccessment")
	public List<SupplierEvaluationResult> getSupplierEvaluationResultListByTenderIdAndAccessment( String tenderId ,String accessment){
		return supplierEvaluationResultService.getSupplierEvaluationResultListByTenderIdAndAccessment(tenderId, accessment);
	}
	
	@GetMapping("/getSupplierEvaluationResultListTenderIdAndeSortNo")
	public List<SupplierEvaluationResult> getSupplierEvaluationResultListTenderIdAndeSortNo( String tenderId ,int sortNo){
		return supplierEvaluationResultService.getSupplierEvaluationResultListTenderIdAndeSortNo(tenderId, sortNo);
	}
	
}