package com.gx.obe.server.management.supplier.service.impl;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.entity.ReflecUtils;
import com.gx.obe.server.common.enumeration.CommonEnum;
import com.gx.obe.server.common.enumeration.EvaluationStepEnum;
import com.gx.obe.server.common.utils.CollectionUtils;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.management.evaluation.entity.EvaluationStep;
import com.gx.obe.server.management.supplier.dao.SupplierEvaluationResultMapper;
import com.gx.obe.server.management.supplier.entity.SupplierEvaluationResult;
import com.gx.obe.server.management.supplier.service.SupplierEvaluationResultService;

/** 
 * @Description: 
 * @author mazc 
 */
@Service
@Transactional
public class SupplierEvaluationResultServiceImpl extends ServiceImpl<SupplierEvaluationResultMapper, SupplierEvaluationResult> implements SupplierEvaluationResultService {

	@Autowired
	private SupplierEvaluationResultMapper supplierEvaluationResultMapper;

	/** 
	 * @Description: 获得评标结果
	 * @author mazc
	 * @param tenderId
	 * @param supplierId
	 * @return 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public SupplierEvaluationResult getOrCreateSupplierEvaluationResult(String tenderId, String supplierId) {
		SupplierEvaluationResult supplierEvaluationResult = new SupplierEvaluationResult();
		String resultId = supplierEvaluationResultMapper.getCreateSupplierEvaluationResultId(tenderId, supplierId);
		if(null == resultId) {
			resultId = IDUtils.getId();
			supplierEvaluationResultMapper.createSupplierEvaluationResultId(resultId,tenderId, supplierId);
		}
		supplierEvaluationResult.setId(resultId);
		supplierEvaluationResult.setTenderId(tenderId);
		supplierEvaluationResult.setSupplierId(supplierId);
		return supplierEvaluationResult;
	}

	/** 
	 * @Description: 批量更新或插入
	 * @author mazc
	 * @param editPriceItemResultList
	 * @param attributes
	 * @return 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public Integer batchSaveOrUpdate(List<SupplierEvaluationResult> editPriceItemResultList, String[] attributes) {
		int iCount = 0;
		List<SupplierEvaluationResult> instrtEditPriceItemResultListTemp = new ArrayList<SupplierEvaluationResult>();
		List<SupplierEvaluationResult> UpdateeditPriceItemResultListTemp = new ArrayList<SupplierEvaluationResult>();
		for(SupplierEvaluationResult supplierEvaluationResult : editPriceItemResultList) {
			SupplierEvaluationResult editPriceItemResultTemp = ReflecUtils.reflex(supplierEvaluationResult, attributes ,SupplierEvaluationResult.class);
			if(null != editPriceItemResultTemp) {
				if(null == editPriceItemResultTemp.getId()) {
					editPriceItemResultTemp.setId(IDUtils.getId());
					instrtEditPriceItemResultListTemp.add(editPriceItemResultTemp);
				}else {
					UpdateeditPriceItemResultListTemp.add(editPriceItemResultTemp);					
				}
			}
		}
		iCount += insertByBatch(instrtEditPriceItemResultListTemp);
		iCount += supplierEvaluationResultMapper.updateObeSupplierEvaluationResultBatchList(UpdateeditPriceItemResultListTemp);
		supplierEvaluationResultMapper.updateUnqObeSupplierEvaluationResultBatchList(UpdateeditPriceItemResultListTemp);
		return iCount;
	}

	private int insertByBatch(List<SupplierEvaluationResult> instrtEditPriceItemResultListTemp) {
	    if (CollectionUtils.isNull(instrtEditPriceItemResultListTemp)) {
            return 0;
        }
        return supplierEvaluationResultMapper.insertByBatch(instrtEditPriceItemResultListTemp);
    }

    /**
	 * @Description: 申请编辑供应商的评审意见
	 * @author wangxiang
	 * @param applyUserId
	 * @param tenderId
	 * @param supplierId
	 * @return 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public Integer applyEditSupplierEvaluationResult(String tenderId, String supplierId, String applyUserId, String opinionEditor) {
		Integer applay  = null;
		applay = supplierEvaluationResultMapper.getApply(tenderId,supplierId,applyUserId,opinionEditor);
		if(applay == 0) {
			Integer count = supplierEvaluationResultMapper.getOpinionCount(tenderId,supplierId);
			if(null == count || count == 0) {
				Integer insertCount = supplierEvaluationResultMapper.insertSupplierOpinion(IDUtils.getId(),tenderId,supplierId,applyUserId,opinionEditor);
				if(insertCount > 0) {
					applay = 1;
				}
			}
		}else {
			applay = 1;
		}
		
		return applay;
	}

	/**
	 * @Description: 统计初审通过（不通过）的供应商数量
	 * @author wangxaing
	 * @param tenderId
	 * @return
	 */
	@Override
	public Integer getSupplierQualifiedCount(String tenderId, String accessment) {
		return supplierEvaluationResultMapper.getSupplierQualifiedCount(tenderId,accessment);
	}

	/**
	 * @Description: 释放编辑供应商评审意见权限
	 * @author wangxiang
	 * @param tenderId
	 * @param supplierId
	 * @param applyUserId
	 * @return 
	 */
	@Override
	public boolean releaseEditSupplierEvaluationResult(String tenderId, String supplierId, String applyUserId) {
		return supplierEvaluationResultMapper.releaseEditSupplierEvaluationResult(tenderId,supplierId,applyUserId) == 1;
	}

	/**
	 * @Description: 保存供应商的评审意见
	 * @author wangxiang
	 * @param tenderId
	 * @param supplierId
	 * @param biddingRequest
	 * @param evaluationOpinion
	 * @return 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public boolean saveSupplierEvaluationOpinion(boolean isSubmit, String tenderId, String supplierId, String biddingRequest, String evaluationOpinion, String vetoType) {
		SupplierEvaluationResult supplierEvaluationResult = getOrCreateSupplierEvaluationResult(tenderId,supplierId);
		int count = 0;
		UpdateWrapper<SupplierEvaluationResult> updateWrapper = new UpdateWrapper<>();
		if(isSubmit){
			updateWrapper.set(SupplierEvaluationResult.APPLY_USER_ID,null)
				.set(SupplierEvaluationResult.BIDDING_REQUEST, biddingRequest)
				.set(SupplierEvaluationResult.EVALUATION_OPINION, evaluationOpinion)
			.set(SupplierEvaluationResult.VETO_TYPE, vetoType);
		}else {
			updateWrapper.set(SupplierEvaluationResult.BIDDING_REQUEST, biddingRequest)
				.set(SupplierEvaluationResult.EVALUATION_OPINION, evaluationOpinion)
				.set(SupplierEvaluationResult.VETO_TYPE, vetoType);
		}
		
		updateWrapper.lambda().eq(SupplierEvaluationResult::getId, supplierEvaluationResult.getId());
		count +=  supplierEvaluationResultMapper.update(supplierEvaluationResult, updateWrapper); 
		return count == 1;
		
	}

	/**
	 * @Description: 修改供应商的废标状态
	 * @author wangxiang
	 * @param tenderId
	 * @param stepIndex 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public boolean updateSupplierEvaluationAccessmentStatus(String tenderId, int stepIndex) {
		return supplierEvaluationResultMapper.updateSupplierEvaluationAccessmentStatus(tenderId,stepIndex,EvaluationStepEnum.UNQUALIFIED,CommonEnum.COMPLETE) == 1;
	}

	/** 
	 * @Description: 根据ID更新指定的字段
	 * @author wangxiang
	 * @param supplierEvaluationResult
	 * @param strings
	 * @return 
	 */
	@Override
	public boolean updateAssignProperty(SupplierEvaluationResult supplierEvaluationResult, String[] strings) {
		SupplierEvaluationResult evaluationStepTemp = ReflecUtils.reflex(supplierEvaluationResult, strings,SupplierEvaluationResult.class);
		return supplierEvaluationResultMapper.updateById(evaluationStepTemp) > 0;
	}
	
	/** 
	 * @Description: 清空供应商评审结果
	 * @author chenxw
	 * @param evaluationStep
	 * @return 
	 */
    public boolean clearSupplierEvaluationResult(EvaluationStep evaluationStep) {
        UpdateWrapper<SupplierEvaluationResult> updateWrapper = new UpdateWrapper<SupplierEvaluationResult>();
        updateWrapper.lambda().set(SupplierEvaluationResult::getSortNo, null)
                              .set(SupplierEvaluationResult::getWinBidSortNo, null)
                              .set(SupplierEvaluationResult::getRecommend, null)
                              .set(SupplierEvaluationResult::getEvaluationScore, null)
                              .set(SupplierEvaluationResult::getEvaluationFinalScore, null);
        if (EvaluationStepEnum.PRICE.equals(evaluationStep.getStepType())) {
            updateWrapper.lambda().set(SupplierEvaluationResult::getPriceScore, null)
                                  .set(SupplierEvaluationResult::getPriceFinalScore, null);
        }
        updateWrapper.lambda().eq(SupplierEvaluationResult::getTenderId, evaluationStep.getTenderId());
        return update(updateWrapper);
    }

	@Override
	public int batchUpdateProperty(List<SupplierEvaluationResult> supplierEvaluationResultList) {
		return supplierEvaluationResultMapper.updateObeSupplierEvaluationResultBatchList(supplierEvaluationResultList);
	}

	@Override
	public SupplierEvaluationResult getSupplierEvaluationResultByTenderIdAndSupplierId(String tenderId,String supplierId) {
		QueryWrapper<SupplierEvaluationResult>  query = new QueryWrapper<SupplierEvaluationResult>();
		query.lambda().eq(SupplierEvaluationResult::getTenderId, tenderId).eq(SupplierEvaluationResult::getSupplierId, supplierId);
		return supplierEvaluationResultMapper.selectOne(query);
	}

	@Override
	public SupplierEvaluationResult getSupplierEvaluationResultBySupplierId(String supplierId) {
		QueryWrapper<SupplierEvaluationResult>  query = new QueryWrapper<SupplierEvaluationResult>();
		query.lambda().eq(SupplierEvaluationResult::getSupplierId, supplierId);
		return supplierEvaluationResultMapper.selectOne(query);
	}

	@Override
	public List<SupplierEvaluationResult> getSupplierEvaluationResultListByTenderId(String tenderId) {
		QueryWrapper<SupplierEvaluationResult>  query = new QueryWrapper<SupplierEvaluationResult>();
		query.lambda().eq(SupplierEvaluationResult::getTenderId, tenderId);
		return supplierEvaluationResultMapper.selectList(query);
 	}

	@Override
	public List<SupplierEvaluationResult> getSupplierEvaluationResultListByTenderIdAndAccessment(String tenderId,
			String accessment) {
		QueryWrapper<SupplierEvaluationResult>  query = new QueryWrapper<SupplierEvaluationResult>();
		query.lambda().eq(SupplierEvaluationResult::getTenderId, tenderId).eq(SupplierEvaluationResult::getAccessment, accessment);
		return supplierEvaluationResultMapper.selectList(query);
	}

	@Override
	public List<SupplierEvaluationResult> getSupplierEvaluationResultListTenderIdAndeSortNo(String tenderId,
			int sortNo) {
		QueryWrapper<SupplierEvaluationResult>  query = new QueryWrapper<SupplierEvaluationResult>();
		query.lambda().eq(SupplierEvaluationResult::getTenderId, tenderId).eq(SupplierEvaluationResult::getSortNo, sortNo);
		return supplierEvaluationResultMapper.selectList(query);
	}
	
}
