package com.gx.obe.server.management.table.controller;

import com.gx.obe.server.common.base.BaseController;
import com.gx.obe.server.common.vo.RequestBodyEntity;
import com.gx.obe.server.management.table.entity.ObeBidPriceTable;
import com.gx.obe.server.management.table.service.ObeBidPriceTableService;
import com.gx.obe.server.management.vo.BidPriceTableDateInfo;
import io.swagger.annotations.Api;
import lombok.AllArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author mazc
 * @Description:
 */
@Api(tags = "")
@RestController
@AllArgsConstructor
@RequestMapping("/obeBidPriceTable")
public class ObeBidPriceTableController extends BaseController<ObeBidPriceTableService, ObeBidPriceTable> {

    @Autowired
    public ObeBidPriceTableService ObeBidPriceTableServices;

    /**
     * @param ObeBidPriceTableList
     * @Description: 批量插入
     * @author mazc
     */
    @PostMapping("/insertByBatch")
    public int insertByBatch(@RequestBody List<ObeBidPriceTable> ObeBidPriceTableList) {
        return ObeBidPriceTableServices.insertByBatch(ObeBidPriceTableList);
    }

    /**
     * @param ObeBidPriceTableList
     * @Description: 批量更新
     * @author mazc
     */
    @PostMapping("/updateByBatch")
    public int updateByBatch(@RequestBody List<ObeBidPriceTable> ObeBidPriceTableList) {
        return ObeBidPriceTableServices.updateByBatch(ObeBidPriceTableList);
    }

    /**
     * @param requestBodyEntity
     * @Description: 清空并保存投标报价明细表信息数据
     * @author mazc
     */
    @PostMapping("/deleteOrSaveBidPriceTableDateInfo")
    public boolean deleteOrSaveBidPriceTableDateInfo(@RequestBody RequestBodyEntity<BidPriceTableDateInfo> requestBodyEntity) {
        String tenderId = requestBodyEntity.getParamsString("tenderId", null);
        String supplierId = requestBodyEntity.getParamsString("supplierId", null);
        BidPriceTableDateInfo bidPriceTableDateInfo = requestBodyEntity.getEntity();
        return ObeBidPriceTableServices.deleteOrSaveBidPriceTableDateInfo(tenderId, supplierId, bidPriceTableDateInfo);
    }

    /**
     * @param supplierId
     * @return
     * @Description: 根据供应商id获取投标报价表
     * @author chenxw
     */
    @GetMapping("/getBidPriceTableInfoBySupplierId")
    public List<ObeBidPriceTable> getBidPriceTableInfoBySupplierId(String tenderId, String supplierId) {
        return ObeBidPriceTableServices.getBidPriceTableInfoBySupplierId(tenderId, supplierId);
    }

    /**
     * @Description: 根据供应商id获取投标报价明细表
     * @author chenxw
     * @param supplierId
     * @return
     */
    @GetMapping("/getBidPriceTableDetailBySupplierId")
    public ObeBidPriceTable getBidPriceTableDetailBySupplierId(String tenderId, String supplierId) {
        return ObeBidPriceTableServices.getBidPriceTableDetailBySupplierId(tenderId, supplierId);
    }

    /**
     * @param id
     * @return
     * @Description: 根据id获取投标报价表
     * @author chenxw
     */
    @GetMapping("/getBidPriceTableInfoById")
    public ObeBidPriceTable getBidPriceTableInfoById(String id) {
        return ObeBidPriceTableServices.getBidPriceTableInfoById(id);
    }

}