package com.gx.obe.server.management.table.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.utils.CollectionUtils;
import com.gx.obe.server.common.utils.ExceptionUtil;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.table.dao.*;
import com.gx.obe.server.management.table.entity.*;
import com.gx.obe.server.management.table.service.*;
import com.gx.obe.server.management.vo.BidPriceTableDateInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

/**
 * @author mazc
 * @Description:
 */
@Service
@Transactional
public class ObeBidPriceTableServiceImpl extends ServiceImpl<ObeBidPriceTableMapper, ObeBidPriceTable> implements ObeBidPriceTableService {

    @Autowired
    private ObeBidPriceTableMapper obeBidPriceTableMapper;

    @Autowired
    private ObeBidPriceTableTitleMapper obeBidPriceTableTitleMapper;

    @Autowired
    private ObeBidPriceTableItemMapper obeBidPriceTableItemMapper;

    @Autowired
    private ObeBidPriceTableValueMapper obeBidPriceTableValueMapper;

    @Autowired
    private ObeBidPriceTableItemCollectMapper obeBidPriceTableItemCollectMapper;

    @Autowired
    private ObeBidPriceTableTitleService obeBidPriceTableTitleService;

    @Autowired
    private ObeBidPriceTableItemService obeBidPriceTableItemService;

    @Autowired
    private ObeBidPriceTableValueService obeBidPriceTableValueService;

    @Autowired
    private ObeBidPriceTableItemCollectService obeBidPriceTableItemCollectService;

    /**
     * @param ObeBidPriceTableList
     * @Description: 批量更新
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer updateByBatch(List<ObeBidPriceTable> ObeBidPriceTableList) {
        return obeBidPriceTableMapper.updateBatchList(ObeBidPriceTableList);
    }

    /**
     * @param ObeBidPriceTableList
     * @Description: 批量插入
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer insertByBatch(List<ObeBidPriceTable> ObeBidPriceTableList) {
        return obeBidPriceTableMapper.insertByBatch(ObeBidPriceTableList);
    }

    /**
     * @param ObeBidPriceTable
     * @param attributes
     * @Description: 指定字段修改
     * @author chenxw
     */
    @Override
    public boolean updateAssignProperty(ObeBidPriceTable ObeBidPriceTable, String[] attributes) {
        return obeBidPriceTableMapper.updateAssignProperty(ObeBidPriceTable, attributes) > 0;
    }

    /**
     * @param ObeBidPriceTableList
     * @param attributes
     * @Description: 批量指定字段修改
     * @author chenxw
     */
    @Override
    public Integer batchUpdateProperty(List<ObeBidPriceTable> ObeBidPriceTableList, String[] attributes) {
        return obeBidPriceTableMapper.batchUpdateProperty(ObeBidPriceTableList, attributes);
    }

    /**
     * @param ObeBidPriceTableList
     * @param attributes
     * @return
     * @Description: 批量添加或跟新
     * @author chenxw
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer batchSaveOrUpdate(List<ObeBidPriceTable> ObeBidPriceTableList, String[] attributes) {
        List<ObeBidPriceTable> insertObeBidPriceTableList = new ArrayList<>();
        List<ObeBidPriceTable> updateObeBidPriceTableList = new ArrayList<>();
        for (ObeBidPriceTable ObeBidPriceTable : ObeBidPriceTableList) {
            if (StringUtils.isEmpty(ObeBidPriceTable.getId())) {
                ObeBidPriceTable.setId(IDUtils.getId());
                insertObeBidPriceTableList.add(ObeBidPriceTable);
            } else {
                updateObeBidPriceTableList.add(ObeBidPriceTable);
            }
        }
        int count = 0;
        if (insertObeBidPriceTableList.size() > 0) {
            count += obeBidPriceTableMapper.insertByBatch(insertObeBidPriceTableList);
        }
        if (updateObeBidPriceTableList.size() > 0) {
            count += obeBidPriceTableMapper.batchUpdateProperty(updateObeBidPriceTableList, attributes);
        }
        return count;
    }

    /**
     * @param tenderId
     * @param bidPriceTableDateInfo
     * @return
     * @Description: 清空并保存投标报价明细表信息数据
     * @author chenxw
     */
    @Transactional(rollbackFor = Exception.class)
    public synchronized boolean deleteOrSaveBidPriceTableDateInfo(String tenderId, String supplierId, BidPriceTableDateInfo bidPriceTableDateInfo) {
        QueryWrapper<ObeBidPriceTable> obeBidPriceTableQueryWrapper = new QueryWrapper<>();
        obeBidPriceTableQueryWrapper.lambda()
                .eq(ObeBidPriceTable::getTenderId, tenderId)
                .eq(ObeBidPriceTable::getSupplierId, supplierId);
        obeBidPriceTableMapper.delete(obeBidPriceTableQueryWrapper);

        QueryWrapper<ObeBidPriceTableTitle> obeBidPriceTableTitleQueryWrapper = new QueryWrapper<>();
        obeBidPriceTableTitleQueryWrapper.lambda()
                .eq(ObeBidPriceTableTitle::getTenderId, tenderId)
                .eq(ObeBidPriceTableTitle::getSupplierId, supplierId);
        obeBidPriceTableTitleMapper.delete(obeBidPriceTableTitleQueryWrapper);

        QueryWrapper<ObeBidPriceTableItem> obeBidPriceTableItemQueryWrapper = new QueryWrapper<>();
        obeBidPriceTableItemQueryWrapper.lambda()
                .eq(ObeBidPriceTableItem::getTenderId, tenderId)
                .eq(ObeBidPriceTableItem::getSupplierId, supplierId);
        obeBidPriceTableItemMapper.delete(obeBidPriceTableItemQueryWrapper);

        QueryWrapper<ObeBidPriceTableValue> obeBidPriceTableValueQueryWrapper = new QueryWrapper<>();
        obeBidPriceTableValueQueryWrapper.lambda()
                .eq(ObeBidPriceTableValue::getTenderId, tenderId)
                .eq(ObeBidPriceTableValue::getSupplierId, supplierId);
        obeBidPriceTableValueMapper.delete(obeBidPriceTableValueQueryWrapper);

        QueryWrapper<ObeBidPriceTableItemCollect> obeBidPriceTableItemCollectQueryWrapper = new QueryWrapper<>();
        obeBidPriceTableItemCollectQueryWrapper.lambda()
                .eq(ObeBidPriceTableItemCollect::getTenderId, tenderId)
                .eq(ObeBidPriceTableItemCollect::getSupplierId, supplierId);
        obeBidPriceTableItemCollectMapper.delete(obeBidPriceTableItemCollectQueryWrapper);

        List<ObeBidPriceTable> obeBidPriceTables = bidPriceTableDateInfo.getBidPriceTableList();
        if (CollectionUtils.isNotNull(obeBidPriceTables))
            ExceptionUtil.equal(obeBidPriceTableMapper.insertByBatch(obeBidPriceTables), obeBidPriceTables.size());

        List<ObeBidPriceTableTitle> obeBidPriceTableTitles = bidPriceTableDateInfo.getBidPriceTableTitleList();
        if (CollectionUtils.isNotNull(obeBidPriceTableTitles))
            ExceptionUtil.equal(obeBidPriceTableTitleMapper.insertByBatch(obeBidPriceTableTitles), obeBidPriceTableTitles.size());

        List<ObeBidPriceTableItem> obeBidPriceTableItems = bidPriceTableDateInfo.getBidPriceTableItemList();
        if (CollectionUtils.isNotNull(obeBidPriceTableItems))
            ExceptionUtil.equal(obeBidPriceTableItemMapper.insertByBatch(obeBidPriceTableItems), obeBidPriceTableItems.size());

        List<ObeBidPriceTableValue> obeBidPriceTableValues = bidPriceTableDateInfo.getBidPriceTableValueList();
        if (CollectionUtils.isNotNull(obeBidPriceTableValues))
            ExceptionUtil.equal(obeBidPriceTableValueMapper.insertByBatch(obeBidPriceTableValues), obeBidPriceTableValues.size());

        List<ObeBidPriceTableItemCollect> obeBidPriceTableItemCollects = bidPriceTableDateInfo.getBidPriceTableItemCollectList();
        if (CollectionUtils.isNotNull(obeBidPriceTableItemCollects))
            ExceptionUtil.equal(obeBidPriceTableItemCollectMapper.insertByBatch(obeBidPriceTableItemCollects), obeBidPriceTableItemCollects.size());

        return true;
    }

    /**
     * @param supplierId
     * @return
     * @Description: 根据供应商id获取投标报价表
     * @author chenxw
     */
    @Override
    public List<ObeBidPriceTable> getBidPriceTableInfoBySupplierId(String tenderId, String supplierId) {
        QueryWrapper<ObeBidPriceTable> obeBidPriceTableQueryWrapper = new QueryWrapper<>();
        obeBidPriceTableQueryWrapper.lambda()
                .eq(ObeBidPriceTable::getTenderId, tenderId)
                .eq(ObeBidPriceTable::getSupplierId, supplierId)
                .eq(ObeBidPriceTable::getBusinessType, "supplier");
        List<ObeBidPriceTable> bidPriceTableList = obeBidPriceTableMapper.selectList(obeBidPriceTableQueryWrapper);
        Optional.ofNullable(bidPriceTableList).map(List::stream).ifPresent(s -> s.forEach(this::setBidPriceTableInfo));
        return bidPriceTableList;
    }

    /**
     * @Description: 根据供应商id获取投标报价明细表
     * @author chenxw
     * @param supplierId
     * @return
     */
    @Override
    public ObeBidPriceTable getBidPriceTableDetailBySupplierId(String tenderId, String supplierId) {
        QueryWrapper<ObeBidPriceTable> obeBidPriceTableQueryWrapper = new QueryWrapper<>();
        obeBidPriceTableQueryWrapper.lambda()
                .eq(ObeBidPriceTable::getTenderId, tenderId)
                .eq(ObeBidPriceTable::getSupplierId, supplierId)
                .eq(ObeBidPriceTable::getBusinessType, "supplier")
                .eq(ObeBidPriceTable::getTableType, "detail");
        ObeBidPriceTable bidPriceTable = obeBidPriceTableMapper.selectOne(obeBidPriceTableQueryWrapper);
        Optional.ofNullable(bidPriceTable).ifPresent(this::setBidPriceTableInfo);
        return bidPriceTable;
    }

    /**
     * @param id
     * @return
     * @Description: 根据id获取投标报价表
     * @author chenxw
     */
    @Override
    public ObeBidPriceTable getBidPriceTableInfoById(String id) {
        ObeBidPriceTable bidPriceTable = getById(id);
        Optional.ofNullable(bidPriceTable).ifPresent(this::setBidPriceTableInfo);
        return bidPriceTable;
    }

    /**
     * @param t
     * @return
     * @Description: 根据id获取投标报价表
     * @author chenxw
     */
    private void setBidPriceTableInfo(ObeBidPriceTable t) {
        String tableType = t.getTableType();
        if ("detail".equals(tableType)) {
            t.setBidPriceTableItemList(obeBidPriceTableItemService.getByPriceTableId(t.getId()));
            t.setBidPriceTableTitleList(obeBidPriceTableTitleService.getByPriceTableId(t.getId()));
            t.setBidPriceTableValueList(obeBidPriceTableValueService.getByPriceTableId(t.getId()));
        }
        if ("collect".equals(tableType)) {
            t.setBidPriceTableItemCollectList(obeBidPriceTableItemCollectService.getByPriceTableId(t.getId()));
        }
    }

}
