package com.gx.obe.server.management.user.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.gx.obe.server.common.base.BaseController;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.user.entity.AuthRoleEntity;
import com.gx.obe.server.management.user.service.AuthRoleService;

import io.swagger.annotations.Api;
import lombok.AllArgsConstructor;

 
 /**  
 * 
 * @author 
 * @Title: AuthRoleController  
 * @Package com.gx.obe.server.management.user.controllers   
 * @Description:   
 * @author: XiaoMa Pro - myzhichao@163.com
 * @date:   2019-04-29   
 * @version V1.0 
 * @Copyright: 2019 www.msypro.com Inc. All rights reserved. 
 *  
 */
@Api(tags = "角色管理")
@RestController
@AllArgsConstructor
@RequestMapping("/authRole")
public class AuthRoleController  extends BaseController<AuthRoleService,AuthRoleEntity>{
	@Autowired
	private AuthRoleService authRoleService;
	
	@GetMapping("/getRoleListByRoleCode")
	public List<AuthRoleEntity> getRoleListByRoleCode(String[] ROLE_CODE){
		QueryWrapper<AuthRoleEntity> queryWrapper = new QueryWrapper<AuthRoleEntity>();
		queryWrapper.lambda().in(AuthRoleEntity::getRoleCode, ROLE_CODE);
		return authRoleService.list(queryWrapper);
	}
	
	/**
	 * @Description: 保存或更新
	 * @author mazc
	 * @return 
	 */
	@PostMapping("/saveOrUpdate")
	public AuthRoleEntity saveOrUpdate(@RequestBody AuthRoleEntity authRoleEntity){
		if(StringUtils.isEmpty(authRoleEntity.getId())) {
			authRoleEntity.setId(IDUtils.getId()); 
		}
		if(authRoleService.saveOrUpdate(authRoleEntity)) {
			return authRoleService.getById(authRoleEntity.getId());
		}
		return null;
	}
	
	/**
	 * @Description: 根据角色类型获取角色列表 
	 * @author mazc
	 * @param roleType
	 * @return 
	 */
	@GetMapping("/getRoleListByRoleType")
	public List<AuthRoleEntity> getRoleListByRoleType(String roleType){
		return authRoleService.getRoleListByRoleType(roleType);
	}
	
	/**
	 * @Description: 根据加色名字查找角色列表
	 * @author mazc
	 * @return 
	 */
	@GetMapping("/getQueryRoleListByRoleName")
	public List<AuthRoleEntity> getQueryRoleListByRoleName(String roleName){
		return authRoleService.getQueryRoleListByRoleName(roleName);
	}
	
	
	/**
	 * @Description: 根据角色状态获取列表
	 * @author mazc
	 * @param useStatus
	 * @return 
	 */
	@GetMapping("/getRoleListByuseStatus")
	public List<AuthRoleEntity> getRoleListByuseStatus(String useStatus){
		return authRoleService.getRoleListByuseStatus(useStatus);
	}
	
	/**
	 * @Description: 更新状态
	 * @author mazc
	 * @param useStatus
	 * @return 
	 */
	@GetMapping("/updateRoleuseStatus")
	public boolean updateRoleuseStatus(String useStatus,String roleId) {
		return authRoleService.updateRoleuseStatus(useStatus, roleId);
	}
}