package com.gx.obe.server.tripartitefunction.bidata.service.impl;

import java.io.File;
import java.io.IOException;
import java.nio.file.FileSystem;
import java.nio.file.FileSystems;
import java.nio.file.FileVisitResult;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.SimpleFileVisitor;
import java.nio.file.attribute.BasicFileAttributes;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Random;
import java.util.concurrent.TimeUnit;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import com.gx.obe.http.HttpResponse;
import com.gx.obe.http.HttpUtil;
import com.gx.obe.http.json.JsonUtil;
import com.gx.obe.server.common.redis.RedisConstants;
import com.gx.obe.server.common.redis.RedisUtil;
import com.gx.obe.server.common.sftp.SFTPClientUtils;
import com.gx.obe.server.common.sftp.SftpConfiguration;
import com.gx.obe.server.common.vo.Result;
import com.gx.obe.server.im.util.MD5Utils;
import com.gx.obe.server.tripartitefunction.bidata.BiDataConstants;
import com.gx.obe.server.tripartitefunction.bidata.SupplierDTO;
import com.gx.obe.server.tripartitefunction.bidata.service.BidataService;
import com.gx.obe.server.tripartitefunction.bidata.vo.BidataMutiNodesOrder;
import com.gx.obe.server.tripartitefunction.bidata.vo.BidataMutiNodesOrderStatus;
import com.gx.obe.server.tripartitefunction.bidata.vo.BidataMutiNodesOrderStatusRequest;
import com.gx.obe.server.tripartitefunction.bidata.vo.BidataMutiNodesRequest;
import com.gx.obe.server.tripartitefunction.bidata.vo.BidataRequest;
import com.gx.obe.server.tripartitefunction.bidata.vo.EntInfo;
import com.gx.obe.server.tripartitefunction.bidata.vo.MutiNodesBody;
import com.gx.obe.server.tripartitefunction.bidata.vo.SupplierInofVo;

import io.netty.util.internal.StringUtil;

@Service
public class BidataServiceImpl implements BidataService {
 	private static Logger LOGGER = LoggerFactory.getLogger(SFTPClientUtils.class);

	@Autowired
	private RedisUtil redisUtil;
	
	 @Value("${upload.temp}")
	 private String tempFile;
	 
 
	 private Object entInfoLock = new Object();
	 private Object mutiNodesLock = new Object();
	 private Object orderStatusLock = new Object();
	 private Object fileContentLock = new Object();
	 
	
	/** 
	 * @Description: 获取供应商的企业信息以及风险
	 * @author mazc
	 * @param mainId
	 * @param id
	 * @param creditcode
	 * @param regno
	 * @param name
	 * @param orgcode
	 * @return 
	 */
	@Override
	public Result<EntInfo> getAllBillTableQDList(String mainId, String id, String creditcode, String regno, String name,String orgcode) {
		Result<EntInfo> result = Result.ok();
		EntInfo entInfo = redisUtil.getCacheObject(RedisConstants.REDIS_BIDATA_ENTINFO + mainId);
		if (entInfo == null) {
			synchronized (entInfoLock) {
				entInfo = redisUtil.getCacheObject(RedisConstants.REDIS_BIDATA_ENTINFO + mainId);
				if(entInfo == null) {
					BidataRequest bidataRequest = getEntInfo(id, creditcode, regno, name, orgcode);
					result.setMsg(bidataRequest.getMSG());
					entInfo = bidataRequest.getENT_INFO();
					if (entInfo != null) {
						redisUtil.setCacheObject(RedisConstants.REDIS_BIDATA_ENTINFO + mainId, entInfo,BiDataConstants.REDIS_TIMEOUT,TimeUnit.DAYS);
					}		
				}
			}
		}
		result.setData(entInfo);
		return result;
	}
	

	/** 
	 * @Description: 获取企业汇总信息
	 * @author mazc
	 * @param mainIdList
	 * @return 
	 */
	@Override
	public synchronized Result<List<SupplierInofVo>> getSupplierlistInfor(List<SupplierDTO> supplierDTOList) {
		Result<List<SupplierInofVo>> result = Result.ok();
		List<SupplierInofVo> list = new ArrayList<SupplierInofVo>();
		for(SupplierDTO supplierDTO : supplierDTOList) {
			String mainId = supplierDTO.getMainId();
			if(!StringUtil.isNullOrEmpty(mainId)) mainId = MD5Utils.getMD5String(supplierDTO.getSupplierName());
			SupplierInofVo  supplierInfo = new SupplierInofVo(mainId , supplierDTO.getSupplierName());
			EntInfo entInfo = redisUtil.getCacheObject(RedisConstants.REDIS_BIDATA_ENTINFO + mainId);
			if (entInfo == null) {
				BidataRequest bidataRequest = getEntInfo(null, null, null, supplierDTO.getSupplierName(), null);
				entInfo = bidataRequest.getENT_INFO();
				if (entInfo != null) {
					redisUtil.setCacheObject(RedisConstants.REDIS_BIDATA_ENTINFO + mainId, entInfo,BiDataConstants.REDIS_TIMEOUT,TimeUnit.DAYS);
				}else {
					result.setMsg(supplierDTO.getSupplierName()+"获取失败");
				}
			}
			processingInformation(supplierInfo , entInfo);
			list.add(supplierInfo);
		}
		result.setData(list);
		return result;
	}
	
	
	/**
	 * @Description: 多节点关联查询生成订单
	 * @author mazc
	 * @param mutiNodesBody
	 * @return 
	 */
	@Override
	public Result<String> muti_nodes(MutiNodesBody mutiNodesBody) {
		Result<String> result = Result.ok();
		BidataMutiNodesRequest bidataRequest = redisUtil.getCacheObject(RedisConstants.REDIS_BIDATA_MUTI_NODES + mutiNodesBody.getId());
		if (bidataRequest == null) {
			synchronized (mutiNodesLock) {
				bidataRequest = redisUtil.getCacheObject(RedisConstants.REDIS_BIDATA_MUTI_NODES + mutiNodesBody.getId());
				if (bidataRequest == null) {
					String respons = getHttpResponsePost(BiDataConstants.MUTI_NODES,JsonUtil.jsonObjectToStr(mutiNodesBody.getMutiNodes()));
					bidataRequest = JsonUtil.jsonToBean(respons, BidataMutiNodesRequest.class);
					LOGGER.info("多节点关联查询生成订单" +"\t"+ "返回值" + respons);
					if (bidataRequest != null && bidataRequest.getCode() == 200) {
						redisUtil.setCacheObject(RedisConstants.REDIS_BIDATA_MUTI_NODES + mutiNodesBody.getId(), bidataRequest,BiDataConstants.REDIS_TIMEOUT, TimeUnit.DAYS);
					} else {
						result = Result.failed();
						result.setCode(Result.FAIL);
						result.setMsg(respons);
						return result;
					}
				}
			}
		}
		result.setMsg(bidataRequest.getMsg());
		result.setData(Optional.ofNullable(bidataRequest.getResult()).map(BidataMutiNodesOrder::getOrder).orElse(""));
		return result;
	}
	

	
	/** 
	 * @Description: 获取订单状态查询
	 * @author mazc
	 * @param order 订单号
	 * @return 
	 */
	public Result<BidataMutiNodesOrderStatus> order_status(String order) {
		Result<BidataMutiNodesOrderStatus> result = Result.ok();
		BidataMutiNodesOrderStatusRequest bidataMutiNodesOrderStatusRequest = redisUtil.getCacheObject(RedisConstants.REDIS_BIDATA_MUTI_NODES_ORDER_STATUS + order);
		if (bidataMutiNodesOrderStatusRequest == null) {
			synchronized (orderStatusLock) {
				Map<String, Object> paramMap = new HashMap<String, Object>();
				paramMap.put("order",order);
				String save = getHttpResponse(BiDataConstants.ORDER_STATUS, paramMap);
				bidataMutiNodesOrderStatusRequest = JsonUtil.jsonToBean(save, BidataMutiNodesOrderStatusRequest.class);
				LOGGER.info("获取订单状态查询" +"\t"+ "返回值" + save);
				if (bidataMutiNodesOrderStatusRequest != null ) {
					if((bidataMutiNodesOrderStatusRequest.getResult().getStatus() == 4||bidataMutiNodesOrderStatusRequest.getResult().getStatus() == 5)) {
						redisUtil.setCacheObject(RedisConstants.REDIS_BIDATA_MUTI_NODES_ORDER_STATUS + order, bidataMutiNodesOrderStatusRequest);
					}
				}else {
					result.setCode(Result.FAIL);
					result.setMsg(save);
					return result;
	 			}
			}
		}
		result.setMsg(bidataMutiNodesOrderStatusRequest.getMsg());
		result.setData(Optional.ofNullable(bidataMutiNodesOrderStatusRequest).map(BidataMutiNodesOrderStatusRequest::getResult).orElse(null));
		return result;
	}
	
	
	
	/** 
	 * @Description: 通过订单号获取文件内容
	 * @author mazc
	 * @param fileId
	 * @return 
	 */
	@Override
	public Result<String> getMutiNodesFileContent(String orderId) {
		Result<String> result = Result.ok();
		String content = redisUtil.getCacheObject(RedisConstants.REDIS_BIDATA_MUTI_NODES_ORDER_FILE_CONTENT + orderId);		
		if (StringUtil.isNullOrEmpty(content)) {
			synchronized (fileContentLock) {
				content = redisUtil.getCacheObject(RedisConstants.REDIS_BIDATA_MUTI_NODES_ORDER_FILE_CONTENT + orderId);		
				if (StringUtil.isNullOrEmpty(content)) {
					content = getFileContentSFTP(orderId);
					LOGGER.info("文件内容" +"\t"+ "返回值" + content);
					if (!StringUtil.isNullOrEmpty(content)) {
						redisUtil.setCacheObject(RedisConstants.REDIS_BIDATA_MUTI_NODES_ORDER_FILE_CONTENT + orderId, content);
					}else {
						result.setCode(Result.FAIL);
						result.setMsg("");
					}
				}
			}
		}
  		result.setData(content);
 		return result;
	}
	
	
	private String getFileContentSFTP(String orderId) {
		String strContent = "";
		File fileContent = null;
		try {
			//下载文件
			boolean downloadStatu = SFTPClientUtils.download(orderId + BiDataConstants.SUFFIX_ZIP, tempFile);
			if (downloadStatu) {
				if (unZip(orderId)) {// 解压文件
					Path deletePath = Paths.get(tempFile, orderId + BiDataConstants.SUFFIX_ZIP);
					File file = new File(deletePath.toString());
					if (file.exists()) {
						file.delete();
					}
					Path path = Paths.get(tempFile, orderId + BiDataConstants.SUFFIX_TXT);
					fileContent = new File(path.toString());
					if (fileContent.exists()) {
						// 读取文件
						strContent = new String(Files.readAllBytes(path), "UTF-8");
					}
				}
			}
		} catch (Exception e) {
			LOGGER.info("获取sftp文件内容getFileContentSFTP" +"\t" + e.getMessage());
			return null;
		}finally {
			if(fileContent != null && fileContent.exists()) {
				fileContent.delete();
			}
		}
		return strContent;
	}
	
	/**
	 * @Description: 解压文件
	 * @author mazc
	 * @param orderId
	 * @return 
	 */
	public boolean unZip(String orderId) {
		try {
			Path sourcePath = Paths.get(tempFile, orderId + BiDataConstants.SUFFIX_ZIP);
			FileSystem fs = FileSystems.newFileSystem(sourcePath, (ClassLoader) null);
			Files.walkFileTree(fs.getPath("/"), new SimpleFileVisitor<Path>() {
				public FileVisitResult visitFile(Path file, BasicFileAttributes attrs) throws IOException {
					Path target = Paths.get(sourcePath.getParent().toString(), file.getFileName().toString());
					if (Files.notExists(target.getParent())) {
						Files.createDirectories(target.getParent());
					}
					File targetFile = new File(target.toString());
					if (targetFile.exists()) {
						targetFile.delete();
					}
					Files.copy(file, target);
					fs.close();
					return FileVisitResult.CONTINUE;
				}
			});
		} catch (Exception e) {
			LOGGER.info("解压文件 unZip" +"\t" + e.getMessage());
			return false;
		}
		return true;
	}
	/**
	 * @Description: 处理企业风险信息
	 * @author mazc
	 * @param supplierInfo
	 * @param entInfo 
	 */
	private void processingInformation(SupplierInofVo supplierInfo, EntInfo entInfo) {
		// 被执行人风险信息
		supplierInfo.setPunishedSize(entInfo.getPUNISHED() == null ? 0 : entInfo.getPUNISHED().size());
		
		// 严重违法 信息
		supplierInfo.setBreaklawSize(entInfo.getBREAKLAW() == null ? 0 : entInfo.getBREAKLAW().size());
		
		// 企业异常名单
		supplierInfo.setExceptionlistSize(entInfo.getEXCEPTIONLIST() == null ? 0 : entInfo.getEXCEPTIONLIST().size());
		
		// 抵押信息
		supplierInfo.setMortgagebasicSize(entInfo.getMORTGAGEBASIC() == null ? 0 : entInfo.getMORTGAGEBASIC().size());
		
		// 行政处罚信息
		supplierInfo.setEntcasebaseinfoSize(entInfo.getENTCASEBASEINFO() == null ? 0 : entInfo.getENTCASEBASEINFO().size());
		
		// 法院判决
		supplierInfo.setJudicialaidSize(entInfo.getJUDICIALAID() == null ? 0 : entInfo.getENTCASEBASEINFO().size());
		
		// 股权出质公告
		supplierInfo.setStockpawnSize(entInfo.getSTOCKPAWN() == null ? 0 : entInfo.getSTOCKPAWN().size());
	}
	
	
	private BidataRequest getEntInfo(String id,String creditcode,String regno, String name, String orgcode) {
		Map<String, Object> paramMap = new HashMap<String, Object>();
 		paramMap.put("version","V1");
		paramMap.put("id",id);
		paramMap.put("creditcode",creditcode);
		paramMap.put("regno",regno);
		paramMap.put("name",name);
		paramMap.put("orgcode",orgcode);
		BidataRequest result = new BidataRequest();
		try {
			String save = getHttpResponse(BiDataConstants.ENTINFO, paramMap);
			LOGGER.info("企业详细信息 getEntInfo " +"\t" + save);
			result = JsonUtil.jsonToBean(save, BidataRequest.class);
  		} catch (Exception e) {
  			result.setMSG(e.getMessage());
	 		return result;
		}
		return result;
	}
	

	/**
	 * @Description: get 请求
	 * @author mazc
	 * @param strWebServerUrl
	 * @param paramMap
	 * @return 
	 */
	public String getHttpResponse(String strWebServerUrl, Map<String, Object> paramMap){
		HttpResponse httpResponse = HttpUtil.createGet(strWebServerUrl).setHeader(getHeaderMap(), true).timeout(50000).form(paramMap).execute();
		return httpResponse.body();
	}

	/**
	 * @Description: post 请求
	 * @author mazc
	 * @param strWebServerUrl
	 * @param body
	 * @return 
	 */
	public String getHttpResponsePost(String strWebServerUrl, String body){
		HttpResponse httpResponse = HttpUtil.createPost(strWebServerUrl).setHeader(getHeaderMap(), true).timeout(50000).body(body).execute();
		return httpResponse.body();
	}
	
	/**
	 * @Description: 封装表头
	 * @author mazc
	 * @return 
	 */
	private Map<String, String> getHeaderMap() {
		String nonce = String.valueOf(new Random().nextInt());
		String timestamp = String.valueOf(System.currentTimeMillis());
		String signature = encrypt(nonce + ";" + SftpConfiguration.SECURITYKEY + ";" + timestamp + ";" +SftpConfiguration.UID+ ";");
		Map<String, String> headerMap = new HashMap<String, String>();
		headerMap.put("Content-Type", "application/json");
		headerMap.put("X-Uid", SftpConfiguration.UID);
		headerMap.put("X-Nonce", nonce);
		headerMap.put("X-Timestamp", timestamp);
		headerMap.put("X-Signature", signature);
		return headerMap;
	}
	
	/**
	 * @Description: 内容进行加密
	 * @author mazc
	 * @param str
	 * @return 
	 */
	private static String encrypt(String str) {
		String result = "";
		try {
			MessageDigest digest = MessageDigest.getInstance("SHA-1");
			digest.update(str.getBytes());
			byte messageDigest[] = digest.digest();	// Create Hex String
			StringBuffer hexString = new StringBuffer();
			for (int i = 0; i < messageDigest.length; i++) {// 字节数组转换为 十六进制 数
				String shaHex = Integer.toHexString(messageDigest[i] & 0xFF);
				if (shaHex.length() < 2) {
					hexString.append(0);
				}
				hexString.append(shaHex);
			}
			result = hexString.toString();
			return result;

		} catch (NoSuchAlgorithmException e) {
		}
		return result;
	}
}
